@@ -1,6 +1,6 @@
 /*-------------------------------------------------------------------------
  *
- * heaptuple.c--
+ * heaptuple.c
  *	  This file contains heap tuple accessor and mutator routines, as well
  *	  as a few various tuple utilities.
  *
@@ -8,7 +8,7 @@
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/access/common/heaptuple.c,v 1.45 1998/10/08 18:29:10 momjian Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/access/common/heaptuple.c,v 1.53 1999/06/12 14:07:20 momjian Exp $
  *
  * NOTES
  *	  The old interface functions have been converted to macros
@@ -36,12 +36,12 @@
 /* Used by heap_getattr() macro, for speed */
 long		heap_sysoffset[] = {
 /* Only the first one is pass-by-ref, and is handled specially in the macro */
-	offsetof(HeapTupleData, t_ctid),
-	offsetof(HeapTupleData, t_oid),
-	offsetof(HeapTupleData, t_xmin),
-	offsetof(HeapTupleData, t_cmin),
-	offsetof(HeapTupleData, t_xmax),
-	offsetof(HeapTupleData, t_cmax)
+	offsetof(HeapTupleHeaderData, t_ctid),
+	offsetof(HeapTupleHeaderData, t_oid),
+	offsetof(HeapTupleHeaderData, t_xmin),
+	offsetof(HeapTupleHeaderData, t_cmin),
+	offsetof(HeapTupleHeaderData, t_xmax),
+	offsetof(HeapTupleHeaderData, t_cmax)
 };
 
 /* ----------------------------------------------------------------
@@ -124,7 +124,7 @@ DataFill(char *data,
 			*bitP |= bitmask;
 		}
 
-		data = (char *)att_align((long)data, att[i]->attlen, att[i]->attalign);
+		data = (char *) att_align((long) data, att[i]->attlen, att[i]->attalign);
 		switch (att[i]->attlen)
 		{
 			case -1:
@@ -151,7 +151,7 @@ DataFill(char *data,
 						att[i]->attlen);
 				break;
 		}
-		data = (char *)att_addlength((long)data, att[i]->attlen, value[i]);
+		data = (char *) att_addlength((long) data, att[i]->attlen, value[i]);
 	}
 }
 
@@ -167,14 +167,14 @@ DataFill(char *data,
 int
 heap_attisnull(HeapTuple tup, int attnum)
 {
-	if (attnum > (int) tup->t_natts)
+	if (attnum > (int) tup->t_data->t_natts)
 		return 1;
 
 	if (HeapTupleNoNulls(tup))
 		return 0;
 
 	if (attnum > 0)
-		return att_isnull(attnum - 1, tup->t_bits);
+		return att_isnull(attnum - 1, tup->t_data->t_bits);
 	else
 		switch (attnum)
 		{
@@ -210,7 +210,7 @@ heap_attisnull(HeapTuple tup, int attnum)
 int
 heap_sysattrlen(AttrNumber attno)
 {
-	HeapTupleData *f = NULL;
+	HeapTupleHeader f = NULL;
 
 	switch (attno)
 	{
@@ -301,6 +301,7 @@ heap_getsysattr(HeapTuple tup, Buffer b, int attnum)
 	}
 	return (Datum) NULL;
 }
+
 #endif
 
 /* ----------------
@@ -311,27 +312,28 @@ heap_getsysattr(HeapTuple tup, Buffer b, int attnum)
  *
  *		This caches attribute offsets in the attribute descriptor.
  *
- *		an alternate way to speed things up would be to cache offsets
+ *		An alternate way to speed things up would be to cache offsets
  *		with the tuple, but that seems more difficult unless you take
  *		the storage hit of actually putting those offsets into the
  *		tuple you send to disk.  Yuck.
  *
  *		This scheme will be slightly slower than that, but should
- *		preform well for queries which hit large #'s of tuples.  After
+ *		perform well for queries which hit large #'s of tuples.  After
  *		you cache the offsets once, examining all the other tuples using
  *		the same attribute descriptor will go much quicker. -cim 5/4/91
  * ----------------
  */
 Datum
-nocachegetattr(HeapTuple tup,
+nocachegetattr(HeapTuple tuple,
 			   int attnum,
 			   TupleDesc tupleDesc,
 			   bool *isnull)
 {
 	char	   *tp;				/* ptr to att in tuple */
+	HeapTupleHeader tup = tuple->t_data;
 	bits8	   *bp = tup->t_bits;		/* ptr to att in tuple */
-	int			slow;			/* do we have to walk nulls? */
 	Form_pg_attribute *att = tupleDesc->attrs;
+	int			slow = 0;		/* do we have to walk nulls? */
 
 
 #if IN_MACRO
@@ -342,6 +344,8 @@ nocachegetattr(HeapTuple tup,
 		*isnull = false;
 #endif
 
+	attnum--;
+
 	/* ----------------
 	 *	 Three cases:
 	 *
@@ -351,10 +355,8 @@ nocachegetattr(HeapTuple tup,
 	 * ----------------
 	 */
 
-	if (HeapTupleNoNulls(tup))
+	if (HeapTupleNoNulls(tuple))
 	{
-		attnum--;
-
 #if IN_MACRO
 /* This is handled in the macro */
 		if (att[attnum]->attcacheoff != -1)
@@ -372,8 +374,6 @@ nocachegetattr(HeapTuple tup,
 			return (Datum) fetchatt(&(att[0]), (char *) tup + tup->t_hoff);
 		}
 #endif
-
-		slow = 0;
 	}
 	else
 	{
@@ -382,10 +382,6 @@ nocachegetattr(HeapTuple tup,
 		 * there's a null somewhere in the tuple
 		 */
 
-		tp = (char *) tup + tup->t_hoff;
-		slow = 0;
-		attnum--;
-
 		/* ----------------
 		 *		check to see if desired att is null
 		 * ----------------
@@ -402,34 +398,28 @@ nocachegetattr(HeapTuple tup,
 #endif
 
 		/* ----------------
-		 *		Now check to see if any preceeding bits are null...
+		 *		Now check to see if any preceding bits are null...
 		 * ----------------
 		 */
 		{
-			int			i = 0;	/* current offset in bp */
-			int			mask;	/* bit in byte we're looking at */
-			char		n;		/* current byte in bp */
-			int			byte,
-						finalbit;
+			int			byte = attnum >> 3;
+			int			finalbit = attnum & 0x07;
 
-			byte = attnum >> 3;
-			finalbit = attnum & 0x07;
-
-			for (; i <= byte && !slow; i++)
+			/* check for nulls "before" final bit of last byte */
+			if ((~bp[byte]) & ((1 << finalbit) - 1))
+				slow = 1;
+			else
 			{
-				n = bp[i];
-				if (i < byte)
+				/* check for nulls in any "earlier" bytes */
+				int			i;
+
+				for (i = 0; i < byte; i++)
 				{
-					/* check for nulls in any "earlier" bytes */
-					if ((~n) != 0)
-						slow = 1;
-				}
-				else
-				{
-					/* check for nulls "before" final bit of last byte */
-					mask = (1 << finalbit) - 1;
-					if ((~n) & mask)
+					if (bp[i] != 0xFF)
+					{
 						slow = 1;
+						break;
+					}
 				}
 			}
 		}
@@ -448,25 +438,31 @@ nocachegetattr(HeapTuple tup,
 									tp + att[attnum]->attcacheoff);
 		}
 		else if (attnum == 0)
-			return (Datum) fetchatt(&(att[0]), (char *) tp);
-		else if (!HeapTupleAllFixed(tup))
+			return (Datum) fetchatt(&(att[0]), tp);
+		else if (!HeapTupleAllFixed(tuple))
 		{
-			int			j = 0;
+			int			j;
 
 			/*
 			 * In for(), we make this <= and not < because we want to test
 			 * if we can go past it in initializing offsets.
 			 */
-			for (j = 0; j <= attnum && !slow; j++)
+			for (j = 0; j <= attnum; j++)
+			{
 				if (att[j]->attlen < 1 && !VARLENA_FIXED_SIZE(att[j]))
+				{
 					slow = 1;
+					break;
+				}
+			}
 		}
 	}
 
 	/*
-	 * if slow is zero, and we got here, we know that we have a tuple with
-	 * no nulls.  We also have to initialize the remainder of the
-	 * attribute cached offset values.
+	 * If slow is zero, and we got here, we know that we have a tuple with
+	 * no nulls or varlenas before the target attribute. If possible, we
+	 * also want to initialize the remainder of the attribute cached
+	 * offset values.
 	 */
 	if (!slow)
 	{
@@ -491,8 +487,8 @@ nocachegetattr(HeapTuple tup,
 		/* Can we compute more?  We will probably need them */
 			 (j < tup->t_natts &&
 			  att[j]->attcacheoff == -1 &&
-			  (HeapTupleNoNulls(tup) || !att_isnull(j, bp)) &&
-			  (HeapTupleAllFixed(tup) ||
+			  (HeapTupleNoNulls(tuple) || !att_isnull(j, bp)) &&
+			  (HeapTupleAllFixed(tuple) ||
 			   att[j]->attlen > 0 || VARLENA_FIXED_SIZE(att[j]))); j++)
 		{
 
@@ -527,7 +523,7 @@ nocachegetattr(HeapTuple tup,
 
 		for (i = 0; i < attnum; i++)
 		{
-			if (!HeapTupleNoNulls(tup))
+			if (!HeapTupleNoNulls(tuple))
 			{
 				if (att_isnull(i, bp))
 				{
@@ -549,7 +545,8 @@ nocachegetattr(HeapTuple tup,
 
 			off = att_addlength(off, att[i]->attlen, tp + off);
 
-			if (att[i]->attlen == -1 && !VARLENA_FIXED_SIZE(att[i]))
+			if (usecache &&
+				att[i]->attlen == -1 && !VARLENA_FIXED_SIZE(att[i]))
 				usecache = false;
 		}
 
@@ -570,14 +567,41 @@ heap_copytuple(HeapTuple tuple)
 {
 	HeapTuple	newTuple;
 
-	if (!HeapTupleIsValid(tuple))
+	if (!HeapTupleIsValid(tuple) || tuple->t_data == NULL)
 		return NULL;
 
-	newTuple = (HeapTuple) palloc(tuple->t_len);
-	memmove((char *) newTuple, (char *) tuple, (int) tuple->t_len);
+	newTuple = (HeapTuple) palloc(HEAPTUPLESIZE + tuple->t_len);
+	newTuple->t_len = tuple->t_len;
+	newTuple->t_self = tuple->t_self;
+	newTuple->t_data = (HeapTupleHeader) ((char *) newTuple + HEAPTUPLESIZE);
+	memmove((char *) newTuple->t_data,
+			(char *) tuple->t_data, (int) tuple->t_len);
 	return newTuple;
 }
 
+/* ----------------
+ *		heap_copytuple_with_tuple
+ *
+ *		returns a copy of an tuple->t_data
+ * ----------------
+ */
+void
+heap_copytuple_with_tuple(HeapTuple src, HeapTuple dest)
+{
+	if (!HeapTupleIsValid(src) || src->t_data == NULL)
+	{
+		dest->t_data = NULL;
+		return;
+	}
+
+	dest->t_len = src->t_len;
+	dest->t_self = src->t_self;
+	dest->t_data = (HeapTupleHeader) palloc(src->t_len);
+	memmove((char *) dest->t_data,
+			(char *) src->t_data, (int) src->t_len);
+	return;
+}
+
 #ifdef NOT_USED
 /* ----------------
  *		heap_deformtuple
@@ -637,8 +661,8 @@ heap_formtuple(TupleDesc tupleDescriptor,
 			   Datum *value,
 			   char *nulls)
 {
-	char	   *tp;				/* tuple pointer */
 	HeapTuple	tuple;			/* return tuple */
+	HeapTupleHeader td;			/* tuple data */
 	int			bitmaplen;
 	long		len;
 	int			hoff;
@@ -646,7 +670,7 @@ heap_formtuple(TupleDesc tupleDescriptor,
 	int			i;
 	int			numberOfAttributes = tupleDescriptor->natts;
 
-	len = offsetof(HeapTupleData, t_bits);
+	len = offsetof(HeapTupleHeaderData, t_bits);
 
 	for (i = 0; i < numberOfAttributes && !hasnull; i++)
 	{
@@ -668,23 +692,24 @@ heap_formtuple(TupleDesc tupleDescriptor,
 
 	len += ComputeDataSize(tupleDescriptor, value, nulls);
 
-	tp = (char *) palloc(len);
-	tuple = (HeapTuple) tp;
+	tuple = (HeapTuple) palloc(HEAPTUPLESIZE + len);
+	td = tuple->t_data = (HeapTupleHeader) ((char *) tuple + HEAPTUPLESIZE);
 
-	MemSet(tp, 0, (int) len);
+	MemSet((char *) td, 0, (int) len);
 
 	tuple->t_len = len;
-	tuple->t_natts = numberOfAttributes;
-	tuple->t_hoff = hoff;
+	ItemPointerSetInvalid(&(tuple->t_self));
+	td->t_natts = numberOfAttributes;
+	td->t_hoff = hoff;
 
-	DataFill((char *) tuple + tuple->t_hoff,
+	DataFill((char *) td + td->t_hoff,
 			 tupleDescriptor,
 			 value,
 			 nulls,
-			 &tuple->t_infomask,
-			 (hasnull ? tuple->t_bits : NULL));
+			 &td->t_infomask,
+			 (hasnull ? td->t_bits : NULL));
 
-	tuple->t_infomask |= HEAP_XMAX_INVALID;
+	td->t_infomask |= HEAP_XMAX_INVALID;
 
 	return tuple;
 }
@@ -738,11 +763,10 @@ heap_modifytuple(HeapTuple tuple,
 
 		if (repl[attoff] == ' ')
 		{
-			value[attoff] =
-				heap_getattr(tuple,
-							 AttrOffsetGetAttrNumber(attoff),
-							 RelationGetDescr(relation),
-							 &isNull);
+			value[attoff] = heap_getattr(tuple,
+										 AttrOffsetGetAttrNumber(attoff),
+										 RelationGetDescr(relation),
+										 &isNull);
 			nulls[attoff] = (isNull) ? 'n' : ' ';
 
 		}
@@ -767,13 +791,15 @@ heap_modifytuple(HeapTuple tuple,
 	 *	copy the header except for t_len, t_natts, t_hoff, t_bits, t_infomask
 	 * ----------------
 	 */
-	infomask = newTuple->t_infomask;
-	memmove((char *) &newTuple->t_oid,	/* XXX */
-			(char *) &tuple->t_oid,
-			((char *) &tuple->t_hoff - (char *) &tuple->t_oid));		/* XXX */
-	newTuple->t_infomask = infomask;
-	newTuple->t_natts = numberOfAttributes;		/* fix t_natts just in
-												 * case */
+	infomask = newTuple->t_data->t_infomask;
+	memmove((char *) &newTuple->t_data->t_oid,	/* XXX */
+			(char *) &tuple->t_data->t_oid,
+			((char *) &tuple->t_data->t_hoff -
+			 (char *) &tuple->t_data->t_oid));	/* XXX */
+	newTuple->t_data->t_infomask = infomask;
+	newTuple->t_data->t_natts = numberOfAttributes;
+	newTuple->t_self = tuple->t_self;
+
 	return newTuple;
 }
 
@@ -787,28 +813,30 @@ heap_addheader(uint32 natts,	/* max domain index */
 			   int structlen,	/* its length */
 			   char *structure) /* pointer to the struct */
 {
-	char	   *tp;				/* tuple data pointer */
-	HeapTuple	tup;
+	HeapTuple	tuple;
+	HeapTupleHeader td;			/* tuple data */
 	long		len;
 	int			hoff;
 
 	AssertArg(natts > 0);
 
-	len = offsetof(HeapTupleData, t_bits);
+	len = offsetof(HeapTupleHeaderData, t_bits);
 
 	hoff = len = DOUBLEALIGN(len);		/* be conservative */
 	len += structlen;
-	tp = (char *) palloc(len);
-	tup = (HeapTuple) tp;
-	MemSet((char *) tup, 0, len);
+	tuple = (HeapTuple) palloc(HEAPTUPLESIZE + len);
+	td = tuple->t_data = (HeapTupleHeader) ((char *) tuple + HEAPTUPLESIZE);
 
-	tup->t_len = len;
-	tp += tup->t_hoff = hoff;
-	tup->t_natts = natts;
-	tup->t_infomask = 0;
-	tup->t_infomask |= HEAP_XMAX_INVALID;
+	MemSet((char *) td, 0, (int) len);
 
-	memmove(tp, structure, structlen);
+	tuple->t_len = len;
+	ItemPointerSetInvalid(&(tuple->t_self));
+	td->t_hoff = hoff;
+	td->t_natts = natts;
+	td->t_infomask = 0;
+	td->t_infomask |= HEAP_XMAX_INVALID;
 
-	return tup;
+	memmove((char *) td + hoff, structure, structlen);
+
+	return tuple;
 }
