@@ -1,6 +1,6 @@
 /*-------------------------------------------------------------------------
  *
- * gist.c--
+ * gist.c
  *	  interface routines for the postgres GiST index access method.
  *
  *
@@ -33,42 +33,32 @@
 #endif
 
 /* non-export function prototypes */
-static InsertIndexResult
-gistdoinsert(Relation r, IndexTuple itup,
+static InsertIndexResult gistdoinsert(Relation r, IndexTuple itup,
 			 GISTSTATE *GISTstate);
-static InsertIndexResult
-gistentryinsert(Relation r, GISTSTACK *stk,
+static InsertIndexResult gistentryinsert(Relation r, GISTSTACK *stk,
 				IndexTuple tup,
 				GISTSTATE *giststate);
-static void
-gistentryinserttwo(Relation r, GISTSTACK *stk, IndexTuple ltup,
+static void gistentryinserttwo(Relation r, GISTSTACK *stk, IndexTuple ltup,
 				   IndexTuple rtup, GISTSTATE *giststate);
-static void
-gistAdjustKeys(Relation r, GISTSTACK *stk, BlockNumber blk,
+static void gistAdjustKeys(Relation r, GISTSTACK *stk, BlockNumber blk,
 			   char *datum, int att_size, GISTSTATE *giststate);
-static void
-gistintinsert(Relation r, GISTSTACK *stk, IndexTuple ltup,
+static void gistintinsert(Relation r, GISTSTACK *stk, IndexTuple ltup,
 			  IndexTuple rtup, GISTSTATE *giststate);
-static InsertIndexResult
-gistSplit(Relation r, Buffer buffer,
+static InsertIndexResult gistSplit(Relation r, Buffer buffer,
 		  GISTSTACK *stack, IndexTuple itup,
 		  GISTSTATE *giststate);
-static void
-gistnewroot(GISTSTATE *giststate, Relation r, IndexTuple lt,
+static void gistnewroot(GISTSTATE *giststate, Relation r, IndexTuple lt,
 			IndexTuple rt);
 static void GISTInitBuffer(Buffer b, uint32 f);
-static BlockNumber
-gistChooseSubtree(Relation r, IndexTuple itup, int level,
+static BlockNumber gistChooseSubtree(Relation r, IndexTuple itup, int level,
 				  GISTSTATE *giststate,
 				  GISTSTACK **retstack, Buffer *leafbuf);
-static OffsetNumber
-gistchoose(Relation r, Page p, IndexTuple it,
+static OffsetNumber gistchoose(Relation r, Page p, IndexTuple it,
 		   GISTSTATE *giststate);
 static int	gistnospace(Page p, IndexTuple it);
 void		gistdelete(Relation r, ItemPointer tid);
 static IndexTuple gist_tuple_replacekey(Relation r, GISTENTRY entry, IndexTuple t);
-static void
-gistcentryinit(GISTSTATE *giststate, GISTENTRY *e, char *pr,
+static void gistcentryinit(GISTSTATE *giststate, GISTENTRY *e, char *pr,
 			   Relation r, Page pg, OffsetNumber o, int b, bool l);
 static char *int_range_out(INTRANGE *r);
 
@@ -87,7 +77,6 @@ gistbuild(Relation heap,
 		  PredInfo *predInfo)
 {
 	HeapScanDesc scan;
-	Buffer		buffer;
 	AttrNumber	i;
 	HeapTuple	htup;
 	IndexTuple	itup;
@@ -112,14 +101,14 @@ gistbuild(Relation heap,
 			   *oldPred;
 	GISTSTATE	giststate;
 	GISTENTRY	tmpcentry;
+	Buffer		buffer = InvalidBuffer;
 	bool	   *compvec;
 
-	/* GiSTs only know how to do stupid locking now */
-	RelationSetLockForWrite(index);
+	/* no locking is needed */
 
-	setheapoverride(TRUE);		/* so we can see the new pg_index tuple */
+	setheapoverride(true);		/* so we can see the new pg_index tuple */
 	initGISTstate(&giststate, index);
-	setheapoverride(FALSE);
+	setheapoverride(false);
 
 	pred = predInfo->pred;
 	oldPred = predInfo->oldPred;
@@ -130,7 +119,7 @@ gistbuild(Relation heap,
 	 */
 
 	if (oldPred == NULL && (nb = RelationGetNumberOfBlocks(index)) != 0)
-		elog(ERROR, "%.16s already contains data", &(index->rd_rel->relname.data[0]));
+		elog(ERROR, "%s already contains data", index->rd_rel->relname.data);
 
 	/* initialize the root page (if this is a new index) */
 	if (oldPred == NULL)
@@ -141,8 +130,8 @@ gistbuild(Relation heap,
 	}
 
 	/* init the tuple descriptors and get set for a heap scan */
-	hd = RelationGetTupleDescriptor(heap);
-	id = RelationGetTupleDescriptor(index);
+	hd = RelationGetDescr(heap);
+	id = RelationGetDescr(index);
 	d = (Datum *) palloc(natts * sizeof(*d));
 	nulls = (bool *) palloc(natts * sizeof(*nulls));
 
@@ -169,16 +158,14 @@ gistbuild(Relation heap,
 		slot = NULL;
 		econtext = NULL;
 	}
-#endif							/* OMIT_PARTIAL_INDEX */
-	scan = heap_beginscan(heap, 0, false, 0, (ScanKey) NULL);
-	htup = heap_getnext(scan, 0, &buffer);
-
+#endif	 /* OMIT_PARTIAL_INDEX */
 	/* int the tuples as we insert them */
 	nh = ni = 0;
 
-	for (; HeapTupleIsValid(htup); htup = heap_getnext(scan, 0, &buffer))
-	{
+	scan = heap_beginscan(heap, 0, SnapshotNow, 0, (ScanKey) NULL);
 
+	while (HeapTupleIsValid(htup = heap_getnext(scan, 0)))
+	{
 		nh++;
 
 		/*
@@ -195,7 +182,7 @@ gistbuild(Relation heap,
 				ni++;
 				continue;
 			}
-#endif							/* OMIT_PARTIAL_INDEX */
+#endif	 /* OMIT_PARTIAL_INDEX */
 		}
 
 		/*
@@ -209,7 +196,7 @@ gistbuild(Relation heap,
 			slot->val = htup;
 			if (ExecQual((List *) pred, econtext) == false)
 				continue;
-#endif							/* OMIT_PARTIAL_INDEX */
+#endif	 /* OMIT_PARTIAL_INDEX */
 		}
 
 		ni++;
@@ -240,8 +227,7 @@ gistbuild(Relation heap,
 									  attoff,
 									  attnum,
 									  finfo,
-									  &attnull,
-									  buffer);
+									  &attnull);
 			nulls[attoff] = (attnull ? 'n' : ' ');
 		}
 
@@ -261,7 +247,7 @@ gistbuild(Relation heap,
 
 		/* form an index tuple and point it at the heap tuple */
 		itup = index_formtuple(id, &d[0], nulls);
-		itup->t_tid = htup->t_ctid;
+		itup->t_tid = htup->t_self;
 
 		/*
 		 * Since we already have the index relation locked, we call
@@ -282,14 +268,13 @@ gistbuild(Relation heap,
 
 	/* okay, all heap tuples are indexed */
 	heap_endscan(scan);
-	RelationUnsetLockForWrite(index);
 
 	if (pred != NULL || oldPred != NULL)
 	{
 #ifndef OMIT_PARTIAL_INDEX
 		ExecDestroyTupleTable(tupleTable, true);
 		pfree(econtext);
-#endif							/* OMIT_PARTIAL_INDEX */
+#endif	 /* OMIT_PARTIAL_INDEX */
 	}
 
 	/*
@@ -302,8 +287,8 @@ gistbuild(Relation heap,
 	 * flushed.  We close them to guarantee that they will be.
 	 */
 
-	hrelid = heap->rd_id;
-	irelid = index->rd_id;
+	hrelid = RelationGetRelid(heap);
+	irelid = RelationGetRelid(index);
 	heap_close(heap);
 	index_close(index);
 
@@ -353,10 +338,15 @@ gistinsert(Relation r, Datum *datum, char *nulls, ItemPointer ht_ctid, Relation
 			compvec[i] = FALSE;
 		datum[i] = (Datum) tmpentry.pred;
 	}
-	itup = index_formtuple(RelationGetTupleDescriptor(r), datum, nulls);
+	itup = index_formtuple(RelationGetDescr(r), datum, nulls);
 	itup->t_tid = *ht_ctid;
 
-	RelationSetLockForWrite(r);
+	/*
+	 * Notes in ExecUtils:ExecOpenIndices()
+	 *
+	 * RelationSetLockForWrite(r);
+	 */
+
 	res = gistdoinsert(r, itup, &giststate);
 	for (i = 0; i < r->rd_att->natts; i++)
 		if (compvec[i] == TRUE)
@@ -364,8 +354,7 @@ gistinsert(Relation r, Datum *datum, char *nulls, ItemPointer ht_ctid, Relation
 	pfree(itup);
 	pfree(compvec);
 
-	/* XXX two-phase locking -- don't unlock the relation until EOT */
-	return (res);
+	return res;
 }
 
 /*
@@ -434,7 +423,7 @@ gistdoinsert(Relation r,
 		res = gistSplit(r, buffer, stack, itup, giststate);
 		gistfreestack(stack);
 		WriteBuffer(buffer);	/* don't forget to release buffer! */
-		return (res);
+		return res;
 	}
 
 	if (PageIsEmpty(page))
@@ -461,7 +450,7 @@ gistdoinsert(Relation r,
 	res = (InsertIndexResult) palloc(sizeof(InsertIndexResultData));
 	ItemPointerSet(&(res->pointerData), blk, l);
 
-	return (res);
+	return res;
 }
 
 
@@ -515,7 +504,7 @@ gistChooseSubtree(Relation r, IndexTuple itup,	/* itup has compressed
 	*retstack = stack;
 	*leafbuf = buffer;
 
-	return (blk);
+	return blk;
 }
 
 
@@ -571,7 +560,7 @@ gistAdjustKeys(Relation r,
 	(*fmgr_faddr(&giststate->equalFn)) (ev0p->pred, datum, &result);
 	if (!result)
 	{
-		TupleDesc	td = RelationGetTupleDescriptor(r);
+		TupleDesc	td = RelationGetDescr(r);
 
 		/* compress datum for storage on page */
 		gistcentryinit(giststate, &centry, datum, ev0p->rel, ev0p->page,
@@ -632,9 +621,7 @@ gistAdjustKeys(Relation r,
 			pfree(datum);
 	}
 	else
-	{
 		ReleaseBuffer(b);
-	}
 	pfree(evec);
 }
 
@@ -834,9 +821,7 @@ gistSplit(Relation r,
 	}
 
 	if ((bufblock = BufferGetBlockNumber(buffer)) != GISTP_ROOT)
-	{
 		PageRestoreTempPage(left, p);
-	}
 	WriteBuffer(leftbuf);
 	WriteBuffer(rightbuf);
 
@@ -876,7 +861,7 @@ gistSplit(Relation r,
 	pfree(ltup);
 	pfree(rtup);
 
-	return (res);
+	return res;
 }
 
 /*
@@ -969,7 +954,7 @@ gistentryinsert(Relation r, GISTSTACK *stk, IndexTuple tup,
 		res = gistSplit(r, b, stk->gs_parent, tup, giststate);
 		WriteBuffer(b);			/* don't forget to release buffer!  -
 								 * 01/31/94 */
-		return (res);
+		return res;
 	}
 	else
 	{
@@ -985,7 +970,7 @@ gistentryinsert(Relation r, GISTSTACK *stk, IndexTuple tup,
 			pfree(tmpentry.pred);
 		if (tup != newtup)
 			pfree(newtup);
-		return (res);
+		return res;
 	}
 }
 
@@ -1086,13 +1071,13 @@ gistchoose(Relation r, Page p, IndexTuple it,	/* it has compressed entry */
 	if (identry.pred != id)
 		pfree(identry.pred);
 
-	return (which);
+	return which;
 }
 
 static int
 gistnospace(Page p, IndexTuple it)
 {
-	return (PageGetFreeSpace(p) < IndexTupleSize(it));
+	return PageGetFreeSpace(p) < IndexTupleSize(it);
 }
 
 void
@@ -1120,8 +1105,12 @@ gistdelete(Relation r, ItemPointer tid)
 	Buffer		buf;
 	Page		page;
 
-	/* must write-lock on delete */
-	RelationSetLockForWrite(r);
+	/*
+	 * Notes in ExecUtils:ExecOpenIndices() Also note that only vacuum
+	 * deletes index tuples now...
+	 *
+	 * RelationSetLockForWrite(r);
+	 */
 
 	blkno = ItemPointerGetBlockNumber(tid);
 	offnum = ItemPointerGetOffsetNumber(tid);
@@ -1137,7 +1126,6 @@ gistdelete(Relation r, ItemPointer tid)
 
 	WriteBuffer(buf);
 
-	/* XXX -- two-phase locking, don't release the write lock */
 }
 
 void
@@ -1151,7 +1139,7 @@ initGISTstate(GISTSTATE *giststate, Relation index)
 				picksplit_proc,
 				equal_proc;
 	HeapTuple	htup;
-	IndexTupleForm itupform;
+	Form_pg_index itupform;
 
 	consistent_proc = index_getprocid(index, 1, GIST_CONSISTENT_PROC);
 	union_proc = index_getprocid(index, 1, GIST_UNION_PROC);
@@ -1169,11 +1157,13 @@ initGISTstate(GISTSTATE *giststate, Relation index)
 	fmgr_info(equal_proc, &giststate->equalFn);
 
 	/* see if key type is different from type of attribute being indexed */
-	htup = SearchSysCacheTuple(INDEXRELID, ObjectIdGetDatum(index->rd_id),
+	htup = SearchSysCacheTuple(INDEXRELID,
+							   ObjectIdGetDatum(RelationGetRelid(index)),
 							   0, 0, 0);
-	itupform = (IndexTupleForm) GETSTRUCT(htup);
+	itupform = (Form_pg_index) GETSTRUCT(htup);
 	if (!HeapTupleIsValid(htup))
-		elog(ERROR, "initGISTstate: index %d not found", index->rd_id);
+		elog(ERROR, "initGISTstate: index %u not found",
+			 RelationGetRelid(index));
 	giststate->haskeytype = itupform->indhaskeytype;
 	if (giststate->haskeytype)
 	{
@@ -1184,11 +1174,11 @@ initGISTstate(GISTSTATE *giststate, Relation index)
 								   0, 0);
 		if (!HeapTupleIsValid(htup))
 		{
-			elog(ERROR, "initGISTstate: no attribute tuple %d %d",
+			elog(ERROR, "initGISTstate: no attribute tuple %u %d",
 				 itupform->indexrelid, FirstOffsetNumber);
 			return;
 		}
-		giststate->keytypbyval = (((AttributeTupleForm) htup)->attbyval);
+		giststate->keytypbyval = (((Form_pg_attribute) htup)->attbyval);
 	}
 	else
 		giststate->keytypbyval = FALSE;
@@ -1215,7 +1205,7 @@ gist_tuple_replacekey(Relation r, GISTENTRY entry, IndexTuple t)
 		/* or in new size */
 		t->t_info |= MAXALIGN(entry.bytes + sizeof(IndexTupleData));
 
-		return (t);
+		return t;
 	}
 	else
 	{
@@ -1233,7 +1223,7 @@ gist_tuple_replacekey(Relation r, GISTENTRY entry, IndexTuple t)
 											  isnull);
 		newtup->t_tid = t->t_tid;
 		pfree(isnull);
-		return (newtup);
+		return newtup;
 	}
 }
 
@@ -1341,45 +1331,17 @@ _gistdump(Relation r)
 	}
 }
 
-#ifdef NOT_USED
-static char *
-text_range_out(TXTRANGE *r)
-{
-	char	   *result;
-	char	   *lower,
-			   *upper;
-
-	if (r == NULL)
-		return (NULL);
-	result = (char *) palloc(16 + VARSIZE(TRLOWER(r)) + VARSIZE(TRUPPER(r))
-							 - 2 * VARHDRSZ);
-
-	lower = (char *) palloc(VARSIZE(TRLOWER(r)) + 1 - VARHDRSZ);
-	memcpy(lower, VARDATA(TRLOWER(r)), VARSIZE(TRLOWER(r)) - VARHDRSZ);
-	lower[VARSIZE(TRLOWER(r)) - VARHDRSZ] = '\0';
-	upper = (char *) palloc(VARSIZE(TRUPPER(r)) + 1 - VARHDRSZ);
-	memcpy(upper, VARDATA(TRUPPER(r)), VARSIZE(TRUPPER(r)) - VARHDRSZ);
-	upper[VARSIZE(TRUPPER(r)) - VARHDRSZ] = '\0';
-
-	sprintf(result, "[%s,%s): %d", lower, upper, r->flag);
-	pfree(lower);
-	pfree(upper);
-	return (result);
-}
-
-#endif
-
 static char *
 int_range_out(INTRANGE *r)
 {
 	char	   *result;
 
 	if (r == NULL)
-		return (NULL);
+		return NULL;
 	result = (char *) palloc(80);
-	sprintf(result, "[%d,%d): %d", r->lower, r->upper, r->flag);
+	snprintf(result, 80, "[%d,%d): %d", r->lower, r->upper, r->flag);
 
-	return (result);
+	return result;
 }
 
-#endif							/* defined GISTDEBUG */
+#endif	 /* defined GISTDEBUG */
