@@ -1,13 +1,13 @@
 /*-------------------------------------------------------------------------
  *
- * hash.c--
+ * hash.c
  *	  Implementation of Margo Seltzer's Hashing package for postgres.
  *
  * Copyright (c) 1994, Regents of the University of California
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/access/hash/hash.c,v 1.22 1998/09/01 04:26:48 momjian Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/access/hash/hash.c,v 1.26 1999/05/25 16:06:54 momjian Exp $
  *
  * NOTES
  *	  This file contains only the public interface routines.
@@ -216,7 +216,7 @@ hashbuild(Relation heap,
 			continue;
 		}
 
-		itup->t_tid = htup->t_ctid;
+		itup->t_tid = htup->t_self;
 		hitem = _hash_formitem(itup);
 		res = _hash_doinsert(index, hitem);
 		pfree(hitem);
@@ -485,10 +485,9 @@ hashrestrpos(IndexScanDesc scan)
 	/* bump lock on currentMarkData and copy to currentItemData */
 	if (ItemPointerIsValid(&(scan->currentMarkData)))
 	{
-		so->hashso_curbuf =
-			_hash_getbuf(scan->relation,
-						 BufferGetBlockNumber(so->hashso_mrkbuf),
-						 HASH_READ);
+		so->hashso_curbuf = _hash_getbuf(scan->relation,
+								 BufferGetBlockNumber(so->hashso_mrkbuf),
+										 HASH_READ);
 
 		scan->currentItemData = scan->currentMarkData;
 	}
