@@ -1,13 +1,13 @@
 /*-------------------------------------------------------------------------
  *
- * hashfunc.c--
+ * hashfunc.c
  *	  Comparison functions for hash access method.
  *
  * Copyright (c) 1994, Regents of the University of California
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/access/hash/hashfunc.c,v 1.12 1998/09/01 04:26:49 momjian Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/access/hash/hashfunc.c,v 1.18 1999/05/25 22:40:49 momjian Exp $
  *
  * NOTES
  *	  These functions are stored in pg_amproc.	For each operator class
@@ -19,6 +19,7 @@
 #include "postgres.h"
 
 #include "access/hash.h"
+#include "utils/int8.h"
 
 uint32
 hashint2(int16 key)
@@ -32,6 +33,12 @@ hashint4(uint32 key)
 	return ~key;
 }
 
+uint32
+hashint8(int64 *key)
+{
+	return ~((uint32) *key);
+}
+
 /* Hash function from Chris Torek. */
 uint32
 hashfloat4(float32 keyp)
