@@ -1,13 +1,13 @@
 /*-------------------------------------------------------------------------
  *
- * hashpage.c--
+ * hashpage.c
  *	  Hash table page management code for the Postgres hash access method
  *
  * Copyright (c) 1994, Regents of the University of California
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/access/hash/hashpage.c,v 1.17 1998/09/01 03:20:58 momjian Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/access/hash/hashpage.c,v 1.20 1999/05/25 16:06:58 momjian Exp $
  *
  * NOTES
  *	  Postgres hash pages look like ordinary relation pages.  The opaque
@@ -81,7 +81,7 @@ _hash_metapinit(Relation rel)
 
 	/* can't be sharing this with anyone, now... */
 	if (USELOCKING)
-		RelationSetLockForWrite(rel);
+		LockRelation(rel, AccessExclusiveLock);
 
 	if ((nblocks = RelationGetNumberOfBlocks(rel)) != 0)
 	{
@@ -169,7 +169,7 @@ _hash_metapinit(Relation rel)
 	_hash_relbuf(rel, metabuf, HASH_WRITE);
 
 	if (USELOCKING)
-		RelationUnsetLockForWrite(rel);
+		UnlockRelation(rel, AccessExclusiveLock);
 }
 
 /*
@@ -316,19 +316,16 @@ _hash_setpagelock(Relation rel,
 				  BlockNumber blkno,
 				  int access)
 {
-	ItemPointerData iptr;
 
 	if (USELOCKING)
 	{
-		ItemPointerSet(&iptr, blkno, 1);
-
 		switch (access)
 		{
-			case HASH_WRITE:
-				RelationSetSingleWLockPage(rel, &iptr);
+				case HASH_WRITE:
+				LockPage(rel, blkno, ExclusiveLock);
 				break;
 			case HASH_READ:
-				RelationSetSingleRLockPage(rel, &iptr);
+				LockPage(rel, blkno, ShareLock);
 				break;
 			default:
 				elog(ERROR, "_hash_setpagelock: invalid access (%d) on blk %x: %s",
@@ -343,19 +340,16 @@ _hash_unsetpagelock(Relation rel,
 					BlockNumber blkno,
 					int access)
 {
-	ItemPointerData iptr;
 
 	if (USELOCKING)
 	{
-		ItemPointerSet(&iptr, blkno, 1);
-
 		switch (access)
 		{
-			case HASH_WRITE:
-				RelationUnsetSingleWLockPage(rel, &iptr);
+				case HASH_WRITE:
+				UnlockPage(rel, blkno, ExclusiveLock);
 				break;
 			case HASH_READ:
-				RelationUnsetSingleRLockPage(rel, &iptr);
+				UnlockPage(rel, blkno, ShareLock);
 				break;
 			default:
 				elog(ERROR, "_hash_unsetpagelock: invalid access (%d) on blk %x: %s",
