@@ -1,13 +1,13 @@
 /*-------------------------------------------------------------------------
  *
- * btinsert.c--
+ * btinsert.c
  *	  Item insertion in Lehman and Yao btrees for Postgres.
  *
  * Copyright (c) 1994, Regents of the University of California
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/access/nbtree/nbtinsert.c,v 1.30 1998/09/01 04:27:01 momjian Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/access/nbtree/nbtinsert.c,v 1.42 1999/05/25 22:04:11 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -18,7 +18,7 @@
 #include <storage/bufpage.h>
 #include <access/nbtree.h>
 #include <access/heapam.h>
-#include <storage/bufmgr.h>
+#include <access/xact.h>
 #include <fmgr.h>
 
 #ifndef HAVE_MEMMOVE
@@ -63,11 +63,11 @@ _bt_doinsert(Relation rel, BTItem btitem, bool index_is_unique, Relation heapRel
 	/* find the page containing this key */
 	stack = _bt_search(rel, natts, itup_scankey, &buf);
 
-	blkno = BufferGetBlockNumber(buf);
-
 	/* trade in our read lock for a write lock */
-	_bt_relbuf(rel, buf, BT_READ);
-	buf = _bt_getbuf(rel, blkno, BT_WRITE);
+	LockBuffer(buf, BUFFER_LOCK_UNLOCK);
+	LockBuffer(buf, BT_WRITE);
+
+l1:
 
 	/*
 	 * If the page was split between the time that we surrendered our read
@@ -78,6 +78,7 @@ _bt_doinsert(Relation rel, BTItem btitem, bool index_is_unique, Relation heapRel
 	 */
 
 	buf = _bt_moveright(rel, buf, natts, itup_scankey, BT_WRITE);
+	blkno = BufferGetBlockNumber(buf);
 
 	/* if we're not allowing duplicates, make sure the key isn't */
 	/* already in the node */
@@ -97,12 +98,12 @@ _bt_doinsert(Relation rel, BTItem btitem, bool index_is_unique, Relation heapRel
 		if (!PageIsEmpty(page) && offset <= maxoff)
 		{
 			TupleDesc	itupdesc;
-			BTItem		btitem;
-			IndexTuple	itup;
-			HeapTuple	htup;
+			BTItem		cbti;
+			HeapTupleData htup;
 			BTPageOpaque opaque;
 			Buffer		nbuf;
-			BlockNumber blkno;
+			BlockNumber nblkno;
+			bool		chtup = true;
 
 			itupdesc = RelationGetDescr(rel);
 			nbuf = InvalidBuffer;
@@ -119,11 +120,44 @@ _bt_doinsert(Relation rel, BTItem btitem, bool index_is_unique, Relation heapRel
 			 */
 			while (_bt_isequal(itupdesc, page, offset, natts, itup_scankey))
 			{					/* they're equal */
-				btitem = (BTItem) PageGetItem(page, PageGetItemId(page, offset));
-				itup = &(btitem->bti_itup);
-				htup = heap_fetch(heapRel, SnapshotSelf, &(itup->t_tid), &buffer);
-				if (htup != (HeapTuple) NULL)
-				{				/* it is a duplicate */
+
+				/*
+				 * Have to check is inserted heap tuple deleted one (i.e.
+				 * just moved to another place by vacuum)!
+				 */
+				if (chtup)
+				{
+					htup.t_self = btitem->bti_itup.t_tid;
+					heap_fetch(heapRel, SnapshotDirty, &htup, &buffer);
+					if (htup.t_data == NULL)	/* YES! */
+						break;
+					/* Live tuple was inserted */
+					ReleaseBuffer(buffer);
+					chtup = false;
+				}
+				cbti = (BTItem) PageGetItem(page, PageGetItemId(page, offset));
+				htup.t_self = cbti->bti_itup.t_tid;
+				heap_fetch(heapRel, SnapshotDirty, &htup, &buffer);
+				if (htup.t_data != NULL)		/* it is a duplicate */
+				{
+					TransactionId xwait =
+					(TransactionIdIsValid(SnapshotDirty->xmin)) ?
+					SnapshotDirty->xmin : SnapshotDirty->xmax;
+
+					/*
+					 * If this tuple is being updated by other transaction
+					 * then we have to wait for its commit/abort.
+					 */
+					ReleaseBuffer(buffer);
+					if (TransactionIdIsValid(xwait))
+					{
+						if (nbuf != InvalidBuffer)
+							_bt_relbuf(rel, nbuf, BT_READ);
+						_bt_relbuf(rel, buf, BT_WRITE);
+						XactLockTableWait(xwait);
+						buf = _bt_getbuf(rel, blkno, BT_WRITE);
+						goto l1;/* continue from the begin */
+					}
 					elog(ERROR, "Cannot insert a duplicate key into a unique index");
 				}
 				/* htup null so no buffer to release */
@@ -142,12 +176,12 @@ _bt_doinsert(Relation rel, BTItem btitem, bool index_is_unique, Relation heapRel
 					 * min key of the right page is the same, ooh - so
 					 * many dead duplicates...
 					 */
-					blkno = opaque->btpo_next;
+					nblkno = opaque->btpo_next;
 					if (nbuf != InvalidBuffer)
 						_bt_relbuf(rel, nbuf, BT_READ);
 					for (nbuf = InvalidBuffer;;)
 					{
-						nbuf = _bt_getbuf(rel, blkno, BT_READ);
+						nbuf = _bt_getbuf(rel, nblkno, BT_READ);
 						page = BufferGetPage(nbuf);
 						maxoff = PageGetMaxOffsetNumber(page);
 						opaque = (BTPageOpaque) PageGetSpecialPointer(page);
@@ -158,10 +192,10 @@ _bt_doinsert(Relation rel, BTItem btitem, bool index_is_unique, Relation heapRel
 						}
 						else
 						{		/* Empty or "pseudo"-empty page - get next */
-							blkno = opaque->btpo_next;
+							nblkno = opaque->btpo_next;
 							_bt_relbuf(rel, nbuf, BT_READ);
 							nbuf = InvalidBuffer;
-							if (blkno == P_NONE)
+							if (nblkno == P_NONE)
 								break;
 						}
 					}
@@ -364,6 +398,7 @@ _bt_insertonpg(Relation rel,
 		OffsetNumber maxoff;
 		bool		shifted = false;
 		bool		left_chained = (lpageop->btpo_flags & BTP_CHAIN) ? true : false;
+		bool		is_root = lpageop->btpo_flags & BTP_ROOT;
 
 		/*
 		 * If we have to split leaf page in the chain of duplicates by new
@@ -536,14 +571,22 @@ _bt_insertonpg(Relation rel,
 		 * reasoning).
 		 */
 
+l_spl:	;
 		if (stack == (BTStack) NULL)
 		{
-
+			if (!is_root)		/* if this page was not root page */
+			{
+				elog(DEBUG, "btree: concurrent ROOT page split");
+				stack = (BTStack) palloc(sizeof(BTStackData));
+				stack->bts_blkno = lpageop->btpo_parent;
+				stack->bts_offset = InvalidOffsetNumber;
+				stack->bts_btitem = (BTItem) palloc(sizeof(BTItemData));
+				/* bts_btitem will be initialized below */
+				stack->bts_parent = NULL;
+				goto l_spl;
+			}
 			/* create a new root node and release the split buffers */
 			_bt_newroot(rel, buf, rbuf);
-			_bt_relbuf(rel, buf, BT_WRITE);
-			_bt_relbuf(rel, rbuf, BT_WRITE);
-
 		}
 		else
 		{
@@ -660,8 +703,8 @@ _bt_insertonpg(Relation rel,
 				{
 					_bt_updateitem(rel, keysz, pbuf,
 								   stack->bts_btitem, lowLeftItem);
-					_bt_relbuf(rel, buf, BT_WRITE);
-					_bt_relbuf(rel, rbuf, BT_WRITE);
+					_bt_wrtbuf(rel, buf);
+					_bt_wrtbuf(rel, rbuf);
 				}
 				else
 				{
@@ -683,11 +726,9 @@ _bt_insertonpg(Relation rel,
 
 					/*
 					 * Unlock the children before doing this
-					 *
-					 * Mmm ... I foresee problems here. - vadim 06/10/97
 					 */
-					_bt_relbuf(rel, buf, BT_WRITE);
-					_bt_relbuf(rel, rbuf, BT_WRITE);
+					_bt_wrtbuf(rel, buf);
+					_bt_wrtbuf(rel, rbuf);
 
 					/*
 					 * A regular _bt_binsrch should find the right place
@@ -712,8 +753,8 @@ _bt_insertonpg(Relation rel,
 			}
 			else
 			{
-				_bt_relbuf(rel, buf, BT_WRITE);
-				_bt_relbuf(rel, rbuf, BT_WRITE);
+				_bt_wrtbuf(rel, buf);
+				_bt_wrtbuf(rel, rbuf);
 			}
 
 			newskey = _bt_mkscankey(rel, &(new_item->bti_itup));
@@ -834,6 +875,8 @@ _bt_split(Relation rel, Buffer buf, OffsetNumber firstright)
 	lopaque->btpo_next = BufferGetBlockNumber(rbuf);
 	ropaque->btpo_next = oopaque->btpo_next;
 
+	lopaque->btpo_parent = ropaque->btpo_parent = oopaque->btpo_parent;
+
 	/*
 	 * If the page we're splitting is not the rightmost page at its level
 	 * in the tree, then the first (0) entry on the page is the high key
@@ -1084,6 +1127,7 @@ _bt_newroot(Relation rel, Buffer lbuf, Buffer rbuf)
 	/* get a new root page */
 	rootbuf = _bt_getbuf(rel, P_NEW, BT_WRITE);
 	rootpage = BufferGetPage(rootbuf);
+	rootbknum = BufferGetBlockNumber(rootbuf);
 	_bt_pageinit(rootpage, BufferGetPageSize(rootbuf));
 
 	/* set btree special data */
@@ -1100,6 +1144,10 @@ _bt_newroot(Relation rel, Buffer lbuf, Buffer rbuf)
 	lpage = BufferGetPage(lbuf);
 	rpage = BufferGetPage(rbuf);
 
+	((BTPageOpaque) PageGetSpecialPointer(lpage))->btpo_parent =
+		((BTPageOpaque) PageGetSpecialPointer(rpage))->btpo_parent =
+		rootbknum;
+
 	/*
 	 * step over the high key on the left page while building the left
 	 * page pointer.
@@ -1137,11 +1185,13 @@ _bt_newroot(Relation rel, Buffer lbuf, Buffer rbuf)
 	pfree(new_item);
 
 	/* write and let go of the root buffer */
-	rootbknum = BufferGetBlockNumber(rootbuf);
 	_bt_wrtbuf(rel, rootbuf);
 
 	/* update metadata page with new root block number */
 	_bt_metaproot(rel, rootbknum, 0);
+
+	_bt_wrtbuf(rel, lbuf);
+	_bt_wrtbuf(rel, rbuf);
 }
 
 /*
@@ -1181,8 +1231,7 @@ _bt_pgaddtup(Relation rel,
 
 		do
 		{
-			chkitem =
-				(BTItem) PageGetItem(page, PageGetItemId(page, itup_off));
+			chkitem = (BTItem) PageGetItem(page, PageGetItemId(page, itup_off));
 			itup_off = OffsetNumberNext(itup_off);
 		} while (!BTItemSame(chkitem, afteritem));
 	}
@@ -1541,6 +1590,7 @@ _bt_shift(Relation rel, Buffer buf, BTStack stack, int keysz,
 		pageop->btpo_flags |= BTP_CHAIN;
 	pageop->btpo_prev = npageop->btpo_prev;		/* restore prev */
 	pageop->btpo_next = nbknum; /* next points to the new page */
+	pageop->btpo_parent = npageop->btpo_parent;
 
 	/* init shifted page opaque */
 	npageop->btpo_prev = bknum = BufferGetBlockNumber(buf);
