@@ -1,6 +1,6 @@
 /*-------------------------------------------------------------------------
  *
- * btree.c--
+ * btree.c
  *	  Implementation of Lehman and Yao's btree management algorithm for
  *	  Postgres.
  *
@@ -8,7 +8,7 @@
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/access/nbtree/nbtree.c,v 1.33 1998/09/07 05:35:33 momjian Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/access/nbtree/nbtree.c,v 1.41 1999/06/07 15:14:54 vadim Exp $
  *
  * NOTES
  *	  This file contains only the public interface routines.
@@ -256,7 +256,7 @@ btbuild(Relation heap,
 		 * if (itup->t_info & INDEX_NULL_MASK) { pfree(itup); continue; }
 		 */
 
-		itup->t_tid = htup->t_ctid;
+		itup->t_tid = htup->t_self;
 		btitem = _bt_formitem(itup);
 
 		/*
@@ -367,16 +367,11 @@ btinsert(Relation rel, Datum *datum, char *nulls, ItemPointer ht_ctid, Relation
 	btitem = _bt_formitem(itup);
 
 	res = _bt_doinsert(rel, btitem,
-							 IndexIsUnique(RelationGetRelid(rel)), heapRel);
+					   IndexIsUnique(RelationGetRelid(rel)), heapRel);
 
 	pfree(btitem);
 	pfree(itup);
 
-#if 0
-	/* adjust any active scans that will be affected by this insertion */
-	_bt_adjscans(rel, &(res->pointerData), BT_INSERT);
-#endif
-
 	return res;
 }
 
@@ -398,13 +393,8 @@ btgettuple(IndexScanDesc scan, ScanDirection dir)
 	{
 
 		/*
-		 * Now we don't adjust scans on insertion (comments in
-		 * nbtscan.c:_bt_scandel()) and I hope that we will unlock current
-		 * index page before leaving index in LLL: this means that current
-		 * index tuple could be moved right before we get here and we have
-		 * to restore our scan position. We save heap TID pointed by
-		 * current index tuple and use it. This will work untill we start
-		 * to re-use (move heap tuples) without vacuum... - vadim 07/29/98
+		 * Restore scan position using heap TID returned by previous call
+		 * to btgettuple(). _bt_restscan() locks buffer.
 		 */
 		_bt_restscan(scan);
 		res = _bt_next(scan, dir);
@@ -412,9 +402,15 @@ btgettuple(IndexScanDesc scan, ScanDirection dir)
 	else
 		res = _bt_first(scan, dir);
 
-	/* Save heap TID to use it in _bt_restscan */
+	/*
+	 * Save heap TID to use it in _bt_restscan. Unlock buffer before
+	 * leaving index !
+	 */
 	if (res)
+	{
 		((BTScanOpaque) scan->opaque)->curHeapIptr = res->heap_iptr;
+		LockBuffer(((BTScanOpaque) scan->opaque)->btso_curbuf, BUFFER_LOCK_UNLOCK);
+	}
 
 	return (char *) res;
 }
@@ -447,18 +443,18 @@ btrescan(IndexScanDesc scan, bool fromEnd, ScanKey scankey)
 
 	so = (BTScanOpaque) scan->opaque;
 
-	/* we hold a read lock on the current page in the scan */
+	/* we don't hold a read lock on the current page in the scan */
 	if (ItemPointerIsValid(iptr = &(scan->currentItemData)))
 	{
-		_bt_relbuf(scan->relation, so->btso_curbuf, BT_READ);
+		ReleaseBuffer(so->btso_curbuf);
 		so->btso_curbuf = InvalidBuffer;
 		ItemPointerSetInvalid(iptr);
 	}
 
-	/* and we hold a read lock on the last marked item in the scan */
+	/* and we don't hold a read lock on the last marked item in the scan */
 	if (ItemPointerIsValid(iptr = &(scan->currentMarkData)))
 	{
-		_bt_relbuf(scan->relation, so->btso_mrkbuf, BT_READ);
+		ReleaseBuffer(so->btso_mrkbuf);
 		so->btso_mrkbuf = InvalidBuffer;
 		ItemPointerSetInvalid(iptr);
 	}
@@ -499,10 +495,10 @@ btmovescan(IndexScanDesc scan, Datum v)
 
 	so = (BTScanOpaque) scan->opaque;
 
-	/* release any locks we still hold */
+	/* we don't hold a read lock on the current page in the scan */
 	if (ItemPointerIsValid(iptr = &(scan->currentItemData)))
 	{
-		_bt_relbuf(scan->relation, so->btso_curbuf, BT_READ);
+		ReleaseBuffer(so->btso_curbuf);
 		so->btso_curbuf = InvalidBuffer;
 		ItemPointerSetInvalid(iptr);
 	}
@@ -522,11 +518,11 @@ btendscan(IndexScanDesc scan)
 
 	so = (BTScanOpaque) scan->opaque;
 
-	/* release any locks we still hold */
+	/* we don't hold any read locks */
 	if (ItemPointerIsValid(iptr = &(scan->currentItemData)))
 	{
 		if (BufferIsValid(so->btso_curbuf))
-			_bt_relbuf(scan->relation, so->btso_curbuf, BT_READ);
+			ReleaseBuffer(so->btso_curbuf);
 		so->btso_curbuf = InvalidBuffer;
 		ItemPointerSetInvalid(iptr);
 	}
@@ -534,7 +530,7 @@ btendscan(IndexScanDesc scan)
 	if (ItemPointerIsValid(iptr = &(scan->currentMarkData)))
 	{
 		if (BufferIsValid(so->btso_mrkbuf))
-			_bt_relbuf(scan->relation, so->btso_mrkbuf, BT_READ);
+			ReleaseBuffer(so->btso_mrkbuf);
 		so->btso_mrkbuf = InvalidBuffer;
 		ItemPointerSetInvalid(iptr);
 	}
@@ -557,20 +553,19 @@ btmarkpos(IndexScanDesc scan)
 
 	so = (BTScanOpaque) scan->opaque;
 
-	/* release lock on old marked data, if any */
+	/* we don't hold any read locks */
 	if (ItemPointerIsValid(iptr = &(scan->currentMarkData)))
 	{
-		_bt_relbuf(scan->relation, so->btso_mrkbuf, BT_READ);
+		ReleaseBuffer(so->btso_mrkbuf);
 		so->btso_mrkbuf = InvalidBuffer;
 		ItemPointerSetInvalid(iptr);
 	}
 
-	/* bump lock on currentItemData and copy to currentMarkData */
+	/* bump pin on current buffer */
 	if (ItemPointerIsValid(&(scan->currentItemData)))
 	{
-		so->btso_mrkbuf = _bt_getbuf(scan->relation,
-								   BufferGetBlockNumber(so->btso_curbuf),
-									 BT_READ);
+		so->btso_mrkbuf = ReadBuffer(scan->relation,
+								  BufferGetBlockNumber(so->btso_curbuf));
 		scan->currentMarkData = scan->currentItemData;
 		so->mrkHeapIptr = so->curHeapIptr;
 	}
@@ -587,20 +582,19 @@ btrestrpos(IndexScanDesc scan)
 
 	so = (BTScanOpaque) scan->opaque;
 
-	/* release lock on current data, if any */
+	/* we don't hold any read locks */
 	if (ItemPointerIsValid(iptr = &(scan->currentItemData)))
 	{
-		_bt_relbuf(scan->relation, so->btso_curbuf, BT_READ);
+		ReleaseBuffer(so->btso_curbuf);
 		so->btso_curbuf = InvalidBuffer;
 		ItemPointerSetInvalid(iptr);
 	}
 
-	/* bump lock on currentMarkData and copy to currentItemData */
+	/* bump pin on marked buffer */
 	if (ItemPointerIsValid(&(scan->currentMarkData)))
 	{
-		so->btso_curbuf = _bt_getbuf(scan->relation,
-								   BufferGetBlockNumber(so->btso_mrkbuf),
-									 BT_READ);
+		so->btso_curbuf = ReadBuffer(scan->relation,
+								  BufferGetBlockNumber(so->btso_mrkbuf));
 
 		scan->currentItemData = scan->currentMarkData;
 		so->curHeapIptr = so->mrkHeapIptr;
@@ -612,31 +606,45 @@ void
 btdelete(Relation rel, ItemPointer tid)
 {
 	/* adjust any active scans that will be affected by this deletion */
-	_bt_adjscans(rel, tid, BT_DELETE);
+	_bt_adjscans(rel, tid);
 
 	/* delete the data from the page */
 	_bt_pagedel(rel, tid);
 }
 
-/*
- * Reasons are in btgettuple... We have to find index item that
- * points to heap tuple returned by previous call to btgettuple().
- */
 static void
 _bt_restscan(IndexScanDesc scan)
 {
 	Relation	rel = scan->relation;
 	BTScanOpaque so = (BTScanOpaque) scan->opaque;
 	Buffer		buf = so->btso_curbuf;
-	Page		page = BufferGetPage(buf);
+	Page		page;
 	ItemPointer current = &(scan->currentItemData);
 	OffsetNumber offnum = ItemPointerGetOffsetNumber(current),
-				maxoff = PageGetMaxOffsetNumber(page);
-	BTPageOpaque opaque = (BTPageOpaque) PageGetSpecialPointer(page);
+				maxoff;
+	BTPageOpaque opaque;
 	ItemPointerData target = so->curHeapIptr;
 	BTItem		item;
 	BlockNumber blkno;
 
+	LockBuffer(buf, BT_READ);		/* lock buffer first! */
+	page = BufferGetPage(buf);
+	maxoff = PageGetMaxOffsetNumber(page);
+	opaque = (BTPageOpaque) PageGetSpecialPointer(page);
+
+	/*
+	 * We use this as flag when first index tuple on page is deleted but
+	 * we do not move left (this would slowdown vacuum) - so we set
+	 * current->ip_posid before first index tuple on the current page
+	 * (_bt_step will move it right)...
+	 */
+	if (!ItemPointerIsValid(&target))
+	{
+		ItemPointerSetOffsetNumber(&(scan->currentItemData),
+		   OffsetNumberPrev(P_RIGHTMOST(opaque) ? P_HIKEY : P_FIRSTKEY));
+		return;
+	}
+
 	if (maxoff >= offnum)
 	{
 
