@@ -1,13 +1,13 @@
 /*-------------------------------------------------------------------------
  *
- * btsearch.c--
+ * btsearch.c
  *	  search code for postgres btrees.
  *
  * Copyright (c) 1994, Regents of the University of California
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/access/nbtree/nbtsearch.c,v 1.39 1998/11/02 15:28:36 momjian Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/access/nbtree/nbtsearch.c,v 1.45 1999/05/25 18:20:30 vadim Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -17,7 +17,6 @@
 #include <access/genam.h>
 #include <fmgr.h>
 #include <storage/bufpage.h>
-#include <storage/bufmgr.h>
 #include <access/nbtree.h>
 #include <catalog/pg_proc.h>
 
@@ -613,7 +612,7 @@ _bt_compare(Relation rel,
 		if (!P_RIGHTMOST(opaque))
 			elog(ERROR, "_bt_compare: invalid comparison to high key");
 
-#if 0
+#ifdef NOT_USED
 
 		/*
 		 * We just have to belive that right answer will not break
@@ -706,15 +705,7 @@ _bt_next(IndexScanDesc scan, ScanDirection dir)
 	so = (BTScanOpaque) scan->opaque;
 	current = &(scan->currentItemData);
 
-	/*
-	 * XXX 10 may 91:  somewhere there's a bug in our management of the
-	 * cached buffer for this scan.  wei discovered it.  the following is
-	 * a workaround so he can work until i figure out what's going on.
-	 */
-
-	if (!BufferIsValid(so->btso_curbuf))
-		so->btso_curbuf = _bt_getbuf(rel, ItemPointerGetBlockNumber(current),
-									 BT_READ);
+	Assert(BufferIsValid(so->btso_curbuf));
 
 	/* we still have the buffer pinned and locked */
 	buf = so->btso_curbuf;
@@ -741,7 +732,8 @@ _bt_next(IndexScanDesc scan, ScanDirection dir)
 			return res;
 		}
 
-	} while (keysok >= so->numberOfFirstKeys);
+	} while (keysok >= so->numberOfFirstKeys ||
+			 (keysok == -1 && ScanDirectionIsBackward(dir)));
 
 	ItemPointerSetInvalid(current);
 	so->btso_curbuf = InvalidBuffer;
@@ -783,6 +775,8 @@ _bt_first(IndexScanDesc scan, ScanDirection dir)
 	BTScanOpaque so;
 	ScanKeyData skdata;
 	Size		keysok;
+	int			i;
+	int			nKeyIndex = -1;
 
 	rel = scan->relation;
 	so = (BTScanOpaque) scan->opaque;
@@ -798,11 +792,34 @@ _bt_first(IndexScanDesc scan, ScanDirection dir)
 	{
 		_bt_orderkeys(rel, so);
 
-		strat = _bt_getstrat(rel, 1, so->keyData[0].sk_procedure);
+		if (ScanDirectionIsBackward(dir))
+		{
+			for (i = 0; i < so->numberOfKeys; i++)
+			{
+				if (so->keyData[i].sk_attno != 1)
+					break;
+				strat = _bt_getstrat(rel, so->keyData[i].sk_attno,
+									 so->keyData[i].sk_procedure);
+				if (strat == BTLessStrategyNumber ||
+					strat == BTLessEqualStrategyNumber ||
+					strat == BTEqualStrategyNumber)
+				{
+					nKeyIndex = i;
+					break;
+				}
+			}
+		}
+		else
+		{
+			strat = _bt_getstrat(rel, 1, so->keyData[0].sk_procedure);
 
-		/* NOTE: it assumes ForwardScanDirection */
-		if (strat == BTLessStrategyNumber ||
-			strat == BTLessEqualStrategyNumber)
+			if (strat == BTLessStrategyNumber ||
+				strat == BTLessEqualStrategyNumber)
+				;
+			else
+				nKeyIndex = 0;
+		}
+		if (nKeyIndex < 0)
 			scan->scanFromEnd = true;
 	}
 	else
@@ -831,8 +848,8 @@ _bt_first(IndexScanDesc scan, ScanDirection dir)
 		return (RetrieveIndexResult) NULL;
 	}
 	proc = index_getprocid(rel, 1, BTORDER_PROC);
-	ScanKeyEntryInitialize(&skdata, so->keyData[0].sk_flags, 1, proc,
-						   so->keyData[0].sk_argument);
+	ScanKeyEntryInitialize(&skdata, so->keyData[nKeyIndex].sk_flags,
+						   1, proc, so->keyData[nKeyIndex].sk_argument);
 
 	stack = _bt_search(rel, 1, &skdata, &buf);
 	_bt_freestack(stack);
@@ -905,7 +922,7 @@ _bt_first(IndexScanDesc scan, ScanDirection dir)
 
 	/* it's yet other place to add some code latter for is(not)null */
 
-	strat = _bt_getstrat(rel, 1, so->keyData[0].sk_procedure);
+	strat = _bt_getstrat(rel, 1, so->keyData[nKeyIndex].sk_procedure);
 
 	switch (strat)
 	{
@@ -922,9 +939,6 @@ _bt_first(IndexScanDesc scan, ScanDirection dir)
 					result = _bt_compare(rel, itupdesc, page, 1, &skdata, offnum);
 				} while (result <= 0);
 
-				/* if this is true, the key we just looked at is gone */
-				if (result > 0)
-					_bt_twostep(scan, &buf, ForwardScanDirection);
 			}
 			break;
 
@@ -954,6 +968,21 @@ _bt_first(IndexScanDesc scan, ScanDirection dir)
 				ItemPointerSetInvalid(&(scan->currentItemData));
 				return (RetrieveIndexResult) NULL;
 			}
+			else if (ScanDirectionIsBackward(dir))
+			{
+				do
+				{
+					if (!_bt_twostep(scan, &buf, ForwardScanDirection))
+						break;
+
+					offnum = ItemPointerGetOffsetNumber(current);
+					page = BufferGetPage(buf);
+					result = _bt_compare(rel, itupdesc, page, 1, &skdata, offnum);
+				} while (result == 0);
+
+				if (result < 0)
+					_bt_twostep(scan, &buf, BackwardScanDirection);
+			}
 			break;
 
 		case BTGreaterEqualStrategyNumber:
@@ -1034,6 +1063,11 @@ _bt_first(IndexScanDesc scan, ScanDirection dir)
 		so->btso_curbuf = buf;
 		return _bt_next(scan, dir);
 	}
+	else if (keysok == -1 && ScanDirectionIsBackward(dir))
+	{
+		so->btso_curbuf = buf;
+		return _bt_next(scan, dir);
+	}
 	else
 	{
 		ItemPointerSetInvalid(current);
@@ -1069,7 +1103,12 @@ _bt_step(IndexScanDesc scan, Buffer *bufP, ScanDirection dir)
 
 	rel = scan->relation;
 	current = &(scan->currentItemData);
-	offnum = ItemPointerGetOffsetNumber(current);
+
+	/*
+	 * Don't use ItemPointerGetOffsetNumber or you risk to get assertion
+	 * due to ability of ip_posid to be equal 0.
+	 */
+	offnum = current->ip_posid;
 	page = BufferGetPage(*bufP);
 	opaque = (BTPageOpaque) PageGetSpecialPointer(page);
 	so = (BTScanOpaque) scan->opaque;
@@ -1398,7 +1437,7 @@ _bt_endpoint(IndexScanDesc scan, ScanDirection dir)
 		 * maxoff). Scanning in BackwardScanDirection is not
 		 * understandable at all. Well - new stuff. - vadim 12/06/96
 		 */
-#if 0
+#ifdef NOT_USED
 		if (PageIsEmpty(page) || start > maxoff)
 		{
 			ItemPointerSet(current, blkno, maxoff);
@@ -1444,7 +1483,7 @@ _bt_endpoint(IndexScanDesc scan, ScanDirection dir)
 		 * empty why do scanning in ForwardScanDirection ??? Well - new
 		 * stuff. - vadim 12/06/96
 		 */
-#if 0
+#ifdef NOT_USED
 		if (PageIsEmpty(page))
 		{
 			ItemPointerSet(current, blkno, FirstOffsetNumber);
@@ -1499,6 +1538,11 @@ _bt_endpoint(IndexScanDesc scan, ScanDirection dir)
 		so->btso_curbuf = buf;
 		return _bt_next(scan, dir);
 	}
+	else if (keysok == -1 && ScanDirectionIsBackward(dir))
+	{
+		so->btso_curbuf = buf;
+		return _bt_next(scan, dir);
+	}
 	else
 	{
 		ItemPointerSetInvalid(current);
