@@ -1,13 +1,13 @@
 /*-------------------------------------------------------------------------
  *
- * btutils.c--
+ * btutils.c
  *	  Utility code for Postgres btree implementation.
  *
  * Copyright (c) 1994, Regents of the University of California
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/access/nbtree/nbtutils.c,v 1.23 1998/09/07 05:35:34 momjian Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/access/nbtree/nbtutils.c,v 1.27 1999/05/25 18:20:31 vadim Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -258,8 +258,7 @@ _bt_orderkeys(Relation relation, BTScanOpaque so)
 		if (init[j])
 		{
 			/* yup, use the appropriate value */
-			test =
-				(long) FMGR_PTR2(&cur->sk_func, cur->sk_argument, xform[j].sk_argument);
+			test = (long) FMGR_PTR2(&cur->sk_func, cur->sk_argument, xform[j].sk_argument);
 			if (test)
 				xform[j].sk_argument = cur->sk_argument;
 			else if (j == (BTEqualStrategyNumber - 1))
@@ -302,9 +301,6 @@ _bt_formitem(IndexTuple itup)
 	btitem = (BTItem) palloc(nbytes_btitem);
 	memmove((char *) &(btitem->bti_itup), (char *) itup, tuplen);
 
-#ifndef BTREE_VERSION_1
-	btitem->bti_oid = newoid();
-#endif
 	return btitem;
 }
 
@@ -368,8 +364,14 @@ _bt_checkkeys(IndexScanDesc scan, IndexTuple tuple, Size *keysok)
 							  &isNull);
 
 		/* btree doesn't support 'A is null' clauses, yet */
-		if (isNull || key[0].sk_flags & SK_ISNULL)
+		if (key[0].sk_flags & SK_ISNULL)
 			return false;
+		if (isNull)
+		{
+			if (*keysok < so->numberOfFirstKeys)
+				*keysok = -1;
+			return false;
+		}
 
 		if (key[0].sk_flags & SK_COMMUTE)
 		{
