@@ -1,13 +1,13 @@
 /*-------------------------------------------------------------------------
  *
- * rtree.c--
+ * rtree.c
  *	  interface routines for the postgres rtree indexed access method.
  *
  * Copyright (c) 1994, Regents of the University of California
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/access/rtree/rtree.c,v 1.28 1998/09/01 04:27:10 momjian Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/access/rtree/rtree.c,v 1.32 1999/05/25 16:07:38 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -107,9 +107,6 @@ rtbuild(Relation heap,
 
 	initRtstate(&rtState, index);
 
-	/* rtrees only know how to do stupid locking now */
-	RelationSetLockForWrite(index);
-
 	pred = predInfo->pred;
 	oldPred = predInfo->oldPred;
 
@@ -233,7 +230,7 @@ rtbuild(Relation heap,
 
 		/* form an index tuple and point it at the heap tuple */
 		itup = index_formtuple(id, &d[0], nulls);
-		itup->t_tid = htup->t_ctid;
+		itup->t_tid = htup->t_self;
 
 		/*
 		 * Since we already have the index relation locked, we call
@@ -250,7 +247,6 @@ rtbuild(Relation heap,
 
 	/* okay, all heap tuples are indexed */
 	heap_endscan(scan);
-	RelationUnsetLockForWrite(index);
 
 	if (pred != NULL || oldPred != NULL)
 	{
@@ -308,10 +304,14 @@ rtinsert(Relation r, Datum *datum, char *nulls, ItemPointer ht_ctid, Relation he
 	itup->t_tid = *ht_ctid;
 	initRtstate(&rtState, r);
 
-	RelationSetLockForWrite(r);
+	/*
+	 * Notes in ExecUtils:ExecOpenIndices()
+	 *
+	 * RelationSetLockForWrite(r);
+	 */
+
 	res = rtdoinsert(r, itup, &rtState);
 
-	/* XXX two-phase locking -- don't unlock the relation until EOT */
 	return res;
 }
 
@@ -946,8 +946,12 @@ rtdelete(Relation r, ItemPointer tid)
 	Buffer		buf;
 	Page		page;
 
-	/* must write-lock on delete */
-	RelationSetLockForWrite(r);
+	/*
+	 * Notes in ExecUtils:ExecOpenIndices() Also note that only vacuum
+	 * deletes index tuples now...
+	 *
+	 * RelationSetLockForWrite(r);
+	 */
 
 	blkno = ItemPointerGetBlockNumber(tid);
 	offnum = ItemPointerGetOffsetNumber(tid);
@@ -963,7 +967,6 @@ rtdelete(Relation r, ItemPointer tid)
 
 	WriteBuffer(buf);
 
-	/* XXX -- two-phase locking, don't release the write lock */
 	return (char *) NULL;
 }
 
