@@ -1,13 +1,13 @@
 /*-------------------------------------------------------------------------
  *
- * rtscan.c--
+ * rtscan.c
  *	  routines to manage scans on index relations
  *
  * Copyright (c) 1994, Regents of the University of California
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/access/rtree/rtscan.c,v 1.19 1998/09/01 04:27:12 momjian Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/access/rtree/rtscan.c,v 1.23 1999/05/25 16:07:40 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -66,7 +66,12 @@ rtbeginscan(Relation r,
 {
 	IndexScanDesc s;
 
-	RelationSetLockForRead(r);
+	/*
+	 * Let index_beginscan does its work...
+	 *
+	 * RelationSetLockForRead(r);
+	 */
+
 	s = RelationGetIndexScan(r, fromEnd, nkeys, key);
 	rtregscan(s);
 
@@ -136,8 +141,7 @@ rtrescan(IndexScanDesc s, bool fromEnd, ScanKey key)
 		s->opaque = p;
 		if (s->numberOfKeys > 0)
 		{
-			p->s_internalKey =
-				(ScanKey) palloc(sizeof(ScanKeyData) * s->numberOfKeys);
+			p->s_internalKey = (ScanKey) palloc(sizeof(ScanKeyData) * s->numberOfKeys);
 
 			/*
 			 * Scans on internal pages use different operators than they
