@@ -1,13 +1,13 @@
 /*-------------------------------------------------------------------------
  *
- * transam.c--
+ * transam.c
  *	  postgres transaction log/time interface routines
  *
  * Copyright (c) 1994, Regents of the University of California
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/access/transam/transam.c,v 1.19 1998/09/01 04:27:15 momjian Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/access/transam/transam.c,v 1.27 1999/06/10 14:17:06 vadim Exp $
  *
  * NOTES
  *	  This file contains the high level access-method interface to the
@@ -172,10 +172,15 @@ TransactionLogTest(TransactionId transactionId, /* transaction id to test */
 
 	if (!fail)
 	{
-		TransactionIdStore(transactionId, &cachedTestXid);
-		cachedTestXidStatus = xidstatus;
-		return (bool)
-			(status == xidstatus);
+		/*
+		 * DO NOT cache status for transactions in unknown state !!!
+		 */
+		if (xidstatus == XID_COMMIT || xidstatus == XID_ABORT)
+		{
+			TransactionIdStore(transactionId, &cachedTestXid);
+			cachedTestXidStatus = xidstatus;
+		}
+		return (bool) (status == xidstatus);
 	}
 
 	/* ----------------
@@ -219,15 +224,15 @@ TransactionLogUpdate(TransactionId transactionId,		/* trans id to update */
 								 status,
 								 &fail);
 
-	/* ----------------
-	 *	 update (invalidate) our single item TransactionLogTest cache.
-	 * ----------------
+	/*
+	 * update (invalidate) our single item TransactionLogTest cache.
+	 *
+	 * if (status != XID_COMMIT)
+	 *
+	 * What's the hell ?! Why != XID_COMMIT ?!
 	 */
-	if (status != XID_COMMIT)
-	{
-		TransactionIdStore(transactionId, &cachedTestXid);
-		cachedTestXidStatus = status;
-	}
+	TransactionIdStore(transactionId, &cachedTestXid);
+	cachedTestXidStatus = status;
 
 }
 
@@ -300,7 +305,7 @@ TransactionLogUpdate(TransactionId transactionId,		/* trans id to update */
 static void
 TransRecover(Relation logRelation)
 {
-#if 0
+#ifdef NOT_USED
 	/* ----------------
 	 *	  first get the last recorded transaction in the log.
 	 * ----------------
@@ -375,7 +380,7 @@ TransRecover(Relation logRelation)
  */
 
 /*
- * InitializeTransactionLog --
+ * InitializeTransactionLog
  *		Initializes transaction logging.
  */
 void
@@ -485,7 +490,7 @@ InitializeTransactionLog(void)
  */
 
 /*
- * TransactionIdDidCommit --
+ * TransactionIdDidCommit
  *		True iff transaction associated with the identifier did commit.
  *
  * Note:
@@ -497,12 +502,11 @@ TransactionIdDidCommit(TransactionId transactionId)
 	if (AMI_OVERRIDE)
 		return true;
 
-	return
-		TransactionLogTest(transactionId, XID_COMMIT);
+	return TransactionLogTest(transactionId, XID_COMMIT);
 }
 
 /*
- * TransactionIdDidAborted --
+ * TransactionIdDidAborted
  *		True iff transaction associated with the identifier did abort.
  *
  * Note:
@@ -515,8 +519,7 @@ TransactionIdDidAbort(TransactionId transactionId)
 	if (AMI_OVERRIDE)
 		return false;
 
-	return
-		TransactionLogTest(transactionId, XID_ABORT);
+	return TransactionLogTest(transactionId, XID_ABORT);
 }
 
 /*
@@ -532,8 +535,7 @@ TransactionIdIsInProgress(TransactionId transactionId)
 	if (AMI_OVERRIDE)
 		return false;
 
-	return
-		TransactionLogTest(transactionId, XID_INPROGRESS);
+	return TransactionLogTest(transactionId, XID_INPROGRESS);
 }
  */
 
@@ -545,7 +547,7 @@ TransactionIdIsInProgress(TransactionId transactionId)
  */
 
 /*
- * TransactionIdCommit --
+ * TransactionIdCommit
  *		Commits the transaction associated with the identifier.
  *
  * Note:
@@ -561,7 +563,7 @@ TransactionIdCommit(TransactionId transactionId)
 }
 
 /*
- * TransactionIdAbort --
+ * TransactionIdAbort
  *		Aborts the transaction associated with the identifier.
  *
  * Note:
@@ -580,15 +582,3 @@ TransactionIdAbort(TransactionId transactionId)
 
 	TransactionLogUpdate(transactionId, XID_ABORT);
 }
-
-#ifdef NOT_USED
-void
-TransactionIdSetInProgress(TransactionId transactionId)
-{
-	if (AMI_OVERRIDE)
-		return;
-
-	TransactionLogUpdate(transactionId, XID_INPROGRESS);
-}
-
-#endif
