@@ -1,13 +1,13 @@
 /*-------------------------------------------------------------------------
  *
- * transsup.c--
+ * transsup.c
  *	  postgres transaction access method support code
  *
  * Copyright (c) 1994, Regents of the University of California
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/access/transam/transsup.c,v 1.17 1998/09/01 04:27:16 momjian Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/access/transam/transsup.c,v 1.21 1999/03/30 01:37:21 momjian Exp $
  *
  * NOTES
  *	  This file contains support functions for the high
@@ -211,8 +211,7 @@ TransBlockGetXidStatus(Block tblock,
 	 *	return the transaction status to the caller
 	 * ----------------
 	 */
-	return (XidStatus)
-		(bit1 | bit2);
+	return (XidStatus) (bit1 | bit2);
 }
 
 /* --------------------------------
@@ -289,18 +288,12 @@ TransBlockNumberGetXidStatus(Relation relation,
 	XidStatus	xstatus;		/* recorded status of xid */
 	bool		localfail;		/* bool used if failP = NULL */
 
-	/* ----------------
-	 *	SOMEDAY place a read lock on the log relation
-	 *	That someday is today 5 Aug 1991 -mer
-	 * ----------------
-	 */
-	RelationSetLockForRead(relation);
-
 	/* ----------------
 	 *	get the page containing the transaction information
 	 * ----------------
 	 */
 	buffer = ReadBuffer(relation, blockNumber);
+	LockBuffer(buffer, BUFFER_LOCK_SHARE);
 	block = BufferGetBlock(buffer);
 
 	/* ----------------
@@ -318,16 +311,10 @@ TransBlockNumberGetXidStatus(Relation relation,
 	 *	release the buffer and return the status
 	 * ----------------
 	 */
+	LockBuffer(buffer, BUFFER_LOCK_UNLOCK);
 	ReleaseBuffer(buffer);
 
-	/* ----------------
-	 *	SOMEDAY release our lock on the log relation
-	 * ----------------
-	 */
-	RelationUnsetLockForRead(relation);
-
-	return
-		xstatus;
+	return xstatus;
 }
 
 /* --------------------------------
@@ -345,19 +332,12 @@ TransBlockNumberSetXidStatus(Relation relation,
 	Block		block;			/* block containing xstatus */
 	bool		localfail;		/* bool used if failP = NULL */
 
-	/* ----------------
-	 *	SOMEDAY gain exclusive access to the log relation
-	 *
-	 *	That someday is today 5 Aug 1991 -mer
-	 * ----------------
-	 */
-	RelationSetLockForWrite(relation);
-
 	/* ----------------
 	 *	get the block containing the transaction status
 	 * ----------------
 	 */
 	buffer = ReadBuffer(relation, blockNumber);
+	LockBuffer(buffer, BUFFER_LOCK_EXCLUSIVE);
 	block = BufferGetBlock(buffer);
 
 	/* ----------------
@@ -372,16 +352,11 @@ TransBlockNumberSetXidStatus(Relation relation,
 
 	TransBlockSetXidStatus(block, xid, xstatus);
 
+	LockBuffer(buffer, BUFFER_LOCK_UNLOCK);
 	if ((*failP) == false)
 		WriteBuffer(buffer);
 	else
 		ReleaseBuffer(buffer);
-
-	/* ----------------
-	 *	SOMEDAY release our lock on the log relation
-	 * ----------------
-	 */
-	RelationUnsetLockForWrite(relation);
 }
 
 /* --------------------------------
