@@ -7,7 +7,7 @@
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/catalog/index.c,v 1.77 1999/06/04 02:19:47 tgl Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/catalog/index.c,v 1.81 1999/07/07 09:27:25 momjian Exp $
  *
  *
  * INTERFACE ROUTINES
@@ -20,6 +20,7 @@
 #include "postgres.h"
 
 #include "access/genam.h"
+#include "access/htup.h"
 #include "access/heapam.h"
 #include "access/istrat.h"
 #include "access/xact.h"
@@ -56,8 +57,10 @@
 /*
  * macros used in guessing how many tuples are on a page.
  */
-#define AVG_TUPLE_SIZE 8
-#define NTUPLES_PER_PAGE(natts) (BLCKSZ/((natts)*AVG_TUPLE_SIZE))
+#define AVG_ATTR_SIZE 8
+#define NTUPLES_PER_PAGE(natts) \
+	((BLCKSZ - MAXALIGN(sizeof (PageHeaderData))) / \
+	((natts) * AVG_ATTR_SIZE + MAXALIGN(sizeof(HeapTupleHeaderData))))
 
 /* non-export function prototypes */
 static Oid GetHeapRelationOid(char *heapRelationName, char *indexRelationName,
@@ -372,7 +375,7 @@ AccessMethodObjectIdGetForm(Oid accessMethodObjectId)
 	HeapScanDesc pg_am_scan;
 	HeapTuple	pg_am_tuple;
 	ScanKeyData key;
-	Form_pg_am	form;
+	Form_pg_am	aform;
 
 	/* ----------------
 	 *	form a scan key for the pg_am relation
@@ -406,13 +409,13 @@ AccessMethodObjectIdGetForm(Oid accessMethodObjectId)
 	 *	if found am tuple, then copy the form and return the copy
 	 * ----------------
 	 */
-	form = (Form_pg_am) palloc(sizeof *form);
-	memcpy(form, GETSTRUCT(pg_am_tuple), sizeof *form);
+	aform = (Form_pg_am) palloc(sizeof *aform);
+	memcpy(aform, GETSTRUCT(pg_am_tuple), sizeof *aform);
 
 	heap_endscan(pg_am_scan);
 	heap_close(pg_am_desc);
 
-	return form;
+	return aform;
 }
 
 /* ----------------------------------------------------------------
@@ -990,9 +993,7 @@ index_create(char *heapRelationName,
 		Oid			relid = RelnameFindRelid(indexRelationName);
 
 		if (relid != InvalidOid)
-		{
 			RelationForgetRelation(relid);
-		}
 	}
 
 	/* save user relation name because heap_create changes it */
