@@ -1,13 +1,13 @@
 /*-------------------------------------------------------------------------
  *
- * pg_proc.c--
+ * pg_proc.c
  *	  routines to support manipulation of the pg_proc relation
  *
  * Copyright (c) 1994, Regents of the University of California
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/catalog/pg_proc.c,v 1.23 1998/09/01 04:27:37 momjian Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/catalog/pg_proc.c,v 1.29 1999/05/25 16:08:11 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -26,6 +26,7 @@
 #include "parser/parse_node.h"
 #include "tcop/tcopprot.h"
 #include "utils/builtins.h"
+#include "utils/fmgrtab.h"
 #include "utils/lsyscache.h"
 #include "utils/sets.h"
 #include "utils/syscache.h"
@@ -37,7 +38,7 @@
 #endif
 
 /* ----------------------------------------------------------------
- *		ProcedureDefine
+ *		ProcedureCreate
  * ----------------------------------------------------------------
  */
 Oid
@@ -66,7 +67,7 @@ ProcedureCreate(char *procedureName,
 	Oid			languageObjectId;
 	Oid			typeObjectId;
 	List	   *x;
-	QueryTreeList *querytree_list;
+	List	   *querytree_list;
 	List	   *plan_list;
 	Oid			typev[8];
 	Oid			relid;
@@ -94,7 +95,7 @@ ProcedureCreate(char *procedureName,
 		if (strcmp(strVal(t), "opaque") == 0)
 		{
 			if (strcmp(languageName, "sql") == 0)
-				elog(ERROR, "ProcedureDefine: sql functions cannot take type \"opaque\"");
+				elog(ERROR, "ProcedureCreate: sql functions cannot take type \"opaque\"");
 			toid = 0;
 		}
 		else
@@ -144,7 +145,7 @@ ProcedureCreate(char *procedureName,
 									  0, 0, 0);
 			pfree(prosrctext);
 			if (HeapTupleIsValid(tup))
-				return tup->t_oid;
+				return tup->t_data->t_oid;
 		}
 	}
 
@@ -155,7 +156,7 @@ ProcedureCreate(char *procedureName,
 	if (!HeapTupleIsValid(tup))
 		elog(ERROR, "ProcedureCreate: no such language %s", languageName);
 
-	languageObjectId = tup->t_oid;
+	languageObjectId = tup->t_data->t_oid;
 
 	if (strcmp(returnTypeName, "opaque") == 0)
 	{
@@ -172,7 +173,7 @@ ProcedureCreate(char *procedureName,
 		{
 			elog(NOTICE, "ProcedureCreate: type '%s' is not yet defined",
 				 returnTypeName);
-#if 0
+#ifdef NOT_USED
 			elog(NOTICE, "ProcedureCreate: creating a shell for type '%s'",
 				 returnTypeName);
 #endif
@@ -220,6 +221,32 @@ ProcedureCreate(char *procedureName,
 		pg_checkretval(typeObjectId, querytree_list);
 	}
 
+	/*
+	 * If this is an internal procedure, check that the given internal
+	 * function name (the 'prosrc' value) is a known builtin function.
+	 *
+	 * NOTE: in Postgres versions before 6.5, the SQL name of the created
+	 * function could not be different from the internal name, and
+	 * 'prosrc' wasn't used.  So there is code out there that does CREATE
+	 * FUNCTION xyz AS '' LANGUAGE 'internal'.	To preserve some modicum
+	 * of backwards compatibility, accept an empty 'prosrc' value as
+	 * meaning the supplied SQL function name.
+	 */
+
+	if (strcmp(languageName, "internal") == 0)
+	{
+		if (strlen(prosrc) == 0)
+			prosrc = procedureName;
+		if (fmgr_lookupByName(prosrc) == (func_ptr) NULL)
+			elog(ERROR,
+			"ProcedureCreate: there is no builtin function named \"%s\"",
+				 prosrc);
+	}
+
+	/*
+	 * All seems OK; prepare the tuple to be inserted into pg_proc.
+	 */
+
 	for (i = 0; i < Natts_pg_proc; ++i)
 	{
 		nulls[i] = ' ';
@@ -276,5 +303,5 @@ ProcedureCreate(char *procedureName,
 		CatalogCloseIndices(Num_pg_proc_indices, idescs);
 	}
 	heap_close(rel);
-	return tup->t_oid;
+	return tup->t_data->t_oid;
 }
