@@ -1,13 +1,13 @@
 /*-------------------------------------------------------------------------
  *
- * dbcommands.c--
+ * dbcommands.c
  *
  *
  * Copyright (c) 1994, Regents of the University of California
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/commands/dbcommands.c,v 1.25 1998/10/05 02:49:36 thomas Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/commands/dbcommands.c,v 1.35 1999/05/25 16:08:21 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -24,13 +24,13 @@
 #include "catalog/catname.h"
 #include "catalog/pg_database.h"
 #include "catalog/pg_shadow.h"
-#include "commands/dbcommands.h"
 #include "fmgr.h"
 #include "miscadmin.h"			/* for DataDir */
 #include "storage/bufmgr.h"
 #include "storage/fd.h"
 #include "storage/lmgr.h"
 #include "tcop/tcopprot.h"
+#include "commands/dbcommands.h"
 #include "utils/rel.h"
 #include "utils/syscache.h"
 
@@ -42,7 +42,7 @@ static HeapTuple get_pg_dbtup(char *command, char *dbname, Relation dbrel);
 static void stop_vacuum(char *dbpath, char *dbname);
 
 void
-createdb(char *dbname, char *dbpath, int encoding)
+createdb(char *dbname, char *dbpath, int encoding, CommandDest dest)
 {
 	Oid			db_id;
 	int4		user_id;
@@ -64,7 +64,7 @@ createdb(char *dbname, char *dbpath, int encoding)
 	{
 		if (*(dbpath + strlen(dbpath) - 1) == SEP_CHAR)
 			*(dbpath + strlen(dbpath) - 1) = '\0';
-		sprintf(loc, "%s%c%s", dbpath, SEP_CHAR, dbname);
+		snprintf(loc, 512, "%s%c%s", dbpath, SEP_CHAR, dbname);
 	}
 	else
 		strcpy(loc, dbname);
@@ -79,31 +79,26 @@ createdb(char *dbname, char *dbpath, int encoding)
 	if (mkdir(lp, S_IRWXU) != 0)
 		elog(ERROR, "Unable to create database directory '%s'", lp);
 
-	sprintf(buf, "%s %s%cbase%ctemplate1%c* %s",
-			COPY_CMD, DataDir, SEP_CHAR, SEP_CHAR, SEP_CHAR, lp);
+	snprintf(buf, 512, "%s %s%cbase%ctemplate1%c* %s",
+			 COPY_CMD, DataDir, SEP_CHAR, SEP_CHAR, SEP_CHAR, lp);
 	system(buf);
 
-#if FALSE
-	sprintf(buf, "insert into pg_database (datname, datdba, datpath) \
-                  values ('%s'::name, '%d'::oid, '%s'::text);",
-			dbname, user_id, dbname);
-#endif
+	snprintf(buf, 512,
+		   "insert into pg_database (datname, datdba, encoding, datpath)"
+		  " values ('%s', '%d', '%d', '%s');", dbname, user_id, encoding,
+			 loc);
 
-	sprintf(buf, "insert into pg_database (datname, datdba, encoding, datpath)"
-			" values ('%s', '%d', '%d', '%s');", dbname, user_id, encoding, loc);
-
-
-	pg_exec_query(buf);
+	pg_exec_query_dest(buf, dest, false);
 }
 
 void
-destroydb(char *dbname)
+destroydb(char *dbname, CommandDest dest)
 {
 	int4		user_id;
 	Oid			db_id;
-	char	   *path;
-	char		dbpath[MAXPGPATH + 1];
-	char		buf[512];
+	char	   *path,
+				dbpath[MAXPGPATH + 1],
+				buf[512];
 
 	/*
 	 * If this call returns, the database exists and we're allowed to
@@ -127,20 +122,19 @@ destroydb(char *dbname)
 	 * remove the pg_database tuple FIRST, this may fail due to
 	 * permissions problems
 	 */
-	sprintf(buf, "delete from pg_database where pg_database.oid = \'%d\'::oid",
-			db_id);
-	pg_exec_query(buf);
+	snprintf(buf, 512,
+	"delete from pg_database where pg_database.oid = \'%u\'::oid", db_id);
+	pg_exec_query_dest(buf, dest, false);
+
+	/* drop pages for this database that are in the shared buffer cache */
+	DropBuffers(db_id);
 
 	/*
 	 * remove the data directory. If the DELETE above failed, this will
 	 * not be reached
 	 */
-
-	sprintf(buf, "rm -r %s", path);
+	snprintf(buf, 512, "rm -r %s", path);
 	system(buf);
-
-	/* drop pages for this database that are in the shared buffer cache */
-	DropBuffers(db_id);
 }
 
 static HeapTuple
@@ -241,7 +235,7 @@ check_permissions(char *command,
 	 * delays when multiple 'createdb's or 'destroydb's are run simult.
 	 * -mer 7/3/91
 	 */
-	RelationSetLockForWrite(dbrel);
+	LockRelation(dbrel, AccessExclusiveLock);
 	dbtup = get_pg_dbtup(command, dbname, dbrel);
 	dbfound = HeapTupleIsValid(dbtup);
 
@@ -251,7 +245,7 @@ check_permissions(char *command,
 									  Anum_pg_database_datdba,
 									  RelationGetDescr(dbrel),
 									  (char *) NULL);
-		*dbIdP = dbtup->t_oid;
+		*dbIdP = dbtup->t_data->t_oid;
 		dbtext = (text *) heap_getattr(dbtup,
 									   Anum_pg_database_datpath,
 									   RelationGetDescr(dbrel),
@@ -305,13 +299,17 @@ stop_vacuum(char *dbpath, char *dbname)
 
 	if (strchr(dbpath, SEP_CHAR) != 0)
 	{
-		sprintf(filename, "%s%cbase%c%s%c%s.vacuum", DataDir, SEP_CHAR, SEP_CHAR,
-				dbname, SEP_CHAR, dbname);
+		snprintf(filename, 256, "%s%cbase%c%s%c%s.vacuum",
+				 DataDir, SEP_CHAR, SEP_CHAR, dbname, SEP_CHAR, dbname);
 	}
 	else
-		sprintf(filename, "%s%c%s.vacuum", dbpath, SEP_CHAR, dbname);
+		snprintf(filename, 256, "%s%c%s.vacuum", dbpath, SEP_CHAR, dbname);
 
+#ifndef __CYGWIN32__
 	if ((fp = AllocateFile(filename, "r")) != NULL)
+#else
+	if ((fp = AllocateFile(filename, "rb")) != NULL)
+#endif
 	{
 		fscanf(fp, "%d", &pid);
 		FreeFile(fp);
