@@ -1,13 +1,13 @@
 /*-------------------------------------------------------------------------
  *
- * rename.c--
+ * rename.c
  *	  renameatt() and renamerel() reside here.
  *
  * Copyright (c) 1994, Regents of the University of California
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/commands/rename.c,v 1.17 1998/09/01 04:27:59 momjian Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/commands/rename.c,v 1.25 1999/05/25 16:08:26 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -18,6 +18,7 @@
 #include <utils/builtins.h>
 #include <catalog/catname.h>
 #include <utils/syscache.h>
+#include <catalog/heap.h>
 #include <catalog/indexing.h>
 #include <catalog/catalog.h>
 #include <commands/copy.h>
@@ -80,7 +81,7 @@ renameatt(char *relname,
 	 *
 	 * normally, only the owner of a class can change its schema.
 	 */
-	if (IsSystemRelationName(relname))
+	if (!allowSystemTableMods && IsSystemRelationName(relname))
 		elog(ERROR, "renameatt: class \"%s\" is a system catalog",
 			 relname);
 #ifndef NO_SECURITY
@@ -106,15 +107,9 @@ renameatt(char *relname,
 		List	   *child,
 				   *children;
 
-		reltup = SearchSysCacheTuple(RELNAME,
-									 PointerGetDatum(relname),
-									 0, 0, 0);
-
-		if (!HeapTupleIsValid(reltup))
+		if ((myrelid = RelnameFindRelid(relname)) == InvalidOid)
 			elog(ERROR, "renameatt: unknown relation: \"%s\"", relname);
 
-		myrelid = reltup->t_oid;
-
 		/* this routine is actually in the planner */
 		children = find_all_inheritors(lconsi(myrelid, NIL), NIL);
 
@@ -135,7 +130,7 @@ renameatt(char *relname,
 										 0, 0, 0);
 			if (!HeapTupleIsValid(reltup))
 			{
-				elog(ERROR, "renameatt: can't find catalog entry for inheriting class with oid %d",
+				elog(ERROR, "renameatt: can't find catalog entry for inheriting class with oid %u",
 					 childrelid);
 			}
 			/* make copy of cache value, could disappear in call */
@@ -147,13 +142,9 @@ renameatt(char *relname,
 		}
 	}
 
-	reltup = SearchSysCacheTuple(RELNAME,
-								 PointerGetDatum(relname),
-								 0, 0, 0);
-	if (!HeapTupleIsValid(reltup))
-		elog(ERROR, "renameatt: relation \"%s\" nonexistent", relname);
 
-	relid = reltup->t_oid;
+	if ((relid = RelnameFindRelid(relname)) == InvalidOid)
+		elog(ERROR, "renameatt: relation \"%s\" nonexistent", relname);
 
 	oldatttup = SearchSysCacheTupleCopy(ATTNAME,
 										ObjectIdGetDatum(relid),
@@ -180,7 +171,7 @@ renameatt(char *relname,
 			newattname, NAMEDATALEN);
 
 	attrelation = heap_openr(AttributeRelationName);
-	heap_replace(attrelation, &oldatttup->t_ctid, oldatttup);
+	heap_replace(attrelation, &oldatttup->t_self, oldatttup, NULL);
 
 	/* keep system catalog indices current */
 	CatalogOpenIndices(Num_pg_attr_indices, Name_pg_attr_indices, irelations);
@@ -210,18 +201,20 @@ renameatt(char *relname,
 void
 renamerel(char *oldrelname, char *newrelname)
 {
+	int			i;
 	Relation	relrelation;	/* for RELATION relation */
-	HeapTuple	oldreltup,
-				newreltup;
+	HeapTuple	oldreltup;
 	char		oldpath[MAXPGPATH],
-				newpath[MAXPGPATH];
+				newpath[MAXPGPATH],
+				toldpath[MAXPGPATH + 10],
+				tnewpath[MAXPGPATH + 10];
 	Relation	irelations[Num_pg_class_indices];
 
-	if (IsSystemRelationName(oldrelname))
+	if (!allowSystemTableMods && IsSystemRelationName(oldrelname))
 		elog(ERROR, "renamerel: system relation \"%s\" not renamed",
 			 oldrelname);
 
-	if (IsSystemRelationName(newrelname))
+	if (!allowSystemTableMods && IsSystemRelationName(newrelname))
 		elog(ERROR, "renamerel: Illegal class name: \"%s\" -- pg_ is reserved for system catalogs",
 			 newrelname);
 
@@ -231,10 +224,7 @@ renamerel(char *oldrelname, char *newrelname)
 	if (!HeapTupleIsValid(oldreltup))
 		elog(ERROR, "renamerel: relation \"%s\" does not exist", oldrelname);
 
-	newreltup = SearchSysCacheTuple(RELNAME,
-									PointerGetDatum(newrelname),
-									0, 0, 0);
-	if (HeapTupleIsValid(newreltup))
+	if (RelnameFindRelid(newrelname) != InvalidOid)
 		elog(ERROR, "renamerel: relation \"%s\" exists", newrelname);
 
 	/* rename the path first, so if this fails the rename's not done */
@@ -243,12 +233,20 @@ renamerel(char *oldrelname, char *newrelname)
 	if (rename(oldpath, newpath) < 0)
 		elog(ERROR, "renamerel: unable to rename file: %s", oldpath);
 
+	for (i = 1;; i++)
+	{
+		sprintf(toldpath, "%s.%d", oldpath, i);
+		sprintf(tnewpath, "%s.%d", newpath, i);
+		if (rename(toldpath, tnewpath) < 0)
+			break;
+	}
+
 	StrNCpy((((Form_pg_class) GETSTRUCT(oldreltup))->relname.data),
 			newrelname, NAMEDATALEN);
 
 	/* insert fixed rel tuple */
 	relrelation = heap_openr(RelationRelationName);
-	heap_replace(relrelation, &oldreltup->t_ctid, oldreltup);
+	heap_replace(relrelation, &oldreltup->t_self, oldreltup, NULL);
 
 	/* keep the system catalog indices current */
 	CatalogOpenIndices(Num_pg_class_indices, Name_pg_class_indices, irelations);
