@@ -1,17 +1,15 @@
 /*-------------------------------------------------------------------------
  *
- * view.c--
+ * view.c
  *	  use rewrite rules to construct views
  *
  * Copyright (c) 1994, Regents of the University of California
  *
- *
- * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/commands/view.c,v 1.26 1998/10/21 16:21:22 momjian Exp $
+ *	$Id: view.c,v 1.33 1999/05/25 16:08:28 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
-#include <stdio.h>				/* for sprintf() */
+#include <stdio.h>
 #include <string.h>
 
 #include <postgres.h>
@@ -97,6 +95,7 @@ DefineVirtualRelation(char *relname, List *tlist)
 	 * nil...
 	 */
 	createStmt.relname = relname;
+	createStmt.istemp = false;
 	createStmt.tableElts = attrList;
 /*	  createStmt.tableType = NULL;*/
 	createStmt.inhRelnames = NIL;
@@ -122,7 +121,7 @@ MakeRetrieveViewRuleName(char *viewName)
 	char	   *buf;
 
 	buf = palloc(strlen(viewName) + 5);
-	sprintf(buf, "_RET%s", viewName);
+	snprintf(buf, strlen(viewName) + 5, "_RET%s", viewName);
 
 	return buf;
 }
@@ -166,17 +165,13 @@ DefineViewRules(char *viewName, Query *viewParse)
 
 #endif
 
-	retrieve_rule =
-		FormViewRetrieveRule(viewName, viewParse);
+	retrieve_rule = FormViewRetrieveRule(viewName, viewParse);
 
 #ifdef NOTYET
 
-	replace_rule =
-		FormViewReplaceRule(viewName, viewParse);
-	append_rule =
-		FormViewAppendRule(viewName, viewParse);
-	delete_rule =
-		FormViewDeleteRule(viewName, viewParse);
+	replace_rule = FormViewReplaceRule(viewName, viewParse);
+	append_rule = FormViewAppendRule(viewName, viewParse);
+	delete_rule = FormViewDeleteRule(viewName, viewParse);
 
 #endif
 
@@ -234,12 +229,10 @@ UpdateRangeTableOfViewParse(char *viewName, Query *viewParse)
 	 * create the 2 new range table entries and form the new range
 	 * table... CURRENT first, then NEW....
 	 */
-	rt_entry1 =
-		addRangeTableEntry(NULL, (char *) viewName, "*CURRENT*",
-						   FALSE, FALSE);
-	rt_entry2 =
-		addRangeTableEntry(NULL, (char *) viewName, "*NEW*",
-						   FALSE, FALSE);
+	rt_entry1 = addRangeTableEntry(NULL, (char *) viewName, "*CURRENT*",
+								   FALSE, FALSE);
+	rt_entry2 = addRangeTableEntry(NULL, (char *) viewName, "*NEW*",
+								   FALSE, FALSE);
 	new_rt = lcons(rt_entry2, old_rt);
 	new_rt = lcons(rt_entry1, new_rt);
 
