@@ -1,6 +1,6 @@
 /*-------------------------------------------------------------------------
  *
- * execTuples.c--
+ * execTuples.c
  *	  Routines dealing with the executor tuple tables.	These are used to
  *	  ensure that the executor frees copies of tuples (made by
  *	  ExecTargetList) properly.
@@ -14,7 +14,7 @@
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/executor/execTuples.c,v 1.21 1998/10/08 18:29:26 momjian Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/executor/execTuples.c,v 1.25 1999/05/25 16:08:39 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -467,6 +467,7 @@ ExecSetSlotPolicy(TupleTableSlot *slot, /* slot to change */
 
 	return old_shouldFree;
 }
+
 #endif
 
 /* --------------------------------
@@ -650,6 +651,7 @@ ExecInitMarkedTupleSlot(EState *estate, MergeJoinState *mergestate)
 	INIT_SLOT_ALLOC;
 	mergestate->mj_MarkedTupleSlot = (TupleTableSlot *) slot;
 }
+
 #endif
 
 /* ----------------
@@ -800,14 +802,6 @@ NodeGetResultTupleSlot(Plan *node)
 			}
 			break;
 
-		case T_Tee:
-			{
-				TeeState   *teestate = ((Tee *) node)->teestate;
-
-				slot = teestate->cstate.cs_ResultTupleSlot;
-			}
-			break;
-
 		default:
 			/* ----------------
 			 *	  should never get here
@@ -870,8 +864,7 @@ ExecCopyTupType(TupleDesc td, int natts)
 	i = 0;
 	while (i < natts)
 		{
-			newTd[i] =
-				(Form_pg_attribute)palloc(sizeof(FormData_pg_attribute));
+			newTd[i] = (Form_pg_attribute)palloc(sizeof(FormData_pg_attribute));
 			memmove(newTd[i], td[i], sizeof(FormData_pg_attribute));
 			i++;
 		}
