@@ -1,13 +1,13 @@
 /*-------------------------------------------------------------------------
  *
- * nodeMergejoin.c--
+ * nodeMergejoin.c
  *	  routines supporting merge joins
  *
  * Copyright (c) 1994, Regents of the University of California
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/executor/nodeMergejoin.c,v 1.19 1998/09/01 04:28:35 momjian Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/executor/nodeMergejoin.c,v 1.27 1999/05/25 16:08:45 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -19,19 +19,19 @@
  *
  * NOTES
  *		Essential operation of the merge join algorithm is as follows:
- *		(** indicates the tuples satisify the merge clause).
+ *		(** indicates the tuples satisfy the merge clause).
  *
  *		Join {												   -
  *			get initial outer and inner tuples				INITIALIZE
  *			Skip Inner										SKIPINNER
  *			mark inner position								JOINMARK
  *			do forever {									   -
- *				while (outer ** inner) {					JOINTEST
+ *				while (outer == inner) {					JOINTEST
  *					join tuples								JOINTUPLES
  *					advance inner position					NEXTINNER
  *				}											   -
  *				advance outer position						NEXTOUTER
- *				if (outer ** mark) {						TESTOUTER
+ *				if (outer == mark) {						TESTOUTER
  *					restore inner position to mark			TESTOUTER
  *					continue								   -
  *				} else {									   -
@@ -42,7 +42,7 @@
  *		}													   -
  *
  *		Skip Outer {										SKIPOUTER
- *			if (inner ** outer) Join Tuples					JOINTUPLES
+ *			if (inner == outer) Join Tuples					JOINTUPLES
  *			while (outer < inner)							SKIPOUTER
  *				advance outer								SKIPOUTER
  *			if (outer > inner)								SKIPOUTER
@@ -50,34 +50,24 @@
  *		}													   -
  *
  *		Skip Inner {										SKIPINNER
- *			if (inner ** outer) Join Tuples					JOINTUPLES
+ *			if (inner == outer) Join Tuples					JOINTUPLES
  *			while (outer > inner)							SKIPINNER
  *				advance inner								SKIPINNER
  *			if (outer < inner)								SKIPINNER
  *				Skip Outer									SKIPOUTER
  *		}													   -
  *
- *		Currently, the merge join operation is coded in the fashion
+ *		The merge join operation is coded in the fashion
  *		of a state machine.  At each state, we do something and then
  *		proceed to another state.  This state is stored in the node's
  *		execution state information and is preserved across calls to
  *		ExecMergeJoin. -cim 10/31/89
  *
- *		Warning:  This code is known to fail for inequality operations
- *				  and is being redesigned.	Specifically, = and > work
- *				  but the logic is not correct for <.  Since mergejoins
- *				  are no better then nestloops for inequalitys, the planner
- *				  should not plan them anyways.  Alternatively, the
- *				  planner could just exchange the inner/outer relations
- *				  if it ever sees a <... -cim 7/1/90
- *
- *		Update:   The executor tuple table has long since alleviated the
- *				  problem described above -cim 4/23/91
- *
  */
 #include "postgres.h"
 
 #include "access/heapam.h"
+#include "catalog/pg_operator.h"
 #include "executor/executor.h"
 #include "executor/execdefs.h"
 #include "executor/nodeMergejoin.h"
@@ -85,6 +75,7 @@
 #include "utils/lsyscache.h"
 #include "utils/psort.h"
 
+
 static bool MergeCompare(List *eqQual, List *compareQual, ExprContext *econtext);
 
 #define MarkInnerTuple(innerTupleSlot, mergestate) \
@@ -95,29 +86,33 @@ static bool MergeCompare(List *eqQual, List *compareQual, ExprContext *econtext)
 				   true) \
 )
 
+
 /* ----------------------------------------------------------------
- *		MJFormOSortopI
+ *		MJFormSkipQual
  *
  *		This takes the mergeclause which is a qualification of the
  *		form ((= expr expr) (= expr expr) ...) and forms a new
  *		qualification like ((> expr expr) (> expr expr) ...) which
  *		is used by ExecMergeJoin() in order to determine if we should
- *		skip tuples.
- *
- * old comments
- *		The 'qual' must be of the form:
- *		   {(= outerkey1 innerkey1)(= outerkey2 innerkey2) ...}
- *		The "sortOp outerkey innerkey" is formed by substituting the "="
- *		by "sortOp".
+ *		skip tuples.  The replacement operators are named either ">"
+ *		or "<" according to the replaceopname parameter, and have the
+ *		same operand data types as the "=" operators they replace.
+ *		(We expect there to be such operators because the "=" operators
+ *		were marked mergejoinable; however, there might be a different
+ *		one needed in each qual clause.)
  * ----------------------------------------------------------------
  */
 static List *
-MJFormOSortopI(List *qualList, Oid sortOp)
+MJFormSkipQual(List *qualList, char *replaceopname)
 {
 	List	   *qualCopy;
 	List	   *qualcdr;
 	Expr	   *qual;
 	Oper	   *op;
+	HeapTuple	optup;
+	Form_pg_operator opform;
+	Oid			oprleft,
+				oprright;
 
 	/* ----------------
 	 *	qualList is a list: ((op .. ..) ...)
@@ -141,75 +136,52 @@ MJFormOSortopI(List *qualList, Oid sortOp)
 		 */
 		op = (Oper *) qual->oper;
 		if (!IsA(op, Oper))
-		{
-			elog(DEBUG, "MJFormOSortopI: op not an Oper!");
-			return NIL;
-		}
+			elog(ERROR, "MJFormSkipQual: op not an Oper!");
 
 		/* ----------------
-		 *	 change it's opid and since Op nodes now carry around a
-		 *	 cached pointer to the associated op function, we have
-		 *	 to make sure we invalidate this.  Otherwise you get bizarre
-		 *	 behavior when someone runs a mergejoin with _exec_repeat_ > 1
-		 *	 -cim 4/23/91
+		 *	 Get the declared left and right operand types of the operator.
+		 *	 Note we do *not* use the actual operand types, since those might
+		 *	 be different in scenarios with binary-compatible data types.
+		 *	 There should be "<" and ">" operators matching a mergejoinable
+		 *	 "=" operator's declared operand types, but we might not find them
+		 *	 if we search with the actual operand types.
 		 * ----------------
 		 */
-		op->opid = sortOp;
+		optup = get_operator_tuple(op->opno);
+		if (!HeapTupleIsValid(optup))	/* shouldn't happen */
+			elog(ERROR, "MJFormSkipQual: operator %u not found", op->opno);
+		opform = (Form_pg_operator) GETSTRUCT(optup);
+		oprleft = opform->oprleft;
+		oprright = opform->oprright;
+
+		/* ----------------
+		 *	 Now look up the matching "<" or ">" operator.	If there isn't one,
+		 *	 whoever marked the "=" operator mergejoinable was a loser.
+		 * ----------------
+		 */
+		optup = SearchSysCacheTuple(OPRNAME,
+									PointerGetDatum(replaceopname),
+									ObjectIdGetDatum(oprleft),
+									ObjectIdGetDatum(oprright),
+									CharGetDatum('b'));
+		if (!HeapTupleIsValid(optup))
+			elog(ERROR,
+			"MJFormSkipQual: mergejoin operator %u has no matching %s op",
+				 op->opno, replaceopname);
+		opform = (Form_pg_operator) GETSTRUCT(optup);
+
+		/* ----------------
+		 *	 And replace the data in the copied operator node.
+		 * ----------------
+		 */
+		op->opno = optup->t_data->t_oid;
+		op->opid = opform->oprcode;
 		op->op_fcache = NULL;
 	}
 
 	return qualCopy;
 }
 
-/* ----------------------------------------------------------------
- *		MJFormISortopO
- *
- *		This does the same thing as MJFormOSortopI() except that
- *		it also reverses the expressions in the qualifications.
- *		For example: ((= expr1 expr2)) produces ((> expr2 expr1))
- *
- * old comments
- *		The 'qual' must be of the form:
- *		   {(= outerkey1 innerkey1) (= outerkey2 innerkey2) ...}
- *		The 'sortOp innerkey1 outerkey" is formed by substituting the "="
- *		by "sortOp" and reversing the positions of the keys.
- *	----------------------------------------------------------------
- */
-static List *
-MJFormISortopO(List *qualList, Oid sortOp)
-{
-	List	   *ISortopO;
-	List	   *qualcdr;
-
-	/* ----------------
-	 *	first generate OSortopI, a list of the form
-	 *	((op outer inner) (op outer inner) ... )
-	 * ----------------
-	 */
-	ISortopO = MJFormOSortopI(qualList, sortOp);
-
-	/* ----------------
-	 *	now swap the cadr and caddr of each qual to form ISortopO,
-	 *	((op inner outer) (op inner outer) ... )
-	 * ----------------
-	 */
-	foreach(qualcdr, ISortopO)
-	{
-		Expr	   *qual;
-		List	   *inner;
-		List	   *outer;
-
-		qual = lfirst(qualcdr);
-
-		inner = lfirst(qual->args);
-		outer = lfirst(lnext(qual->args));
-		lfirst(qual->args) = outer;
-		lfirst(lnext(qual->args)) = inner;
-	}
-
-	return ISortopO;
-}
-
 /* ----------------------------------------------------------------
  *		MergeCompare
  *
@@ -224,6 +196,7 @@ MJFormISortopO(List *qualList, Oid sortOp)
  *		the first keys being most significant. Therefore, the clauses
  *		are evaluated in order and the 'compareQual' is satisfied
  *		if (key1i > key2i) is true and (key1j = key2j) for 0 < j < i.
+ *		We use the original mergeclause items to detect equality.
  * ----------------------------------------------------------------
  */
 static bool
@@ -296,6 +269,9 @@ MergeCompare(List *eqQual, List *compareQual, ExprContext *econtext)
  * ----------------------------------------------------------------
  */
 #ifdef EXEC_MERGEJOINDEBUG
+void
+			ExecMergeTupleDumpInner(ExprContext *econtext);
+
 void
 ExecMergeTupleDumpInner(ExprContext *econtext)
 {
@@ -306,10 +282,13 @@ ExecMergeTupleDumpInner(ExprContext *econtext)
 	if (TupIsNull(innerSlot))
 		printf("(nil)\n");
 	else
-		debugtup(innerSlot->val,
-				 innerSlot->ttc_tupleDescriptor);
+		MJ_debugtup(innerSlot->val,
+					innerSlot->ttc_tupleDescriptor);
 }
 
+void
+			ExecMergeTupleDumpOuter(ExprContext *econtext);
+
 void
 ExecMergeTupleDumpOuter(ExprContext *econtext)
 {
@@ -320,10 +299,13 @@ ExecMergeTupleDumpOuter(ExprContext *econtext)
 	if (TupIsNull(outerSlot))
 		printf("(nil)\n");
 	else
-		debugtup(outerSlot->val,
-				 outerSlot->ttc_tupleDescriptor);
+		MJ_debugtup(outerSlot->val,
+					outerSlot->ttc_tupleDescriptor);
 }
 
+void ExecMergeTupleDumpMarked(ExprContext *econtext,
+						 MergeJoinState *mergestate);
+
 void
 ExecMergeTupleDumpMarked(ExprContext *econtext,
 						 MergeJoinState *mergestate)
@@ -336,10 +318,13 @@ ExecMergeTupleDumpMarked(ExprContext *econtext,
 	if (TupIsNull(markedSlot))
 		printf("(nil)\n");
 	else
-		debugtup(markedSlot->val,
-				 markedSlot->ttc_tupleDescriptor);
+		MJ_debugtup(markedSlot->val,
+					markedSlot->ttc_tupleDescriptor);
 }
 
+void
+			ExecMergeTupleDump(ExprContext *econtext, MergeJoinState *mergestate);
+
 void
 ExecMergeTupleDump(ExprContext *econtext, MergeJoinState *mergestate)
 {
@@ -383,12 +368,16 @@ ExecMergeTupleDump(ExprContext *econtext, MergeJoinState *mergestate)
  *
  *		Therefore, when initializing the merge-join node, the executor
  *		creates the "greater/smaller" clause by substituting the "="
- *		operator in the join clauses with the sort operator used to
- *		sort the outer and inner relation forming (outerKey sortOp innerKey).
- *		The sort operator is "<" if the relations are in ascending order
- *		otherwise, it is ">" if the relations are in descending order.
- *		The opposite "smaller/greater" clause is formed by reversing the
- *		outer and inner keys forming (innerKey sortOp outerKey).
+ *		operator in the join clauses with the corresponding ">" operator.
+ *		The opposite "smaller/greater" clause is formed by substituting "<".
+ *
+ *		Note: prior to v6.5, the relational clauses were formed using the
+ *		sort op used to sort the inner relation, which of course would fail
+ *		if the outer and inner keys were of different data types.
+ *		In the current code, we instead assume that operators named "<" and ">"
+ *		will do the right thing.  This should be true since the mergejoin "="
+ *		operator's pg_operator entry will have told the planner to sort by
+ *		"<" for each of the left and right sides.
  *
  *		(2) repositioning inner "cursor"
  *
@@ -423,6 +412,17 @@ ExecMergeJoin(MergeJoin *node)
 
 	ExprContext *econtext;
 
+#ifdef ENABLE_OUTER_JOINS
+
+	/*
+	 * These should be set from the expression context! - thomas
+	 * 1999-02-20
+	 */
+	static bool isLeftJoin = true;
+	static bool isRightJoin = false;
+
+#endif
+
 	/* ----------------
 	 *	get information from node
 	 * ----------------
@@ -438,13 +438,13 @@ ExecMergeJoin(MergeJoin *node)
 
 	if (ScanDirectionIsForward(direction))
 	{
-		outerSkipQual = mergestate->mj_OSortopI;
-		innerSkipQual = mergestate->mj_ISortopO;
+		outerSkipQual = mergestate->mj_OuterSkipQual;
+		innerSkipQual = mergestate->mj_InnerSkipQual;
 	}
 	else
 	{
-		outerSkipQual = mergestate->mj_ISortopO;
-		innerSkipQual = mergestate->mj_OSortopI;
+		outerSkipQual = mergestate->mj_InnerSkipQual;
+		innerSkipQual = mergestate->mj_OuterSkipQual;
 	}
 
 	/* ----------------
@@ -476,20 +476,18 @@ ExecMergeJoin(MergeJoin *node)
 		{
 
 				/*
-				 * ******************************** EXEC_MJ_INITIALIZE
-				 * means that this is the first time ExecMergeJoin() has
-				 * been called and so we have to initialize the inner,
-				 * outer and marked tuples as well as various stuff in the
-				 * expression context. ********************************
-				 *
+				 * EXEC_MJ_INITIALIZE means that this is the first time
+				 * ExecMergeJoin() has been called and so we have to
+				 * initialize the inner, outer and marked tuples as well
+				 * as various stuff in the expression context.
 				 */
 			case EXEC_MJ_INITIALIZE:
 				MJ_printf("ExecMergeJoin: EXEC_MJ_INITIALIZE\n");
-				/* ----------------
-				 *	 Note: at this point, if either of our inner or outer
-				 *	 tuples are nil, then the join ends immediately because
-				 *	 we know one of the subplans is empty.
-				 * ----------------
+
+				/*
+				 * Note: at this point, if either of our inner or outer
+				 * tuples are nil, then the join ends immediately because
+				 * we know one of the subplans is empty.
 				 */
 				innerTupleSlot = ExecProcNode(innerPlan, (Plan *) node);
 				if (TupIsNull(innerTupleSlot))
@@ -523,11 +521,10 @@ ExecMergeJoin(MergeJoin *node)
 				break;
 
 				/*
-				 * ******************************** EXEC_MJ_JOINMARK means
-				 * we have just found a new outer tuple and a possible
-				 * matching inner tuple. This is the case after the
-				 * INITIALIZE, SKIPOUTER or SKIPINNER states. ********************************
-				 *
+				 * EXEC_MJ_JOINMARK means we have just found a new outer
+				 * tuple and a possible matching inner tuple. This is the
+				 * case after the INITIALIZE, SKIPOUTER or SKIPINNER
+				 * states.
 				 */
 			case EXEC_MJ_JOINMARK:
 				MJ_printf("ExecMergeJoin: EXEC_MJ_JOINMARK\n");
@@ -539,15 +536,13 @@ ExecMergeJoin(MergeJoin *node)
 				break;
 
 				/*
-				 * ******************************** EXEC_MJ_JOINTEST means
-				 * we have two tuples which might satisify the merge
-				 * clause, so we test them.
+				 * EXEC_MJ_JOINTEST means we have two tuples which might
+				 * satisfy the merge clause, so we test them.
 				 *
-				 * If they do satisify, then we join them and move on to the
+				 * If they do satisfy, then we join them and move on to the
 				 * next inner tuple (EXEC_MJ_JOINTUPLES).
 				 *
-				 * If they do not satisify then advance to next outer tuple. ********************************
-				 *
+				 * If they do not satisfy then advance to next outer tuple.
 				 */
 			case EXEC_MJ_JOINTEST:
 				MJ_printf("ExecMergeJoin: EXEC_MJ_JOINTEST\n");
@@ -562,11 +557,9 @@ ExecMergeJoin(MergeJoin *node)
 				break;
 
 				/*
-				 * ******************************** EXEC_MJ_JOINTUPLES
-				 * means we have two tuples which satisified the merge
-				 * clause so we join them and then proceed to get the next
-				 * inner tuple (EXEC_NEXT_INNER). ********************************
-				 *
+				 * EXEC_MJ_JOINTUPLES means we have two tuples which
+				 * satisified the merge clause so we join them and then
+				 * proceed to get the next inner tuple (EXEC_NEXT_INNER).
 				 */
 			case EXEC_MJ_JOINTUPLES:
 				MJ_printf("ExecMergeJoin: EXEC_MJ_JOINTUPLES\n");
@@ -597,11 +590,9 @@ ExecMergeJoin(MergeJoin *node)
 				break;
 
 				/*
-				 * ******************************** EXEC_MJ_NEXTINNER
-				 * means advance the inner scan to the next tuple.	If the
-				 * tuple is not nil, we then proceed to test it against
-				 * the join qualification. ********************************
-				 *
+				 * EXEC_MJ_NEXTINNER means advance the inner scan to the
+				 * next tuple. If the tuple is not nil, we then proceed to
+				 * test it against the join qualification.
 				 */
 			case EXEC_MJ_NEXTINNER:
 				MJ_printf("ExecMergeJoin: EXEC_MJ_NEXTINNER\n");
@@ -620,18 +611,21 @@ ExecMergeJoin(MergeJoin *node)
 					mergestate->mj_JoinState = EXEC_MJ_JOINTEST;
 				break;
 
-				/*
-				 * ******************************** EXEC_MJ_NEXTOUTER
-				 * means
+				/*-------------------------------------------
+				 * EXEC_MJ_NEXTOUTER means
 				 *
-				 * outer inner outer tuple -  5		5  - marked tuple 5		5
-				 * 6 6	- inner tuple 7		7
+				 *				outer inner
+				 * outer tuple -  5		5  - marked tuple
+				 *				  5		5
+				 *				  6		6  - inner tuple
+				 *				  7		7
 				 *
-				 * we know we just bumped into the first inner tuple >
-				 * current outer tuple so get a new outer tuple and then
+				 * we know we just bumped into the
+				 * first inner tuple > current outer tuple
+				 * so get a new outer tuple and then
 				 * proceed to test it against the marked tuple
-				 * (EXEC_MJ_TESTOUTER) ********************************
-				 *
+				 * (EXEC_MJ_TESTOUTER)
+				 *------------------------------------------------
 				 */
 			case EXEC_MJ_NEXTOUTER:
 				MJ_printf("ExecMergeJoin: EXEC_MJ_NEXTOUTER\n");
@@ -654,35 +648,39 @@ ExecMergeJoin(MergeJoin *node)
 				mergestate->mj_JoinState = EXEC_MJ_TESTOUTER;
 				break;
 
-				/*
-				 * ******************************** EXEC_MJ_TESTOUTER If
-				 * the new outer tuple and the marked tuple satisify the
-				 * merge clause then we know we have duplicates in the
-				 * outer scan so we have to restore the inner scan to the
-				 * marked tuple and proceed to join the new outer tuples
-				 * with the inner tuples (EXEC_MJ_JOINTEST)
+				/*--------------------------------------------------------
+				 * EXEC_MJ_TESTOUTER If the new outer tuple and the marked
+				 * tuple satisfy the merge clause then we know we have
+				 * duplicates in the outer scan so we have to restore the
+				 * inner scan to the marked tuple and proceed to join the
+				 * new outer tuples with the inner tuples (EXEC_MJ_JOINTEST)
 				 *
 				 * This is the case when
+				 *						  outer inner
+				 *							4	  5  - marked tuple
+				 *			 outer tuple -	5	  5
+				 *		 new outer tuple -	5	  5
+				 *							6	  8  - inner tuple
+				 *							7	 12
 				 *
-				 * outer inner 4	 5	- marked tuple outer tuple -  5		5
-				 * new outer tuple -  5		5 6		8  - inner tuple 7 12
+				 *				new outer tuple = marked tuple
 				 *
-				 * new outer tuple = marked tuple
+				 *		If the outer tuple fails the test, then we know we have
+				 *		to proceed to skip outer tuples until outer >= inner
+				 *		(EXEC_MJ_SKIPOUTER).
 				 *
-				 * If the outer tuple fails the test, then we know we have to
-				 * proceed to skip outer tuples until outer >= inner
-				 * (EXEC_MJ_SKIPOUTER).
+				 *		This is the case when
 				 *
-				 * This is the case when
-				 *
-				 * outer inner 5	 5	- marked tuple outer tuple -  5		5
-				 * new outer tuple -  6		8  - inner tuple 7	  12
-				 *
-				 * new outer tuple > marked tuple
-				 *
-				***************************
+				 *						  outer inner
+				 *							5	  5  - marked tuple
+				 *			 outer tuple -	5	  5
+				 *		 new outer tuple -	6	  8  - inner tuple
+				 *							7	 12
 				 *
 				 *
+				 *		 new outer tuple > marked tuple
+				 *
+				 *---------------------------------------------------------
 				 */
 			case EXEC_MJ_TESTOUTER:
 				MJ_printf("ExecMergeJoin: EXEC_MJ_TESTOUTER\n");
@@ -725,11 +723,11 @@ ExecMergeJoin(MergeJoin *node)
 					 *	tuple didn't match the marked outer tuple then
 					 *	we may have the case:
 					 *
-					 *				outer	inner
-					 *					4	  4   - marked tuple
-					 *		new outer - 5	  4
-					 *					6	 nil  - inner tuple
-					 *					7
+					 *			 outer inner
+					 *			   4	 4	- marked tuple
+					 * new outer - 5	 4
+					 *			   6	nil - inner tuple
+					 *			   7
 					 *
 					 *	which means that all subsequent outer tuples will be
 					 *	larger than our inner tuples.
@@ -737,7 +735,15 @@ ExecMergeJoin(MergeJoin *node)
 					 */
 					if (TupIsNull(innerTupleSlot))
 					{
-						MJ_printf("ExecMergeJoin: **** wierd case 1 ****\n");
+#ifdef ENABLE_OUTER_JOINS
+						if (isLeftJoin)
+						{
+							/* continue on to null fill outer tuples */
+							mergestate->mj_JoinState = EXEC_MJ_FILLOUTER;
+							break;
+						}
+#endif
+						MJ_printf("ExecMergeJoin: **** weird case 1 ****\n");
 						return NULL;
 					}
 
@@ -746,33 +752,28 @@ ExecMergeJoin(MergeJoin *node)
 				}
 				break;
 
-				/*
-				 * ******************************** EXEC_MJ_SKIPOUTER
-				 * means skip over tuples in the outer plan until we find
-				 * an outer tuple > current inner tuple.
+				/*----------------------------------------------------------
+				 * EXEC_MJ_SKIPOUTER means skip over tuples in the outer plan
+				 * until we find an outer tuple > current inner tuple.
 				 *
 				 * For example:
 				 *
-				 * outer inner 5	 5 5	 5 outer tuple -  6		8  -
-				 * inner tuple 7	12 8	14
-				 *
-				 * we have to advance the outer scan until we find the outer
-				 * 8.
-				 *
-				**************************
-				 *
-				 *
-				 *
-				 *
-				 *
-				 *
+				 *				outer inner
+				 *				  5		5
+				 *				  5		5
+				 * outer tuple -  6		8  - inner tuple
+				 *				  7    12
+				 *				  8    14
 				 *
+				 * we have to advance the outer scan
+				 * until we find the outer 8.
+				 *----------------------------------------------------------
 				 */
 			case EXEC_MJ_SKIPOUTER:
 				MJ_printf("ExecMergeJoin: EXEC_MJ_SKIPOUTER\n");
 				/* ----------------
 				 *	before we advance, make sure the current tuples
-				 *	do not satisify the mergeclauses.  If they do, then
+				 *	do not satisfy the mergeclauses.  If they do, then
 				 *	we update the marked tuple and go join them.
 				 * ----------------
 				 */
@@ -806,6 +807,17 @@ ExecMergeJoin(MergeJoin *node)
 				 */
 				if (compareResult)
 				{
+#ifdef ENABLE_OUTER_JOINS
+					/* ----------------
+					 *	if this is a left or full outer join, then fill
+					 * ----------------
+					 */
+					if (isLeftJoin)
+					{
+						mergestate->mj_JoinState = EXEC_MJ_FILLOUTER;
+						break;
+					}
+#endif
 
 					outerTupleSlot = ExecProcNode(outerPlan, (Plan *) node);
 					MJ_DEBUG_PROC_NODE(outerTupleSlot);
@@ -848,33 +860,29 @@ ExecMergeJoin(MergeJoin *node)
 					mergestate->mj_JoinState = EXEC_MJ_JOINMARK;
 				break;
 
-				/*
-				 * ******************************** EXEC_MJ_SKIPINNER
-				 * means skip over tuples in the inner plan until we find
-				 * an inner tuple > current outer tuple.
+				/*-----------------------------------------------------------
+				 * EXEC_MJ_SKIPINNER means skip over tuples in the inner plan
+				 * until we find an inner tuple > current outer tuple.
 				 *
 				 * For example:
 				 *
-				 * outer inner 5	 5 5	 5 outer tuple - 12		8 - inner
-				 * tuple 14    10 17	12
-				 *
-				 * we have to advance the inner scan until we find the inner
-				 * 12.
-				 *
-				**************************
-				 *
-				 *
-				 *
-				 *
-				 *
+				 *				outer inner
+				 *				  5		5
+				 *				  5		5
+				 * outer tuple - 12		8  - inner tuple
+				 *				 14    10
+				 *				 17    12
 				 *
+				 * we have to advance the inner scan
+				 * until we find the inner 12.
 				 *
+				 *-------------------------------------------------------
 				 */
 			case EXEC_MJ_SKIPINNER:
 				MJ_printf("ExecMergeJoin: EXEC_MJ_SKIPINNER\n");
 				/* ----------------
 				 *	before we advance, make sure the current tuples
-				 *	do not satisify the mergeclauses.  If they do, then
+				 *	do not satisfy the mergeclauses.  If they do, then
 				 *	we update the marked tuple and go join them.
 				 * ----------------
 				 */
@@ -908,6 +916,18 @@ ExecMergeJoin(MergeJoin *node)
 				 */
 				if (compareResult)
 				{
+#ifdef ENABLE_OUTER_JOINS
+					/* ----------------
+					 *	if this is a right or full outer join, then fill
+					 * ----------------
+					 */
+					if (isRightJoin)
+					{
+						mergestate->mj_JoinState = EXEC_MJ_FILLINNER;
+						break;
+					}
+#endif
+
 					/* ----------------
 					 *	now try and get a new inner tuple
 					 * ----------------
@@ -939,7 +959,7 @@ ExecMergeJoin(MergeJoin *node)
 						 *	This means the join should end.
 						 * ----------------
 						 */
-						MJ_printf("ExecMergeJoin: **** wierd case 2 ****\n");
+						MJ_printf("ExecMergeJoin: **** weird case 2 ****\n");
 						return NULL;
 					}
 
@@ -970,11 +990,111 @@ ExecMergeJoin(MergeJoin *node)
 
 				break;
 
+#ifdef ENABLE_OUTER_JOINS
+
 				/*
-				 * ******************************** if we get here it
-				 * means our code is fucked up and so we just end the join
-				 * prematurely. ********************************
-				 *
+				 * EXEC_MJ_FILLINNER means we have an unmatched inner
+				 * tuple which must be null-expanded into the projection
+				 * tuple. get the next inner tuple and reset markers
+				 * (EXEC_MJ_JOINMARK).
+				 */
+			case EXEC_MJ_FILLINNER:
+				MJ_printf("ExecMergeJoin: EXEC_MJ_FILLINNER\n");
+				mergestate->mj_JoinState = EXEC_MJ_JOINMARK;
+
+				/* ----------------
+				 *	project the inner tuple into the result
+				 * ----------------
+				 */
+				MJ_printf("ExecMergeJoin: project inner tuple into the result (not yet implemented)\n");
+
+				/* ----------------
+				 *	now skip this inner tuple
+				 * ----------------
+				 */
+				innerTupleSlot = ExecProcNode(innerPlan, (Plan *) node);
+				MJ_DEBUG_PROC_NODE(innerTupleSlot);
+				econtext->ecxt_innertuple = innerTupleSlot;
+
+				/* ----------------
+				 *	if the inner tuple is null then we know
+				 *	we have to restore the inner scan
+				 *	and advance to the next outer tuple
+				 * ----------------
+				 */
+				if (TupIsNull(innerTupleSlot))
+				{
+					if (isLeftJoin && !TupIsNull(outerTupleSlot))
+					{
+						mergestate->mj_JoinState = EXEC_MJ_FILLOUTER;
+						MJ_printf("ExecMergeJoin: try to complete outer fill\n");
+						break;
+					}
+
+					MJ_printf("ExecMergeJoin: **** weird case 2 ****\n");
+					return NULL;
+				}
+
+				/* ----------------
+				 *	otherwise test the new tuple against the skip qual.
+				 *	(we move to the EXEC_MJ_JOINMARK state)
+				 * ----------------
+				 */
+				break;
+
+				/*
+				 * EXEC_MJ_FILLOUTER means we have an unmatched outer
+				 * tuple which must be null-expanded into the projection
+				 * tuple. get the next outer tuple and reset markers
+				 * (EXEC_MJ_JOINMARK).
+				 */
+			case EXEC_MJ_FILLOUTER:
+				MJ_printf("ExecMergeJoin: EXEC_MJ_FILLOUTER\n");
+				mergestate->mj_JoinState = EXEC_MJ_JOINMARK;
+
+				/* ----------------
+				 *	project the outer tuple into the result
+				 * ----------------
+				 */
+				MJ_printf("ExecMergeJoin: project outer tuple into the result (not yet implemented)\n");
+
+				/* ----------------
+				 *	now skip this outer tuple
+				 * ----------------
+				 */
+				outerTupleSlot = ExecProcNode(outerPlan, (Plan *) node);
+				MJ_DEBUG_PROC_NODE(outerTupleSlot);
+				econtext->ecxt_outertuple = outerTupleSlot;
+
+				/* ----------------
+				 *	if the outer tuple is null then we know
+				 *	we are done with the left half of the join
+				 * ----------------
+				 */
+				if (TupIsNull(outerTupleSlot))
+				{
+					if (isRightJoin && !TupIsNull(innerTupleSlot))
+					{
+						mergestate->mj_JoinState = EXEC_MJ_FILLINNER;
+						MJ_printf("ExecMergeJoin: try to complete inner fill\n");
+						break;
+					}
+
+					MJ_printf("ExecMergeJoin: **** outerTuple is nil ****\n");
+					return NULL;
+				}
+
+				/* ----------------
+				 *	otherwise test the new tuple against the skip qual.
+				 *	(we move to the EXEC_MJ_JOINMARK state)
+				 * ----------------
+				 */
+				break;
+#endif
+
+				/*
+				 * if we get here it means our code is fouled up and so we
+				 * just end the join prematurely.
 				 */
 			default:
 				elog(NOTICE, "ExecMergeJoin: invalid join state. aborting");
@@ -996,14 +1116,8 @@ ExecInitMergeJoin(MergeJoin *node, EState *estate, Plan *parent)
 {
 	MergeJoinState *mergestate;
 	List	   *joinclauses;
-	RegProcedure rightsortop;
-	RegProcedure leftsortop;
-	RegProcedure sortop;
 	TupleTableSlot *mjSlot;
 
-	List	   *OSortopI;
-	List	   *ISortopO;
-
 	MJ1_printf("ExecInitMergeJoin: %s\n",
 			   "initializing node");
 
@@ -1019,14 +1133,14 @@ ExecInitMergeJoin(MergeJoin *node, EState *estate, Plan *parent)
 	 * ----------------
 	 */
 	mergestate = makeNode(MergeJoinState);
-	mergestate->mj_OSortopI = NIL;
-	mergestate->mj_ISortopO = NIL;
+	mergestate->mj_OuterSkipQual = NIL;
+	mergestate->mj_InnerSkipQual = NIL;
 	mergestate->mj_JoinState = 0;
 	mergestate->mj_MarkedTupleSlot = NULL;
 	node->mergestate = mergestate;
 
 	/* ----------------
-	 *	Miscellanious initialization
+	 *	Miscellaneous initialization
 	 *
 	 *		 +	assign node's base_id
 	 *		 +	assign debugging hooks and
@@ -1051,40 +1165,17 @@ ExecInitMergeJoin(MergeJoin *node, EState *estate, Plan *parent)
 	mergestate->mj_MarkedTupleSlot = mjSlot;
 
 	/* ----------------
-	 *	get merge sort operators.
-	 *
-	 *	XXX for now we assume all quals in the joinclauses were
-	 *		sorted with the same operator in both the inner and
-	 *		outer relations. -cim 11/2/89
+	 *	form merge skip qualifications
 	 * ----------------
 	 */
 	joinclauses = node->mergeclauses;
+	mergestate->mj_OuterSkipQual = MJFormSkipQual(joinclauses, "<");
+	mergestate->mj_InnerSkipQual = MJFormSkipQual(joinclauses, ">");
 
-	rightsortop = get_opcode(node->mergerightorder[0]);
-	leftsortop = get_opcode(node->mergeleftorder[0]);
-
-	if (leftsortop != rightsortop)
-		elog(NOTICE, "ExecInitMergeJoin: %s",
-			 "left and right sortop's are unequal!");
-
-	sortop = rightsortop;
-
-	/* ----------------
-	 *	form merge skip qualifications
-	 *
-	 *	XXX MJform routines need to be extended
-	 *		to take a list of sortops.. -cim 11/2/89
-	 * ----------------
-	 */
-	OSortopI = MJFormOSortopI(joinclauses, sortop);
-	ISortopO = MJFormISortopO(joinclauses, sortop);
-	mergestate->mj_OSortopI = OSortopI;
-	mergestate->mj_ISortopO = ISortopO;
-
-	MJ_printf("\nExecInitMergeJoin: OSortopI is ");
-	MJ_nodeDisplay(OSortopI);
-	MJ_printf("\nExecInitMergeJoin: ISortopO is ");
-	MJ_nodeDisplay(ISortopO);
+	MJ_printf("\nExecInitMergeJoin: OuterSkipQual is ");
+	MJ_nodeDisplay(mergestate->mj_OuterSkipQual);
+	MJ_printf("\nExecInitMergeJoin: InnerSkipQual is ");
+	MJ_nodeDisplay(mergestate->mj_InnerSkipQual);
 	MJ_printf("\n");
 
 	/* ----------------
