@@ -1,59 +1,34 @@
 /*-------------------------------------------------------------------------
  *
- * spi.c--
+ * spi.c
  *				Server Programming Interface
  *
+ * $Id: spi.c,v 1.39 1999/05/25 22:41:02 momjian Exp $
+ *
  *-------------------------------------------------------------------------
  */
 #include "executor/spi.h"
+#include "executor/spi_priv.h"
 #include "catalog/pg_type.h"
 #include "access/printtup.h"
 #include "fmgr.h"
 
-typedef struct
-{
-	QueryTreeList *qtlist;		/* malloced */
-	uint32		processed;		/* by Executor */
-	SPITupleTable *tuptable;
-	Portal		portal;			/* portal per procedure */
-	MemoryContext savedcxt;
-	CommandId	savedId;
-} _SPI_connection;
-
 static Portal _SPI_portal = (Portal) NULL;
 static _SPI_connection *_SPI_stack = NULL;
 static _SPI_connection *_SPI_current = NULL;
 static int	_SPI_connected = -1;
 static int	_SPI_curid = -1;
 
-uint32		SPI_processed = 0;
-SPITupleTable *SPI_tuptable;
-int			SPI_result;
-
-void		spi_printtup(HeapTuple tuple, TupleDesc tupdesc);
-
-typedef struct
-{
-	QueryTreeList *qtlist;
-	List	   *ptlist;
-	int			nargs;
-	Oid		   *argtypes;
-} _SPI_plan;
+DLLIMPORT uint32 SPI_processed = 0;
+DLLIMPORT SPITupleTable *SPI_tuptable;
+DLLIMPORT int SPI_result;
 
 static int	_SPI_execute(char *src, int tcount, _SPI_plan *plan);
 static int	_SPI_pquery(QueryDesc *queryDesc, EState *state, int tcount);
 
-#if 0
-static void _SPI_fetch(FetchStmt *stmt);
-
-#endif
 static int _SPI_execute_plan(_SPI_plan *plan,
 				  Datum *Values, char *Nulls, int tcount);
 
-#define _SPI_CPLAN_CURCXT	0
-#define _SPI_CPLAN_PROCXT	1
-#define _SPI_CPLAN_TOPCXT	2
-
 static _SPI_plan *_SPI_copy_plan(_SPI_plan *plan, int location);
 
 static int	_SPI_begin_call(bool execmem);
@@ -128,7 +103,7 @@ SPI_connect()
 	_SPI_current->tuptable = NULL;
 
 	/* Create Portal for this procedure ... */
-	sprintf(pname, "<SPI %d>", _SPI_connected);
+	snprintf(pname, 64, "<SPI %d>", _SPI_connected);
 	_SPI_current->portal = CreatePortal(pname);
 	if (!PortalIsValid(_SPI_current->portal))
 		elog(FATAL, "SPI_connect: initialization failed");
@@ -182,6 +157,18 @@ SPI_finish()
 
 }
 
+void
+SPI_push(void)
+{
+	_SPI_curid++;
+}
+
+void
+SPI_pop(void)
+{
+	_SPI_curid--;
+}
+
 int
 SPI_exec(char *src, int tcount)
 {
@@ -356,11 +343,12 @@ SPI_modifytuple(Relation rel, HeapTuple tuple, int natts, int *attnum,
 	if (i == natts)				/* no errors in *attnum */
 	{
 		mtuple = heap_formtuple(rel->rd_att, v, n);
-		infomask = mtuple->t_infomask;
-		memmove(&(mtuple->t_ctid), &(tuple->t_ctid),
-				((char *) &(tuple->t_hoff) - (char *) &(tuple->t_ctid)));
-		mtuple->t_infomask = infomask;
-		mtuple->t_natts = numberOfAttributes;
+		infomask = mtuple->t_data->t_infomask;
+		memmove(&(mtuple->t_data->t_oid), &(tuple->t_data->t_oid),
+				((char *) &(tuple->t_data->t_hoff) -
+				 (char *) &(tuple->t_data->t_oid)));
+		mtuple->t_data->t_infomask = infomask;
+		mtuple->t_data->t_natts = numberOfAttributes;
 	}
 	else
 	{
@@ -410,10 +398,11 @@ SPI_getvalue(HeapTuple tuple, TupleDesc tupdesc, int fnumber)
 {
 	Datum		val;
 	bool		isnull;
-	Oid			foutoid;
+	Oid			foutoid,
+				typelem;
 
 	SPI_result = 0;
-	if (tuple->t_natts < fnumber || fnumber <= 0)
+	if (tuple->t_data->t_natts < fnumber || fnumber <= 0)
 	{
 		SPI_result = SPI_ERROR_NOATTRIBUTE;
 		return NULL;
@@ -422,15 +411,14 @@ SPI_getvalue(HeapTuple tuple, TupleDesc tupdesc, int fnumber)
 	val = heap_getattr(tuple, fnumber, tupdesc, &isnull);
 	if (isnull)
 		return NULL;
-	foutoid = typtoout((Oid) tupdesc->attrs[fnumber - 1]->atttypid);
-	if (!OidIsValid(foutoid))
+	if (!getTypeOutAndElem((Oid) tupdesc->attrs[fnumber - 1]->atttypid,
+						   &foutoid, &typelem))
 	{
 		SPI_result = SPI_ERROR_NOOUTFUNC;
 		return NULL;
 	}
 
-	return (fmgr(foutoid, val,
-				 gettypelem(tupdesc->attrs[fnumber - 1]->atttypid),
+	return (fmgr(foutoid, val, typelem,
 				 tupdesc->attrs[fnumber - 1]->atttypmod));
 }
 
@@ -441,7 +429,7 @@ SPI_getbinval(HeapTuple tuple, TupleDesc tupdesc, int fnumber, bool *isnull)
 
 	*isnull = true;
 	SPI_result = 0;
-	if (tuple->t_natts < fnumber || fnumber <= 0)
+	if (tuple->t_data->t_natts < fnumber || fnumber <= 0)
 	{
 		SPI_result = SPI_ERROR_NOATTRIBUTE;
 		return (Datum) NULL;
@@ -561,13 +549,13 @@ SPI_pfree(void *pointer)
 /* =================== private functions =================== */
 
 /*
- * spi_printtup --
+ * spi_printtup
  *		store tuple retrieved by Executor into SPITupleTable
  *		of current SPI procedure
  *
  */
 void
-spi_printtup(HeapTuple tuple, TupleDesc tupdesc)
+spi_printtup(HeapTuple tuple, TupleDesc tupdesc, DestReceiver *self)
 {
 	SPITupleTable *tuptable;
 	MemoryContext oldcxt;
@@ -614,18 +602,18 @@ spi_printtup(HeapTuple tuple, TupleDesc tupdesc)
 static int
 _SPI_execute(char *src, int tcount, _SPI_plan *plan)
 {
-	QueryTreeList *queryTree_list;
+	List	   *queryTree_list;
 	List	   *planTree_list;
+	List	   *queryTree_list_item;
 	List	   *ptlist;
 	QueryDesc  *qdesc;
 	Query	   *queryTree;
 	Plan	   *planTree;
 	EState	   *state;
-	int			qlen;
 	int			nargs = 0;
 	Oid		   *argtypes = NULL;
-	int			res;
-	int			i;
+	int			res = 0;
+	bool		islastquery;
 
 	/* Increment CommandCounter to see changes made by now */
 	CommandCounterIncrement();
@@ -640,18 +628,18 @@ _SPI_execute(char *src, int tcount, _SPI_plan *plan)
 		nargs = plan->nargs;
 		argtypes = plan->argtypes;
 	}
-	ptlist = planTree_list = (List *)
+	ptlist = planTree_list =
 		pg_parse_and_plan(src, argtypes, nargs, &queryTree_list, None, FALSE);
 
 	_SPI_current->qtlist = queryTree_list;
 
-	qlen = queryTree_list->len;
-	for (i = 0;; i++)
+	foreach(queryTree_list_item, queryTree_list)
 	{
-		queryTree = (Query *) (queryTree_list->qtrees[i]);
+		queryTree = (Query *) lfirst(queryTree_list_item);
 		planTree = lfirst(planTree_list);
-
 		planTree_list = lnext(planTree_list);
+		islastquery = (planTree_list == NIL);	/* assume lists are same
+												 * len */
 
 		if (queryTree->commandType == CMD_UTILITY)
 		{
@@ -671,32 +659,32 @@ _SPI_execute(char *src, int tcount, _SPI_plan *plan)
 			if (plan == NULL)
 			{
 				ProcessUtility(queryTree->utilityStmt, None);
-				if (i < qlen - 1)
+				if (!islastquery)
 					CommandCounterIncrement();
 				else
 					return res;
 			}
-			else if (i >= qlen - 1)
+			else if (islastquery)
 				break;
 		}
 		else if (plan == NULL)
 		{
 			qdesc = CreateQueryDesc(queryTree, planTree,
-									(i < qlen - 1) ? None : SPI);
+									islastquery ? SPI : None);
 			state = CreateExecutorState();
-			res = _SPI_pquery(qdesc, state, (i < qlen - 1) ? 0 : tcount);
-			if (res < 0 || i >= qlen - 1)
+			res = _SPI_pquery(qdesc, state, islastquery ? tcount : 0);
+			if (res < 0 || islastquery)
 				return res;
 			CommandCounterIncrement();
 		}
 		else
 		{
 			qdesc = CreateQueryDesc(queryTree, planTree,
-									(i < qlen - 1) ? None : SPI);
-			res = _SPI_pquery(qdesc, NULL, (i < qlen - 1) ? 0 : tcount);
+									islastquery ? SPI : None);
+			res = _SPI_pquery(qdesc, NULL, islastquery ? tcount : 0);
 			if (res < 0)
 				return res;
-			if (i >= qlen - 1)
+			if (islastquery)
 				break;
 		}
 	}
@@ -705,23 +693,22 @@ _SPI_execute(char *src, int tcount, _SPI_plan *plan)
 	plan->ptlist = ptlist;
 
 	return res;
-
 }
 
 static int
 _SPI_execute_plan(_SPI_plan *plan, Datum *Values, char *Nulls, int tcount)
 {
-	QueryTreeList *queryTree_list = plan->qtlist;
+	List	   *queryTree_list = plan->qtlist;
 	List	   *planTree_list = plan->ptlist;
+	List	   *queryTree_list_item;
 	QueryDesc  *qdesc;
 	Query	   *queryTree;
 	Plan	   *planTree;
 	EState	   *state;
 	int			nargs = plan->nargs;
-	int			qlen = queryTree_list->len;
-	int			res;
-	int			i,
-				k;
+	int			res = 0;
+	bool		islastquery;
+	int			k;
 
 	/* Increment CommandCounter to see changes made by now */
 	CommandCounterIncrement();
@@ -731,17 +718,18 @@ _SPI_execute_plan(_SPI_plan *plan, Datum *Values, char *Nulls, int tcount)
 	_SPI_current->tuptable = NULL;
 	_SPI_current->qtlist = NULL;
 
-	for (i = 0;; i++)
+	foreach(queryTree_list_item, queryTree_list)
 	{
-		queryTree = (Query *) (queryTree_list->qtrees[i]);
+		queryTree = (Query *) lfirst(queryTree_list_item);
 		planTree = lfirst(planTree_list);
-
 		planTree_list = lnext(planTree_list);
+		islastquery = (planTree_list == NIL);	/* assume lists are same
+												 * len */
 
 		if (queryTree->commandType == CMD_UTILITY)
 		{
 			ProcessUtility(queryTree->utilityStmt, None);
-			if (i < qlen - 1)
+			if (!islastquery)
 				CommandCounterIncrement();
 			else
 				return SPI_OK_UTILITY;
@@ -749,7 +737,7 @@ _SPI_execute_plan(_SPI_plan *plan, Datum *Values, char *Nulls, int tcount)
 		else
 		{
 			qdesc = CreateQueryDesc(queryTree, planTree,
-									(i < qlen - 1) ? None : SPI);
+									islastquery ? SPI : None);
 			state = CreateExecutorState();
 			if (nargs > 0)
 			{
@@ -768,15 +756,14 @@ _SPI_execute_plan(_SPI_plan *plan, Datum *Values, char *Nulls, int tcount)
 			}
 			else
 				state->es_param_list_info = NULL;
-			res = _SPI_pquery(qdesc, state, (i < qlen - 1) ? 0 : tcount);
-			if (res < 0 || i >= qlen - 1)
+			res = _SPI_pquery(qdesc, state, islastquery ? tcount : 0);
+			if (res < 0 || islastquery)
 				return res;
 			CommandCounterIncrement();
 		}
 	}
 
 	return res;
-
 }
 
 static int
@@ -791,6 +778,8 @@ _SPI_pquery(QueryDesc *queryDesc, EState *state, int tcount)
 	bool		isRetrieveIntoRelation = false;
 	char	   *intoName = NULL;
 	int			res;
+	Const		tcount_const;
+	Node	   *count = NULL;
 
 	switch (operation)
 	{
@@ -825,6 +814,39 @@ _SPI_pquery(QueryDesc *queryDesc, EState *state, int tcount)
 			return SPI_ERROR_OPUNKNOWN;
 	}
 
+	/* ----------------
+	 * Get the query LIMIT tuple count
+	 * ----------------
+	 */
+	if (parseTree->limitCount != NULL)
+	{
+		/* ----------------
+		 * A limit clause in the parsetree overrides the
+		 * tcount parameter
+		 * ----------------
+		 */
+		count = parseTree->limitCount;
+	}
+	else
+	{
+		/* ----------------
+		 * No LIMIT clause in parsetree. Use a local Const node
+		 * to put tcount into it
+		 * ----------------
+		 */
+		memset(&tcount_const, 0, sizeof(tcount_const));
+		tcount_const.type = T_Const;
+		tcount_const.consttype = INT4OID;
+		tcount_const.constlen = sizeof(int4);
+		tcount_const.constvalue = (Datum) tcount;
+		tcount_const.constisnull = FALSE;
+		tcount_const.constbyval = TRUE;
+		tcount_const.constisset = FALSE;
+		tcount_const.constiscast = FALSE;
+
+		count = (Node *) &tcount_const;
+	}
+
 	if (state == NULL)			/* plan preparation */
 		return res;
 #ifdef SPI_EXECUTOR_STATS
@@ -845,7 +867,7 @@ _SPI_pquery(QueryDesc *queryDesc, EState *state, int tcount)
 		return SPI_OK_CURSOR;
 	}
 
-	ExecutorRun(queryDesc, state, EXEC_FOR, tcount);
+	ExecutorRun(queryDesc, state, EXEC_FOR, parseTree->limitOffset, count);
 
 	_SPI_current->processed = state->es_processed;
 	if (operation == CMD_SELECT && queryDesc->dest == SPI)
@@ -875,46 +897,6 @@ _SPI_pquery(QueryDesc *queryDesc, EState *state, int tcount)
 
 }
 
-#if 0
-static void
-_SPI_fetch(FetchStmt *stmt)
-{
-	char	   *name = stmt->portalname;
-	int			feature = (stmt->direction == FORWARD) ? EXEC_FOR : EXEC_BACK;
-	int			count = stmt->howMany;
-	Portal		portal;
-	QueryDesc  *queryDesc;
-	EState	   *state;
-	MemoryContext context;
-
-	if (name == NULL)
-		elog(FATAL, "SPI_fetch from blank portal unsupported");
-
-	portal = GetPortalByName(name);
-	if (!PortalIsValid(portal))
-		elog(FATAL, "SPI_fetch: portal \"%s\" not found", name);
-
-	context = MemoryContextSwitchTo((MemoryContext) PortalGetHeapMemory(portal));
-
-	queryDesc = PortalGetQueryDesc(portal);
-	state = PortalGetState(portal);
-
-	ExecutorRun(queryDesc, state, feature, count);
-
-	MemoryContextSwitchTo(context);		/* switch to the normal Executor
-										 * context */
-
-	_SPI_current->processed = state->es_processed;
-	if (_SPI_checktuples())
-		elog(FATAL, "SPI_fetch: # of processed tuples check failed");
-
-	SPI_processed = _SPI_current->processed;
-	SPI_tuptable = _SPI_current->tuptable;
-
-}
-
-#endif
-
 static MemoryContext
 _SPI_execmem()
 {
@@ -942,7 +924,7 @@ _SPI_procmem()
 }
 
 /*
- * _SPI_begin_call --
+ * _SPI_begin_call
  *
  */
 static int
@@ -972,12 +954,7 @@ _SPI_end_call(bool procmem)
 	 */
 	_SPI_curid--;
 
-	if (_SPI_current->qtlist)	/* free _SPI_plan allocations */
-	{
-		free(_SPI_current->qtlist->qtrees);
-		free(_SPI_current->qtlist);
-		_SPI_current->qtlist = NULL;
-	}
+	_SPI_current->qtlist = NULL;
 
 	if (procmem)				/* switch to the procedure memory context */
 	{							/* but free Executor memory before */
@@ -1017,7 +994,6 @@ _SPI_copy_plan(_SPI_plan *plan, int location)
 {
 	_SPI_plan  *newplan;
 	MemoryContext oldcxt = NULL;
-	int			i;
 
 	if (location == _SPI_CPLAN_PROCXT)
 		oldcxt = MemoryContextSwitchTo((MemoryContext)
@@ -1026,14 +1002,7 @@ _SPI_copy_plan(_SPI_plan *plan, int location)
 		oldcxt = MemoryContextSwitchTo(TopMemoryContext);
 
 	newplan = (_SPI_plan *) palloc(sizeof(_SPI_plan));
-	newplan->qtlist = (QueryTreeList *) palloc(sizeof(QueryTreeList));
-	newplan->qtlist->len = plan->qtlist->len;
-	newplan->qtlist->qtrees = (Query **) palloc(plan->qtlist->len *
-												sizeof(Query *));
-	for (i = 0; i < plan->qtlist->len; i++)
-		newplan->qtlist->qtrees[i] = (Query *)
-			copyObject(plan->qtlist->qtrees[i]);
-
+	newplan->qtlist = (List *) copyObject(plan->qtlist);
 	newplan->ptlist = (List *) copyObject(plan->ptlist);
 	newplan->nargs = plan->nargs;
 	if (plan->nargs > 0)
