@@ -1,13 +1,13 @@
 /*-------------------------------------------------------------------------
  *
- * readfuncs.c--
+ * readfuncs.c
  *	  Reader functions for Postgres tree nodes.
  *
  * Copyright (c) 1994, Regents of the University of California
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/nodes/readfuncs.c,v 1.38.2.1 1998/11/24 02:05:08 momjian Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/nodes/readfuncs.c,v 1.64 1999/05/25 16:09:11 momjian Exp $
  *
  * NOTES
  *	  Most of the read functions for plan nodes are tested. (In fact, they
@@ -37,6 +37,7 @@
 
 #include "utils/lsyscache.h"
 #include "utils/syscache.h"
+#include "catalog/pg_index.h"
 #include "catalog/pg_type.h"
 
 #include "nodes/primnodes.h"
@@ -120,6 +121,10 @@ _readQuery()
 	token = lsptok(NULL, &length);		/* get isBinary */
 	local_node->isBinary = (token[0] == 't') ? true : false;
 
+	token = lsptok(NULL, &length);		/* skip :isTemp */
+	token = lsptok(NULL, &length);		/* get isTemp */
+	local_node->isTemp = (token[0] == 't') ? true : false;
+
 	token = lsptok(NULL, &length);		/* skip :unionall */
 	token = lsptok(NULL, &length);		/* get unionall */
 	local_node->unionall = (token[0] == 't') ? true : false;
@@ -163,12 +168,20 @@ _readQuery()
 	token = lsptok(NULL, &length);		/* skip :unionClause */
 	local_node->unionClause = nodeRead(true);
 
+	/***S*I***/
+	token = lsptok(NULL, &length);		/* skip :intersectClause */
+	local_node->intersectClause = nodeRead(true);
+
+
 	token = lsptok(NULL, &length);		/* skip :limitOffset */
 	local_node->limitOffset = nodeRead(true);
 
 	token = lsptok(NULL, &length);		/* skip :limitCount */
 	local_node->limitCount = nodeRead(true);
 
+	token = lsptok(NULL, &length);		/* skip :rowMark */
+	local_node->rowMark = nodeRead(true);
+
 	return local_node;
 }
 
@@ -208,13 +221,14 @@ _readGroupClause()
 
 	local_node = makeNode(GroupClause);
 
-	token = lsptok(NULL, &length);		/* skip the :entry */
-	local_node->entry = nodeRead(true);
-
 	token = lsptok(NULL, &length);		/* skip :grpOpoid */
 	token = lsptok(NULL, &length);		/* get grpOpoid */
 	local_node->grpOpoid = strtoul(token, NULL, 10);
 
+	token = lsptok(NULL, &length);		/* skip :tleGroupref */
+	token = lsptok(NULL, &length);		/* get tleGroupref */
+	local_node->tleGroupref = strtoul(token, NULL, 10);
+
 	return local_node;
 }
 
@@ -413,10 +427,6 @@ _readMergeJoin()
 	token = lsptok(NULL, &length);		/* eat :mergeclauses */
 	local_node->mergeclauses = nodeRead(true);	/* now read it */
 
-	token = lsptok(NULL, &length);		/* eat :mergejoinop */
-	token = lsptok(NULL, &length);		/* get mergejoinop */
-	local_node->mergejoinop = atol(token);
-
 	return local_node;
 }
 
@@ -444,18 +454,6 @@ _readHashJoin()
 	token = lsptok(NULL, &length);		/* get hashjoinop */
 	local_node->hashjoinop = strtoul(token, NULL, 10);
 
-	token = lsptok(NULL, &length);		/* eat :hashjointable */
-	token = lsptok(NULL, &length);		/* eat hashjointable */
-	local_node->hashjointable = NULL;
-
-	token = lsptok(NULL, &length);		/* eat :hashjointablekey */
-	token = lsptok(NULL, &length);		/* eat hashjointablekey */
-	local_node->hashjointablekey = 0;
-
-	token = lsptok(NULL, &length);		/* eat :hashjointablesize */
-	token = lsptok(NULL, &length);		/* eat hashjointablesize */
-	local_node->hashjointablesize = 0;
-
 	token = lsptok(NULL, &length);		/* eat :hashdone */
 	token = lsptok(NULL, &length);		/* eat hashdone */
 	local_node->hashdone = false;
@@ -540,38 +538,37 @@ _readIndexScan()
 	_getScan((Scan *) local_node);
 
 	token = lsptok(NULL, &length);		/* eat :indxid */
-	local_node->indxid =
-		toIntList(nodeRead(true));		/* now read it */
+	local_node->indxid = toIntList(nodeRead(true));		/* now read it */
 
 	token = lsptok(NULL, &length);		/* eat :indxqual */
 	local_node->indxqual = nodeRead(true);		/* now read it */
 
 	token = lsptok(NULL, &length);		/* eat :indxqualorig */
-	local_node->indxqualorig = nodeRead(true);		/* now read it */
+	local_node->indxqualorig = nodeRead(true);	/* now read it */
 
 	return local_node;
 }
 
 /* ----------------
- *		_readTemp
+ *		_readNoname
  *
- *	Temp is a subclass of Plan
+ *	Noname is a subclass of Plan
  * ----------------
  */
-static Temp *
-_readTemp()
+static Noname *
+_readNoname()
 {
-	Temp	   *local_node;
+	Noname	   *local_node;
 	char	   *token;
 	int			length;
 
-	local_node = makeNode(Temp);
+	local_node = makeNode(Noname);
 
 	_getPlan((Plan *) local_node);
 
-	token = lsptok(NULL, &length);		/* eat :tempid */
-	token = lsptok(NULL, &length);		/* get tempid */
-	local_node->tempid = atol(token);
+	token = lsptok(NULL, &length);		/* eat :nonameid */
+	token = lsptok(NULL, &length);		/* get nonameid */
+	local_node->nonameid = atol(token);
 
 	token = lsptok(NULL, &length);		/* eat :keycount */
 	token = lsptok(NULL, &length);		/* get keycount */
@@ -583,7 +580,7 @@ _readTemp()
 /* ----------------
  *		_readSort
  *
- *	Sort is a subclass of Temp
+ *	Sort is a subclass of Noname
  * ----------------
  */
 static Sort *
@@ -597,9 +594,9 @@ _readSort()
 
 	_getPlan((Plan *) local_node);
 
-	token = lsptok(NULL, &length);		/* eat :tempid */
-	token = lsptok(NULL, &length);		/* get tempid */
-	local_node->tempid = atol(token);
+	token = lsptok(NULL, &length);		/* eat :nonameid */
+	token = lsptok(NULL, &length);		/* get nonameid */
+	local_node->nonameid = atol(token);
 
 	token = lsptok(NULL, &length);		/* eat :keycount */
 	token = lsptok(NULL, &length);		/* get keycount */
@@ -627,7 +624,7 @@ _readAgg()
 /* ----------------
  *		_readUnique
  *
- * For some reason, unique is a subclass of Temp.
+ * For some reason, unique is a subclass of Noname.
  */
 static Unique *
 _readUnique()
@@ -640,9 +637,9 @@ _readUnique()
 
 	_getPlan((Plan *) local_node);
 
-	token = lsptok(NULL, &length);		/* eat :tempid */
-	token = lsptok(NULL, &length);		/* get :tempid */
-	local_node->tempid = atol(token);
+	token = lsptok(NULL, &length);		/* eat :nonameid */
+	token = lsptok(NULL, &length);		/* get :nonameid */
+	local_node->nonameid = atol(token);
 
 	token = lsptok(NULL, &length);		/* eat :keycount */
 	token = lsptok(NULL, &length);		/* get :keycount */
@@ -654,7 +651,7 @@ _readUnique()
 /* ----------------
  *		_readHash
  *
- *	Hash is a subclass of Temp
+ *	Hash is a subclass of Noname
  * ----------------
  */
 static Hash *
@@ -671,18 +668,6 @@ _readHash()
 	token = lsptok(NULL, &length);		/* eat :hashkey */
 	local_node->hashkey = (Var *) nodeRead(true);
 
-	token = lsptok(NULL, &length);		/* eat :hashtable */
-	token = lsptok(NULL, &length);		/* eat hashtable address */
-	local_node->hashtable = NULL;
-
-	token = lsptok(NULL, &length);		/* eat :hashtablekey */
-	token = lsptok(NULL, &length);		/* get hashtablekey */
-	local_node->hashtablekey = 0;
-
-	token = lsptok(NULL, &length);		/* eat :hashtablesize */
-	token = lsptok(NULL, &length);		/* get hashtablesize */
-	local_node->hashtablesize = 0;
-
 	return local_node;
 }
 
@@ -736,9 +721,13 @@ _readResdom()
 	token = lsptok(NULL, &length);		/* get reskeyop */
 	local_node->reskeyop = (Oid) atol(token);
 
+	token = lsptok(NULL, &length);		/* eat :resgroupref */
+	token = lsptok(NULL, &length);		/* get resgroupref */
+	local_node->resgroupref = strtoul(token, NULL, 10);
+
 	token = lsptok(NULL, &length);		/* eat :resjunk */
 	token = lsptok(NULL, &length);		/* get resjunk */
-	local_node->resjunk = atoi(token);
+	local_node->resjunk = (token[0] == 't') ? true : false;
 
 	return local_node;
 }
@@ -786,6 +775,50 @@ _readExpr()
 	return local_node;
 }
 
+/* ----------------
+ *		_readCaseExpr
+ *
+ *	CaseExpr is a subclass of Node
+ * ----------------
+ */
+static CaseExpr *
+_readCaseExpr()
+{
+	CaseExpr   *local_node;
+	char	   *token;
+	int			length;
+
+	local_node = makeNode(CaseExpr);
+
+	local_node->args = nodeRead(true);
+	token = lsptok(NULL, &length);		/* eat :default */
+	local_node->defresult = nodeRead(true);
+
+	return local_node;
+}
+
+/* ----------------
+ *		_readCaseWhen
+ *
+ *	CaseWhen is a subclass of Node
+ * ----------------
+ */
+static CaseWhen *
+_readCaseWhen()
+{
+	CaseWhen   *local_node;
+	char	   *token;
+	int			length;
+
+	local_node = makeNode(CaseWhen);
+
+	local_node->expr = nodeRead(true);
+	token = lsptok(NULL, &length);		/* eat :then */
+	local_node->result = nodeRead(true);
+
+	return local_node;
+}
+
 /* ----------------
  *		_readVar
  *
@@ -1109,19 +1142,19 @@ _readParam()
 }
 
 /* ----------------
- *		_readAggreg
+ *		_readAggref
  *
- *	Aggreg is a subclass of Node
+ *	Aggref is a subclass of Node
  * ----------------
  */
-static Aggreg *
-_readAggreg()
+static Aggref *
+_readAggref()
 {
-	Aggreg	   *local_node;
+	Aggref	   *local_node;
 	char	   *token;
 	int			length;
 
-	local_node = makeNode(Aggreg);
+	local_node = makeNode(Aggref);
 
 	token = lsptok(NULL, &length);		/* eat :aggname */
 	token = lsptok(NULL, &length);		/* get aggname */
@@ -1240,8 +1273,7 @@ _readRelOptInfo()
 	local_node = makeNode(RelOptInfo);
 
 	token = lsptok(NULL, &length);		/* get :relids */
-	local_node->relids =
-		toIntList(nodeRead(true));		/* now read it */
+	local_node->relids = toIntList(nodeRead(true));		/* now read it */
 
 	token = lsptok(NULL, &length);		/* get :indexed */
 	token = lsptok(NULL, &length);		/* now read it */
@@ -1279,12 +1311,6 @@ _readRelOptInfo()
 	 * This can be changed later, if necessary.
 	 */
 
-	token = lsptok(NULL, &length);		/* get :unorderpath */
-	token = lsptok(NULL, &length);		/* get @ */
-	token = lsptok(NULL, &length);		/* now read it */
-
-	sscanf(token, "%x", (unsigned int *) &local_node->unorderedpath);
-
 	token = lsptok(NULL, &length);		/* get :cheapestpath */
 	token = lsptok(NULL, &length);		/* get @ */
 	token = lsptok(NULL, &length);		/* now read it */
@@ -1292,8 +1318,8 @@ _readRelOptInfo()
 	sscanf(token, "%x", (unsigned int *) &local_node->cheapestpath);
 
 
-	token = lsptok(NULL, &length);		/* get :clauseinfo */
-	local_node->clauseinfo = nodeRead(true);	/* now read it */
+	token = lsptok(NULL, &length);		/* get :restrictinfo */
+	local_node->restrictinfo = nodeRead(true);	/* now read it */
 
 	token = lsptok(NULL, &length);		/* get :joininfo */
 	local_node->joininfo = nodeRead(true);		/* now read it */
@@ -1378,6 +1404,73 @@ _readRangeTblEntry()
 	return local_node;
 }
 
+static RowMark *
+_readRowMark()
+{
+	RowMark    *local_node = makeNode(RowMark);
+	char	   *token;
+	int			length;
+
+	token = lsptok(NULL, &length);		/* eat :rti */
+	token = lsptok(NULL, &length);		/* get :rti */
+	local_node->rti = strtoul(token, NULL, 10);
+
+	token = lsptok(NULL, &length);		/* eat :info */
+	token = lsptok(NULL, &length);		/* get :info */
+	local_node->info = strtoul(token, NULL, 10);
+
+	return local_node;
+}
+
+/* ----------------
+ *		_readPathOrder
+ *
+ *	PathOrder is part of Path and it's subclasses.
+ * ----------------
+ */
+static PathOrder *
+_readPathOrder()
+{
+	PathOrder  *local_node;
+	char	   *token;
+	int			length;
+
+	local_node = makeNode(PathOrder);
+
+	token = lsptok(NULL, &length);		/* get :ordtype */
+	token = lsptok(NULL, &length);		/* now read it */
+	local_node->ordtype = atol(token);
+
+	if (local_node->ordtype == SORTOP_ORDER)
+	{
+		token = lsptok(NULL, &length);	/* get :sortop */
+
+		if (length == 0)
+			local_node->ord.sortop = NULL;
+		else
+		{
+			int			i = -1;
+
+			local_node->ord.sortop = palloc(sizeof(Oid) * (INDEX_MAX_KEYS + 1));
+
+			do
+			{
+				i++;
+				Assert(i <= INDEX_MAX_KEYS);
+				token = lsptok(NULL, &length);	/* now read it */
+				local_node->ord.sortop[i] = strtoul(token, NULL, 10);
+			} while (local_node->ord.sortop[i] != 0);
+		}
+	}
+	else
+	{
+		token = lsptok(NULL, &length);	/* get :merge */
+		local_node->ord.merge = nodeRead(true); /* now read it */
+	}
+
+	return local_node;
+}
+
 /* ----------------
  *		_readPath
  *
@@ -1401,14 +1494,11 @@ _readPath()
 	token = lsptok(NULL, &length);		/* now read it */
 	local_node->path_cost = (Cost) atof(token);
 
-#if 0
-	token = lsptok(NULL, &length);		/* get :p_ordering */
-	local_node->p_ordering =
-		nodeRead(true);			/* now read it */
-#endif
+	token = lsptok(NULL, &length);		/* get :pathorder */
+	local_node->pathorder = nodeRead(true);		/* now read it */
 
-	token = lsptok(NULL, &length);		/* get :keys */
-	local_node->keys = nodeRead(true);	/* now read it */
+	token = lsptok(NULL, &length);		/* get :pathkeys */
+	local_node->pathkeys = nodeRead(true);		/* now read it */
 
 	return local_node;
 }
@@ -1436,17 +1526,14 @@ _readIndexPath()
 	token = lsptok(NULL, &length);		/* now read it */
 	local_node->path.path_cost = (Cost) atof(token);
 
-#if 0
-	token = lsptok(NULL, &length);		/* get :p_ordering */
-	local_node->path.p_ordering = nodeRead(true);		/* now read it */
-#endif
+	token = lsptok(NULL, &length);		/* get :pathorder */
+	local_node->path.pathorder = nodeRead(true);		/* now read it */
 
-	token = lsptok(NULL, &length);		/* get :keys */
-	local_node->path.keys = nodeRead(true);		/* now read it */
+	token = lsptok(NULL, &length);		/* get :pathkeys */
+	local_node->path.pathkeys = nodeRead(true); /* now read it */
 
 	token = lsptok(NULL, &length);		/* get :indexid */
-	local_node->indexid =
-		toIntList(nodeRead(true));
+	local_node->indexid = toIntList(nodeRead(true));
 
 	token = lsptok(NULL, &length);		/* get :indexqual */
 	local_node->indexqual = nodeRead(true);		/* now read it */
@@ -1455,20 +1542,20 @@ _readIndexPath()
 }
 
 /* ----------------
- *		_readJoinPath
+ *		_readNestPath
  *
- *	JoinPath is a subclass of Path
+ *	NestPath is a subclass of Path
  * ----------------
  */
-static JoinPath *
-_readJoinPath()
+static NestPath *
+_readNestPath()
 {
-	JoinPath   *local_node;
+	NestPath   *local_node;
 	char	   *token;
 	int			length;
 
 
-	local_node = makeNode(JoinPath);
+	local_node = makeNode(NestPath);
 
 	token = lsptok(NULL, &length);		/* get :pathtype */
 	token = lsptok(NULL, &length);		/* now read it */
@@ -1478,16 +1565,14 @@ _readJoinPath()
 	token = lsptok(NULL, &length);		/* now read it */
 	local_node->path.path_cost = (Cost) atof(token);
 
-#if 0
-	token = lsptok(NULL, &length);		/* get :p_ordering */
-	local_node->path.p_ordering = nodeRead(true);		/* now read it */
-#endif
+	token = lsptok(NULL, &length);		/* get :pathorder */
+	local_node->path.pathorder = nodeRead(true);		/* now read it */
 
-	token = lsptok(NULL, &length);		/* get :keys */
-	local_node->path.keys = nodeRead(true);		/* now read it */
+	token = lsptok(NULL, &length);		/* get :pathkeys */
+	local_node->path.pathkeys = nodeRead(true); /* now read it */
 
-	token = lsptok(NULL, &length);		/* get :pathclauseinfo */
-	local_node->pathclauseinfo = nodeRead(true);		/* now read it */
+	token = lsptok(NULL, &length);		/* get :pathinfo */
+	local_node->pathinfo = nodeRead(true);		/* now read it */
 
 	/*
 	 * Not sure if these are nodes; they're declared as "struct path *".
@@ -1515,8 +1600,7 @@ _readJoinPath()
 	local_node->path.outerjoincost = (Cost) atof(token);
 
 	token = lsptok(NULL, &length);		/* get :joinid */
-	local_node->path.joinid =
-		toIntList(nodeRead(true));		/* now read it */
+	local_node->path.joinid = toIntList(nodeRead(true));		/* now read it */
 
 	return local_node;
 }
@@ -1524,7 +1608,7 @@ _readJoinPath()
 /* ----------------
  *		_readMergePath
  *
- *	MergePath is a subclass of JoinPath.
+ *	MergePath is a subclass of NestPath.
  * ----------------
  */
 static MergePath *
@@ -1546,16 +1630,14 @@ _readMergePath()
 
 	local_node->jpath.path.path_cost = (Cost) atof(token);
 
-#if 0
-	token = lsptok(NULL, &length);		/* get :p_ordering */
-	local_node->jpath.path.p_ordering = nodeRead(true); /* now read it */
-#endif
+	token = lsptok(NULL, &length);		/* get :pathorder */
+	local_node->jpath.path.pathorder = nodeRead(true);	/* now read it */
 
-	token = lsptok(NULL, &length);		/* get :keys */
-	local_node->jpath.path.keys = nodeRead(true);		/* now read it */
+	token = lsptok(NULL, &length);		/* get :pathkeys */
+	local_node->jpath.path.pathkeys = nodeRead(true);	/* now read it */
 
-	token = lsptok(NULL, &length);		/* get :pathclauseinfo */
-	local_node->jpath.pathclauseinfo = nodeRead(true);	/* now read it */
+	token = lsptok(NULL, &length);		/* get :pathinfo */
+	local_node->jpath.pathinfo = nodeRead(true);		/* now read it */
 
 	/*
 	 * Not sure if these are nodes; they're declared as "struct path *".
@@ -1583,8 +1665,7 @@ _readMergePath()
 	local_node->jpath.path.outerjoincost = (Cost) atof(token);
 
 	token = lsptok(NULL, &length);		/* get :joinid */
-	local_node->jpath.path.joinid =
-		toIntList(nodeRead(true));		/* now read it */
+	local_node->jpath.path.joinid = toIntList(nodeRead(true));	/* now read it */
 
 	token = lsptok(NULL, &length);		/* get :path_mergeclauses */
 	local_node->path_mergeclauses = nodeRead(true);		/* now read it */
@@ -1601,7 +1682,7 @@ _readMergePath()
 /* ----------------
  *		_readHashPath
  *
- *	HashPath is a subclass of JoinPath.
+ *	HashPath is a subclass of NestPath.
  * ----------------
  */
 static HashPath *
@@ -1623,16 +1704,14 @@ _readHashPath()
 
 	local_node->jpath.path.path_cost = (Cost) atof(token);
 
-#if 0
-	token = lsptok(NULL, &length);		/* get :p_ordering */
-	local_node->jpath.path.p_ordering = nodeRead(true); /* now read it */
-#endif
+	token = lsptok(NULL, &length);		/* get :pathorder */
+	local_node->jpath.path.pathorder = nodeRead(true);	/* now read it */
 
-	token = lsptok(NULL, &length);		/* get :keys */
-	local_node->jpath.path.keys = nodeRead(true);		/* now read it */
+	token = lsptok(NULL, &length);		/* get :pathkeys */
+	local_node->jpath.path.pathkeys = nodeRead(true);	/* now read it */
 
-	token = lsptok(NULL, &length);		/* get :pathclauseinfo */
-	local_node->jpath.pathclauseinfo = nodeRead(true);	/* now read it */
+	token = lsptok(NULL, &length);		/* get :pathinfo */
+	local_node->jpath.pathinfo = nodeRead(true);		/* now read it */
 
 	/*
 	 * Not sure if these are nodes; they're declared as "struct path *".
@@ -1660,8 +1739,7 @@ _readHashPath()
 	local_node->jpath.path.outerjoincost = (Cost) atof(token);
 
 	token = lsptok(NULL, &length);		/* get :joinid */
-	local_node->jpath.path.joinid =
-		toIntList(nodeRead(true));		/* now read it */
+	local_node->jpath.path.joinid = toIntList(nodeRead(true));	/* now read it */
 
 	token = lsptok(NULL, &length);		/* get :path_hashclauses */
 	local_node->path_hashclauses = nodeRead(true);		/* now read it */
@@ -1770,19 +1848,19 @@ _readMergeOrder()
 }
 
 /* ----------------
- *		_readClauseInfo
+ *		_readRestrictInfo
  *
- *	ClauseInfo is a subclass of Node.
+ *	RestrictInfo is a subclass of Node.
  * ----------------
  */
-static ClauseInfo *
-_readClauseInfo()
+static RestrictInfo *
+_readRestrictInfo()
 {
-	ClauseInfo *local_node;
+	RestrictInfo *local_node;
 	char	   *token;
 	int			length;
 
-	local_node = makeNode(ClauseInfo);
+	local_node = makeNode(RestrictInfo);
 
 	token = lsptok(NULL, &length);		/* get :clause */
 	local_node->clause = nodeRead(true);		/* now read it */
@@ -1839,19 +1917,19 @@ _readJoinMethod()
 }
 
 /* ----------------
- *		_readHInfo
+ *		_readHashInfo
  *
- * HInfo is a subclass of JoinMethod.
+ * HashInfo is a subclass of JoinMethod.
  * ----------------
  */
-static HInfo *
-_readHInfo()
+static HashInfo *
+_readHashInfo()
 {
-	HInfo	   *local_node;
+	HashInfo   *local_node;
 	char	   *token;
 	int			length;
 
-	local_node = makeNode(HInfo);
+	local_node = makeNode(HashInfo);
 
 	token = lsptok(NULL, &length);		/* get :hashop */
 	token = lsptok(NULL, &length);		/* now read it */
@@ -1882,12 +1960,11 @@ _readJoinInfo()
 
 	local_node = makeNode(JoinInfo);
 
-	token = lsptok(NULL, &length);		/* get :otherrels */
-	local_node->otherrels =
-		toIntList(nodeRead(true));		/* now read it */
+	token = lsptok(NULL, &length);		/* get :unjoined_relids */
+	local_node->unjoined_relids = toIntList(nodeRead(true));	/* now read it */
 
-	token = lsptok(NULL, &length);		/* get :jinfoclauseinfo */
-	local_node->jinfoclauseinfo = nodeRead(true);		/* now read it */
+	token = lsptok(NULL, &length);		/* get :jinfo_restrictinfo */
+	local_node->jinfo_restrictinfo = nodeRead(true);	/* now read it */
 
 	token = lsptok(NULL, &length);		/* get :mergejoinable */
 
@@ -1965,12 +2042,12 @@ parsePlanString(void)
 		return_value = _readSeqScan();
 	else if (!strncmp(token, "INDEXSCAN", length))
 		return_value = _readIndexScan();
-	else if (!strncmp(token, "TEMP", length))
-		return_value = _readTemp();
+	else if (!strncmp(token, "NONAME", length))
+		return_value = _readNoname();
 	else if (!strncmp(token, "SORT", length))
 		return_value = _readSort();
 	else if (!strncmp(token, "AGGREG", length))
-		return_value = _readAggreg();
+		return_value = _readAggref();
 	else if (!strncmp(token, "SUBLINK", length))
 		return_value = _readSubLink();
 	else if (!strncmp(token, "AGG", length))
@@ -2005,12 +2082,14 @@ parsePlanString(void)
 		return_value = _readTargetEntry();
 	else if (!strncmp(token, "RTE", length))
 		return_value = _readRangeTblEntry();
+	else if (!strncmp(token, "PATHORDER", length))
+		return_value = _readPathOrder();
 	else if (!strncmp(token, "PATH", length))
 		return_value = _readPath();
 	else if (!strncmp(token, "INDEXPATH", length))
 		return_value = _readIndexPath();
-	else if (!strncmp(token, "JOINPATH", length))
-		return_value = _readJoinPath();
+	else if (!strncmp(token, "NESTPATH", length))
+		return_value = _readNestPath();
 	else if (!strncmp(token, "MERGEPATH", length))
 		return_value = _readMergePath();
 	else if (!strncmp(token, "HASHPATH", length))
@@ -2021,14 +2100,14 @@ parsePlanString(void)
 		return_value = _readJoinKey();
 	else if (!strncmp(token, "MERGEORDER", length))
 		return_value = _readMergeOrder();
-	else if (!strncmp(token, "CLAUSEINFO", length))
-		return_value = _readClauseInfo();
+	else if (!strncmp(token, "RETRICTINFO", length))
+		return_value = _readRestrictInfo();
 	else if (!strncmp(token, "JOINMETHOD", length))
 		return_value = _readJoinMethod();
 	else if (!strncmp(token, "JOININFO", length))
 		return_value = _readJoinInfo();
-	else if (!strncmp(token, "HINFO", length))
-		return_value = _readHInfo();
+	else if (!strncmp(token, "HASHINFO", length))
+		return_value = _readHashInfo();
 	else if (!strncmp(token, "ITER", length))
 		return_value = _readIter();
 	else if (!strncmp(token, "QUERY", length))
@@ -2037,6 +2116,12 @@ parsePlanString(void)
 		return_value = _readSortClause();
 	else if (!strncmp(token, "GROUPCLAUSE", length))
 		return_value = _readGroupClause();
+	else if (!strncmp(token, "CASE", length))
+		return_value = _readCaseExpr();
+	else if (!strncmp(token, "WHEN", length))
+		return_value = _readCaseWhen();
+	else if (!strncmp(token, "ROWMARK", length))
+		return_value = _readRowMark();
 	else
 		elog(ERROR, "badly formatted planstring \"%.10s\"...\n", token);
 
