@@ -1,13 +1,13 @@
 /*-------------------------------------------------------------------------
  *
- * allpaths.c--
+ * allpaths.c
  *	  Routines to find possible search paths for processing a query
  *
  * Copyright (c) 1994, Regents of the University of California
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/optimizer/path/allpaths.c,v 1.23 1998/09/01 04:29:27 momjian Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/optimizer/path/allpaths.c,v 1.46 1999/05/25 22:41:25 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -43,23 +43,24 @@ bool		_use_geqo_ = false;
 int32		_use_geqo_rels_ = GEQO_RELS;
 
 
-static void find_rel_paths(Query *root, List *rels);
-static List *find_join_paths(Query *root, List *outer_rels, int levels_needed);
+static void set_base_rel_pathlist(Query *root, List *rels);
+static RelOptInfo *make_one_rel_by_joins(Query *root, List *rels,
+					  int levels_needed);
 
 #ifdef OPTIMIZER_DEBUG
-static void debug_print_rel(Query *root, RelOptInfo * rel);
+static void debug_print_rel(Query *root, RelOptInfo *rel);
 
 #endif
 
 /*
- * find-paths--
- *	  Finds all possible access paths for executing a query, returning the
- *	  top level list of relation entries.
+ * make_one_rel
+ *	  Finds all possible access paths for executing a query, returning a
+ *	  single rel.
  *
  * 'rels' is the list of single relation entries appearing in the query
  */
-List *
-find_paths(Query *root, List *rels)
+RelOptInfo *
+make_one_rel(Query *root, List *rels)
 {
 	int			levels_needed;
 
@@ -69,12 +70,9 @@ find_paths(Query *root, List *rels)
 	levels_needed = length(rels);
 
 	if (levels_needed <= 0)
-		return NIL;
+		return NULL;
 
-	/*
-	 * Find the base relation paths.
-	 */
-	find_rel_paths(root, rels);
+	set_base_rel_pathlist(root, rels);
 
 	if (levels_needed <= 1)
 	{
@@ -82,23 +80,23 @@ find_paths(Query *root, List *rels)
 		/*
 		 * Unsorted single relation, no more processing is required.
 		 */
-		return rels;
+		return lfirst(rels);
 	}
 	else
 	{
 
 		/*
-		 * this means that joins or sorts are required. set selectivities
+		 * This means that joins or sorts are required. set selectivities
 		 * of clauses that have not been set by an index.
 		 */
 		set_rest_relselec(root, rels);
 
-		return find_join_paths(root, rels, levels_needed);
+		return make_one_rel_by_joins(root, rels, levels_needed);
 	}
 }
 
 /*
- * find-rel-paths--
+ * set_base_rel_pathlist
  *	  Finds all paths available for scanning each relation entry in
  *	  'rels'.  Sequential scan and any available indices are considered
  *	  if possible(indices are not considered for lower nesting levels).
@@ -107,10 +105,9 @@ find_paths(Query *root, List *rels)
  *	  MODIFIES: rels
  */
 static void
-find_rel_paths(Query *root, List *rels)
+set_base_rel_pathlist(Query *root, List *rels)
 {
 	List	   *temp;
-	List	   *lastpath;
 
 	foreach(temp, rels)
 	{
@@ -121,33 +118,26 @@ find_rel_paths(Query *root, List *rels)
 
 		sequential_scan_list = lcons(create_seqscan_path(rel), NIL);
 
-		rel_index_scan_list = find_index_paths(root,
-											   rel,
+		rel_index_scan_list = create_index_paths(root,
+												 rel,
 										find_relation_indices(root, rel),
-											   rel->clauseinfo,
-											   rel->joininfo);
+												 rel->restrictinfo,
+												 rel->joininfo);
 
-		or_index_scan_list = create_or_index_paths(root, rel, rel->clauseinfo);
+		or_index_scan_list = create_or_index_paths(root, rel, rel->restrictinfo);
 
 		rel->pathlist = add_pathlist(rel,
 									 sequential_scan_list,
-									 append(rel_index_scan_list,
-											or_index_scan_list));
+									 nconc(rel_index_scan_list,
+										   or_index_scan_list));
 
-		/*
-		 * The unordered path is always the last in the list. If it is not
-		 * the cheapest path, prune it.
-		 */
-		lastpath = rel->pathlist;
-		while (lnext(lastpath) != NIL)
-			lastpath = lnext(lastpath);
-		prune_rel_path(rel, (Path *) lfirst(lastpath));
+		set_cheapest(rel, rel->pathlist);
 
 		/*
 		 * if there is a qualification of sequential scan the selec. value
 		 * is not set -- so set it explicitly -- Sunita
 		 */
-		set_rest_selec(root, rel->clauseinfo);
+		set_rest_selec(root, rel->restrictinfo);
 		rel->size = compute_rel_size(rel);
 		rel->width = compute_rel_width(rel);
 	}
@@ -155,37 +145,34 @@ find_rel_paths(Query *root, List *rels)
 }
 
 /*
- * find-join-paths--
+ * make_one_rel_by_joins
  *	  Find all possible joinpaths for a query by successively finding ways
  *	  to join single relations into join relations.
  *
- *	  if BushyPlanFlag is set, bushy tree plans will be generated:
  *	  Find all possible joinpaths(bushy trees) for a query by systematically
  *	  finding ways to join relations(both original and derived) together.
  *
- * 'outer-rels' is the current list of relations for which join paths
- *				are to be found, i.e., he current list of relations that
+ * 'rels' is the current list of relations for which join paths
+ *				are to be found, i.e., the current list of relations that
  *				have already been derived.
- * 'levels-needed' is the number of iterations needed
+ * 'levels_needed' is the number of iterations needed
  *
  * Returns the final level of join relations, i.e., the relation that is
  * the result of joining all the original relations together.
  */
-static List *
-find_join_paths(Query *root, List *outer_rels, int levels_needed)
+static RelOptInfo *
+make_one_rel_by_joins(Query *root, List *rels, int levels_needed)
 {
 	List	   *x;
-	List	   *new_rels = NIL;
+	List	   *joined_rels = NIL;
 	RelOptInfo *rel;
 
 	/*******************************************
 	 * genetic query optimizer entry point	   *
 	 *	  <utesch@aut.tu-freiberg.de>		   *
 	 *******************************************/
-
 	if ((_use_geqo_) && length(root->base_rel_list) >= _use_geqo_rels_)
-		return lcons(geqo(root), NIL);	/* returns *one* RelOptInfo, so
-										 * lcons it */
+		return geqo(root);
 
 	/*******************************************
 	 * rest will be deprecated in case of GEQO *
@@ -196,80 +183,48 @@ find_join_paths(Query *root, List *outer_rels, int levels_needed)
 
 		/*
 		 * Determine all possible pairs of relations to be joined at this
-		 * level. Determine paths for joining these relation pairs and
-		 * modify 'new-rels' accordingly, then eliminate redundant join
+		 * level.  Determine paths for joining these relation pairs and
+		 * modify 'joined_rels' accordingly, then eliminate redundant join
 		 * relations.
 		 */
-		new_rels = find_join_rels(root, outer_rels);
+		joined_rels = make_rels_by_joins(root, rels);
 
-		find_all_join_paths(root, new_rels);
+		update_rels_pathlist_for_joins(root, joined_rels);
 
-		prune_joinrels(new_rels);
+		merge_rels_with_same_relids(joined_rels);
 
-#if 0
+		root->join_rel_list = rels = joined_rels;
+
+#ifdef NOT_USED
 
 		/*
 		 * * for each expensive predicate in each path in each distinct
 		 * rel, * consider doing pullup  -- JMH
 		 */
 		if (XfuncMode != XFUNC_NOPULL && XfuncMode != XFUNC_OFF)
-			foreach(x, new_rels)
+			foreach(x, joined_rels)
 				xfunc_trypullup((RelOptInfo *) lfirst(x));
 #endif
 
-		prune_rel_paths(new_rels);
+		rels_set_cheapest(joined_rels);
 
-		if (BushyPlanFlag)
-		{
-
-			/*
-			 * In case of bushy trees if there is still a join between a
-			 * join relation and another relation, add a new joininfo that
-			 * involves the join relation to the joininfo list of the
-			 * other relation
-			 */
-			add_new_joininfos(root, new_rels, outer_rels);
-		}
-
-		foreach(x, new_rels)
+		foreach(x, joined_rels)
 		{
 			rel = (RelOptInfo *) lfirst(x);
+
 			if (rel->size <= 0)
 				rel->size = compute_rel_size(rel);
 			rel->width = compute_rel_width(rel);
 
-			/* #define OPTIMIZER_DEBUG */
 #ifdef OPTIMIZER_DEBUG
 			printf("levels left: %d\n", levels_needed);
 			debug_print_rel(root, rel);
 #endif
 		}
 
-		if (BushyPlanFlag)
-		{
-
-			/*
-			 * prune rels that have been completely incorporated into new
-			 * join rels
-			 */
-			outer_rels = prune_oldrels(outer_rels);
-
-			/*
-			 * merge join rels if then contain the same list of base rels
-			 */
-			outer_rels = merge_joinrels(new_rels, outer_rels);
-			root->join_rel_list = outer_rels;
-		}
-		else
-			root->join_rel_list = new_rels;
-		if (!BushyPlanFlag)
-			outer_rels = new_rels;
 	}
 
-	if (BushyPlanFlag)
-		return final_join_rels(outer_rels);
-	else
-		return new_rels;
+	return get_cheapest_complete_rel(rels);
 }
 
 /*****************************************************************************
@@ -285,7 +240,7 @@ print_joinclauses(Query *root, List *clauses)
 
 	foreach(l, clauses)
 	{
-		ClauseInfo *c = lfirst(l);
+		RestrictInfo *c = lfirst(l);
 
 		print_expr((Node *) c->clause, root->rtable);
 		if (lnext(l))
@@ -314,7 +269,7 @@ print_path(Query *root, Path *path, int indent)
 			ptype = "IdxScan";
 			join = false;
 			break;
-		case T_JoinPath:
+		case T_NestPath:
 			ptype = "Nestloop";
 			join = true;
 			break;
@@ -342,8 +297,7 @@ print_path(Query *root, Path *path, int indent)
 				for (i = 0; i < indent + 1; i++)
 					printf("\t");
 				printf("   clauses=(");
-				print_joinclauses(root,
-								  ((JoinPath *) path)->pathclauseinfo);
+				print_joinclauses(root, ((JoinPath *) path)->pathinfo);
 				printf(")\n");
 
 				if (nodeTag(path) == T_MergePath)
@@ -379,8 +333,8 @@ print_path(Query *root, Path *path, int indent)
 			List	   *k,
 					   *l;
 
-			printf(" keys=");
-			foreach(k, path->keys)
+			printf(" pathkeys=");
+			foreach(k, path->pathkeys)
 			{
 				printf("(");
 				foreach(l, lfirst(k))
@@ -401,7 +355,7 @@ print_path(Query *root, Path *path, int indent)
 }
 
 static void
-debug_print_rel(Query *root, RelOptInfo * rel)
+debug_print_rel(Query *root, RelOptInfo *rel)
 {
 	List	   *l;
 
