@@ -1,13 +1,13 @@
 /*-------------------------------------------------------------------------
  *
- * createplan.c--
+ * createplan.c
  *	  Routines to create the desired plan for processing a query
  *
  * Copyright (c) 1994, Regents of the University of California
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/optimizer/plan/createplan.c,v 1.32.2.1 1998/11/24 02:05:09 momjian Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/optimizer/plan/createplan.c,v 1.57 1999/05/25 22:41:38 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -31,7 +31,8 @@
 #include "utils/palloc.h"
 #include "utils/builtins.h"
 
-#include "optimizer/clauseinfo.h"
+#include "optimizer/restrictinfo.h"
+#include "optimizer/cost.h"
 #include "optimizer/clauses.h"
 #include "optimizer/planmain.h"
 #include "optimizer/tlist.h"
@@ -40,17 +41,19 @@
 #include "optimizer/internal.h"
 
 
-#define TEMP_SORT		1
-#define TEMP_MATERIAL	2
+#define NONAME_SORT		1
+#define NONAME_MATERIAL 2
 
 static List *switch_outer(List *clauses);
+static Oid *generate_merge_input_sortorder(List *pathkeys,
+							   MergeOrder *mergeorder);
 static Scan *create_scan_node(Path *best_path, List *tlist);
 static Join *create_join_node(JoinPath *best_path, List *tlist);
 static SeqScan *create_seqscan_node(Path *best_path, List *tlist,
 					List *scan_clauses);
 static IndexScan *create_indexscan_node(IndexPath *best_path, List *tlist,
 					  List *scan_clauses);
-static NestLoop *create_nestloop_node(JoinPath *best_path, List *tlist,
+static NestLoop *create_nestloop_node(NestPath *best_path, List *tlist,
 					 List *clauses, Plan *outer_node, List *outer_tlist,
 					 Plan *inner_node, List *inner_tlist);
 static MergeJoin *create_mergejoin_node(MergePath *best_path, List *tlist,
@@ -60,25 +63,25 @@ static HashJoin *create_hashjoin_node(HashPath *best_path, List *tlist,
 					 List *clauses, Plan *outer_node, List *outer_tlist,
 					 Plan *inner_node, List *inner_tlist);
 static Node *fix_indxqual_references(Node *clause, Path *index_path);
-static Temp *make_temp(List *tlist, List *keys, Oid *operators,
-		  Plan *plan_node, int temptype);
+static Noname *make_noname(List *tlist, List *pathkeys, Oid *operators,
+			Plan *plan_node, int nonametype);
 static IndexScan *make_indexscan(List *qptlist, List *qpqual, Index scanrelid,
-			   List *indxid, List *indxqual, List *indxqualorig, Cost cost);
+			   List *indxid, List *indxqual, List *indxqualorig);
 static NestLoop *make_nestloop(List *qptlist, List *qpqual, Plan *lefttree,
 			  Plan *righttree);
 static HashJoin *make_hashjoin(List *tlist, List *qpqual,
 			  List *hashclauses, Plan *lefttree, Plan *righttree);
 static Hash *make_hash(List *tlist, Var *hashkey, Plan *lefttree);
 static MergeJoin *make_mergejoin(List *tlist, List *qpqual,
-			   List *mergeclauses, Oid opcode, Oid *rightorder,
-			   Oid *leftorder, Plan *righttree, Plan *lefttree);
-static Material *make_material(List *tlist, Oid tempid, Plan *lefttree,
+			   List *mergeclauses, Plan *righttree, Plan *lefttree);
+static Material *make_material(List *tlist, Oid nonameid, Plan *lefttree,
 			  int keycount);
+static void copy_costsize(Plan *dest, Plan *src);
 
 /*
- * create_plan--
+ * create_plan
  *	  Creates the access plan for a query by tracing backwards through the
- *	  desired chain of pathnodes, starting at the node 'best-path'.  For
+ *	  desired chain of pathnodes, starting at the node 'best_path'.  For
  *	  every pathnode found:
  *	  (1) Create a corresponding plan node containing appropriate id,
  *		  target list, and qualification information.
@@ -86,7 +89,7 @@ static Material *make_material(List *tlist, Oid tempid, Plan *lefttree,
  *		  relative values.
  *	  (3) Target lists are not modified, but will be in another routine.
  *
- *	  best-path is the best access path
+ *	  best_path is the best access path
  *
  *	  Returns the optimal(?) access plan.
  */
@@ -130,7 +133,7 @@ create_plan(Path *best_path)
 		pages = 1;
 	plan_node->plan_tupperpage = tuples / pages;
 
-#if 0							/* fix xfunc */
+#ifdef NOT_USED					/* fix xfunc */
 	/* sort clauses by cost/(1-selectivity) -- JMH 2/26/92 */
 	if (XfuncMode != XFUNC_OFF)
 	{
@@ -147,10 +150,10 @@ create_plan(Path *best_path)
 }
 
 /*
- * create_scan_node--
- *	 Create a scan path for the parent relation of 'best-path'.
+ * create_scan_node
+ *	 Create a scan path for the parent relation of 'best_path'.
  *
- *	 tlist is the targetlist for the base relation scanned by 'best-path'
+ *	 tlist is the targetlist for the base relation scanned by 'best_path'
  *
  *	 Returns the scan node.
  */
@@ -170,7 +173,7 @@ create_scan_node(Path *best_path, List *tlist)
 	 * xfunc_trypullup(), we get the relevant clauses from the path
 	 * itself, not its parent relation.   --- JMH, 6/15/92
 	 */
-	scan_clauses = fix_opids(get_actual_clauses(best_path->locclauseinfo));
+	scan_clauses = fix_opids(get_actual_clauses(best_path->loc_restrictinfo));
 
 	switch (best_path->pathtype)
 	{
@@ -194,12 +197,12 @@ create_scan_node(Path *best_path, List *tlist)
 }
 
 /*
- * create_join_node --
- *	  Create a join path for 'best-path' and(recursively) paths for its
+ * create_join_node
+ *	  Create a join path for 'best_path' and(recursively) paths for its
  *	  inner and outer paths.
  *
  *	  'tlist' is the targetlist for the join relation corresponding to
- *		'best-path'
+ *		'best_path'
  *
  *	  Returns the join node.
  */
@@ -219,7 +222,7 @@ create_join_node(JoinPath *best_path, List *tlist)
 	inner_node = create_plan((Path *) best_path->innerjoinpath);
 	inner_tlist = inner_node->targetlist;
 
-	clauses = get_actual_clauses(best_path->pathclauseinfo);
+	clauses = get_actual_clauses(best_path->pathinfo);
 
 	switch (best_path->path.pathtype)
 	{
@@ -242,7 +245,7 @@ create_join_node(JoinPath *best_path, List *tlist)
 												   inner_tlist);
 			break;
 		case T_NestLoop:
-			retval = (Join *) create_nestloop_node((JoinPath *) best_path,
+			retval = (Join *) create_nestloop_node((NestPath *) best_path,
 												   tlist,
 												   clauses,
 												   outer_node,
@@ -260,14 +263,14 @@ create_join_node(JoinPath *best_path, List *tlist)
 
 	/*
 	 * * Expensive function pullups may have pulled local predicates *
-	 * into this path node.  Put them in the qpqual of the plan node. * --
+	 * into this path node.  Put them in the qpqual of the plan node. *
 	 * JMH, 6/15/92
 	 */
-	if (get_locclauseinfo(best_path) != NIL)
+	if (get_loc_restrictinfo(best_path) != NIL)
 		set_qpqual((Plan) retval,
 				   nconc(get_qpqual((Plan) retval),
 						 fix_opids(get_actual_clauses
-								   (get_locclauseinfo(best_path)))));
+								   (get_loc_restrictinfo(best_path)))));
 #endif
 
 	return retval;
@@ -281,9 +284,9 @@ create_join_node(JoinPath *best_path, List *tlist)
 
 
 /*
- * create_seqscan_node--
- *	 Returns a seqscan node for the base relation scanned by 'best-path'
- *	 with restriction clauses 'scan-clauses' and targetlist 'tlist'.
+ * create_seqscan_node
+ *	 Returns a seqscan node for the base relation scanned by 'best_path'
+ *	 with restriction clauses 'scan_clauses' and targetlist 'tlist'.
  */
 static SeqScan *
 create_seqscan_node(Path *best_path, List *tlist, List *scan_clauses)
@@ -309,9 +312,9 @@ create_seqscan_node(Path *best_path, List *tlist, List *scan_clauses)
 }
 
 /*
- * create_indexscan_node--
- *	  Returns a indexscan node for the base relation scanned by 'best-path'
- *	  with restriction clauses 'scan-clauses' and targetlist 'tlist'.
+ * create_indexscan_node
+ *	  Returns a indexscan node for the base relation scanned by 'best_path'
+ *	  with restriction clauses 'scan_clauses' and targetlist 'tlist'.
  */
 static IndexScan *
 create_indexscan_node(IndexPath *best_path,
@@ -321,7 +324,7 @@ create_indexscan_node(IndexPath *best_path,
 
 	/*
 	 * Extract the(first if conjunct, only if disjunct) clause from the
-	 * clauseinfo list.
+	 * restrictinfo list.
 	 */
 	Expr	   *index_clause = (Expr *) NULL;
 	List	   *indxqual = NIL;
@@ -364,8 +367,7 @@ create_indexscan_node(IndexPath *best_path,
 										 ObjectIdGetDatum(lfirsti(ixid)),
 										 0, 0, 0);
 		if (!HeapTupleIsValid(indexTuple))
-			elog(ERROR, "create_plan: index %d not found",
-				 lfirsti(ixid));
+			elog(ERROR, "create_plan: index %u not found", lfirsti(ixid));
 		index = (Form_pg_index) GETSTRUCT(indexTuple);
 		if (index->indislossy)
 			lossy = TRUE;
@@ -385,8 +387,7 @@ create_indexscan_node(IndexPath *best_path,
 								lcons(index_clause, NIL));
 
 		if (lossy)
-			qpqual = nconc(qpqual,
-						   lcons((List *) copyObject(index_clause), NIL));
+			qpqual = lappend(qpqual, (List *) copyObject(index_clause));
 	}
 	else
 	{
@@ -396,17 +397,16 @@ create_indexscan_node(IndexPath *best_path,
 						   (List *) copyObject(lfirst(indxqual)));
 	}
 
-	fixed_indxqual =
-		(List *) fix_indxqual_references((Node *) indxqual, (Path *) best_path);
+	fixed_indxqual = (List *) fix_indxqual_references((Node *) indxqual, (Path *) best_path);
 
-	scan_node =
-		make_indexscan(tlist,
-					   qpqual,
-					   lfirsti(best_path->path.parent->relids),
-					   best_path->indexid,
-					   fixed_indxqual,
-					   indxqual,
-					   best_path->path.path_cost);
+	scan_node = make_indexscan(tlist,
+							   qpqual,
+							   lfirsti(best_path->path.parent->relids),
+							   best_path->indexid,
+							   fixed_indxqual,
+							   indxqual);
+
+	scan_node->scan.plan.cost = best_path->path.path_cost;
 
 	return scan_node;
 }
@@ -418,7 +418,7 @@ create_indexscan_node(IndexPath *best_path,
  *****************************************************************************/
 
 static NestLoop *
-create_nestloop_node(JoinPath *best_path,
+create_nestloop_node(NestPath *best_path,
 					 List *tlist,
 					 List *clauses,
 					 Plan *outer_node,
@@ -437,7 +437,7 @@ create_nestloop_node(JoinPath *best_path,
 		 * used in the inner scan path, so we need only consider the first
 		 * set of qualifications in indxqual.
 		 *
-		 * But there may be more than one clauses in this "first set" in the
+		 * But there may be more than one clause in this "first set" in the
 		 * case of multi-column indices. - vadim 03/18/97
 		 */
 
@@ -470,21 +470,19 @@ create_nestloop_node(JoinPath *best_path,
 			List	   *new_inner_qual = NIL;
 
 			clauses = set_difference(clauses, inner_indxqual);	/* XXX */
-			new_inner_qual =
-				index_outerjoin_references(inner_indxqual,
-										   outer_node->targetlist,
+			new_inner_qual = index_outerjoin_references(inner_indxqual,
+												  outer_node->targetlist,
 									   ((Scan *) inner_node)->scanrelid);
-			((IndexScan *) inner_node)->indxqual =
-				lcons(new_inner_qual, NIL);
+			((IndexScan *) inner_node)->indxqual = lcons(new_inner_qual, NIL);
 		}
 	}
 	else if (IsA_Join(inner_node))
 	{
-		inner_node = (Plan *) make_temp(inner_tlist,
-										NIL,
-										NULL,
-										inner_node,
-										TEMP_MATERIAL);
+		inner_node = (Plan *) make_noname(inner_tlist,
+										  NIL,
+										  NULL,
+										  inner_node,
+										  NONAME_MATERIAL);
 	}
 
 	join_node = make_nestloop(tlist,
@@ -510,9 +508,6 @@ create_mergejoin_node(MergePath *best_path,
 {
 	List	   *qpqual,
 			   *mergeclauses;
-	RegProcedure opcode;
-	Oid		   *outer_order,
-			   *inner_order;
 	MergeJoin  *join_node;
 
 
@@ -533,53 +528,39 @@ create_mergejoin_node(MergePath *best_path,
 												outer_tlist,
 												inner_tlist));
 
-	opcode =
-		get_opcode((best_path->jpath.path.p_ordering.ord.merge)->join_operator);
-
-	outer_order = (Oid *) palloc(sizeof(Oid) * 2);
-	outer_order[0] =
-		(best_path->jpath.path.p_ordering.ord.merge)->left_operator;
-	outer_order[1] = 0;
-
-	inner_order = (Oid *) palloc(sizeof(Oid) * 2);
-	inner_order[0] =
-		(best_path->jpath.path.p_ordering.ord.merge)->right_operator;
-	inner_order[1] = 0;
-
 	/*
 	 * Create explicit sort paths for the outer and inner join paths if
 	 * necessary.  The sort cost was already accounted for in the path.
 	 */
 	if (best_path->outersortkeys)
 	{
-		Temp	   *sorted_outer_node = make_temp(outer_tlist,
+		Oid		   *outer_order = generate_merge_input_sortorder(
 												best_path->outersortkeys,
-												  outer_order,
-												  outer_node,
-												  TEMP_SORT);
+							 best_path->jpath.path.pathorder->ord.merge);
 
-		sorted_outer_node->plan.cost = outer_node->cost;
-		outer_node = (Plan *) sorted_outer_node;
+		outer_node = (Plan *) make_noname(outer_tlist,
+										  best_path->outersortkeys,
+										  outer_order,
+										  outer_node,
+										  NONAME_SORT);
 	}
 
 	if (best_path->innersortkeys)
 	{
-		Temp	   *sorted_inner_node = make_temp(inner_tlist,
+		Oid		   *inner_order = generate_merge_input_sortorder(
 												best_path->innersortkeys,
-												  inner_order,
-												  inner_node,
-												  TEMP_SORT);
+							 best_path->jpath.path.pathorder->ord.merge);
 
-		sorted_inner_node->plan.cost = outer_node->cost;
-		inner_node = (Plan *) sorted_inner_node;
+		inner_node = (Plan *) make_noname(inner_tlist,
+										  best_path->innersortkeys,
+										  inner_order,
+										  inner_node,
+										  NONAME_SORT);
 	}
 
 	join_node = make_mergejoin(tlist,
 							   qpqual,
 							   mergeclauses,
-							   opcode,
-							   inner_order,
-							   outer_order,
 							   inner_node,
 							   outer_node);
 
@@ -615,20 +596,18 @@ create_hashjoin_node(HashPath *best_path,
 	 * Separate the hashclauses from the other join qualification clauses
 	 * and set those clauses to contain references to lower attributes.
 	 */
-	qpqual =
-		join_references(set_difference(clauses,
-									   best_path->path_hashclauses),
-						outer_tlist,
-						inner_tlist);
+	qpqual = join_references(set_difference(clauses,
+											best_path->path_hashclauses),
+							 outer_tlist,
+							 inner_tlist);
 
 	/*
 	 * Now set the references in the hashclauses and rearrange them so
 	 * that the outer variable is always on the left.
 	 */
-	hashclauses =
-		switch_outer(join_references(best_path->path_hashclauses,
-									 outer_tlist,
-									 inner_tlist));
+	hashclauses = switch_outer(join_references(best_path->path_hashclauses,
+											   outer_tlist,
+											   inner_tlist));
 
 	innerhashkey = get_rightop(lfirst(hashclauses));
 
@@ -638,6 +617,7 @@ create_hashjoin_node(HashPath *best_path,
 							  hashclauses,
 							  outer_node,
 							  (Plan *) hash_node);
+
 	join_node->join.cost = best_path->jpath.path.path_cost;
 
 	return join_node;
@@ -650,15 +630,23 @@ create_hashjoin_node(HashPath *best_path,
  *
  *****************************************************************************/
 
+/*
+ * fix_indxqual_references
+ *	Adjust a qual clause to refer to an index instead of the original relation.
+ *
+ * Returns a modified copy of the given clause --- the original is not changed.
+ */
+
 static Node *
 fix_indxqual_references(Node *clause, Path *index_path)
 {
-	Node	   *newclause;
-
-	if (IsA(clause, Var))
+	if (clause == NULL)
+		return NULL;
+	else if (IsA(clause, Var))
 	{
 		if (lfirsti(index_path->parent->relids) == ((Var *) clause)->varno)
 		{
+			Node	   *newclause;
 			int			pos = 0;
 			int			varatt = ((Var *) clause)->varattno;
 			int		   *indexkeys = ((IndexPath *) index_path)->indexkeys;
@@ -672,131 +660,140 @@ fix_indxqual_references(Node *clause, Path *index_path)
 					pos++;
 				}
 			}
-			newclause = copyObject((Node *) clause);
+			newclause = copyObject(clause);
 			((Var *) newclause)->varattno = pos + 1;
 			return newclause;
 		}
-		else
-			return clause;
-	}
-	else if (IsA(clause, Const))
-		return clause;
-	else if (IsA(clause, Param))
-	{
-		/* Function parameter used as index scan arg.  DZ - 27-8-1996 */
-		return clause;
+		/* The Var is not part of the indexed relation, leave it alone */
+		return copyObject(clause);
 	}
+	else if (single_node(clause))
+		return copyObject(clause);
 	else if (is_opclause(clause) &&
 			 is_funcclause((Node *) get_leftop((Expr *) clause)) &&
 	((Func *) ((Expr *) get_leftop((Expr *) clause))->oper)->funcisindex)
 	{
-		Var		   *newvar =
-		makeVar((Index) lfirsti(index_path->parent->relids),
-				1,				/* func indices have one key */
-				((Func *) ((Expr *) clause)->oper)->functype,
-				-1,
-				0,
-				(Index) lfirsti(index_path->parent->relids),
-				0);
 
-		return
-			((Node *) make_opclause((Oper *) ((Expr *) clause)->oper,
-									newvar,
-									get_rightop((Expr *) clause)));
+		/*
+		 * This looks pretty seriously wrong to me, but I'm not sure what
+		 * it's supposed to be doing ... tgl 5/99
+		 */
+		Var		   *newvar = makeVar((Index) lfirsti(index_path->parent->relids),
+									 1, /* func indices have one key */
+							((Func *) ((Expr *) clause)->oper)->functype,
+									 -1,
+									 0,
+							 (Index) lfirsti(index_path->parent->relids),
+									 0);
+
+		return ((Node *) make_opclause((Oper *) ((Expr *) clause)->oper,
+									   newvar,
+									   get_rightop((Expr *) clause)));
 
 	}
 	else if (IsA(clause, Expr))
 	{
 		Expr	   *expr = (Expr *) clause;
 		List	   *new_subclauses = NIL;
-		Node	   *subclause = NULL;
-		List	   *i = NIL;
+		List	   *i;
 
 		foreach(i, expr->args)
 		{
-			subclause = lfirst(i);
-			if (subclause)
-				new_subclauses =
-					lappend(new_subclauses,
-							fix_indxqual_references(subclause,
-													index_path));
+			Node	   *subclause = lfirst(i);
 
+			new_subclauses = lappend(new_subclauses,
+									 fix_indxqual_references(subclause,
+															 index_path));
 		}
 
-		/*
-		 * XXX new_subclauses should be a list of the form: ( (var var)
-		 * (var const) ...) ?
-		 */
-		if (new_subclauses)
+		return (Node *) make_clause(expr->opType, expr->oper, new_subclauses);
+	}
+	else if (IsA(clause, List))
+	{
+		List	   *new_subclauses = NIL;
+		List	   *i;
+
+		foreach(i, (List *) clause)
 		{
-			return (Node *)
-				make_clause(expr->opType, expr->oper, new_subclauses);
+			Node	   *subclause = lfirst(i);
+
+			new_subclauses = lappend(new_subclauses,
+									 fix_indxqual_references(subclause,
+															 index_path));
 		}
-		else
-			return clause;
+
+		return (Node *) new_subclauses;
+	}
+	else if (IsA(clause, ArrayRef))
+	{
+		ArrayRef   *oldnode = (ArrayRef *) clause;
+		ArrayRef   *newnode = makeNode(ArrayRef);
+
+		newnode->refattrlength = oldnode->refattrlength;
+		newnode->refelemlength = oldnode->refelemlength;
+		newnode->refelemtype = oldnode->refelemtype;
+		newnode->refelembyval = oldnode->refelembyval;
+		newnode->refupperindexpr = (List *)
+			fix_indxqual_references((Node *) oldnode->refupperindexpr,
+									index_path);
+		newnode->reflowerindexpr = (List *)
+			fix_indxqual_references((Node *) oldnode->reflowerindexpr,
+									index_path);
+		newnode->refexpr =
+			fix_indxqual_references(oldnode->refexpr, index_path);
+		newnode->refassgnexpr =
+			fix_indxqual_references(oldnode->refassgnexpr, index_path);
+
+		return (Node *) newnode;
 	}
 	else
 	{
-		List	   *oldclauses = (List *) clause;
-		List	   *new_subclauses = NIL;
-		Node	   *subclause = NULL;
-		List	   *i = NIL;
-
-		foreach(i, oldclauses)
-		{
-			subclause = lfirst(i);
-			if (subclause)
-				new_subclauses =
-					lappend(new_subclauses,
-							fix_indxqual_references(subclause,
-													index_path));
-
-		}
-
-		/*
-		 * XXX new_subclauses should be a list of the form: ( (var var)
-		 * (var const) ...) ?
-		 */
-		if (new_subclauses)
-			return (Node *) new_subclauses;
-		else
-			return clause;
+		elog(ERROR, "fix_indxqual_references: Cannot handle node type %d",
+			 nodeTag(clause));
+		return NULL;
 	}
 }
 
 
 /*
- * switch_outer--
+ * switch_outer
  *	  Given a list of merge clauses, rearranges the elements within the
  *	  clauses so the outer join variable is on the left and the inner is on
- *	  the right.
- *
- *	  Returns the rearranged list ?
- *
- *	  XXX Shouldn't the operator be commuted?!
+ *	  the right.  The original list is not touched; a modified list
+ *	  is returned.
  */
 static List *
 switch_outer(List *clauses)
 {
 	List	   *t_list = NIL;
-	Expr	   *temp = NULL;
-	List	   *i = NIL;
+	Expr	   *temp;
+	List	   *i;
 	Expr	   *clause;
 	Node	   *op;
 
 	foreach(i, clauses)
 	{
 		clause = lfirst(i);
+		Assert(is_opclause((Node *) clause));
 		op = (Node *) get_rightop(clause);
+		Assert(op != (Node *) NULL);
 		if (IsA(op, ArrayRef))
 			op = ((ArrayRef *) op)->refexpr;
 		Assert(IsA(op, Var));
 		if (var_is_outer((Var *) op))
 		{
+
+			/*
+			 * Duplicate just enough of the structure to allow commuting
+			 * the clause without changing the original list.  Could use
+			 * copyObject, but a complete deep copy is overkill.
+			 */
 			temp = make_clause(clause->opType, clause->oper,
-							   lcons(get_rightop(clause),
-									 lcons(get_leftop(clause),
+							   lcons(get_leftop(clause),
+									 lcons(get_rightop(clause),
 										   NIL)));
+			/* Commute it --- note this modifies the temp node in-place. */
+			CommuteClause((Node *) temp);
 			t_list = lappend(t_list, temp);
 		}
 		else
@@ -806,7 +803,47 @@ switch_outer(List *clauses)
 }
 
 /*
- * set-temp-tlist-operators--
+ * generate_merge_input_sortorder
+ *
+ * Generate the list of sort ops needed to sort one of the input paths for
+ * a merge.  We may have to use either left or right sortop for each item,
+ * since the original mergejoin clause may or may not have been commuted
+ * (compare switch_outer above).
+ *
+ * XXX This is largely a crock.  It works only because group_clauses_by_order
+ * only groups together mergejoin clauses that have identical MergeOrder info,
+ * which means we can safely use a single MergeOrder input to deal with all
+ * the data.  There should be a more general data structure that allows coping
+ * with groups of mergejoin clauses that have different join operators.
+ */
+static Oid *
+generate_merge_input_sortorder(List *pathkeys, MergeOrder *mergeorder)
+{
+	int			listlength = length(pathkeys);
+	Oid		   *result = (Oid *) palloc(sizeof(Oid) * (listlength + 1));
+	Oid		   *nextsortop = result;
+	List	   *p;
+
+	foreach(p, pathkeys)
+	{
+		Var		   *pkey = (Var *) lfirst((List *) lfirst(p));
+
+		Assert(IsA(pkey, Var));
+		if (pkey->vartype == mergeorder->left_type)
+			*nextsortop++ = mergeorder->left_operator;
+		else if (pkey->vartype == mergeorder->right_type)
+			*nextsortop++ = mergeorder->right_operator;
+		else
+			elog(ERROR,
+			 "generate_merge_input_sortorder: can't handle data type %d",
+				 pkey->vartype);
+	}
+	*nextsortop++ = InvalidOid;
+	return result;
+}
+
+/*
+ * set_noname_tlist_operators
  *	  Sets the key and keyop fields of resdom nodes in a target list.
  *
  *	  'tlist' is the target list
@@ -814,97 +851,116 @@ switch_outer(List *clauses)
  *				corresponding to vars in the target list that are to
  *				be sorted or hashed
  *	  'operators' is the corresponding list of N sort or hash operators
- *	  'keyno' is the first key number
- *	  XXX - keyno ? doesn't exist - jeff
  *
- *	  Returns the modified target list.
+ *	  Returns the modified-in-place target list.
  */
 static List *
-set_temp_tlist_operators(List *tlist, List *pathkeys, Oid *operators)
+set_noname_tlist_operators(List *tlist, List *pathkeys, Oid *operators)
 {
-	Node	   *keys = NULL;
 	int			keyno = 1;
-	Resdom	   *resdom = (Resdom *) NULL;
-	List	   *i = NIL;
+	Node	   *pathkey;
+	Resdom	   *resdom;
+	List	   *i;
 
 	foreach(i, pathkeys)
 	{
-		keys = lfirst((List *) lfirst(i));
-		resdom = tlist_member((Var *) keys, tlist);
+		pathkey = lfirst((List *) lfirst(i));
+		resdom = tlist_member((Var *) pathkey, tlist);
 		if (resdom)
 		{
 
 			/*
-			 * Order the resdom keys and replace the operator OID for each
-			 * key with the regproc OID.
-			 *
-			 * XXX Note that the optimizer only generates merge joins with 1
-			 * operator (see create_mergejoin_node)  - ay 2/95
+			 * Order the resdom pathkey and replace the operator OID for
+			 * each key with the regproc OID.
 			 */
 			resdom->reskey = keyno;
-			resdom->reskeyop = get_opcode(operators[0]);
+			resdom->reskeyop = get_opcode(operators[keyno - 1]);
 		}
 		keyno += 1;
 	}
 	return tlist;
 }
 
+/*
+ * Copy cost and size info from a lower plan node to an inserted node.
+ * This is not critical, since the decisions have already been made,
+ * but it helps produce more reasonable-looking EXPLAIN output.
+ */
+
+static void
+copy_costsize(Plan *dest, Plan *src)
+{
+	if (src)
+	{
+		dest->cost = src->cost;
+		dest->plan_size = src->plan_size;
+		dest->plan_width = src->plan_width;
+	}
+	else
+	{
+		dest->cost = 0;
+		dest->plan_size = 0;
+		dest->plan_width = 0;
+	}
+}
+
+
 /*****************************************************************************
  *
  *
  *****************************************************************************/
 
 /*
- * make_temp--
- *	  Create plan nodes to sort or materialize relations into temporaries. The
- *	  result returned for a sort will look like (SEQSCAN(SORT(plan-node)))
- *	  or (SEQSCAN(MATERIAL(plan-node)))
+ * make_noname
+ *	  Create plan nodes to sort or materialize relations into noname. The
+ *	  result returned for a sort will look like (SEQSCAN(SORT(plan_node)))
+ *	  or (SEQSCAN(MATERIAL(plan_node)))
  *
  *	  'tlist' is the target list of the scan to be sorted or hashed
- *	  'keys' is the list of keys which the sort or hash will be done on
+ *	  'pathkeys' is the list of keys which the sort or hash will be done on
  *	  'operators' is the operators with which the sort or hash is to be done
  *		(a list of operator OIDs)
- *	  'plan-node' is the node which yields tuples for the sort
- *	  'temptype' indicates which operation(sort or hash) to perform
+ *	  'plan_node' is the node which yields tuples for the sort
+ *	  'nonametype' indicates which operation(sort or hash) to perform
  */
-static Temp *
-make_temp(List *tlist,
-		  List *keys,
-		  Oid *operators,
-		  Plan *plan_node,
-		  int temptype)
+static Noname *
+make_noname(List *tlist,
+			List *pathkeys,
+			Oid *operators,
+			Plan *plan_node,
+			int nonametype)
 {
-	List	   *temp_tlist;
-	Temp	   *retval = NULL;
+	List	   *noname_tlist;
+	Noname	   *retval = NULL;
 
-	/* Create a new target list for the temporary, with keys set. */
-	temp_tlist = set_temp_tlist_operators(new_unsorted_tlist(tlist),
-										  keys,
-										  operators);
-	switch (temptype)
+	/* Create a new target list for the noname, with keys set. */
+	noname_tlist = set_noname_tlist_operators(new_unsorted_tlist(tlist),
+											  pathkeys,
+											  operators);
+	switch (nonametype)
 	{
-		case TEMP_SORT:
-			retval = (Temp *) make_seqscan(tlist,
-										   NIL,
-										   _TEMP_RELATION_ID_,
-										   (Plan *) make_sort(temp_tlist,
-													  _TEMP_RELATION_ID_,
-															  plan_node,
-														  length(keys)));
+		case NONAME_SORT:
+			retval = (Noname *) make_seqscan(tlist,
+											 NIL,
+											 _NONAME_RELATION_ID_,
+										 (Plan *) make_sort(noname_tlist,
+													_NONAME_RELATION_ID_,
+															plan_node,
+													  length(pathkeys)));
 			break;
 
-		case TEMP_MATERIAL:
-			retval = (Temp *) make_seqscan(tlist,
-										   NIL,
-										   _TEMP_RELATION_ID_,
-									   (Plan *) make_material(temp_tlist,
-													  _TEMP_RELATION_ID_,
-															  plan_node,
-														  length(keys)));
+		case NONAME_MATERIAL:
+			retval = (Noname *) make_seqscan(tlist,
+											 NIL,
+											 _NONAME_RELATION_ID_,
+									 (Plan *) make_material(noname_tlist,
+													_NONAME_RELATION_ID_,
+															plan_node,
+													  length(pathkeys)));
 			break;
 
 		default:
-			elog(ERROR, "make_temp: unknown temp type %d", temptype);
+			elog(ERROR, "make_noname: unknown noname type %d", nonametype);
 
 	}
 	return retval;
@@ -920,7 +976,7 @@ make_seqscan(List *qptlist,
 	SeqScan    *node = makeNode(SeqScan);
 	Plan	   *plan = &node->plan;
 
-	plan->cost = (lefttree ? lefttree->cost : 0);
+	copy_costsize(plan, lefttree);
 	plan->state = (EState *) NULL;
 	plan->targetlist = qptlist;
 	plan->qual = qpqual;
@@ -938,13 +994,12 @@ make_indexscan(List *qptlist,
 			   Index scanrelid,
 			   List *indxid,
 			   List *indxqual,
-			   List *indxqualorig,
-			   Cost cost)
+			   List *indxqualorig)
 {
 	IndexScan  *node = makeNode(IndexScan);
 	Plan	   *plan = &node->scan.plan;
 
-	plan->cost = cost;
+	copy_costsize(plan, NULL);
 	plan->state = (EState *) NULL;
 	plan->targetlist = qptlist;
 	plan->qual = qpqual;
@@ -969,6 +1024,10 @@ make_nestloop(List *qptlist,
 	NestLoop   *node = makeNode(NestLoop);
 	Plan	   *plan = &node->join;
 
+	/*
+	 * this cost estimate is entirely bogus... hopefully it will be
+	 * overwritten by caller.
+	 */
 	plan->cost = (lefttree ? lefttree->cost : 0) +
 		(righttree ? righttree->cost : 0);
 	plan->state = (EState *) NULL;
@@ -991,18 +1050,18 @@ make_hashjoin(List *tlist,
 	HashJoin   *node = makeNode(HashJoin);
 	Plan	   *plan = &node->join;
 
+	/*
+	 * this cost estimate is entirely bogus... hopefully it will be
+	 * overwritten by caller.
+	 */
 	plan->cost = (lefttree ? lefttree->cost : 0) +
 		(righttree ? righttree->cost : 0);
-	plan->cost = 0.0;
 	plan->state = (EState *) NULL;
 	plan->targetlist = tlist;
 	plan->qual = qpqual;
 	plan->lefttree = lefttree;
 	plan->righttree = righttree;
 	node->hashclauses = hashclauses;
-	node->hashjointable = NULL;
-	node->hashjointablekey = 0;
-	node->hashjointablesize = 0;
 	node->hashdone = false;
 
 	return node;
@@ -1014,17 +1073,13 @@ make_hash(List *tlist, Var *hashkey, Plan *lefttree)
 	Hash	   *node = makeNode(Hash);
 	Plan	   *plan = &node->plan;
 
-	plan->cost = (lefttree ? lefttree->cost : 0);
-	plan->cost = 0.0;
+	copy_costsize(plan, lefttree);
 	plan->state = (EState *) NULL;
 	plan->targetlist = tlist;
 	plan->qual = NULL;
 	plan->lefttree = lefttree;
 	plan->righttree = NULL;
 	node->hashkey = hashkey;
-	node->hashtable = NULL;
-	node->hashtablekey = 0;
-	node->hashtablesize = 0;
 
 	return node;
 }
@@ -1033,15 +1088,16 @@ static MergeJoin *
 make_mergejoin(List *tlist,
 			   List *qpqual,
 			   List *mergeclauses,
-			   Oid opcode,
-			   Oid *rightorder,
-			   Oid *leftorder,
 			   Plan *righttree,
 			   Plan *lefttree)
 {
 	MergeJoin  *node = makeNode(MergeJoin);
 	Plan	   *plan = &node->join;
 
+	/*
+	 * this cost estimate is entirely bogus... hopefully it will be
+	 * overwritten by caller.
+	 */
 	plan->cost = (lefttree ? lefttree->cost : 0) +
 		(righttree ? righttree->cost : 0);
 	plan->state = (EState *) NULL;
@@ -1050,26 +1106,24 @@ make_mergejoin(List *tlist,
 	plan->lefttree = lefttree;
 	plan->righttree = righttree;
 	node->mergeclauses = mergeclauses;
-	node->mergejoinop = opcode;
-	node->mergerightorder = rightorder;
-	node->mergeleftorder = leftorder;
 
 	return node;
 }
 
 Sort *
-make_sort(List *tlist, Oid tempid, Plan *lefttree, int keycount)
+make_sort(List *tlist, Oid nonameid, Plan *lefttree, int keycount)
 {
 	Sort	   *node = makeNode(Sort);
 	Plan	   *plan = &node->plan;
 
-	plan->cost = (lefttree ? lefttree->cost : 0);
+	copy_costsize(plan, lefttree);
+	plan->cost += cost_sort(NULL, plan->plan_size, plan->plan_width);
 	plan->state = (EState *) NULL;
 	plan->targetlist = tlist;
 	plan->qual = NIL;
 	plan->lefttree = lefttree;
 	plan->righttree = NULL;
-	node->tempid = tempid;
+	node->nonameid = nonameid;
 	node->keycount = keycount;
 
 	return node;
@@ -1077,20 +1131,20 @@ make_sort(List *tlist, Oid tempid, Plan *lefttree, int keycount)
 
 static Material *
 make_material(List *tlist,
-			  Oid tempid,
+			  Oid nonameid,
 			  Plan *lefttree,
 			  int keycount)
 {
 	Material   *node = makeNode(Material);
 	Plan	   *plan = &node->plan;
 
-	plan->cost = (lefttree ? lefttree->cost : 0);
+	copy_costsize(plan, lefttree);
 	plan->state = (EState *) NULL;
 	plan->targetlist = tlist;
 	plan->qual = NIL;
 	plan->lefttree = lefttree;
 	plan->righttree = NULL;
-	node->tempid = tempid;
+	node->nonameid = nonameid;
 	node->keycount = keycount;
 
 	return node;
@@ -1101,7 +1155,7 @@ make_agg(List *tlist, Plan *lefttree)
 {
 	Agg		   *node = makeNode(Agg);
 
-	node->plan.cost = (lefttree ? lefttree->cost : 0);
+	copy_costsize(&node->plan, lefttree);
 	node->plan.state = (EState *) NULL;
 	node->plan.qual = NULL;
 	node->plan.targetlist = tlist;
@@ -1121,7 +1175,7 @@ make_group(List *tlist,
 {
 	Group	   *node = makeNode(Group);
 
-	node->plan.cost = (lefttree ? lefttree->plan.cost : 0);
+	copy_costsize(&node->plan, (Plan *) lefttree);
 	node->plan.state = (EState *) NULL;
 	node->plan.qual = NULL;
 	node->plan.targetlist = tlist;
@@ -1148,13 +1202,13 @@ make_unique(List *tlist, Plan *lefttree, char *uniqueAttr)
 	Unique	   *node = makeNode(Unique);
 	Plan	   *plan = &node->plan;
 
-	plan->cost = (lefttree ? lefttree->cost : 0);
+	copy_costsize(plan, lefttree);
 	plan->state = (EState *) NULL;
 	plan->targetlist = tlist;
 	plan->qual = NIL;
 	plan->lefttree = lefttree;
 	plan->righttree = NULL;
-	node->tempid = _TEMP_RELATION_ID_;
+	node->nonameid = _NONAME_RELATION_ID_;
 	node->keycount = 0;
 	if (strcmp(uniqueAttr, "*") == 0)
 		node->uniqueAttr = NULL;
@@ -1208,8 +1262,7 @@ generate_fjoin(List *tlist)
 									  inner,
 									  results,
 									  alwaysDone);
-		tempList = lcons(fjoinNode, NIL);
-		tempList = nconc(tempList, fjoinList);
+		tempList = lcons(fjoinNode, fjoinList);
 		newTlist = lappend(newTlist, tempList);
 	}
 	return newTlist;
