@@ -1,13 +1,13 @@
 /*-------------------------------------------------------------------------
  *
- * prepqual.c--
+ * prepqual.c
  *	  Routines for preprocessing the parse tree qualification
  *
  * Copyright (c) 1994, Regents of the University of California
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/optimizer/prep/prepqual.c,v 1.11 1998/10/04 03:30:56 momjian Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/optimizer/prep/prepqual.c,v 1.15 1999/05/25 16:09:45 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -106,10 +106,18 @@ find_nots(Expr *qual)
 
 	if (is_opclause((Node *) qual))
 	{
-		return (make_clause(qual->opType, qual->oper,
-							lcons(find_nots((Expr *) get_leftop(qual)),
-							 lcons(find_nots((Expr *) get_rightop(qual)),
-								   NIL))));
+		Expr	   *left = (Expr *) get_leftop(qual);
+		Expr	   *right = (Expr *) get_rightop(qual);
+
+		if (right)
+			return make_clause(qual->opType, qual->oper,
+							   lcons(find_nots(left),
+									 lcons(find_nots(right),
+										   NIL)));
+		else
+			return make_clause(qual->opType, qual->oper,
+							   lcons(find_nots(left),
+									 NIL));
 	}
 	else if (and_clause((Node *) qual))
 	{
@@ -155,12 +163,18 @@ normalize(Expr *qual)
 
 	if (is_opclause((Node *) qual))
 	{
-		Expr	   *expr = (Expr *) qual;
+		Expr	   *left = (Expr *) get_leftop(qual);
+		Expr	   *right = (Expr *) get_rightop(qual);
 
-		return (make_clause(expr->opType, expr->oper,
-							lcons(normalize((Expr *) get_leftop(qual)),
-							 lcons(normalize((Expr *) get_rightop(qual)),
-								   NIL))));
+		if (right)
+			return make_clause(qual->opType, qual->oper,
+							   lcons(normalize(left),
+									 lcons(normalize(right),
+										   NIL)));
+		else
+			return make_clause(qual->opType, qual->oper,
+							   lcons(normalize(left),
+									 NIL));
 	}
 	else if (and_clause((Node *) qual))
 	{
@@ -217,10 +231,18 @@ qual_cleanup(Expr *qual)
 
 	if (is_opclause((Node *) qual))
 	{
-		return ((List *) make_clause(qual->opType, qual->oper,
-							lcons(qual_cleanup((Expr *) get_leftop(qual)),
-						   lcons(qual_cleanup((Expr *) get_rightop(qual)),
-								 NIL))));
+		Expr	   *left = (Expr *) get_leftop(qual);
+		Expr	   *right = (Expr *) get_rightop(qual);
+
+		if (right)
+			return (List *) make_clause(qual->opType, qual->oper,
+										lcons(qual_cleanup(left),
+											  lcons(qual_cleanup(right),
+													NIL)));
+		else
+			return (List *) make_clause(qual->opType, qual->oper,
+										lcons(qual_cleanup(left),
+											  NIL));
 	}
 	else if (and_clause((Node *) qual))
 	{
@@ -276,10 +298,18 @@ pull_args(Expr *qual)
 
 	if (is_opclause((Node *) qual))
 	{
-		return (make_clause(qual->opType, qual->oper,
-							lcons(pull_args((Expr *) get_leftop(qual)),
-							 lcons(pull_args((Expr *) get_rightop(qual)),
-								   NIL))));
+		Expr	   *left = (Expr *) get_leftop(qual);
+		Expr	   *right = (Expr *) get_rightop(qual);
+
+		if (right)
+			return make_clause(qual->opType, qual->oper,
+							   lcons(pull_args(left),
+									 lcons(pull_args(right),
+										   NIL)));
+		else
+			return make_clause(qual->opType, qual->oper,
+							   lcons(pull_args(left),
+									 NIL));
 	}
 	else if (and_clause((Node *) qual))
 	{
@@ -384,8 +414,7 @@ push_nots(Expr *qual)
 											   0, NULL);
 
 			op->op_fcache = (FunctionCache *) NULL;
-			return
-				(make_opclause(op, get_leftop(qual), get_rightop(qual)));
+			return make_opclause(op, get_leftop(qual), get_rightop(qual));
 		}
 		else
 			return make_notclause(qual);
@@ -458,10 +487,9 @@ or_normalize(List *orlist)
 
 	if (new_orlist)
 	{
-		return
-			(or_normalize(lcons(distribute_args(lfirst(new_orlist),
+		return (or_normalize(lcons(distribute_args(lfirst(new_orlist),
 										 ((Expr *) distributable)->args),
-								lnext(new_orlist))));
+								   lnext(new_orlist))));
 	}
 	else
 		return orlist;
@@ -513,10 +541,18 @@ remove_ands(Expr *qual)
 		return NIL;
 	if (is_opclause((Node *) qual))
 	{
-		return ((List *) make_clause(qual->opType, qual->oper,
-							lcons(remove_ands((Expr *) get_leftop(qual)),
-						   lcons(remove_ands((Expr *) get_rightop(qual)),
-								 NIL))));
+		Expr	   *left = (Expr *) get_leftop(qual);
+		Expr	   *right = (Expr *) get_rightop(qual);
+
+		if (right)
+			return (List *) make_clause(qual->opType, qual->oper,
+										lcons(remove_ands(left),
+											  lcons(remove_ands(right),
+													NIL)));
+		else
+			return (List *) make_clause(qual->opType, qual->oper,
+										lcons(remove_ands(left),
+											  NIL));
 	}
 	else if (and_clause((Node *) qual))
 	{
