@@ -1,13 +1,13 @@
 /*-------------------------------------------------------------------------
  *
- * pathnode.c--
+ * pathnode.c
  *	  Routines to manipulate pathlists and create path nodes
  *
  * Copyright (c) 1994, Regents of the University of California
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/optimizer/util/pathnode.c,v 1.13 1998/09/21 15:41:28 momjian Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/optimizer/util/pathnode.c,v 1.42 1999/05/25 22:41:49 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -20,7 +20,7 @@
 
 #include "optimizer/internal.h"
 #include "optimizer/pathnode.h"
-#include "optimizer/clauseinfo.h"
+#include "optimizer/restrictinfo.h"
 #include "optimizer/plancat.h"
 #include "optimizer/cost.h"
 #include "optimizer/keys.h"
@@ -29,7 +29,7 @@
 
 #include "parser/parsetree.h"	/* for getrelid() */
 
-static Path *better_path(Path *new_path, List *unique_paths, bool *noOther);
+static Path *better_path(Path *new_path, List *unique_paths, bool *is_new);
 
 
 /*****************************************************************************
@@ -37,7 +37,7 @@ static Path *better_path(Path *new_path, List *unique_paths, bool *noOther);
  *****************************************************************************/
 
 /*
- * path-is-cheaper--
+ * path_is_cheaper
  *	  Returns t iff 'path1' is cheaper than 'path2'.
  *
  */
@@ -51,18 +51,18 @@ path_is_cheaper(Path *path1, Path *path2)
 }
 
 /*
- * set_cheapest--
+ * set_cheapest
  *	  Finds the minimum cost path from among a relation's paths.
  *
- * 'parent-rel' is the parent relation
- * 'pathlist' is a list of path nodes corresponding to 'parent-rel'
+ * 'parent_rel' is the parent relation
+ * 'pathlist' is a list of path nodes corresponding to 'parent_rel'
  *
  * Returns and sets the relation entry field with the pathnode that
  * is minimum.
  *
  */
 Path *
-set_cheapest(RelOptInfo * parent_rel, List *pathlist)
+set_cheapest(RelOptInfo *parent_rel, List *pathlist)
 {
 	List	   *p;
 	Path	   *cheapest_so_far;
@@ -86,36 +86,39 @@ set_cheapest(RelOptInfo * parent_rel, List *pathlist)
 }
 
 /*
- * add_pathlist--
- *	  For each path in the list 'new-paths', add to the list 'unique-paths'
+ * add_pathlist
+ *	  For each path in the list 'new_paths', add to the list 'unique_paths'
  *	  only those paths that are unique (i.e., unique ordering and ordering
  *	  keys).  Should a conflict arise, the more expensive path is thrown out,
  *	  thereby pruning the plan space.  But we don't prune if xfunc
  *	  told us not to.
  *
- * 'parent-rel' is the relation entry to which these paths correspond.
+ * 'parent_rel' is the relation entry to which these paths correspond.
  *
  * Returns the list of unique pathnodes.
  *
  */
 List *
-add_pathlist(RelOptInfo * parent_rel, List *unique_paths, List *new_paths)
+add_pathlist(RelOptInfo *parent_rel, List *unique_paths, List *new_paths)
 {
-	List	   *x;
-	Path	   *new_path;
-	Path	   *old_path;
-	bool		noOther;
+	List	   *p1;
 
-	foreach(x, new_paths)
+	foreach(p1, new_paths)
 	{
-		new_path = (Path *) lfirst(x);
+		Path	   *new_path = (Path *) lfirst(p1);
+		Path	   *old_path;
+		bool		is_new;
+
+		/* Is this new path already in unique_paths? */
 		if (member(new_path, unique_paths))
 			continue;
-		old_path = better_path(new_path, unique_paths, &noOther);
 
-		if (noOther)
+		/* Find best matching path */
+		old_path = better_path(new_path, unique_paths, &is_new);
+
+		if (is_new)
 		{
-			/* Is a brand new path.  */
+			/* This is a brand new path.  */
 			new_path->parent = parent_rel;
 			unique_paths = lcons(new_path, unique_paths);
 		}
@@ -137,54 +140,129 @@ add_pathlist(RelOptInfo * parent_rel, List *unique_paths, List *new_paths)
 }
 
 /*
- * better_path--
- *	  Determines whether 'new-path' has the same ordering and keys as some
- *	  path in the list 'unique-paths'.	If there is a redundant path,
+ * better_path
+ *	  Determines whether 'new_path' has the same ordering and keys as some
+ *	  path in the list 'unique_paths'.	If there is a redundant path,
  *	  eliminate the more expensive path.
  *
  * Returns:
- *	  The old path - if 'new-path' matches some path in 'unique-paths' and is
+ *	  The old path - if 'new_path' matches some path in 'unique_paths' and is
  *				cheaper
- *	  nil - if 'new-path' matches but isn't cheaper
+ *	  nil - if 'new_path' matches but isn't cheaper
  *	  t - if there is no path in the list with the same ordering and keys
  *
  */
 static Path *
-better_path(Path *new_path, List *unique_paths, bool *noOther)
+better_path(Path *new_path, List *unique_paths, bool *is_new)
 {
-	Path	   *old_path = (Path *) NULL;
 	Path	   *path = (Path *) NULL;
 	List	   *temp = NIL;
-	Path	   *retval = NULL;
+	int			better_key;
+	int			better_sort;
+
+#ifdef OPTDUP_DEBUG
+	printf("better_path entry\n");
+	printf("new\n");
+	pprint(new_path);
+	printf("unique_paths\n");
+	pprint(unique_paths);
+#endif
 
-	/*
-	 * XXX - added the following two lines which weren't int the lisp
-	 * planner, but otherwise, doesn't seem to work for the case where
-	 * new_path is 'nil
-	 */
 	foreach(temp, unique_paths)
 	{
 		path = (Path *) lfirst(temp);
 
-		if ((equal_path_path_ordering(&new_path->p_ordering,
-									  &path->p_ordering) &&
-			 samekeys(new_path->keys, path->keys)))
+#ifdef OPTDUP_DEBUG
+		if (!pathkeys_match(new_path->pathkeys, path->pathkeys, &better_key) ||
+			better_key != 0)
 		{
-			old_path = path;
-			break;
+			printf("betterkey = %d\n", better_key);
+			printf("newpath\n");
+			pprint(new_path->pathkeys);
+			printf("oldpath\n");
+			pprint(path->pathkeys);
+			if (path->pathkeys && new_path->pathkeys &&
+				length(lfirst(path->pathkeys)) >= 2		/* &&
+														 * length(lfirst(path->pa
+														 * thkeys)) <
+														 * length(lfirst(new_path
+					->pathkeys)) */ )
+				sleep(0);		/* set breakpoint here */
+		}
+		if (!pathorder_match(new_path->pathorder, path->pathorder,
+							 &better_sort) ||
+			better_sort != 0)
+		{
+			printf("neword\n");
+			pprint(new_path->pathorder);
+			printf("oldord\n");
+			pprint(path->pathorder);
+		}
+#endif
+
+		if (pathkeys_match(new_path->pathkeys, path->pathkeys,
+						   &better_key) &&
+			pathorder_match(new_path->pathorder, path->pathorder,
+							&better_sort))
+		{
+
+			/*
+			 * Replace pathkeys that match exactly, {{1,2}}, {{1,2}}
+			 * Replace pathkeys {{1,2}} with {{1,2,3}}} if the latter is
+			 * not more expensive and replace unordered path with ordered
+			 * path if it is not more expensive.  Favor sorted keys over
+			 * unsorted keys in the same way.
+			 */
+			/* same keys, and new is cheaper, use it */
+			if ((better_key == 0 && better_sort == 0 &&
+				 new_path->path_cost < path->path_cost) ||
+
+			/* new is better, and cheaper, use it */
+				(((better_key == 1 && better_sort != 2) ||
+				  (better_key != 2 && better_sort == 1)) &&
+				 new_path->path_cost <= path->path_cost))
+			{
+#ifdef OPTDUP_DEBUG
+				printf("replace with new %p old %p better key %d better sort %d\n", &new_path, &path, better_key, better_sort);
+				printf("new\n");
+				pprint(new_path);
+				printf("old\n");
+				pprint(path);
+#endif
+				*is_new = false;
+				return path;
+			}
+
+			/* same keys, new is more expensive, stop */
+			if ((better_key == 0 && better_sort == 0 &&
+				 new_path->path_cost >= path->path_cost) ||
+
+			/* old is better, and less expensive, stop */
+				(((better_key == 2 && better_sort != 1) ||
+				  (better_key != 1 && better_sort == 2)) &&
+				 new_path->path_cost >= path->path_cost))
+			{
+#ifdef OPTDUP_DEBUG
+				printf("skip new %p old %p better key %d better sort %d\n", &new_path, &path, better_key, better_sort);
+				printf("new\n");
+				pprint(new_path);
+				printf("old\n");
+				pprint(path);
+#endif
+				*is_new = false;
+				return NULL;
+			}
 		}
 	}
 
-	if (old_path == NULL)
-		*noOther = true;
-	else
-	{
-		*noOther = false;
-		if (path_is_cheaper(new_path, old_path))
-			retval = old_path;
-	}
+#ifdef OPTDUP_DEBUG
+	printf("add new %p old %p better key %d better sort %d\n", &new_path, &path, better_key, better_sort);
+	printf("new\n");
+	pprint(new_path);
+#endif
 
-	return retval;
+	*is_new = true;
+	return NULL;
 }
 
 
@@ -194,13 +272,13 @@ better_path(Path *new_path, List *unique_paths, bool *noOther)
  *****************************************************************************/
 
 /*
- * create_seqscan_path--
+ * create_seqscan_path
  *	  Creates a path corresponding to a sequential scan, returning the
  *	  pathnode.
  *
  */
 Path *
-create_seqscan_path(RelOptInfo * rel)
+create_seqscan_path(RelOptInfo *rel)
 {
 	int			relid = 0;
 
@@ -209,16 +287,16 @@ create_seqscan_path(RelOptInfo * rel)
 	pathnode->pathtype = T_SeqScan;
 	pathnode->parent = rel;
 	pathnode->path_cost = 0.0;
-	pathnode->p_ordering.ordtype = SORTOP_ORDER;
-	pathnode->p_ordering.ord.sortop = NULL;
-	pathnode->keys = NIL;
+	pathnode->pathorder = makeNode(PathOrder);
+	pathnode->pathorder->ordtype = SORTOP_ORDER;
+	pathnode->pathorder->ord.sortop = NULL;
+	pathnode->pathkeys = NIL;
 
 	/*
-	 * copy clauseinfo list into path for expensive function processing --
+	 * copy restrictinfo list into path for expensive function processing
 	 * JMH, 7/7/92
 	 */
-	pathnode->locclauseinfo =
-		(List *) copyObject((Node *) rel->clauseinfo);
+	pathnode->loc_restrictinfo = (List *) copyObject((Node *) rel->restrictinfo);
 
 	if (rel->relids != NULL)
 		relid = lfirsti(rel->relids);
@@ -226,24 +304,21 @@ create_seqscan_path(RelOptInfo * rel)
 	pathnode->path_cost = cost_seqscan(relid,
 									   rel->pages, rel->tuples);
 	/* add in expensive functions cost!  -- JMH, 7/7/92 */
-#if 0
+#ifdef NOT_USED
 	if (XfuncMode != XFUNC_OFF)
-	{
-		pathnode->path_cost +=
-			xfunc_get_path_cost(pathnode);
-	}
+		pathnode->path_cost += xfunc_get_path_cost(pathnode);
 #endif
 	return pathnode;
 }
 
 /*
- * create_index_path--
+ * create_index_path
  *	  Creates a single path node for an index scan.
  *
  * 'rel' is the parent rel
  * 'index' is the pathnode for the index on 'rel'
- * 'restriction-clauses' is a list of restriction clause nodes.
- * 'is-join-scan' is a flag indicating whether or not the index is being
+ * 'restriction_clauses' is a list of restriction clause nodes.
+ * 'is_join_scan' is a flag indicating whether or not the index is being
  *		considered because of its sort order.
  *
  * Returns the new path node.
@@ -251,8 +326,8 @@ create_seqscan_path(RelOptInfo * rel)
  */
 IndexPath  *
 create_index_path(Query *root,
-				  RelOptInfo * rel,
-				  RelOptInfo * index,
+				  RelOptInfo *rel,
+				  RelOptInfo *index,
 				  List *restriction_clauses,
 				  bool is_join_scan)
 {
@@ -260,29 +335,29 @@ create_index_path(Query *root,
 
 	pathnode->path.pathtype = T_IndexScan;
 	pathnode->path.parent = rel;
-	pathnode->path.p_ordering.ordtype = SORTOP_ORDER;
-	pathnode->path.p_ordering.ord.sortop = index->ordering;
+	pathnode->path.pathorder = makeNode(PathOrder);
+	pathnode->path.pathorder->ordtype = SORTOP_ORDER;
+	pathnode->path.pathorder->ord.sortop = index->ordering;
 
 	pathnode->indexid = index->relids;
 	pathnode->indexkeys = index->indexkeys;
 	pathnode->indexqual = NIL;
 
 	/*
-	 * copy clauseinfo list into path for expensive function processing --
+	 * copy restrictinfo list into path for expensive function processing
 	 * JMH, 7/7/92
 	 */
-	pathnode->path.locclauseinfo =
-		set_difference((List *) copyObject((Node *) rel->clauseinfo),
-					   (List *) restriction_clauses);
+	pathnode->path.loc_restrictinfo = set_difference((List *) copyObject((Node *) rel->restrictinfo),
+										   (List *) restriction_clauses);
 
 	/*
 	 * The index must have an ordering for the path to have (ordering)
 	 * keys, and vice versa.
 	 */
-	if (pathnode->path.p_ordering.ord.sortop)
+	if (pathnode->path.pathorder->ord.sortop)
 	{
-		pathnode->path.keys = collect_index_pathkeys(index->indexkeys,
-													 rel->targetlist);
+		pathnode->path.pathkeys = collect_index_pathkeys(index->indexkeys,
+														 rel->targetlist);
 
 		/*
 		 * Check that the keys haven't 'disappeared', since they may no
@@ -290,11 +365,11 @@ create_index_path(Query *root,
 		 * relevant to the scan are not applied to the scan path node, so
 		 * if no index keys were found, we can't order the path).
 		 */
-		if (pathnode->path.keys == NULL)
-			pathnode->path.p_ordering.ord.sortop = NULL;
+		if (pathnode->path.pathkeys == NULL)
+			pathnode->path.pathorder->ord.sortop = NULL;
 	}
 	else
-		pathnode->path.keys = NULL;
+		pathnode->path.pathkeys = NULL;
 
 	if (is_join_scan || restriction_clauses == NULL)
 	{
@@ -306,22 +381,20 @@ create_index_path(Query *root,
 		 */
 /* is the statement above really true?	what about IndexScan as the
    inner of a join? */
-		pathnode->path.path_cost =
-			cost_index(lfirsti(index->relids),
-					   index->pages,
-					   1.0,
-					   rel->pages,
-					   rel->tuples,
-					   index->pages,
-					   index->tuples,
-					   false);
+		pathnode->path.path_cost = cost_index(lfirsti(index->relids),
+											  index->pages,
+											  1.0,
+											  rel->pages,
+											  rel->tuples,
+											  index->pages,
+											  index->tuples,
+											  false);
+#ifdef NOT_USED
 		/* add in expensive functions cost!  -- JMH, 7/7/92 */
-#if 0
 		if (XfuncMode != XFUNC_OFF)
 		{
-			pathnode->path_cost =
-				(pathnode->path_cost +
-				 xfunc_get_path_cost((Path *) pathnode));
+			pathnode->path_cost = (pathnode->path_cost +
+								 xfunc_get_path_cost((Path *) pathnode));
 		}
 #endif
 	}
@@ -355,25 +428,22 @@ create_index_path(Query *root,
 						  &npages,
 						  &selec);
 		/* each clause gets an equal selectivity */
-		clausesel =	pow(selec, 1.0 / (double) length(restriction_clauses));
+		clausesel = pow(selec, 1.0 / (double) length(restriction_clauses));
 
 		pathnode->indexqual = restriction_clauses;
 		pathnode->path.path_cost = cost_index(lfirsti(index->relids),
-											   (int) npages,
-											   selec,
-											   rel->pages,
-											   rel->tuples,
-											   index->pages,
-											   index->tuples,
-											   false);
+											  (int) npages,
+											  selec,
+											  rel->pages,
+											  rel->tuples,
+											  index->pages,
+											  index->tuples,
+											  false);
 
-#if 0
+#ifdef NOT_USED
 		/* add in expensive functions cost!  -- JMH, 7/7/92 */
 		if (XfuncMode != XFUNC_OFF)
-		{
-			pathnode->path_cost +=
-				xfunc_get_path_cost((Path *) pathnode);
-		}
+			pathnode->path_cost += xfunc_get_path_cost((Path *) pathnode);
 #endif
 
 		/*
@@ -389,7 +459,7 @@ create_index_path(Query *root,
 }
 
 /*
- * create_nestloop_path--
+ * create_nestloop_path
  *	  Creates a pathnode corresponding to a nestloop join between two
  *	  relations.
  *
@@ -397,61 +467,54 @@ create_index_path(Query *root,
  * 'outer_rel' is the outer join relation
  * 'outer_path' is the outer join path.
  * 'inner_path' is the inner join path.
- * 'keys' are the keys of the path
+ * 'pathkeys' are the keys of the path
  *
  * Returns the resulting path node.
  *
  */
-JoinPath   *
-create_nestloop_path(RelOptInfo * joinrel,
-					 RelOptInfo * outer_rel,
+NestPath   *
+create_nestloop_path(RelOptInfo *joinrel,
+					 RelOptInfo *outer_rel,
 					 Path *outer_path,
 					 Path *inner_path,
-					 List *keys)
+					 List *pathkeys)
 {
-	JoinPath   *pathnode = makeNode(JoinPath);
+	NestPath   *pathnode = makeNode(NestPath);
 
 	pathnode->path.pathtype = T_NestLoop;
 	pathnode->path.parent = joinrel;
 	pathnode->outerjoinpath = outer_path;
 	pathnode->innerjoinpath = inner_path;
-	pathnode->pathclauseinfo = joinrel->clauseinfo;
-	pathnode->path.keys = keys;
+	pathnode->pathinfo = joinrel->restrictinfo;
+	pathnode->path.pathkeys = pathkeys;
 	pathnode->path.joinid = NIL;
 	pathnode->path.outerjoincost = (Cost) 0.0;
-	pathnode->path.locclauseinfo = NIL;
+	pathnode->path.loc_restrictinfo = NIL;
+	pathnode->path.pathorder = makeNode(PathOrder);
 
-	if (keys)
+	if (pathkeys)
 	{
-		pathnode->path.p_ordering.ordtype =
-			outer_path->p_ordering.ordtype;
-		if (outer_path->p_ordering.ordtype == SORTOP_ORDER)
-		{
-			pathnode->path.p_ordering.ord.sortop =
-				outer_path->p_ordering.ord.sortop;
-		}
+		pathnode->path.pathorder->ordtype = outer_path->pathorder->ordtype;
+		if (outer_path->pathorder->ordtype == SORTOP_ORDER)
+			pathnode->path.pathorder->ord.sortop = outer_path->pathorder->ord.sortop;
 		else
-		{
-			pathnode->path.p_ordering.ord.merge =
-				outer_path->p_ordering.ord.merge;
-		}
+			pathnode->path.pathorder->ord.merge = outer_path->pathorder->ord.merge;
 	}
 	else
 	{
-		pathnode->path.p_ordering.ordtype = SORTOP_ORDER;
-		pathnode->path.p_ordering.ord.sortop = NULL;
+		pathnode->path.pathorder->ordtype = SORTOP_ORDER;
+		pathnode->path.pathorder->ord.sortop = NULL;
 	}
 
-	pathnode->path.path_cost =
-		cost_nestloop(outer_path->path_cost,
-					  inner_path->path_cost,
-					  outer_rel->size,
-					  inner_path->parent->size,
-					  page_size(outer_rel->size,
-								outer_rel->width),
-					  IsA(inner_path, IndexPath));
+	pathnode->path.path_cost = cost_nestloop(outer_path->path_cost,
+											 inner_path->path_cost,
+											 outer_rel->size,
+											 inner_path->parent->size,
+											 page_size(outer_rel->size,
+													   outer_rel->width),
+											 IsA(inner_path, IndexPath));
 	/* add in expensive function costs -- JMH 7/7/92 */
-#if 0
+#ifdef NOT_USED
 	if (XfuncMode != XFUNC_OFF)
 		pathnode->path_cost += xfunc_get_path_cost((Path *) pathnode);
 #endif
@@ -459,7 +522,7 @@ create_nestloop_path(RelOptInfo * joinrel,
 }
 
 /*
- * create_mergejoin_path--
+ * create_mergejoin_path
  *	  Creates a pathnode corresponding to a mergejoin join between
  *	  two relations
  *
@@ -470,7 +533,7 @@ create_nestloop_path(RelOptInfo * joinrel,
  * 'innerwidth' is the number of bytes per tuple in the inner relation
  * 'outer_path' is the outer path
  * 'inner_path' is the inner path
- * 'keys' are the new keys of the join relation
+ * 'pathkeys' are the new keys of the join relation
  * 'order' is the sort order required for the merge
  * 'mergeclauses' are the applicable join/restriction clauses
  * 'outersortkeys' are the sort varkeys for the outer relation
@@ -478,14 +541,14 @@ create_nestloop_path(RelOptInfo * joinrel,
  *
  */
 MergePath  *
-create_mergejoin_path(RelOptInfo * joinrel,
+create_mergejoin_path(RelOptInfo *joinrel,
 					  int outersize,
 					  int innersize,
 					  int outerwidth,
 					  int innerwidth,
 					  Path *outer_path,
 					  Path *inner_path,
-					  List *keys,
+					  List *pathkeys,
 					  MergeOrder *order,
 					  List *mergeclauses,
 					  List *outersortkeys,
@@ -497,30 +560,27 @@ create_mergejoin_path(RelOptInfo * joinrel,
 	pathnode->jpath.path.parent = joinrel;
 	pathnode->jpath.outerjoinpath = outer_path;
 	pathnode->jpath.innerjoinpath = inner_path;
-	pathnode->jpath.pathclauseinfo = joinrel->clauseinfo;
-	pathnode->jpath.path.keys = keys;
-	pathnode->jpath.path.p_ordering.ordtype = MERGE_ORDER;
-	pathnode->jpath.path.p_ordering.ord.merge = order;
+	pathnode->jpath.pathinfo = joinrel->restrictinfo;
+	pathnode->jpath.path.pathkeys = pathkeys;
+	pathnode->jpath.path.pathorder = makeNode(PathOrder);
+	pathnode->jpath.path.pathorder->ordtype = MERGE_ORDER;
+	pathnode->jpath.path.pathorder->ord.merge = order;
 	pathnode->path_mergeclauses = mergeclauses;
-	pathnode->jpath.path.locclauseinfo = NIL;
+	pathnode->jpath.path.loc_restrictinfo = NIL;
 	pathnode->outersortkeys = outersortkeys;
 	pathnode->innersortkeys = innersortkeys;
-	pathnode->jpath.path.path_cost =
-		cost_mergejoin(outer_path->path_cost,
-					   inner_path->path_cost,
-					   outersortkeys,
-					   innersortkeys,
-					   outersize,
-					   innersize,
-					   outerwidth,
-					   innerwidth);
+	pathnode->jpath.path.path_cost = cost_mergejoin(outer_path->path_cost,
+													inner_path->path_cost,
+													outersortkeys,
+													innersortkeys,
+													outersize,
+													innersize,
+													outerwidth,
+													innerwidth);
 	/* add in expensive function costs -- JMH 7/7/92 */
-#if 0
+#ifdef NOT_USED
 	if (XfuncMode != XFUNC_OFF)
-	{
-		pathnode->path_cost +=
-			xfunc_get_path_cost((Path *) pathnode);
-	}
+		pathnode->path_cost += xfunc_get_path_cost((Path *) pathnode);
 #endif
 	return pathnode;
 }
@@ -536,7 +596,7 @@ create_mergejoin_path(RelOptInfo * joinrel,
  * 'innerwidth' is the number of bytes per tuple in the inner relation
  * 'outer_path' is the outer path
  * 'inner_path' is the inner path
- * 'keys' are the new keys of the join relation
+ * 'pathkeys' are the new keys of the join relation
  * 'operator' is the hashjoin operator
  * 'hashclauses' are the applicable join/restriction clauses
  * 'outerkeys' are the sort varkeys for the outer relation
@@ -544,14 +604,14 @@ create_mergejoin_path(RelOptInfo * joinrel,
  *
  */
 HashPath   *
-create_hashjoin_path(RelOptInfo * joinrel,
+create_hashjoin_path(RelOptInfo *joinrel,
 					 int outersize,
 					 int innersize,
 					 int outerwidth,
 					 int innerwidth,
 					 Path *outer_path,
 					 Path *inner_path,
-					 List *keys,
+					 List *pathkeys,
 					 Oid operator,
 					 List *hashclauses,
 					 List *outerkeys,
@@ -563,31 +623,28 @@ create_hashjoin_path(RelOptInfo * joinrel,
 	pathnode->jpath.path.parent = joinrel;
 	pathnode->jpath.outerjoinpath = outer_path;
 	pathnode->jpath.innerjoinpath = inner_path;
-	pathnode->jpath.pathclauseinfo = joinrel->clauseinfo;
-	pathnode->jpath.path.locclauseinfo = NIL;
-	pathnode->jpath.path.keys = keys;
-	pathnode->jpath.path.p_ordering.ordtype = SORTOP_ORDER;
-	pathnode->jpath.path.p_ordering.ord.sortop = NULL;
+	pathnode->jpath.pathinfo = joinrel->restrictinfo;
+	pathnode->jpath.path.loc_restrictinfo = NIL;
+	pathnode->jpath.path.pathkeys = pathkeys;
+	pathnode->jpath.path.pathorder = makeNode(PathOrder);
+	pathnode->jpath.path.pathorder->ordtype = SORTOP_ORDER;
+	pathnode->jpath.path.pathorder->ord.sortop = NULL;
 	pathnode->jpath.path.outerjoincost = (Cost) 0.0;
-	pathnode->jpath.path.joinid = (Relid) NULL;
+	pathnode->jpath.path.joinid = (Relids) NULL;
 	/* pathnode->hashjoinoperator = operator;  */
 	pathnode->path_hashclauses = hashclauses;
 	pathnode->outerhashkeys = outerkeys;
 	pathnode->innerhashkeys = innerkeys;
-	pathnode->jpath.path.path_cost =
-		cost_hashjoin(outer_path->path_cost,
-					  inner_path->path_cost,
-					  outerkeys,
-					  innerkeys,
-					  outersize, innersize,
-					  outerwidth, innerwidth);
+	pathnode->jpath.path.path_cost = cost_hashjoin(outer_path->path_cost,
+												   inner_path->path_cost,
+												   outerkeys,
+												   innerkeys,
+												   outersize, innersize,
+												 outerwidth, innerwidth);
 	/* add in expensive function costs -- JMH 7/7/92 */
-#if 0
+#ifdef NOT_USED
 	if (XfuncMode != XFUNC_OFF)
-	{
-		pathnode->path_cost +=
-			xfunc_get_path_cost((Path *) pathnode);
-	}
+		pathnode->path_cost += xfunc_get_path_cost((Path *) pathnode);
 #endif
 	return pathnode;
 }
