@@ -1,13 +1,13 @@
 /*-------------------------------------------------------------------------
  *
- * tlist.c--
+ * tlist.c
  *	  Target list manipulation routines
  *
  * Copyright (c) 1994, Regents of the University of California
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/optimizer/util/tlist.c,v 1.20 1998/09/22 21:48:27 momjian Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/optimizer/util/tlist.c,v 1.33 1999/05/25 22:41:51 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -34,13 +34,13 @@ static Node *flatten_tlistentry(Node *tlistentry, List *flat_tlist);
  *****************************************************************************/
 
 /*
- * tlistentry-member--
+ * tlistentry_member
  *
  * RETURNS:  the leftmost member of sequence "targetlist" that satisfies
  *			 the predicate "var_equal"
  * MODIFIES: nothing
  * REQUIRES: test = function which can operate on a lispval union
- *			 var = valid var-node
+ *			 var = valid var_node
  *			 targetlist = valid sequence
  */
 TargetEntry *
@@ -48,7 +48,7 @@ tlistentry_member(Var *var, List *targetlist)
 {
 	if (var)
 	{
-		List	   *temp = NIL;
+		List	   *temp;
 
 		foreach(temp, targetlist)
 		{
@@ -61,7 +61,7 @@ tlistentry_member(Var *var, List *targetlist)
 }
 
 /*
- * matching_tlvar--
+ * matching_tlist_var
  *
  * RETURNS:  var node in a target list which is var_equal to 'var',
  *			 if one exists.
@@ -69,7 +69,7 @@ tlistentry_member(Var *var, List *targetlist)
  *
  */
 Expr *
-matching_tlvar(Var *var, List *targetlist)
+matching_tlist_var(Var *var, List *targetlist)
 {
 	TargetEntry *tlentry;
 
@@ -81,7 +81,7 @@ matching_tlvar(Var *var, List *targetlist)
 }
 
 /*
- * add_tl_element--
+ * add_var_to_tlist
  *	  Creates a targetlist entry corresponding to the supplied var node
  *
  * 'var' and adds the new targetlist entry to the targetlist field of
@@ -90,14 +90,14 @@ matching_tlvar(Var *var, List *targetlist)
  * RETURNS: nothing
  * MODIFIES: vartype and varid fields of leftmost varnode that matches
  *			 argument "var" (sometimes).
- * CREATES:  new var-node iff no matching var-node exists in targetlist
+ * CREATES:  new var_node iff no matching var_node exists in targetlist
  */
 void
-add_tl_element(RelOptInfo * rel, Var *var)
+add_var_to_tlist(RelOptInfo *rel, Var *var)
 {
-	Expr	   *oldvar = (Expr *) NULL;
+	Expr	   *oldvar;
 
-	oldvar = matching_tlvar(var, rel->targetlist);
+	oldvar = matching_tlist_var(var, rel->targetlist);
 
 	/*
 	 * If 'var' is not already in 'rel's target list, add a new node.
@@ -113,39 +113,37 @@ add_tl_element(RelOptInfo * rel, Var *var)
 									 var->varno,
 									 var->varoattno);
 
-		rel->targetlist =
-			lappend(tlist,
-					create_tl_element(newvar,
-									  length(tlist) + 1));
+		rel->targetlist = lappend(tlist,
+								  create_tl_element(newvar,
+													length(tlist) + 1));
 
 	}
 }
 
 /*
- * create_tl_element--
+ * create_tl_element
  *	  Creates a target list entry node and its associated (resdom var) pair
  *	  with its resdom number equal to 'resdomno' and the joinlist field set
  *	  to 'joinlist'.
  *
- * RETURNS:  newly created tlist-entry
+ * RETURNS:  newly created tlist_entry
  * CREATES:  new targetlist entry (always).
  */
 TargetEntry *
 create_tl_element(Var *var, int resdomno)
 {
-
 	return makeTargetEntry(makeResdom(resdomno,
 									  var->vartype,
 									  var->vartypmod,
 									  NULL,
 									  (Index) 0,
 									  (Oid) 0,
-									  0),
+									  false),
 						   (Node *) var);
 }
 
 /*
- * get-actual-tlist--
+ * get_actual_tlist
  *	  Returns the targetlist elements from a relation tlist.
  *
  */
@@ -156,7 +154,7 @@ get_actual_tlist(List *tlist)
 	/*
 	 * this function is not making sense. - ay 10/94
 	 */
-#if 0
+#ifdef NOT_USED
 	List	   *element = NIL;
 	List	   *result = NIL;
 
@@ -185,40 +183,30 @@ get_actual_tlist(List *tlist)
  *****************************************************************************/
 
 /*
- * tlist-member--
+ * tlist_member
  *	  Determines whether a var node is already contained within a
  *	  target list.
  *
  * 'var' is the var node
  * 'tlist' is the target list
- * 'dots' is t if we must match dotfields to determine uniqueness
  *
- * Returns the resdom entry of the matching var node.
+ * Returns the resdom entry of the matching var node, or NULL if no match.
  *
  */
 Resdom *
 tlist_member(Var *var, List *tlist)
 {
-	List	   *i = NIL;
-	TargetEntry *temp_tle = (TargetEntry *) NULL;
-	TargetEntry *tl_elt = (TargetEntry *) NULL;
-
 	if (var)
 	{
+		List	   *i;
+
 		foreach(i, tlist)
 		{
-			temp_tle = (TargetEntry *) lfirst(i);
-			if (var_equal(var, get_expr(temp_tle)))
-			{
-				tl_elt = temp_tle;
-				break;
-			}
-		}
+			TargetEntry *tle = (TargetEntry *) lfirst(i);
 
-		if (tl_elt != NULL)
-			return tl_elt->resdom;
-		else
-			return (Resdom *) NULL;
+			if (var_equal(var, get_expr(tle)))
+				return tle->resdom;
+		}
 	}
 	return (Resdom *) NULL;
 }
@@ -229,14 +217,13 @@ tlist_member(Var *var, List *tlist)
 Resdom *
 tlist_resdom(List *tlist, Resdom *resnode)
 {
-	Resdom	   *resdom = (Resdom *) NULL;
-	List	   *i = NIL;
-	TargetEntry *temp_tle = (TargetEntry *) NULL;
+	List	   *i;
 
 	foreach(i, tlist)
 	{
-		temp_tle = (TargetEntry *) lfirst(i);
-		resdom = temp_tle->resdom;
+		TargetEntry *tle = (TargetEntry *) lfirst(i);
+		Resdom	   *resdom = tle->resdom;
+
 		/* Since resnos are supposed to be unique */
 		if (resnode->resno == resdom->resno)
 			return resdom;
@@ -246,7 +233,7 @@ tlist_resdom(List *tlist, Resdom *resnode)
 
 
 /*
- * match_varid--
+ * match_varid
  *	  Searches a target list for an entry with some desired varid.
  *
  * 'varid' is the desired id
@@ -289,7 +276,6 @@ match_varid(Var *test_var, List *tlist)
 		if (tlvar->varnoold == test_var->varnoold &&
 			tlvar->varoattno == test_var->varoattno)
 		{
-
 			if (tlvar->vartype == type_var)
 				return entry;
 		}
@@ -300,7 +286,7 @@ match_varid(Var *test_var, List *tlist)
 
 
 /*
- * new-unsorted-tlist--
+ * new_unsorted_tlist
  *	  Creates a copy of a target list by creating new resdom nodes
  *	  without sort information.
  *
@@ -313,7 +299,7 @@ List *
 new_unsorted_tlist(List *targetlist)
 {
 	List	   *new_targetlist = (List *) copyObject((Node *) targetlist);
-	List	   *x = NIL;
+	List	   *x;
 
 	foreach(x, new_targetlist)
 	{
@@ -326,7 +312,7 @@ new_unsorted_tlist(List *targetlist)
 }
 
 /*
- * copy-vars--
+ * copy_vars
  *	  Replaces the var nodes in the first target list with those from
  *	  the second target list.  The two target lists are assumed to be
  *	  identical except their actual resdoms and vars are different.
@@ -356,7 +342,7 @@ copy_vars(List *target, List *source)
 }
 
 /*
- * flatten-tlist--
+ * flatten_tlist
  *	  Create a target list that only contains unique variables.
  *
  *
@@ -375,13 +361,10 @@ flatten_tlist(List *tlist)
 
 	foreach(temp, tlist)
 	{
-		TargetEntry *temp_entry = NULL;
-		List	   *vars;
+		TargetEntry *temp_entry = (TargetEntry *) lfirst(temp);
 
-		temp_entry = lfirst(temp);
-		vars = pull_var_clause((Node *) get_expr(temp_entry));
-		if (vars != NULL)
-			tlist_vars = nconc(tlist_vars, vars);
+		tlist_vars = nconc(tlist_vars,
+						 pull_var_clause((Node *) get_expr(temp_entry)));
 	}
 
 	foreach(temp, tlist_vars)
@@ -398,7 +381,7 @@ flatten_tlist(List *tlist)
 						   NULL,
 						   (Index) 0,
 						   (Oid) 0,
-						   0);
+						   false);
 			last_resdomno++;
 			new_tlist = lappend(new_tlist, makeTargetEntry(r, (Node *) var));
 		}
@@ -408,13 +391,13 @@ flatten_tlist(List *tlist)
 }
 
 /*
- * flatten-tlist-vars--
+ * flatten_tlist_vars
  *	  Redoes the target list of a query with no nested attributes by
  *	  replacing vars within computational expressions with vars from
  *	  the 'flattened' target list of the query.
  *
- * 'full-tlist' is the actual target list
- * 'flat-tlist' is the flattened (var-only) target list
+ * 'full_tlist' is the actual target list
+ * 'flat_tlist' is the flattened (var-only) target list
  *
  * Returns the modified actual target list.
  *
@@ -422,28 +405,28 @@ flatten_tlist(List *tlist)
 List *
 flatten_tlist_vars(List *full_tlist, List *flat_tlist)
 {
-	List	   *x = NIL;
 	List	   *result = NIL;
+	List	   *x;
 
 	foreach(x, full_tlist)
 	{
 		TargetEntry *tle = lfirst(x);
 
 		result = lappend(result, makeTargetEntry(tle->resdom,
-							   		flatten_tlistentry((Node *) get_expr(tle),
-												  		flat_tlist)));
+							   flatten_tlistentry((Node *) get_expr(tle),
+												  flat_tlist)));
 	}
 
 	return result;
 }
 
 /*
- * flatten-tlistentry--
+ * flatten_tlistentry
  *	  Replaces vars within a target list entry with vars from a flattened
  *	  target list.
  *
  * 'tlistentry' is the target list entry to be modified
- * 'flat-tlist' is the flattened target list
+ * 'flat_tlist' is the flattened target list
  *
  * Returns the (modified) target_list entry from the target list.
  *
@@ -451,15 +434,15 @@ flatten_tlist_vars(List *full_tlist, List *flat_tlist)
 static Node *
 flatten_tlistentry(Node *tlistentry, List *flat_tlist)
 {
+	List	   *temp;
+
 	if (tlistentry == NULL)
-	{
 		return NULL;
-	}
 	else if (IsA(tlistentry, Var))
-	{
-		return ((Node *) get_expr(match_varid((Var *) tlistentry,
-										   flat_tlist)));
-	}
+		return (Node *) get_expr(match_varid((Var *) tlistentry,
+											 flat_tlist));
+	else if (single_node(tlistentry))
+		return tlistentry;
 	else if (IsA(tlistentry, Iter))
 	{
 		((Iter *) tlistentry)->iterexpr =
@@ -467,62 +450,66 @@ flatten_tlistentry(Node *tlistentry, List *flat_tlist)
 							   flat_tlist);
 		return tlistentry;
 	}
-	else if (single_node(tlistentry))
+	else if (is_subplan(tlistentry))
 	{
+		/* do we need to support this case? */
+		elog(ERROR, "flatten_tlistentry: subplan case not implemented");
 		return tlistentry;
 	}
-	else if (is_funcclause(tlistentry))
+	else if (IsA(tlistentry, Expr))
 	{
-		Expr	   *expr = (Expr *) tlistentry;
-		List	   *temp_result = NIL;
-		List	   *elt = NIL;
-
-		foreach(elt, expr->args)
-			temp_result = lappend(temp_result,
-							flatten_tlistentry(lfirst(elt), flat_tlist));
-
-		return ((Node *) make_funcclause((Func *) expr->oper, temp_result));
 
+		/*
+		 * Recursively scan the arguments of an expression. NOTE: this
+		 * must come after is_subplan() case since subplan is a kind of
+		 * Expr node.
+		 */
+		foreach(temp, ((Expr *) tlistentry)->args)
+			lfirst(temp) = flatten_tlistentry(lfirst(temp), flat_tlist);
+		return tlistentry;
 	}
-	else if (IsA(tlistentry, Aggreg))
+	else if (IsA(tlistentry, Aggref))
 	{
+
+		/*
+		 * XXX shouldn't this be recursing into the agg's target? Seems to
+		 * work though, so will leave it alone ... tgl 5/99
+		 */
 		return tlistentry;
 	}
 	else if (IsA(tlistentry, ArrayRef))
 	{
 		ArrayRef   *aref = (ArrayRef *) tlistentry;
-		List	   *temp = NIL;
-		List	   *elt = NIL;
-
-		foreach(elt, aref->refupperindexpr)
-			temp = lappend(temp, flatten_tlistentry(lfirst(elt), flat_tlist));
-		aref->refupperindexpr = temp;
-
-		temp = NIL;
-		foreach(elt, aref->reflowerindexpr)
-			temp = lappend(temp, flatten_tlistentry(lfirst(elt), flat_tlist));
-		aref->reflowerindexpr = temp;
 
+		foreach(temp, aref->refupperindexpr)
+			lfirst(temp) = flatten_tlistentry(lfirst(temp), flat_tlist);
+		foreach(temp, aref->reflowerindexpr)
+			lfirst(temp) = flatten_tlistentry(lfirst(temp), flat_tlist);
 		aref->refexpr = flatten_tlistentry(aref->refexpr, flat_tlist);
-
 		aref->refassgnexpr = flatten_tlistentry(aref->refassgnexpr, flat_tlist);
 
 		return tlistentry;
 	}
+	else if (case_clause(tlistentry))
+	{
+		CaseExpr   *cexpr = (CaseExpr *) tlistentry;
+
+		foreach(temp, cexpr->args)
+		{
+			CaseWhen   *cwhen = (CaseWhen *) lfirst(temp);
+
+			cwhen->expr = flatten_tlistentry(cwhen->expr, flat_tlist);
+			cwhen->result = flatten_tlistentry(cwhen->result, flat_tlist);
+		}
+		cexpr->defresult = flatten_tlistentry(cexpr->defresult, flat_tlist);
+
+		return tlistentry;
+	}
 	else
 	{
-		Expr	   *expr = (Expr *) tlistentry;
-
-		Var 	   *left = (Var *) flatten_tlistentry((Node *) get_leftop(expr),
-								   flat_tlist);
-		Var		   *right = (Var *) flatten_tlistentry((Node *) get_rightop(expr),
-								   flat_tlist);
-		Expr	   *final = make_opclause((Oper *) expr->oper, left, right);
-
-		Assert(IsA(tlistentry, Expr));
-		final->opType = expr->opType;
-		final->typeOid = expr->typeOid;
-		return (Node *)final;
+		elog(ERROR, "flatten_tlistentry: Cannot handle node type %d",
+			 nodeTag(tlistentry));
+		return tlistentry;
 	}
 }
 
@@ -537,6 +524,25 @@ get_expr(TargetEntry *tle)
 }
 
 
+Var *
+get_groupclause_expr(GroupClause *groupClause, List *targetList)
+{
+	List	   *l;
+	TargetEntry *tle;
+
+	foreach(l, targetList)
+	{
+		tle = (TargetEntry *) lfirst(l);
+		if (tle->resdom->resgroupref == groupClause->tleGroupref)
+			return get_expr(tle);
+	}
+
+	elog(ERROR,
+	"get_groupclause_expr: GROUP BY expression not found in targetlist");
+	return NULL;
+}
+
+
 /*****************************************************************************
  *
  *****************************************************************************/
@@ -547,6 +553,11 @@ get_expr(TargetEntry *tle)
  *	  in there.
  */
 #ifdef NOT_USED
+/*
+ * WARNING!!! If this ever get's used again, the new reference
+ * mechanism from group clause to targetlist entry must be implemented
+ * here too. Jan
+ */
 void
 AddGroupAttrToTlist(List *tlist, List *grpCl)
 {
@@ -568,7 +579,7 @@ AddGroupAttrToTlist(List *tlist, List *grpCl)
 						   NULL,
 						   (Index) 0,
 						   (Oid) 0,
-						   0);
+						   false);
 			last_resdomno++;
 			tlist = lappend(tlist, makeTargetEntry(r, (Node *) var));
 		}
