@@ -1,13 +1,13 @@
 /*-------------------------------------------------------------------------
  *
- * parse_clause.c--
+ * parse_clause.c
  *	  handle clauses in parser
  *
  * Copyright (c) 1994, Regents of the University of California
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/parser/parse_clause.c,v 1.26 1998/09/25 13:36:03 thomas Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/parser/parse_clause.c,v 1.37 1999/05/26 12:55:35 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -26,7 +26,9 @@
 #include "parser/parse_relation.h"
 #include "parser/parse_target.h"
 #include "parser/parse_coerce.h"
+#include "nodes/print.h"
 
+#include "parse.h"
 
 
 #define ORDER_CLAUSE 0
@@ -37,33 +39,43 @@ static char *clauseText[] = {"ORDER", "GROUP"};
 static TargetEntry *
 			findTargetlistEntry(ParseState *pstate, Node *node, List *tlist, int clause);
 
-static void parseFromClause(ParseState *pstate, List *frmList);
+static void parseFromClause(ParseState *pstate, List *frmList, Node **qual);
+
+#ifdef ENABLE_OUTER_JOINS
+Node	   *transformUsingClause(ParseState *pstate, List *onList, char *lname, char *rname);
+
+#endif
+
+static char	*transformTableEntry(ParseState *pstate, RangeVar *r);
 
 
 /*
  * makeRangeTable -
  *	  make a range table with the specified relation (optional) and the
- *	  from-clause.
+ *	  from_clause.
  */
 void
-makeRangeTable(ParseState *pstate, char *relname, List *frmList)
+makeRangeTable(ParseState *pstate, char *relname, List *frmList, Node **qual)
 {
 	RangeTblEntry *rte;
 	int			sublevels_up;
 
-	parseFromClause(pstate, frmList);
+	parseFromClause(pstate, frmList, qual);
 
 	if (relname == NULL)
 		return;
 
-	if (refnameRangeTablePosn(pstate, relname, &sublevels_up) == 0 ||
-		sublevels_up != 0)
+	if ((refnameRangeTablePosn(pstate, relname, &sublevels_up) == 0)
+		|| (sublevels_up != 0))
 		rte = addRangeTableEntry(pstate, relname, relname, FALSE, FALSE);
 	else
 		rte = refnameRangeTableEntry(pstate, relname);
 
+	/* This could only happen for multi-action rules */
+	if (pstate->p_target_relation != NULL)
+		heap_close(pstate->p_target_relation);
+
 	pstate->p_target_rangetblentry = rte;
-	Assert(pstate->p_target_relation == NULL);
 	pstate->p_target_relation = heap_open(rte->relid);
 	/* will close relation later */
 }
@@ -72,17 +84,36 @@ makeRangeTable(ParseState *pstate, char *relname, List *frmList)
  * transformWhereClause -
  *	  transforms the qualification and make sure it is of type Boolean
  *
+ * Now accept an additional argument, which is a qualification derived
+ * from the JOIN/ON or JOIN/USING syntax.
+ * - thomas 1998-12-16
  */
 Node *
-transformWhereClause(ParseState *pstate, Node *a_expr)
+transformWhereClause(ParseState *pstate, Node *a_expr, Node *o_expr)
 {
+	A_Expr	   *expr;
 	Node	   *qual;
 
-	if (a_expr == NULL)
+	if ((a_expr == NULL) && (o_expr == NULL))
 		return NULL;			/* no qualifiers */
 
+	if ((a_expr != NULL) && (o_expr != NULL))
+	{
+		A_Expr	   *a = makeNode(A_Expr);
+
+		a->oper = AND;
+		a->opname = NULL;
+		a->lexpr = o_expr;
+		a->rexpr = a_expr;
+		expr = a;
+	}
+	else if (o_expr != NULL)
+		expr = (A_Expr *) o_expr;
+	else
+		expr = (A_Expr *) a_expr;
+
 	pstate->p_in_where_clause = true;
-	qual = transformExpr(pstate, a_expr, EXPR_COLUMN_FIRST);
+	qual = transformExpr(pstate, (Node *) expr, EXPR_COLUMN_FIRST);
 	pstate->p_in_where_clause = false;
 
 	if (exprType(qual) != BOOLOID)
@@ -93,6 +124,115 @@ transformWhereClause(ParseState *pstate, Node *a_expr)
 	return qual;
 }
 
+#ifdef NOT_USED
+static Attr *
+makeAttr(char *relname, char *attname)
+{
+	Attr	   *a = makeNode(Attr);
+
+	a->relname = relname;
+	a->paramNo = NULL;
+	a->attrs = lcons(makeString(attname), NIL);
+	a->indirection = NULL;
+
+	return a;
+}
+#endif
+
+#ifdef ENABLE_OUTER_JOINS
+/* transformUsingClause()
+ * Take an ON or USING clause from a join expression and expand if necessary.
+ */
+Node *
+transformUsingClause(ParseState *pstate, List *onList, char *lname, char *rname)
+{
+	A_Expr	   *expr = NULL;
+	List	   *on;
+	Node	   *qual;
+
+	foreach(on, onList)
+	{
+		qual = lfirst(on);
+
+		/*
+		 * Ident node means it is just a column name from a real USING
+		 * clause...
+		 */
+		if (IsA(qual, Ident))
+		{
+			Ident	   *i = (Ident *) qual;
+			Attr	   *lattr = makeAttr(lname, i->name);
+			Attr	   *rattr = makeAttr(rname, i->name);
+			A_Expr	   *e = makeNode(A_Expr);
+
+			e->oper = OP;
+			e->opname = "=";
+			e->lexpr = (Node *) lattr;
+			e->rexpr = (Node *) rattr;
+
+			if (expr != NULL)
+			{
+				A_Expr	   *a = makeNode(A_Expr);
+
+				a->oper = AND;
+				a->opname = NULL;
+				a->lexpr = (Node *) expr;
+				a->rexpr = (Node *) e;
+				expr = a;
+			}
+			else
+				expr = e;
+		}
+
+		/* otherwise, we have an expression from an ON clause... */
+		else
+		{
+			if (expr != NULL)
+			{
+				A_Expr	   *a = makeNode(A_Expr);
+
+				a->oper = AND;
+				a->opname = NULL;
+				a->lexpr = (Node *) expr;
+				a->rexpr = (Node *) qual;
+				expr = a;
+			}
+			else
+				expr = (A_Expr *) qual;
+		}
+	}
+	return ((Node *) transformExpr(pstate, (Node *) expr, EXPR_COLUMN_FIRST));
+}
+
+#endif
+
+static char *
+transformTableEntry(ParseState *pstate, RangeVar *r)
+{
+	RelExpr    *baserel = r->relExpr;
+	char	   *relname = baserel->relname;
+	char	   *refname = r->name;
+	RangeTblEntry *rte;
+
+	if (refname == NULL)
+		refname = relname;
+
+	/*
+	 * marks this entry to indicate it comes from the FROM clause. In SQL,
+	 * the target list can only refer to range variables specified in the
+	 * from clause but we follow the more powerful POSTQUEL semantics and
+	 * automatically generate the range variable if not specified. However
+	 * there are times we need to know whether the entries are legitimate.
+	 *
+	 * eg. select * from foo f where f.x = 1; will generate wrong answer if
+	 * we expand * to foo.x.
+	 */
+
+	rte = addRangeTableEntry(pstate, relname, refname, baserel->inh, TRUE);
+
+	return refname;
+}
+
 /*
  * parseFromClause -
  *	  turns the table references specified in the from-clause into a
@@ -101,22 +241,23 @@ transformWhereClause(ParseState *pstate, Node *a_expr)
  *	  allow references to relations not specified in the from-clause. We
  *	  also allow now as an extension.)
  *
+ * The FROM clause can now contain JoinExpr nodes, which contain parsing info
+ * for inner and outer joins. The USING clause must be expanded into a qualification
+ * for an inner join at least, since that is compatible with the old syntax.
+ * Not sure yet how to handle outer joins, but it will become clear eventually?
+ * - thomas 1998-12-16
  */
 static void
-parseFromClause(ParseState *pstate, List *frmList)
+parseFromClause(ParseState *pstate, List *frmList, Node **qual)
 {
 	List	   *fl;
 
+	if (qual != NULL)
+		*qual = NULL;
+
 	foreach(fl, frmList)
 	{
-		RangeVar   *r = lfirst(fl);
-		RelExpr    *baserel = r->relExpr;
-		char	   *relname = baserel->relname;
-		char	   *refname = r->name;
-		RangeTblEntry *rte;
-
-		if (refname == NULL)
-			refname = relname;
+		Node	   *n = lfirst(fl);
 
 		/*
 		 * marks this entry to indicate it comes from the FROM clause. In
@@ -129,7 +270,69 @@ parseFromClause(ParseState *pstate, List *frmList)
 		 * eg. select * from foo f where f.x = 1; will generate wrong answer
 		 * if we expand * to foo.x.
 		 */
-		rte = addRangeTableEntry(pstate, relname, refname, baserel->inh, TRUE);
+		if (IsA(n, RangeVar))
+			transformTableEntry(pstate, (RangeVar *) n);
+		else if (IsA(n, JoinExpr))
+		{
+			JoinExpr   *j = (JoinExpr *) n;
+
+#ifdef ENABLE_OUTER_JOINS
+			char	   *lname = transformTableEntry(pstate, (RangeVar *) j->larg);
+
+#endif
+			char	   *rname;
+
+			if (IsA((Node *) j->rarg, RangeVar))
+				rname = transformTableEntry(pstate, (RangeVar *) j->rarg);
+			else
+				elog(ERROR, "Nested JOINs are not yet supported");
+
+#ifdef ENABLE_OUTER_JOINS
+			if (j->jointype == INNER_P)
+			{
+
+				/*
+				 * This is an inner join, so rip apart the join node and
+				 * transform into a traditional FROM list. NATURAL JOIN
+				 * and USING clauses both change the shape of the result.
+				 * Need to generate a list of result columns to use for
+				 * target list expansion and validation. Not doing this
+				 * yet though!
+				 */
+				if (IsA(j->quals, List))
+					j->quals = lcons(transformUsingClause(pstate, (List *) j->quals, lname, rname), NIL);
+
+				Assert(qual != NULL);
+
+				if (*qual == NULL)
+					*qual = lfirst(j->quals);
+				else
+					elog(ERROR, "Multiple JOIN/ON clauses not handled (internal error)");
+
+				/*
+				 * if we are transforming this node back into a FROM list,
+				 * then we will need to replace the node with two nodes.
+				 * Will need access to the previous list item to change
+				 * the link pointer to reference these new nodes. Try
+				 * accumulating and returning a new list. - thomas
+				 * 1999-01-08 Not doing this yet though!
+				 */
+
+			}
+			else if ((j->jointype == LEFT)
+					 || (j->jointype == RIGHT)
+					 || (j->jointype == FULL))
+				elog(ERROR, "OUTER JOIN is not implemented");
+			else
+				elog(ERROR, "Unrecognized JOIN clause; tag is %d (internal error)",
+					 j->jointype);
+#else
+			elog(ERROR, "JOIN expressions are not yet implemented");
+#endif
+		}
+		else
+			elog(ERROR, "parseFromClause: unexpected FROM clause node (internal error)"
+				 "\n\t%s", nodeToString(n));
 	}
 }
 
@@ -286,14 +489,14 @@ findTargetlistEntry(ParseState *pstate, Node *node, List *tlist, int clause)
 			case T_Attr:
 				target_result = MakeTargetEntryIdent(pstate, node,
 										 &((Attr *) node)->relname, NULL,
-										 ((Attr *) node)->relname, TRUE);
+										 ((Attr *) node)->relname, true);
 				lappend(tlist, target_result);
 				break;
 
 			case T_Ident:
 				target_result = MakeTargetEntryIdent(pstate, node,
 										   &((Ident *) node)->name, NULL,
-										   ((Ident *) node)->name, TRUE);
+										   ((Ident *) node)->name, true);
 				lappend(tlist, target_result);
 				break;
 
@@ -308,7 +511,7 @@ findTargetlistEntry(ParseState *pstate, Node *node, List *tlist, int clause)
 
 			case T_FuncCall:
 			case T_A_Expr:
-				target_result = MakeTargetEntryExpr(pstate, "resjunk", expr, FALSE, TRUE);
+				target_result = MakeTargetEntryExpr(pstate, "resjunk", expr, false, true);
 				lappend(tlist, target_result);
 				break;
 
@@ -343,13 +546,19 @@ transformGroupClause(ParseState *pstate, List *grouplist, List *targetlist)
 
 		restarget = findTargetlistEntry(pstate, lfirst(grouplist), targetlist, GROUP_CLAUSE);
 
-		grpcl->entry = restarget;
 		resdom = restarget->resdom;
 		grpcl->grpOpoid = oprid(oper("<",
 									 resdom->restype,
 									 resdom->restype, false));
 		if (glist == NIL)
+		{
+			int			groupref = length(glist) + 1;
+
+			restarget->resdom->resgroupref = groupref;
+			grpcl->tleGroupref = groupref;
+
 			gl = glist = lcons(grpcl, NIL);
+		}
 		else
 		{
 			List	   *i;
@@ -358,11 +567,17 @@ transformGroupClause(ParseState *pstate, List *grouplist, List *targetlist)
 			{
 				GroupClause *gcl = (GroupClause *) lfirst(i);
 
-				if (gcl->entry == grpcl->entry)
+				if (equal(get_groupclause_expr(gcl, targetlist),
+						  restarget->expr))
 					break;
 			}
 			if (i == NIL)		/* not in grouplist already */
 			{
+				int			groupref = length(glist) + 1;
+
+				restarget->resdom->resgroupref = groupref;
+				grpcl->tleGroupref = groupref;
+
 				lnext(gl) = lcons(grpcl, NIL);
 				gl = lnext(gl);
 			}
@@ -529,27 +744,39 @@ transformSortClause(ParseState *pstate,
  * Let's just try matching in pairs for now (right to left) and see if it works.
  * - thomas 1998-05-22
  */
-List *
+#ifdef NOT_USED
+static List *
 transformUnionClause(List *unionClause, List *targetlist)
 {
 	List	   *union_list = NIL;
-	QueryTreeList *qlist;
-	int			i;
+	List	   *qlist,
+			   *qlist_item;
 
 	if (unionClause)
 	{
 		/* recursion */
 		qlist = parse_analyze(unionClause, NULL);
 
-		for (i = 0; i < qlist->len; i++)
+		foreach(qlist_item, qlist)
 		{
+			Query	   *query = (Query *) lfirst(qlist_item);
 			List	   *prev_target = targetlist;
 			List	   *next_target;
+			int			prev_len = 0,
+						next_len = 0;
 
-			if (length(targetlist) != length(qlist->qtrees[i]->targetList))
+			foreach(prev_target, targetlist)
+				if (!((TargetEntry *) lfirst(prev_target))->resdom->resjunk)
+				prev_len++;
+
+			foreach(next_target, query->targetList)
+				if (!((TargetEntry *) lfirst(next_target))->resdom->resjunk)
+				next_len++;
+
+			if (prev_len != next_len)
 				elog(ERROR, "Each UNION clause must have the same number of columns");
 
-			foreach(next_target, qlist->qtrees[i]->targetList)
+			foreach(next_target, query->targetList)
 			{
 				Oid			itype;
 				Oid			otype;
@@ -600,10 +827,11 @@ transformUnionClause(List *unionClause, List *targetlist)
 				}
 				prev_target = lnext(prev_target);
 			}
-			union_list = lappend(union_list, qlist->qtrees[i]);
+			union_list = lappend(union_list, query);
 		}
 		return union_list;
 	}
 	else
 		return NIL;
-}	/* transformUnionClause() */
+}
+#endif
