@@ -1,13 +1,13 @@
 /*-------------------------------------------------------------------------
  *
- * proc.c--
+ * proc.c
  *	  routines to manage per-process shared memory data structure
  *
  * Copyright (c) 1994, Regents of the University of California
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/storage/lmgr/proc.c,v 1.43 1998/09/01 04:32:02 momjian Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/storage/lmgr/proc.c,v 1.57 1999/05/25 22:42:03 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -46,7 +46,7 @@
  *		This is so that we can support more backends. (system-wide semaphore
  *		sets run out pretty fast.)				  -ay 4/95
  *
- * $Header: /usr/local/cvsroot/pgsql/src/backend/storage/lmgr/proc.c,v 1.43 1998/09/01 04:32:02 momjian Exp $
+ * $Header: /usr/local/cvsroot/pgsql/src/backend/storage/lmgr/proc.c,v 1.57 1999/05/25 22:42:03 momjian Exp $
  */
 #include <sys/time.h>
 #include <unistd.h>
@@ -78,7 +78,7 @@
 #include "utils/trace.h"
 
 static void HandleDeadLock(int sig);
-static PROC *ProcWakeup(PROC *proc, int errType);
+static void ProcFreeAllSemaphores(void);
 
 #define DeadlockCheckTimer pg_options[OPT_DEADLOCKTIMEOUT]
 
@@ -105,16 +105,29 @@ static void ProcKill(int exitStatus, int pid);
 static void ProcGetNewSemKeyAndNum(IPCKey *key, int *semNum);
 static void ProcFreeSem(IpcSemaphoreKey semKey, int semNum);
 
+static char *DeadLockMessage = "Deadlock detected -- See the lock(l) manual page for a possible cause.";
+
 /*
  * InitProcGlobal -
  *	  initializes the global process table. We put it here so that
- *	  the postmaster can do this initialization. (ProcFreeAllSem needs
+ *	  the postmaster can do this initialization. (ProcFreeAllSemaphores needs
  *	  to read this table on exiting the postmaster. If we have the first
  *	  backend do this, starting up and killing the postmaster without
  *	  starting any backends will be a problem.)
+ *
+ *	  We also allocate all the per-process semaphores we will need to support
+ *	  the requested number of backends.  We used to allocate semaphores
+ *	  only when backends were actually started up, but that is bad because
+ *	  it lets Postgres fail under load --- a lot of Unix systems are
+ *	  (mis)configured with small limits on the number of semaphores, and
+ *	  running out when trying to start another backend is a common failure.
+ *	  So, now we grab enough semaphores to support the desired max number
+ *	  of backends immediately at initialization --- if the sysadmin has set
+ *	  MaxBackends higher than his kernel will support, he'll find out sooner
+ *	  rather than later.
  */
 void
-InitProcGlobal(IPCKey key)
+InitProcGlobal(IPCKey key, int maxBackends)
 {
 	bool		found = false;
 
@@ -124,6 +137,8 @@ InitProcGlobal(IPCKey key)
 
 	/* --------------------
 	 * We're the first - initialize.
+	 * XXX if found should ever be true, it is a sign of impending doom ...
+	 * ought to complain if so?
 	 * --------------------
 	 */
 	if (!found)
@@ -134,6 +149,37 @@ InitProcGlobal(IPCKey key)
 		ProcGlobal->currKey = IPCGetProcessSemaphoreInitKey(key);
 		for (i = 0; i < MAX_PROC_SEMS / PROC_NSEMS_PER_SET; i++)
 			ProcGlobal->freeSemMap[i] = 0;
+
+		/*
+		 * Arrange to delete semas on exit --- set this up now so that we
+		 * will clean up if pre-allocation fails...
+		 */
+		on_shmem_exit(ProcFreeAllSemaphores, NULL);
+
+		/*
+		 * Pre-create the semaphores for the first maxBackends processes,
+		 * unless we are running as a standalone backend.
+		 */
+		if (key != PrivateIPCKey)
+		{
+			for (i = 0;
+				 i < (maxBackends + PROC_NSEMS_PER_SET - 1) / PROC_NSEMS_PER_SET;
+				 i++)
+			{
+				IPCKey		semKey = ProcGlobal->currKey + i;
+				int			semId;
+				int			semstat;
+
+				semId = IpcSemaphoreCreate(semKey,
+										   PROC_NSEMS_PER_SET,
+										   IPCProtection,
+										   IpcSemaphoreDefaultStartValue,
+										   0,
+										   &semstat);
+				/* mark this sema set allocated */
+				ProcGlobal->freeSemMap[i] = (1 << PROC_NSEMS_PER_SET);
+			}
+		}
 	}
 }
 
@@ -222,6 +268,12 @@ InitProcess(IPCKey key)
 
 		ProcGetNewSemKeyAndNum(&semKey, &semNum);
 
+		/*
+		 * Note: because of the pre-allocation done in InitProcGlobal,
+		 * this call should always attach to an existing semaphore. It
+		 * will (try to) create a new group of semaphores only if the
+		 * postmaster tries to start more backends than it said it would.
+		 */
 		semId = IpcSemaphoreCreate(semKey,
 								   PROC_NSEMS_PER_SET,
 								   IPCProtection,
@@ -252,9 +304,7 @@ InitProcess(IPCKey key)
 
 	MyProc->pid = MyProcPid;
 	MyProc->xid = InvalidTransactionId;
-#ifdef LowLevelLocking
 	MyProc->xmin = InvalidTransactionId;
-#endif
 
 	/* ----------------
 	 * Start keeping spin lock stats from here on.	Any botch before
@@ -442,68 +492,79 @@ ProcQueueInit(PROC_QUEUE *queue)
  */
 int
 ProcSleep(PROC_QUEUE *waitQueue,/* lock->waitProcs */
-		  SPINLOCK spinlock,
+		  LOCKMETHODCTL *lockctl,
 		  int token,			/* lockmode */
-		  int prio,
-		  LOCK *lock,
-		  TransactionId xid)	/* needed by user locks, see below */
+		  LOCK *lock)
 {
 	int			i;
+	SPINLOCK	spinlock = lockctl->masterLock;
 	PROC	   *proc;
+	int			myMask = (1 << token);
+	int			waitMask = lock->waitMask;
+	int			aheadHolders[MAX_LOCKMODES];
+	bool		selfConflict = (lockctl->conflictTab[token] & myMask),
+				prevSame = false;
+	bool		deadlock_checked = false;
 	struct itimerval timeval,
 				dummy;
 
-	/*
-	 * If the first entries in the waitQueue have a greater priority than
-	 * we have, we must be a reader, and they must be a writers, and we
-	 * must be here because the current holder is a writer or a reader but
-	 * we don't share shared locks if a writer is waiting. We put
-	 * ourselves after the writers.  This way, we have a FIFO, but keep
-	 * the readers together to give them decent priority, and no one
-	 * starves.  Because we group all readers together, a non-empty queue
-	 * only has a few possible configurations:
-	 *
-	 * [readers] [writers] [readers][writers] [writers][readers]
-	 * [writers][readers][writers]
-	 *
-	 * In a full queue, we would have a reader holding a lock, then a writer
-	 * gets the lock, then a bunch of readers, made up of readers who
-	 * could not share the first readlock because a writer was waiting,
-	 * and new readers arriving while the writer had the lock.
-	 *
-	 */
-	proc = (PROC *) MAKE_PTR(waitQueue->links.prev);
-
-	/* If we are a reader, and they are writers, skip past them */
-	for (i = 0; i < waitQueue->size && proc->prio > prio; i++)
-		proc = (PROC *) MAKE_PTR(proc->links.prev);
-
-	/* The rest of the queue is FIFO, with readers first, writers last */
-	for (; i < waitQueue->size && proc->prio <= prio; i++)
-		proc = (PROC *) MAKE_PTR(proc->links.prev);
-
-	MyProc->prio = prio;
 	MyProc->token = token;
 	MyProc->waitLock = lock;
 
-#ifdef USER_LOCKS
-	/* -------------------
-	 * Currently, we only need this for the ProcWakeup routines.
-	 * This must be 0 for user lock, so we can't just use the value
-	 * from GetCurrentTransactionId().
-	 * -------------------
-	 */
-	TransactionIdStore(xid, &MyProc->xid);
-#else
-#ifndef LowLevelLocking
-	/* -------------------
-	 * currently, we only need this for the ProcWakeup routines
-	 * -------------------
-	 */
-	TransactionIdStore((TransactionId) GetCurrentTransactionId(), &MyProc->xid);
-#endif
-#endif
+	proc = (PROC *) MAKE_PTR(waitQueue->links.prev);
 
+	/* if we don't conflict with any waiter - be first in queue */
+	if (!(lockctl->conflictTab[token] & waitMask))
+		goto ins;
+
+	for (i = 1; i < MAX_LOCKMODES; i++)
+		aheadHolders[i] = lock->activeHolders[i];
+	(aheadHolders[token])++;
+
+	for (i = 0; i < waitQueue->size; i++)
+	{
+		/* am I waiting for him ? */
+		if (lockctl->conflictTab[token] & proc->holdLock)
+		{
+			/* is he waiting for me ? */
+			if (lockctl->conflictTab[proc->token] & MyProc->holdLock)
+			{
+				MyProc->errType = STATUS_ERROR;
+				elog(NOTICE, DeadLockMessage);
+				goto rt;
+			}
+			/* being waiting for him - go past */
+		}
+		/* if he waits for me */
+		else if (lockctl->conflictTab[proc->token] & MyProc->holdLock)
+			break;
+		/* if conflicting locks requested */
+		else if (lockctl->conflictTab[proc->token] & myMask)
+		{
+
+			/*
+			 * If I request non self-conflicting lock and there are others
+			 * requesting the same lock just before me - stay here.
+			 */
+			if (!selfConflict && prevSame)
+				break;
+		}
+
+		/*
+		 * Last attempt to don't move any more: if we don't conflict with
+		 * rest waiters in queue.
+		 */
+		else if (!(lockctl->conflictTab[token] & waitMask))
+			break;
+
+		prevSame = (proc->token == token);
+		(aheadHolders[proc->token])++;
+		if (aheadHolders[proc->token] == lock->holders[proc->token])
+			waitMask &= ~(1 << proc->token);
+		proc = (PROC *) MAKE_PTR(proc->links.prev);
+	}
+
+ins:;
 	/* -------------------
 	 * assume that these two operations are atomic (because
 	 * of the spinlock).
@@ -512,6 +573,7 @@ ProcSleep(PROC_QUEUE *waitQueue,/* lock->waitProcs */
 	SHMQueueInsertTL(&(proc->links), &(MyProc->links));
 	waitQueue->size++;
 
+	lock->waitMask |= myMask;
 	SpinRelease(spinlock);
 
 	/* --------------
@@ -531,8 +593,10 @@ ProcSleep(PROC_QUEUE *waitQueue,/* lock->waitProcs */
 	{
 		MyProc->errType = NO_ERROR;		/* reset flag after deadlock check */
 
-		if (setitimer(ITIMER_REAL, &timeval, &dummy))
-			elog(FATAL, "ProcSleep: Unable to set timer for process wakeup");
+		if (!deadlock_checked)
+			if (setitimer(ITIMER_REAL, &timeval, &dummy))
+				elog(FATAL, "ProcSleep: Unable to set timer for process wakeup");
+		deadlock_checked = true;
 
 		/* --------------
 		 * if someone wakes us between SpinRelease and IpcSemaphoreLock,
@@ -560,6 +624,8 @@ ProcSleep(PROC_QUEUE *waitQueue,/* lock->waitProcs */
 	 */
 	SpinAcquire(spinlock);
 
+rt:;
+
 #ifdef LOCK_MGR_DEBUG
 	/* Just to get meaningful debug messages from DumpLocks() */
 	MyProc->waitLock = (LOCK *) NULL;
@@ -575,7 +641,7 @@ ProcSleep(PROC_QUEUE *waitQueue,/* lock->waitProcs */
  *	 remove the process from the wait queue and set its links invalid.
  *	 RETURN: the next process in the wait queue.
  */
-static PROC *
+PROC *
 ProcWakeup(PROC *proc, int errType)
 {
 	PROC	   *retProc;
@@ -607,9 +673,9 @@ int
 ProcLockWakeup(PROC_QUEUE *queue, LOCKMETHOD lockmethod, LOCK *lock)
 {
 	PROC	   *proc;
-	int			count;
+	int			count = 0;
 	int			trace_flag;
-	int			last_locktype = -1;
+	int			last_locktype = 0;
 	int			queue_size = queue->size;
 
 	Assert(queue->size >= 0);
@@ -618,7 +684,6 @@ ProcLockWakeup(PROC_QUEUE *queue, LOCKMETHOD lockmethod, LOCK *lock)
 		return STATUS_NOT_FOUND;
 
 	proc = (PROC *) MAKE_PTR(queue->links.prev);
-	count = 0;
 	while ((queue_size--) && (proc))
 	{
 
@@ -630,7 +695,7 @@ ProcLockWakeup(PROC_QUEUE *queue, LOCKMETHOD lockmethod, LOCK *lock)
 			continue;
 
 		/*
-		 * This proc conflicts with locks held by others, ignored.
+		 * Does this proc conflict with locks held by others ?
 		 */
 		if (LockResolveConflicts(lockmethod,
 								 lock,
@@ -638,6 +703,8 @@ ProcLockWakeup(PROC_QUEUE *queue, LOCKMETHOD lockmethod, LOCK *lock)
 								 proc->xid,
 								 (XIDLookupEnt *) NULL) != STATUS_OK)
 		{
+			if (count != 0)
+				break;
 			last_locktype = proc->token;
 			continue;
 		}
@@ -740,10 +807,10 @@ HandleDeadLock(int sig)
 	DumpAllLocks();
 #endif
 
-	if (!DeadLockCheck(&(MyProc->lockQueue), MyProc->waitLock, true))
+	MyProc->errType = STATUS_NOT_FOUND;
+	if (!DeadLockCheck(MyProc, MyProc->waitLock))
 	{
 		UnlockLockTable();
-		MyProc->errType = STATUS_NOT_FOUND;
 		return;
 	}
 
@@ -780,7 +847,7 @@ HandleDeadLock(int sig)
 	 */
 	UnlockLockTable();
 
-	elog(NOTICE, "Deadlock detected -- See the lock(l) manual page for a possible cause.");
+	elog(NOTICE, DeadLockMessage);
 	return;
 }
 
@@ -820,20 +887,20 @@ ProcGetNewSemKeyAndNum(IPCKey *key, int *semNum)
 {
 	int			i;
 	int32	   *freeSemMap = ProcGlobal->freeSemMap;
-	unsigned int fullmask;
+	int32		fullmask = (1 << (PROC_NSEMS_PER_SET + 1)) - 1;
 
 	/*
 	 * we hold ProcStructLock when entering this routine. We scan through
 	 * the bitmap to look for a free semaphore.
 	 */
-	fullmask = ~0 >> (32 - PROC_NSEMS_PER_SET);
+
 	for (i = 0; i < MAX_PROC_SEMS / PROC_NSEMS_PER_SET; i++)
 	{
 		int			mask = 1;
 		int			j;
 
 		if (freeSemMap[i] == fullmask)
-			continue;			/* none free for this set */
+			continue;			/* this set is fully allocated */
 
 		for (j = 0; j < PROC_NSEMS_PER_SET; j++)
 		{
@@ -841,9 +908,10 @@ ProcGetNewSemKeyAndNum(IPCKey *key, int *semNum)
 			{
 
 				/*
-				 * a free semaphore found. Mark it as allocated.
+				 * a free semaphore found. Mark it as allocated. Also set
+				 * the bit indicating whole set is allocated.
 				 */
-				freeSemMap[i] |= mask;
+				freeSemMap[i] |= mask + (1 << PROC_NSEMS_PER_SET);
 
 				*key = ProcGlobal->currKey + i;
 				*semNum = j;
@@ -859,8 +927,7 @@ ProcGetNewSemKeyAndNum(IPCKey *key, int *semNum)
 
 /*
  * ProcFreeSem -
- *	  free up our semaphore in the semaphore set. If we're the last one
- *	  in the set, also remove the semaphore set.
+ *	  free up our semaphore in the semaphore set.
  */
 static void
 ProcFreeSem(IpcSemaphoreKey semKey, int semNum)
@@ -873,16 +940,22 @@ ProcFreeSem(IpcSemaphoreKey semKey, int semNum)
 	mask = ~(1 << semNum);
 	freeSemMap[i] &= mask;
 
-	if (freeSemMap[i] == 0)
-		IpcSemaphoreKill(semKey);
+	/*
+	 * Formerly we'd release a semaphore set if it was now completely
+	 * unused, but now we keep the semaphores to ensure we won't run out
+	 * when starting new backends --- cf. InitProcGlobal.  Note that the
+	 * PROC_NSEMS_PER_SET+1'st bit of the freeSemMap entry remains set to
+	 * indicate it is still allocated; ProcFreeAllSemaphores() needs that.
+	 */
 }
 
 /*
  * ProcFreeAllSemaphores -
- *	  on exiting the postmaster, we free up all the semaphores allocated
- *	  to the lmgrs of the backends.
+ *	  called at shmem_exit time, ie when exiting the postmaster or
+ *	  destroying shared state for a failed set of backends.
+ *	  Free up all the semaphores allocated to the lmgrs of the backends.
  */
-void
+static void
 ProcFreeAllSemaphores()
 {
 	int			i;
