@@ -7,7 +7,7 @@
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/utils/adt/acl.c,v 1.36 1999/05/25 16:11:49 momjian Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/utils/adt/acl.c,v 1.37 1999/07/09 03:28:51 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -20,6 +20,7 @@
 #include "utils/syscache.h"
 #include "catalog/catalog.h"
 #include "catalog/pg_shadow.h"
+#include "catalog/pg_type.h"
 #include "miscadmin.h"
 
 static char *getid(char *s, char *n);
@@ -682,6 +683,9 @@ makeAclStmt(char *privileges, List *rel_list, char *grantee,
 	ChangeACLStmt *n = makeNode(ChangeACLStmt);
 	char		str[MAX_PARSE_BUFFER];
 
+	/* see comment in pg_type.h */
+	Assert(ACLITEMSIZE == sizeof(AclItem));
+
 	n->aclitem = (AclItem *) palloc(sizeof(AclItem));
 
 	/* the grantee string is "G <group_name>", "U  <user_name>", or "ALL" */
