@@ -1,6 +1,6 @@
 /*-------------------------------------------------------------------------
  *
- * date.c--
+ * date.c
  *	  Utilities for the built-in type "AbsoluteTime" (defined in nabstime).
  *	  Functions for the built-in type "RelativeTime".
  *	  Functions for the built-in type "TimeInterval".
@@ -9,7 +9,7 @@
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/utils/adt/date.c,v 1.28 1998/10/08 18:30:05 momjian Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/utils/adt/date.c,v 1.32 1999/05/25 16:11:56 momjian Exp $
  *
  * NOTES
  *	 This code is actually (almost) unused.
@@ -73,7 +73,7 @@
 #define ABSTIMEMIN(t1, t2) abstimele((t1),(t2)) ? (t1) : (t2)
 #define ABSTIMEMAX(t1, t2) abstimelt((t1),(t2)) ? (t2) : (t1)
 
-#if FALSE
+#ifdef NOT_USED
 static char *unit_tab[] = {
 	"second", "seconds", "minute", "minutes",
 	"hour", "hours", "day", "days", "week", "weeks",
@@ -94,9 +94,9 @@ static int	sec_tab[] = {
  * Function prototypes -- internal to this file only
  */
 
-static void reltime2tm(int32 time, struct tm * tm);
+static void reltime2tm(RelativeTime time, struct tm * tm);
 
-#if FALSE
+#ifdef NOT_USED
 static int	correct_unit(char *unit, int *unptr);
 static int	correct_dir(char *direction, int *signptr);
 
@@ -161,7 +161,7 @@ reltimein(char *str)
  *		reltimeout		- converts the internal format to a reltime string
  */
 char *
-reltimeout(int32 time)
+reltimeout(RelativeTime time)
 {
 	char	   *result;
 	struct tm	tt,
@@ -193,7 +193,7 @@ do { \
 } while(0)
 
 static void
-reltime2tm(int32 time, struct tm * tm)
+reltime2tm(RelativeTime time, struct tm * tm)
 {
 	TMODULO(time, tm->tm_year, 31536000);
 	TMODULO(time, tm->tm_mon, 2592000);
@@ -205,7 +205,7 @@ reltime2tm(int32 time, struct tm * tm)
 	return;
 }	/* reltime2tm() */
 
-#if FALSE
+#ifdef NOT_USED
 int
 dummyfunc()
 {
@@ -988,9 +988,10 @@ isreltime(char *str)
 
 	return 0;
 }	/* isreltime() */
+
 #endif
 
-#if FALSE
+#ifdef NOT_USED
 int
 dummyfunc()
 {
