@@ -1,13 +1,13 @@
 /*-------------------------------------------------------------------------
  *
- * datetime.c--
+ * datetime.c
  *	  implements DATE and TIME data types specified in SQL-92 standard
  *
  * Copyright (c) 1994-5, Regents of the University of California
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/utils/adt/datetime.c,v 1.25.2.1 1998/12/31 16:34:47 thomas Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/utils/adt/datetime.c,v 1.33 1999/05/25 16:11:58 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -28,26 +28,6 @@
 static int	date2tm(DateADT dateVal, int *tzp, struct tm * tm, double *fsec, char **tzn);
 
 
-static int	day_tab[2][12] = {
-	{31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31},
-{31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31}};
-
-#define isleap(y) (((y % 4) == 0 && (y % 100) != 0) || (y % 400) == 0)
-
-#define UTIME_MINYEAR (1901)
-#define UTIME_MINMONTH (12)
-#define UTIME_MINDAY (14)
-#define UTIME_MAXYEAR (2038)
-#define UTIME_MAXMONTH (01)
-#define UTIME_MAXDAY (18)
-
-#define IS_VALID_UTIME(y,m,d) (((y > UTIME_MINYEAR) \
- || ((y == UTIME_MINYEAR) && ((m > UTIME_MINMONTH) \
-  || ((m == UTIME_MINMONTH) && (d >= UTIME_MINDAY))))) \
- && ((y < UTIME_MAXYEAR) \
- || ((y == UTIME_MAXYEAR) && ((m < UTIME_MAXMONTH) \
-  || ((m == UTIME_MAXMONTH) && (d <= UTIME_MAXDAY))))))
-
 /*****************************************************************************
  *	 Date ADT
  *****************************************************************************/
@@ -78,7 +58,7 @@ date_in(char *str)
 #endif
 	if ((ParseDateTime(str, lowstr, field, ftype, MAXDATEFIELDS, &nf) != 0)
 	 || (DecodeDateTime(field, ftype, nf, &dtype, tm, &fsec, &tzp) != 0))
-		elog(ERROR, "Bad date external representation %s", str);
+		elog(ERROR, "Bad date external representation '%s'", str);
 
 	switch (dtype)
 	{
@@ -96,17 +76,9 @@ date_in(char *str)
 			break;
 
 		default:
-			elog(ERROR, "Unrecognized date external representation %s", str);
+			elog(ERROR, "Unrecognized date external representation '%s'", str);
 	}
 
-	if (tm->tm_year < 0 || tm->tm_year > 32767)
-		elog(ERROR, "date_in: year must be limited to values 0 through 32767 in '%s'", str);
-	if (tm->tm_mon < 1 || tm->tm_mon > 12)
-		elog(ERROR, "date_in: month must be limited to values 1 through 12 in '%s'", str);
-	if (tm->tm_mday < 1 || tm->tm_mday > day_tab[isleap(tm->tm_year)][tm->tm_mon - 1])
-		elog(ERROR, "date_in: day must be limited to values 1 through %d in '%s'",
-			 day_tab[isleap(tm->tm_year)][tm->tm_mon - 1], str);
-
 	date = (date2j(tm->tm_year, tm->tm_mon, tm->tm_mday) - date2j(2000, 1, 1));
 
 	return date;
@@ -339,7 +311,7 @@ abstime_date(AbsoluteTime abstime)
  *	and then convert again to try to get the time zones correct.
  */
 static int
-date2tm(DateADT dateVal, int *tzp, struct tm *tm, double *fsec, char **tzn)
+date2tm(DateADT dateVal, int *tzp, struct tm * tm, double *fsec, char **tzn)
 {
 	struct tm  *tx;
 	time_t		utime;
@@ -386,9 +358,13 @@ date2tm(DateADT dateVal, int *tzp, struct tm *tm, double *fsec, char **tzn)
 		/* tm_gmtoff is Sun/DEC-ism */
 		*tzp = -(tm->tm_gmtoff);
 		if (tzn != NULL)
-			*tzn = (char *)tm->tm_zone;
+			*tzn = (char *) tm->tm_zone;
 #elif defined(HAVE_INT_TIMEZONE)
+#ifdef __CYGWIN__
+		*tzp = (tm->tm_isdst ? (_timezone - 3600) : _timezone);
+#else
 		*tzp = (tm->tm_isdst ? (timezone - 3600) : timezone);
+#endif
 		if (tzn != NULL)
 			*tzn = tzname[(tm->tm_isdst > 0)];
 #else
@@ -417,13 +393,13 @@ date2tm(DateADT dateVal, int *tzp, struct tm *tm, double *fsec, char **tzn)
 
 #ifdef DATEDEBUG
 #if defined(HAVE_TM_ZONE)
-		printf("date2tm- %d.%02d.%02d %02d:%02d:%02.0f (%d %s) dst=%d\n",
-			   tm->tm_year, tm->tm_mon, tm->tm_mday, tm->tm_hour, tm->tm_min, (double) tm->tm_sec,
-			   *tzp, tm->tm_zone, tm->tm_isdst);
+	printf("date2tm- %d.%02d.%02d %02d:%02d:%02.0f (%d %s) dst=%d\n",
+		   tm->tm_year, tm->tm_mon, tm->tm_mday, tm->tm_hour, tm->tm_min, (double) tm->tm_sec,
+		   *tzp, tm->tm_zone, tm->tm_isdst);
 #elif defined(HAVE_INT_TIMEZONE)
-		printf("date2tm- %d.%02d.%02d %02d:%02d:%02.0f (%d %s %s) dst=%d\n",
-			   tm->tm_year, tm->tm_mon, tm->tm_mday, tm->tm_hour, tm->tm_min, (double) tm->tm_sec,
-			   *tzp, tzname[0], tzname[1], tm->tm_isdst);
+	printf("date2tm- %d.%02d.%02d %02d:%02d:%02.0f (%d %s %s) dst=%d\n",
+		   tm->tm_year, tm->tm_mon, tm->tm_mday, tm->tm_hour, tm->tm_min, (double) tm->tm_sec,
+		   *tzp, tzname[0], tzname[1], tm->tm_isdst);
 #endif
 #endif
 
@@ -458,13 +434,6 @@ time_in(char *str)
 		|| (DecodeTimeOnly(field, ftype, nf, &dtype, tm, &fsec) != 0))
 		elog(ERROR, "Bad time external representation '%s'", str);
 
-	if ((tm->tm_hour < 0) || (tm->tm_hour > 23))
-		elog(ERROR, "Hour must be limited to values 0 through 23 in '%s'", str);
-	if ((tm->tm_min < 0) || (tm->tm_min > 59))
-		elog(ERROR, "Minute must be limited to values 0 through 59 in '%s'", str);
-	if ((tm->tm_sec < 0) || ((tm->tm_sec + fsec) >= 60))
-		elog(ERROR, "Second must be limited to values 0 through < 60 in '%s'", str);
-
 	time = palloc(sizeof(TimeADT));
 
 	*time = ((((tm->tm_hour * 60) + tm->tm_min) * 60) + tm->tm_sec + fsec);
