@@ -1,13 +1,13 @@
  /*-------------------------------------------------------------------------
  *
- * regproc.c--
+ * regproc.c
  *	  Functions for the built-in type "RegProcedure".
  *
  * Copyright (c) 1994, Regents of the University of California
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/utils/adt/regproc.c,v 1.32 1998/10/02 05:31:28 momjian Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/utils/adt/regproc.c,v 1.37 1999/05/25 16:12:17 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -36,19 +36,23 @@
 /*
  *		regprocin		- converts "proname" or "proid" to proid
  *
- *		proid of NULL signifies unknown
+ *		proid of '-' signifies unknown, for consistency with regprocout
  */
 int32
 regprocin(char *pro_name_or_oid)
 {
-	HeapTuple	 proctup = NULL;
+	HeapTuple	proctup = NULL;
+	HeapTupleData tuple;
 	RegProcedure result = InvalidOid;
 
 	if (pro_name_or_oid == NULL)
 		return InvalidOid;
+	if (pro_name_or_oid[0] == '-' && pro_name_or_oid[1] == '\0')
+		return InvalidOid;
 
 	if (!IsBootstrapProcessingMode())
 	{
+
 		/*
 		 * we need to use the oid because there can be multiple entries
 		 * with the same name.	We accept int4eq_1323 and 1323.
@@ -60,7 +64,7 @@ regprocin(char *pro_name_or_oid)
 								ObjectIdGetDatum(oidin(pro_name_or_oid)),
 										  0, 0, 0);
 			if (HeapTupleIsValid(proctup))
-				result = (RegProcedure) proctup->t_oid;
+				result = (RegProcedure) proctup->t_data->t_oid;
 			else
 				elog(ERROR, "No procedure with oid %s", pro_name_or_oid);
 		}
@@ -73,26 +77,27 @@ regprocin(char *pro_name_or_oid)
 			RetrieveIndexResult indexRes;
 			Buffer		buffer;
 			int			matches = 0;
-		
+
 			ScanKeyEntryInitialize(&skey[0],
 								   (bits16) 0x0,
 								   (AttrNumber) 1,
 								   (RegProcedure) F_NAMEEQ,
 								   PointerGetDatum(pro_name_or_oid));
-		
+
 			hdesc = heap_openr(ProcedureRelationName);
 			idesc = index_openr(ProcedureNameIndex);
-		
+
 			sd = index_beginscan(idesc, false, 1, skey);
 			while ((indexRes = index_getnext(sd, ForwardScanDirection)))
 			{
-				proctup = heap_fetch(hdesc, SnapshotNow,
-									&indexRes->heap_iptr,
-									&buffer);
+				tuple.t_self = indexRes->heap_iptr;
+				heap_fetch(hdesc, SnapshotNow,
+						   &tuple,
+						   &buffer);
 				pfree(indexRes);
-				if (HeapTupleIsValid(proctup))
+				if (tuple.t_data != NULL)
 				{
-					result = (RegProcedure) proctup->t_oid;
+					result = (RegProcedure) tuple.t_data->t_oid;
 					ReleaseBuffer(buffer);
 
 					if (++matches > 1)
@@ -225,7 +230,7 @@ regprocout(RegProcedure proid)
 			if (!isnull)
 				StrNCpy(result, s, NAMEDATALEN);
 			else
-				elog(FATAL, "regprocout: null procedure %d", proid);
+				elog(FATAL, "regprocout: null procedure %u", proid);
 		}
 		else
 		{
