@@ -3,7 +3,7 @@
  *			  out of it's tuple
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/utils/adt/ruleutils.c,v 1.6 1998/10/26 01:01:35 tgl Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/utils/adt/ruleutils.c,v 1.18 1999/06/02 11:52:28 wieck Exp $
  *
  *	  This software is copyrighted by Jan Wieck - Hamburg.
  *
@@ -57,14 +57,16 @@
 #include "catalog/pg_opclass.h"
 #include "fmgr.h"
 
+#define BUFSIZE 8192
 
 /* ----------
  * Local data types
  * ----------
  */
-typedef struct QryHier {
-	struct QryHier		*parent;
-	Query			*query;
+typedef struct QryHier
+{
+	struct QryHier *parent;
+	Query	   *query;
 } QryHier;
 
 
@@ -89,7 +91,7 @@ static char *query_getopclass = "SELECT * FROM pg_opclass WHERE oid = $1";
  */
 text	   *pg_get_ruledef(NameData *rname);
 text	   *pg_get_viewdef(NameData *rname);
-text       *pg_get_indexdef(Oid indexrelid);
+text	   *pg_get_indexdef(Oid indexrelid);
 NameData   *pg_get_userbyid(int4 uid);
 
 
@@ -313,27 +315,27 @@ pg_get_viewdef(NameData *rname)
  *				  only return the SELECT part of a view
  * ----------
  */
-text       *
+text *
 pg_get_indexdef(Oid indexrelid)
 {
-	text		*indexdef;
+	text	   *indexdef;
 	HeapTuple	ht_idx;
 	HeapTuple	ht_idxrel;
 	HeapTuple	ht_indrel;
 	HeapTuple	spi_tup;
 	TupleDesc	spi_ttc;
-	int		spi_fno;
-	Form_pg_index	idxrec;
-	Form_pg_class	idxrelrec;
-	Form_pg_class	indrelrec;
+	int			spi_fno;
+	Form_pg_index idxrec;
+	Form_pg_class idxrelrec;
+	Form_pg_class indrelrec;
 	Datum		spi_args[1];
 	char		spi_nulls[2];
-	int		spirc;
-	int		len;
-	int		keyno;
-	char		buf[8192];
-	char		keybuf[8192];
-	char		*sep;
+	int			spirc;
+	int			len;
+	int			keyno;
+	char		buf[BUFSIZE];
+	char		keybuf[BUFSIZE];
+	char	   *sep;
 
 	/* ----------
 	 * Connect to SPI manager
@@ -370,30 +372,30 @@ pg_get_indexdef(Oid indexrelid)
 	 * ----------
 	 */
 	ht_idx = SearchSysCacheTuple(INDEXRELID,
-			ObjectIdGetDatum(indexrelid), 0, 0, 0);
+								 ObjectIdGetDatum(indexrelid), 0, 0, 0);
 	if (!HeapTupleIsValid(ht_idx))
-		elog(ERROR, "syscache lookup for index %d failed", indexrelid);
-	idxrec = (Form_pg_index)GETSTRUCT(ht_idx);
+		elog(ERROR, "syscache lookup for index %u failed", indexrelid);
+	idxrec = (Form_pg_index) GETSTRUCT(ht_idx);
 
 	/* ----------
 	 * Fetch the pg_class tuple of the index relation
 	 * ----------
 	 */
 	ht_idxrel = SearchSysCacheTuple(RELOID,
-			ObjectIdGetDatum(idxrec->indexrelid), 0, 0, 0);
+						  ObjectIdGetDatum(idxrec->indexrelid), 0, 0, 0);
 	if (!HeapTupleIsValid(ht_idxrel))
-		elog(ERROR, "syscache lookup for relid %d failed", idxrec->indexrelid);
-	idxrelrec = (Form_pg_class)GETSTRUCT(ht_idxrel);
+		elog(ERROR, "syscache lookup for relid %u failed", idxrec->indexrelid);
+	idxrelrec = (Form_pg_class) GETSTRUCT(ht_idxrel);
 
 	/* ----------
 	 * Fetch the pg_class tuple of the indexed relation
 	 * ----------
 	 */
 	ht_indrel = SearchSysCacheTuple(RELOID,
-			ObjectIdGetDatum(idxrec->indrelid), 0, 0, 0);
+							ObjectIdGetDatum(idxrec->indrelid), 0, 0, 0);
 	if (!HeapTupleIsValid(ht_indrel))
-		elog(ERROR, "syscache lookup for relid %d failed", idxrec->indrelid);
-	indrelrec = (Form_pg_class)GETSTRUCT(ht_indrel);
+		elog(ERROR, "syscache lookup for relid %u failed", idxrec->indrelid);
+	indrelrec = (Form_pg_class) GETSTRUCT(ht_indrel);
 
 	/* ----------
 	 * Get the am name for the index relation
@@ -416,11 +418,11 @@ pg_get_indexdef(Oid indexrelid)
 	 * ----------
 	 */
 	sprintf(buf, "CREATE %sINDEX \"%s\" ON \"%s\" USING %s (",
-		idxrec->indisunique ? "UNIQUE " : "",
-		nameout(&(idxrelrec->relname)),
-		nameout(&(indrelrec->relname)),
-		SPI_getvalue(spi_tup, spi_ttc, spi_fno));
-			
+			idxrec->indisunique ? "UNIQUE " : "",
+			nameout(&(idxrelrec->relname)),
+			nameout(&(indrelrec->relname)),
+			SPI_getvalue(spi_tup, spi_ttc, spi_fno));
+
 	/* ----------
 	 * Collect the indexed attributes
 	 * ----------
@@ -444,7 +446,7 @@ pg_get_indexdef(Oid indexrelid)
 			strcat(keybuf, "oid");
 		else
 			strcat(keybuf, get_attribute_name(idxrec->indrelid,
-						idxrec->indkey[keyno]));
+											  idxrec->indkey[keyno]));
 		strcat(keybuf, "\"");
 
 		/* ----------
@@ -458,9 +460,9 @@ pg_get_indexdef(Oid indexrelid)
 			spi_nulls[1] = '\0';
 			spirc = SPI_execp(plan_getopclass, spi_args, spi_nulls, 1);
 			if (spirc != SPI_OK_SELECT)
-				elog(ERROR, "failed to get pg_opclass tuple %d", idxrec->indclass[keyno]);
+				elog(ERROR, "failed to get pg_opclass tuple %u", idxrec->indclass[keyno]);
 			if (SPI_processed != 1)
-				elog(ERROR, "failed to get pg_opclass tuple %d", idxrec->indclass[keyno]);
+				elog(ERROR, "failed to get pg_opclass tuple %u", idxrec->indclass[keyno]);
 			spi_tup = SPI_tuptable->vals[0];
 			spi_ttc = SPI_tuptable->tupdesc;
 			spi_fno = SPI_fnumber(spi_ttc, "opcname");
@@ -477,12 +479,12 @@ pg_get_indexdef(Oid indexrelid)
 	if (idxrec->indproc != InvalidOid)
 	{
 		HeapTuple	proctup;
-		Form_pg_proc	procStruct;
+		Form_pg_proc procStruct;
 
 		proctup = SearchSysCacheTuple(PROOID,
-								ObjectIdGetDatum(idxrec->indproc), 0, 0, 0);
+							 ObjectIdGetDatum(idxrec->indproc), 0, 0, 0);
 		if (!HeapTupleIsValid(proctup))
-			elog(ERROR, "cache lookup for proc %d failed", idxrec->indproc);
+			elog(ERROR, "cache lookup for proc %u failed", idxrec->indproc);
 
 		procStruct = (Form_pg_proc) GETSTRUCT(proctup);
 		strcat(buf, "\"");
@@ -496,9 +498,9 @@ pg_get_indexdef(Oid indexrelid)
 		spi_nulls[1] = '\0';
 		spirc = SPI_execp(plan_getopclass, spi_args, spi_nulls, 1);
 		if (spirc != SPI_OK_SELECT)
-			elog(ERROR, "failed to get pg_opclass tuple %d", idxrec->indclass[0]);
+			elog(ERROR, "failed to get pg_opclass tuple %u", idxrec->indclass[0]);
 		if (SPI_processed != 1)
-			elog(ERROR, "failed to get pg_opclass tuple %d", idxrec->indclass[0]);
+			elog(ERROR, "failed to get pg_opclass tuple %u", idxrec->indclass[0]);
 		spi_tup = SPI_tuptable->vals[0];
 		spi_ttc = SPI_tuptable->tupdesc;
 		spi_fno = SPI_fnumber(spi_ttc, "opcname");
@@ -507,13 +509,11 @@ pg_get_indexdef(Oid indexrelid)
 		strcat(buf, "\"");
 	}
 	else
-	/* ----------
-	 * For the others say 'attr opclass [, ...]'
-	 * ----------
-	 */
-	{
+		/* ----------
+		 * For the others say 'attr opclass [, ...]'
+		 * ----------
+		 */
 		strcat(buf, keybuf);
-	}
 
 	/* ----------
 	 * Finish
@@ -550,8 +550,8 @@ NameData   *
 pg_get_userbyid(int4 uid)
 {
 	HeapTuple	usertup;
-	Form_pg_shadow	user_rec;
-	NameData	*result;
+	Form_pg_shadow user_rec;
+	NameData   *result;
 
 	/* ----------
 	 * Allocate space for the result
@@ -565,16 +565,14 @@ pg_get_userbyid(int4 uid)
 	 * ----------
 	 */
 	usertup = SearchSysCacheTuple(USESYSID,
-			ObjectIdGetDatum(uid), 0, 0, 0);
+								  ObjectIdGetDatum(uid), 0, 0, 0);
 	if (HeapTupleIsValid(usertup))
 	{
-		user_rec = (Form_pg_shadow)GETSTRUCT(usertup);
+		user_rec = (Form_pg_shadow) GETSTRUCT(usertup);
 		StrNCpy(result->data, (&(user_rec->usename))->data, NAMEDATALEN);
 	}
 	else
-	{
-		sprintf((char *)result, "unknown (UID=%d)", uid);
-	}
+		sprintf((char *) result, "unknown (UID=%d)", uid);
 
 	return result;
 }
@@ -603,7 +601,7 @@ make_ruledef(HeapTuple ruletup, TupleDesc rulettc)
 	 * Allocate space for the returned rule definition text
 	 * ----------
 	 */
-	buf = palloc(8192);
+	buf = palloc(BUFSIZE);
 
 	/* ----------
 	 * Get the attribute values from the rules tuple
@@ -678,16 +676,16 @@ make_ruledef(HeapTuple ruletup, TupleDesc rulettc)
 	/* If the rule has an event qualification, add it */
 	if (ev_qual == NULL)
 		ev_qual = "";
-	if (strlen(ev_qual) > 0)
+	if (strlen(ev_qual) > 0 && strcmp(ev_qual, "<>") != 0)
 	{
 		Node	   *qual;
 		Query	   *query;
-		QryHier    qh;
+		QryHier		qh;
 
 		qual = stringToNode(ev_qual);
 		query = (Query *) lfirst(actions);
 		qh.parent = NULL;
-		qh.query  = query;
+		qh.query = query;
 
 		strcat(buf, " WHERE ");
 		strcat(buf, get_rule_expr(&qh, 0, qual, TRUE));
@@ -746,7 +744,7 @@ make_ruledef(HeapTuple ruletup, TupleDesc rulettc)
 static char *
 make_viewdef(HeapTuple ruletup, TupleDesc rulettc)
 {
-	char		buf[8192];
+	char		buf[BUFSIZE];
 	Query	   *query;
 	char		ev_type;
 	Oid			ev_class;
@@ -787,7 +785,7 @@ make_viewdef(HeapTuple ruletup, TupleDesc rulettc)
 
 	query = (Query *) lfirst(actions);
 
-	if (ev_type != '1' || ev_attr >= 0 || !is_instead || strcmp(ev_qual, ""))
+	if (ev_type != '1' || ev_attr >= 0 || !is_instead || strcmp(ev_qual, "<>"))
 		return "Not a view";
 
 	strcpy(buf, get_query_def(query, NULL));
@@ -813,11 +811,11 @@ get_query_def(Query *query, QryHier *parentqh)
 	QryHier		qh;
 
 	qh.parent = parentqh;
-	qh.query  = query;
+	qh.query = query;
 
 	switch (query->commandType)
 	{
-			case CMD_SELECT:
+		case CMD_SELECT:
 			return get_select_query_def(query, &qh);
 			break;
 
@@ -854,7 +852,7 @@ get_query_def(Query *query, QryHier *parentqh)
 static char *
 get_select_query_def(Query *query, QryHier *qh)
 {
-	char		buf[8192];
+	char		buf[BUFSIZE];
 	char	   *sep;
 	TargetEntry *tle;
 	RangeTblEntry *rte;
@@ -1024,7 +1022,7 @@ get_select_query_def(Query *query, QryHier *qh)
 static char *
 get_insert_query_def(Query *query, QryHier *qh)
 {
-	char		buf[8192];
+	char		buf[BUFSIZE];
 	char	   *sep;
 	TargetEntry *tle;
 	RangeTblEntry *rte;
@@ -1117,7 +1115,7 @@ get_insert_query_def(Query *query, QryHier *qh)
 		strcat(buf, ")");
 	}
 	else
-		strcat(buf, get_query_def(query, qh));
+		strcat(buf, get_select_query_def(query, qh));
 
 	/* ----------
 	 * Copy the query string into allocated space and return it
@@ -1134,7 +1132,7 @@ get_insert_query_def(Query *query, QryHier *qh)
 static char *
 get_update_query_def(Query *query, QryHier *qh)
 {
-	char		buf[8192];
+	char		buf[BUFSIZE];
 	char	   *sep;
 	TargetEntry *tle;
 	RangeTblEntry *rte;
@@ -1187,7 +1185,7 @@ get_update_query_def(Query *query, QryHier *qh)
 static char *
 get_delete_query_def(Query *query, QryHier *qh)
 {
-	char		buf[8192];
+	char		buf[BUFSIZE];
 	RangeTblEntry *rte;
 
 	/* ----------
@@ -1221,7 +1219,7 @@ get_delete_query_def(Query *query, QryHier *qh)
 static char *
 get_rule_expr(QryHier *qh, int rt_index, Node *node, bool varprefix)
 {
-	char		buf[8192];
+	char		buf[BUFSIZE];
 
 	if (node == NULL)
 		return pstrdup("");
@@ -1245,15 +1243,15 @@ get_rule_expr(QryHier *qh, int rt_index, Node *node, bool varprefix)
 			}
 			break;
 
-		case T_Aggreg:
+		case T_Aggref:
 			{
-				Aggreg	   *agg = (Aggreg *) node;
+				Aggref	   *aggref = (Aggref *) node;
 
 				strcat(buf, "\"");
-				strcat(buf, agg->aggname);
+				strcat(buf, aggref->aggname);
 				strcat(buf, "\"(");
 				strcat(buf, get_rule_expr(qh, rt_index,
-									 (Node *) (agg->target), varprefix));
+								  (Node *) (aggref->target), varprefix));
 				strcat(buf, ")");
 				return pstrdup(buf);
 			}
@@ -1262,9 +1260,23 @@ get_rule_expr(QryHier *qh, int rt_index, Node *node, bool varprefix)
 		case T_GroupClause:
 			{
 				GroupClause *grp = (GroupClause *) node;
+				List	   *l;
+				TargetEntry *tle = NULL;
 
-				return get_rule_expr(qh, rt_index,
-									 (Node *) (grp->entry), varprefix);
+				foreach(l, qh->query->targetList)
+				{
+					if (((TargetEntry *) lfirst(l))->resdom->resgroupref ==
+						grp->tleGroupref)
+					{
+						tle = (TargetEntry *) lfirst(l);
+						break;
+					}
+				}
+
+				if (tle == NULL)
+					elog(ERROR, "GROUP BY expression not found in targetlist");
+
+				return get_rule_expr(qh, rt_index, (Node *) tle, varprefix);
 			}
 			break;
 
@@ -1343,9 +1355,10 @@ get_rule_expr(QryHier *qh, int rt_index, Node *node, bool varprefix)
 			{
 				Var		   *var = (Var *) node;
 				RangeTblEntry *rte;
-				int sup = var->varlevelsup;
+				int			sup = var->varlevelsup;
 
-				while(sup-- > 0) qh = qh->parent;
+				while (sup-- > 0)
+					qh = qh->parent;
 
 				rte = (RangeTblEntry *) nth(var->varno - 1, qh->query->rtable);
 
@@ -1354,7 +1367,7 @@ get_rule_expr(QryHier *qh, int rt_index, Node *node, bool varprefix)
 				else
 				{
 					if (!strcmp(rte->refname, "*CURRENT*"))
-						strcat(buf, "current.");
+						strcat(buf, "old.");
 					else
 					{
 						if (strcmp(rte->relname, rte->refname) != 0)
@@ -1381,9 +1394,7 @@ get_rule_expr(QryHier *qh, int rt_index, Node *node, bool varprefix)
 			break;
 
 		case T_SubLink:
-			{
-				return get_sublink_expr(qh, rt_index, node, varprefix);
-			}
+			return get_sublink_expr(qh, rt_index, node, varprefix);
 			break;
 
 		case T_Const:
@@ -1408,7 +1419,7 @@ get_rule_expr(QryHier *qh, int rt_index, Node *node, bool varprefix)
 static char *
 get_func_expr(QryHier *qh, int rt_index, Expr *expr, bool varprefix)
 {
-	char		buf[8192];
+	char		buf[BUFSIZE];
 	HeapTuple	proctup;
 	Form_pg_proc procStruct;
 	List	   *l;
@@ -1423,7 +1434,7 @@ get_func_expr(QryHier *qh, int rt_index, Expr *expr, bool varprefix)
 	proctup = SearchSysCacheTuple(PROOID,
 								ObjectIdGetDatum(func->funcid), 0, 0, 0);
 	if (!HeapTupleIsValid(proctup))
-		elog(ERROR, "cache lookup for proc %d failed", func->funcid);
+		elog(ERROR, "cache lookup for proc %u failed", func->funcid);
 
 	procStruct = (Form_pg_proc) GETSTRUCT(proctup);
 	proname = nameout(&(procStruct->proname));
@@ -1515,7 +1526,7 @@ get_tle_expr(QryHier *qh, int rt_index, TargetEntry *tle, bool varprefix)
 	proctup = SearchSysCacheTuple(PROOID,
 								ObjectIdGetDatum(func->funcid), 0, 0, 0);
 	if (!HeapTupleIsValid(proctup))
-		elog(ERROR, "cache lookup for proc %d failed", func->funcid);
+		elog(ERROR, "cache lookup for proc %u failed", func->funcid);
 
 	procStruct = (Form_pg_proc) GETSTRUCT(proctup);
 
@@ -1564,7 +1575,7 @@ get_const_expr(Const *constval)
 	FmgrInfo	finfo_output;
 	char	   *extval;
 	bool		isnull = FALSE;
-	char		buf[8192];
+	char		buf[BUFSIZE];
 	char		namebuf[64];
 
 	if (constval->constisnull)
@@ -1598,10 +1609,10 @@ get_sublink_expr(QryHier *qh, int rt_index, Node *node, bool varprefix)
 {
 	SubLink    *sublink = (SubLink *) node;
 	Query	   *query = (Query *) (sublink->subselect);
-	Expr       *expr;
+	Expr	   *expr;
 	List	   *l;
 	char	   *sep;
-	char       buf[8192];
+	char		buf[BUFSIZE];
 
 	buf[0] = '\0';
 
@@ -1625,32 +1636,33 @@ get_sublink_expr(QryHier *qh, int rt_index, Node *node, bool varprefix)
 			strcat(buf, " ");
 	}
 
-	switch (sublink->subLinkType) {
+	switch (sublink->subLinkType)
+	{
 		case EXISTS_SUBLINK:
 			strcat(buf, "EXISTS ");
 			break;
 
 		case ANY_SUBLINK:
-			expr = (Expr *)lfirst(sublink->oper);
+			expr = (Expr *) lfirst(sublink->oper);
 			strcat(buf, get_opname(((Oper *) (expr->oper))->opno));
 			strcat(buf, " ANY ");
 			break;
 
 		case ALL_SUBLINK:
-			expr = (Expr *)lfirst(sublink->oper);
+			expr = (Expr *) lfirst(sublink->oper);
 			strcat(buf, get_opname(((Oper *) (expr->oper))->opno));
 			strcat(buf, " ALL ");
 			break;
 
 		case EXPR_SUBLINK:
-			expr = (Expr *)lfirst(sublink->oper);
+			expr = (Expr *) lfirst(sublink->oper);
 			strcat(buf, get_opname(((Oper *) (expr->oper))->opno));
 			strcat(buf, " ");
 			break;
 
 		default:
 			elog(ERROR, "unupported sublink type %d",
-					sublink->subLinkType);
+				 sublink->subLinkType);
 			break;
 	}
 
@@ -1675,7 +1687,7 @@ get_relation_name(Oid relid)
 	classtup = SearchSysCacheTuple(RELOID,
 								   ObjectIdGetDatum(relid), 0, 0, 0);
 	if (!HeapTupleIsValid(classtup))
-		elog(ERROR, "cache lookup of relation %d failed", relid);
+		elog(ERROR, "cache lookup of relation %u failed", relid);
 
 	classStruct = (Form_pg_class) GETSTRUCT(classtup);
 	return nameout(&(classStruct->relname));
@@ -1696,7 +1708,7 @@ get_attribute_name(Oid relid, int2 attnum)
 	atttup = SearchSysCacheTuple(ATTNUM,
 						  ObjectIdGetDatum(relid), (Datum) attnum, 0, 0);
 	if (!HeapTupleIsValid(atttup))
-		elog(ERROR, "cache lookup of attribute %d in relation %d failed",
+		elog(ERROR, "cache lookup of attribute %d in relation %u failed",
 			 attnum, relid);
 
 	attStruct = (Form_pg_attribute) GETSTRUCT(atttup);
@@ -1727,22 +1739,17 @@ check_if_rte_used(int rt_index, Node *node, int sup)
 			}
 			break;
 
-		case T_Aggreg:
+		case T_Aggref:
 			{
-				Aggreg	   *agg = (Aggreg *) node;
+				Aggref	   *aggref = (Aggref *) node;
 
 				return check_if_rte_used(rt_index,
-										 (Node *) (agg->target), sup);
+										 (Node *) (aggref->target), sup);
 			}
 			break;
 
 		case T_GroupClause:
-			{
-				GroupClause *grp = (GroupClause *) node;
-
-				return check_if_rte_used(rt_index,
-										 (Node *) (grp->entry), sup);
-			}
+			return FALSE;
 			break;
 
 		case T_Expr:
