@@ -1,6 +1,6 @@
 /*-------------------------------------------------------------------------
  *
- * sets.c--
+ * sets.c
  *	  Functions for sets, which are defined by queries.
  *	  Example:	 a set is defined as being the result of the query
  *			retrieve (X.all)
@@ -9,7 +9,7 @@
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/utils/adt/sets.c,v 1.18 1998/09/01 04:32:51 momjian Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/utils/adt/sets.c,v 1.21 1999/02/13 23:19:33 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -109,7 +109,7 @@ SetDefine(char *querystr, char *typename)
 
 		/* change the pg_proc tuple */
 		procrel = heap_openr(ProcedureRelationName);
-		RelationSetLockForWrite(procrel);
+		LockRelation(procrel, AccessExclusiveLock);
 
 		tup = SearchSysCacheTuple(PROOID,
 								  ObjectIdGetDatum(setoid),
@@ -123,10 +123,10 @@ SetDefine(char *querystr, char *typename)
 									  repl);
 
 			setheapoverride(true);
-			heap_replace(procrel, &tup->t_ctid, newtup);
+			heap_replace(procrel, &tup->t_self, newtup, NULL);
 			setheapoverride(false);
 
-			setoid = newtup->t_oid;
+			setoid = newtup->t_data->t_oid;
 		}
 		else
 			elog(ERROR, "setin: could not find new set oid tuple");
@@ -139,7 +139,7 @@ SetDefine(char *querystr, char *typename)
 			CatalogIndexInsert(idescs, Num_pg_proc_indices, procrel, newtup);
 			CatalogCloseIndices(Num_pg_proc_indices, idescs);
 		}
-		RelationUnsetLockForWrite(procrel);
+		UnlockRelation(procrel, AccessExclusiveLock);
 		heap_close(procrel);
 	}
 	return setoid;
