@@ -1,17 +1,13 @@
 /*-------------------------------------------------------------------------
  *
- * catcache.c--
+ * catcache.c
  *	  System catalog cache for tuples matching a key.
  *
  * Copyright (c) 1994, Regents of the University of California
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/utils/cache/catcache.c,v 1.35 1998/10/12 00:53:33 momjian Exp $
- *
- * Notes:
- *		XXX This needs to use exception.h to handle recovery when
- *				an abort occurs during DisableCache.
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/utils/cache/catcache.c,v 1.43 1999/06/04 02:19:45 tgl Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -66,10 +62,11 @@ static long comphash(long l, char *v);
 #define CACHE6_elog(a,b,c,d,e,f,g)
 #endif
 
-CatCache   *Caches = NULL;
-GlobalMemory CacheCxt;
+static CatCache   *Caches = NULL; /* head of list of caches */
+
+GlobalMemory CacheCxt;			/* context in which caches are allocated */
+/* CacheCxt is global because relcache uses it too. */
 
-static int	DisableCache;
 
 /* ----------------
  *		EQPROC is used in CatalogCacheInitializeCache
@@ -183,7 +180,7 @@ CatalogCacheInitializeCache(struct catcache * cache,
 	cache->relationId = RelationGetRelid(relation);
 	tupdesc = cache->cc_tupdesc = RelationGetDescr(relation);
 
-	CACHE3_elog(DEBUG, "CatalogCacheInitializeCache: relid %d, %d keys",
+	CACHE3_elog(DEBUG, "CatalogCacheInitializeCache: relid %u, %d keys",
 				cache->relationId, cache->cc_nkeys);
 
 	/* ----------------
@@ -211,8 +208,7 @@ CatalogCacheInitializeCache(struct catcache * cache,
 			else
 				cache->cc_klen[i] = tupdesc->attrs[cache->cc_key[i] - 1]->attlen;
 
-			cache->cc_skey[i].sk_procedure =
-				EQPROC(tupdesc->attrs[cache->cc_key[i] - 1]->atttypid);
+			cache->cc_skey[i].sk_procedure = EQPROC(tupdesc->attrs[cache->cc_key[i] - 1]->atttypid);
 
 			fmgr_info(cache->cc_skey[i].sk_procedure,
 					  &cache->cc_skey[i].sk_func);
@@ -279,7 +275,7 @@ CatalogCacheSetId(CatCache *cacheInOutP, int id)
 #endif
 
 /* ----------------
- * comphash --
+ * comphash
  *		Compute a hash value, somehow.
  *
  * XXX explain algorithm here.
@@ -386,7 +382,7 @@ CatalogCacheComputeTupleHashIndex(struct catcache * cacheInOutP,
 		case 4:
 			cacheInOutP->cc_skey[3].sk_argument =
 				(cacheInOutP->cc_key[3] == ObjectIdAttributeNumber)
-				? (Datum) tuple->t_oid
+				? (Datum) tuple->t_data->t_oid
 				: fastgetattr(tuple,
 							  cacheInOutP->cc_key[3],
 							  RelationGetDescr(relation),
@@ -396,7 +392,7 @@ CatalogCacheComputeTupleHashIndex(struct catcache * cacheInOutP,
 		case 3:
 			cacheInOutP->cc_skey[2].sk_argument =
 				(cacheInOutP->cc_key[2] == ObjectIdAttributeNumber)
-				? (Datum) tuple->t_oid
+				? (Datum) tuple->t_data->t_oid
 				: fastgetattr(tuple,
 							  cacheInOutP->cc_key[2],
 							  RelationGetDescr(relation),
@@ -406,7 +402,7 @@ CatalogCacheComputeTupleHashIndex(struct catcache * cacheInOutP,
 		case 2:
 			cacheInOutP->cc_skey[1].sk_argument =
 				(cacheInOutP->cc_key[1] == ObjectIdAttributeNumber)
-				? (Datum) tuple->t_oid
+				? (Datum) tuple->t_data->t_oid
 				: fastgetattr(tuple,
 							  cacheInOutP->cc_key[1],
 							  RelationGetDescr(relation),
@@ -416,7 +412,7 @@ CatalogCacheComputeTupleHashIndex(struct catcache * cacheInOutP,
 		case 1:
 			cacheInOutP->cc_skey[0].sk_argument =
 				(cacheInOutP->cc_key[0] == ObjectIdAttributeNumber)
-				? (Datum) tuple->t_oid
+				? (Datum) tuple->t_data->t_oid
 				: fastgetattr(tuple,
 							  cacheInOutP->cc_key[0],
 							  RelationGetDescr(relation),
@@ -430,8 +426,7 @@ CatalogCacheComputeTupleHashIndex(struct catcache * cacheInOutP,
 			break;
 	}
 
-	return
-		CatalogCacheComputeHashIndex(cacheInOutP);
+	return CatalogCacheComputeHashIndex(cacheInOutP);
 }
 
 /* --------------------------------
@@ -513,7 +508,7 @@ CatalogCacheIdInvalidate(int cacheId,	/* XXX */
 			 elt = DLGetSucc(elt))
 		{
 			ct = (CatCTup *) DLE_VAL(elt);
-			if (ItemPointerEquals(pointer, &ct->ct_tup->t_ctid))
+			if (ItemPointerEquals(pointer, &ct->ct_tup->t_self))
 				break;
 		}
 
@@ -561,16 +556,7 @@ ResetSystemCache()
 	MemoryContext oldcxt;
 	struct catcache *cache;
 
-	/* ----------------
-	 *	sanity checks
-	 * ----------------
-	 */
 	CACHE1_elog(DEBUG, "ResetSystemCache called");
-	if (DisableCache)
-	{
-		elog(ERROR, "ResetSystemCache: Called while cache disabled");
-		return;
-	}
 
 	/* ----------------
 	 *	first switch to the cache context so our allocations
@@ -604,11 +590,13 @@ ResetSystemCache()
 			{
 				nextelt = DLGetSucc(elt);
 				CatCacheRemoveCTup(cache, elt);
-				if (cache->cc_ntup == -1)
-					elog(ERROR, "ResetSystemCache: cc_ntup<0 (software error)");
+				if (cache->cc_ntup < 0)
+					elog(NOTICE,
+						 "ResetSystemCache: cc_ntup<0 (software error)");
 			}
 		}
 		cache->cc_ntup = 0;		/* in case of WARN error above */
+		cache->busy = false;	/* to recover from recursive-use error */
 	}
 
 	CACHE1_elog(DEBUG, "end of ResetSystemCache call");
@@ -623,10 +611,18 @@ ResetSystemCache()
 /* --------------------------------
  *		SystemCacheRelationFlushed
  *
- *	RelationFlushRelation() frees some information referenced in the
- *	cache structures. So we get informed when this is done and arrange
- *	for the next SearchSysCache() call that this information is setup
- *	again.
+ *	This is called by RelationFlushRelation() to clear out cached information
+ *	about a relation being dropped.  (This could be a DROP TABLE command,
+ *	or a temp table being dropped at end of transaction, or a table created
+ *	during the current transaction that is being dropped because of abort.)
+ *	Remove all cache entries relevant to the specified relation OID.
+ *
+ *	A special case occurs when relId is itself one of the cacheable system
+ *	tables --- although those'll never be dropped, they can get flushed from
+ *	the relcache (VACUUM causes this, for example).  In that case we need to
+ *	force the next SearchSysCache() call to reinitialize the cache itself,
+ *	because we have info (such as cc_tupdesc) that is pointing at the about-
+ *	to-be-deleted relcache entry.
  * --------------------------------
  */
 void
@@ -634,6 +630,18 @@ SystemCacheRelationFlushed(Oid relId)
 {
 	struct catcache *cache;
 
+	/*
+	 * XXX Ideally we'd search the caches and just zap entries that actually
+	 * refer to the indicated relation.  For now, we take the brute-force
+	 * approach: just flush the caches entirely.
+	 */
+	ResetSystemCache();
+
+	/*
+	 * If relcache is dropping a system relation's cache entry, mark the
+	 * associated cache structures invalid, so we can rebuild them from
+	 * scratch (not just repopulate them) next time they are used.
+	 */
 	for (cache = Caches; PointerIsValid(cache); cache = cache->cc_next)
 	{
 		if (cache->relationId == relId)
@@ -653,7 +661,7 @@ SystemCacheRelationFlushed(Oid relId)
 #ifdef CACHEDEBUG
 #define InitSysCache_DEBUG1 \
 do { \
-	elog(DEBUG, "InitSysCache: rid=%d id=%d nkeys=%d size=%d\n", \
+	elog(DEBUG, "InitSysCache: rid=%u id=%d nkeys=%d size=%d\n", \
 		cp->relationId, cp->id, cp->cc_nkeys, cp->cc_size); \
 	for (i = 0; i < nkeys; i += 1) \
 	{ \
@@ -748,6 +756,7 @@ InitSysCache(char *relname,
 	cp->cc_indname = indname;
 	cp->cc_tupdesc = (TupleDesc) NULL;
 	cp->id = id;
+	cp->busy = false;
 	cp->cc_maxtup = MAXTUP;
 	cp->cc_size = NCCBUCK;
 	cp->cc_nkeys = nkeys;
@@ -878,16 +887,18 @@ SearchSysCache(struct catcache * cache,
 
 	/* ----------------
 	 *	if we found a tuple in the cache, move it to the top of the
-	 *	lru list, and return it.
+	 *	lru list, and return it.  We also move it to the front of the
+	 *	list for its hashbucket, in order to speed subsequent searches.
+	 *	(The most frequently accessed elements in any hashbucket will
+	 *	tend to be near the front of the hashbucket's list.)
 	 * ----------------
 	 */
 	if (elt)
 	{
-		Dlelem	   *old_lru_elt;
+		Dlelem	   *old_lru_elt = ((CatCTup *) DLE_VAL(elt))->ct_node;
 
-		old_lru_elt = ((CatCTup *) DLE_VAL(elt))->ct_node;
-		DLRemove(old_lru_elt);
-		DLAddHead(cache->cc_lrulist, old_lru_elt);
+		DLMoveToFront(old_lru_elt);
+		DLMoveToFront(elt);
 
 #ifdef CACHEDEBUG
 		relation = heap_open(cache->relationId);
@@ -902,19 +913,23 @@ SearchSysCache(struct catcache * cache,
 	/* ----------------
 	 *	Tuple was not found in cache, so we have to try and
 	 *	retrieve it directly from the relation.  If it's found,
-	 *	we add it to the cache.  We must avoid recursion here,
-	 *	so we disable cache operations.  If operations are
-	 *	currently disabled and we couldn't find the requested item
-	 *	in the cache, then this may be a recursive request, and we
-	 *	abort with an error.
+	 *	we add it to the cache.
+	 *
+	 *	To guard against possible infinite recursion, we mark this cache
+	 *	"busy" while trying to load a new entry for it.  It is OK to
+	 *	recursively invoke SearchSysCache for a different cache, but
+	 *	a recursive call for the same cache will error out.  (We could
+	 *	store the specific key(s) being looked for, and consider only
+	 *	a recursive request for the same key to be an error, but this
+	 *	simple scheme is sufficient for now.)
 	 * ----------------
 	 */
 
-	if (DisableCache)
+	if (cache->busy)
 	{
-		elog(ERROR, "SearchSysCache: Called while cache disabled");
-		return (HeapTuple) NULL;
+		elog(ERROR, "SearchSysCache: recursive use of cache %d", cache->id);
 	}
+	cache->busy = true;
 
 	/* ----------------
 	 *	open the relation associated with the cache
@@ -925,10 +940,9 @@ SearchSysCache(struct catcache * cache,
 				RelationGetRelationName(relation));
 
 	/* ----------------
-	 *	DisableCache and then switch to the cache memory context.
+	 *	Switch to the cache memory context.
 	 * ----------------
 	 */
-	DisableCache = 1;
 
 	if (!CacheCxt)
 		CacheCxt = CreateGlobalMemory("Cache");
@@ -1011,7 +1025,7 @@ SearchSysCache(struct catcache * cache,
 		MemoryContextSwitchTo((MemoryContext) CacheCxt);
 	}
 
-	DisableCache = 0;
+	cache->busy = false;
 
 	/* ----------------
 	 *	scan is complete.  if tup is valid, we copy it and add the copy to
@@ -1046,7 +1060,8 @@ SearchSysCache(struct catcache * cache,
 		DLAddHead(cache->cc_cache[hash], elt);
 
 		/* ----------------
-		 *	deal with hash bucket overflow
+		 *	If we've exceeded the desired size of this cache,
+		 *	throw away the least recently used entry.
 		 * ----------------
 		 */
 		if (++cache->cc_ntup > cache->cc_maxtup)
@@ -1056,13 +1071,12 @@ SearchSysCache(struct catcache * cache,
 			elt = DLGetTail(cache->cc_lrulist);
 			ct = (CatCTup *) DLE_VAL(elt);
 
-			if (ct != nct)
+			if (ct != nct)		/* shouldn't be possible, but be safe... */
 			{
 				CACHE2_elog(DEBUG, "SearchSysCache(%s): Overflow, LRU removal",
 							RelationGetRelationName(relation));
 
 				CatCacheRemoveCTup(cache, elt);
-
 			}
 		}
 
@@ -1141,7 +1155,7 @@ RelationInvalidateCatalogCacheTuple(Relation relation,
 
 		(*function) (ccp->id,
 				 CatalogCacheComputeTupleHashIndex(ccp, relation, tuple),
-					 &tuple->t_ctid);
+					 &tuple->t_self);
 
 		heap_close(relation);
 	}
