@@ -1,13 +1,13 @@
 /*-------------------------------------------------------------------------
  *
- * relcache.c--
+ * relcache.c
  *	  POSTGRES relation descriptor cache code
  *
  * Copyright (c) 1994, Regents of the University of California
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/utils/cache/relcache.c,v 1.50 1998/09/01 04:33:02 momjian Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/utils/cache/relcache.c,v 1.62 1999/05/25 16:12:23 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -77,7 +77,6 @@
 #include "utils/memutils.h"
 #include "utils/rel.h"
 #include "utils/relcache.h"
-#include "utils/relcache.h"
 #include "utils/syscache.h"
 
 
@@ -87,13 +86,6 @@ static Relation RelationNameCacheGetRelation(char *relationName);
 static void init_irels(void);
 static void write_irels(void);
 
-/* ----------------
- *		defines
- * ----------------
- */
-#define private static
-#define INIT_FILENAME	"pg_internal.init"
-
 /* ----------------
  *		externs
  * ----------------
@@ -235,7 +227,7 @@ do { \
 static void formrdesc(char *relationName, u_int natts,
 		  FormData_pg_attribute *att);
 
-#if 0							/* See comments at line 1304 */
+#ifdef NOT_USED					/* See comments at line 1304 */
 static void RelationFlushIndexes(Relation *r, Oid accessMethodId);
 
 #endif
@@ -288,7 +280,7 @@ BuildDescInfoError(RelationBuildDescInfo buildinfo)
 	switch (buildinfo.infotype)
 	{
 		case INFO_RELID:
-			sprintf(errBuf, "(relation id %d)", buildinfo.i.info_id);
+			sprintf(errBuf, "(relation id %u)", buildinfo.i.info_id);
 			break;
 		case INFO_RELNAME:
 			sprintf(errBuf, "(relation name %s)", buildinfo.i.info_name);
@@ -363,8 +355,6 @@ scan_pg_rel_seq(RelationBuildDescInfo buildinfo)
 	 * ----------------
 	 */
 	pg_class_desc = heap_openr(RelationRelationName);
-	if (!IsInitProcessingMode())
-		RelationSetLockForRead(pg_class_desc);
 	pg_class_scan = heap_beginscan(pg_class_desc, 0, SnapshotNow, 1, &key);
 	pg_class_tuple = heap_getnext(pg_class_scan, 0);
 
@@ -383,16 +373,11 @@ scan_pg_rel_seq(RelationBuildDescInfo buildinfo)
 		 *	this bug is discovered and killed by wei on 9/27/91.
 		 * -------------------
 		 */
-		return_tuple = (HeapTuple) palloc((Size) pg_class_tuple->t_len);
-		memmove((char *) return_tuple,
-				(char *) pg_class_tuple,
-				(int) pg_class_tuple->t_len);
+		return_tuple = heap_copytuple(pg_class_tuple);
 	}
 
 	/* all done */
 	heap_endscan(pg_class_scan);
-	if (!IsInitProcessingMode())
-		RelationUnsetLockForRead(pg_class_desc);
 	heap_close(pg_class_desc);
 
 	return return_tuple;
@@ -406,7 +391,7 @@ scan_pg_rel_ind(RelationBuildDescInfo buildinfo)
 
 	pg_class_desc = heap_openr(RelationRelationName);
 	if (!IsInitProcessingMode())
-		RelationSetLockForRead(pg_class_desc);
+		LockRelation(pg_class_desc, AccessShareLock);
 
 	switch (buildinfo.infotype)
 	{
@@ -431,7 +416,7 @@ scan_pg_rel_ind(RelationBuildDescInfo buildinfo)
 
 	/* all done */
 	if (!IsInitProcessingMode())
-		RelationUnsetLockForRead(pg_class_desc);
+		UnlockRelation(pg_class_desc, AccessShareLock);
 	heap_close(pg_class_desc);
 
 	return return_tuple;
@@ -560,7 +545,7 @@ build_tupdesc_seq(RelationBuildDescInfo buildinfo,
 	}
 
 	if (need > 0)
-		elog(ERROR, "catalog is missing %d attribute%s for relid %d",
+		elog(ERROR, "catalog is missing %d attribute%s for relid %u",
 			 need, (need == 1 ? "" : "s"), RelationGetRelid(relation));
 
 	/* ----------------
@@ -718,29 +703,24 @@ RelationBuildRuleLock(Relation relation)
 
 		rule = (RewriteRule *) palloc(sizeof(RewriteRule));
 
-		rule->ruleId = pg_rewrite_tuple->t_oid;
+		rule->ruleId = pg_rewrite_tuple->t_data->t_oid;
 
-		rule->event =
-			(int) heap_getattr(pg_rewrite_tuple,
+		rule->event = (int) heap_getattr(pg_rewrite_tuple,
 							 Anum_pg_rewrite_ev_type, pg_rewrite_tupdesc,
-							   &isnull) - 48;
-		rule->attrno =
-			(int) heap_getattr(pg_rewrite_tuple,
+										 &isnull) - 48;
+		rule->attrno = (int) heap_getattr(pg_rewrite_tuple,
 							 Anum_pg_rewrite_ev_attr, pg_rewrite_tupdesc,
-							   &isnull);
-		rule->isInstead =
-			!!heap_getattr(pg_rewrite_tuple,
-						   Anum_pg_rewrite_is_instead, pg_rewrite_tupdesc,
-						   &isnull);
+										  &isnull);
+		rule->isInstead = !!heap_getattr(pg_rewrite_tuple,
+						  Anum_pg_rewrite_is_instead, pg_rewrite_tupdesc,
+										 &isnull);
 
-		ruleaction =
-			heap_getattr(pg_rewrite_tuple,
-						 Anum_pg_rewrite_ev_action, pg_rewrite_tupdesc,
-						 &isnull);
-		rule_evqual_string =
-			heap_getattr(pg_rewrite_tuple,
-						 Anum_pg_rewrite_ev_qual, pg_rewrite_tupdesc,
-						 &isnull);
+		ruleaction = heap_getattr(pg_rewrite_tuple,
+						   Anum_pg_rewrite_ev_action, pg_rewrite_tupdesc,
+								  &isnull);
+		rule_evqual_string = heap_getattr(pg_rewrite_tuple,
+							 Anum_pg_rewrite_ev_qual, pg_rewrite_tupdesc,
+										  &isnull);
 
 		ruleaction = PointerGetDatum(textout((struct varlena *) DatumGetPointer(ruleaction)));
 		rule_evqual_string = PointerGetDatum(textout((struct varlena *) DatumGetPointer(rule_evqual_string)));
@@ -752,8 +732,7 @@ RelationBuildRuleLock(Relation relation)
 		if (numlocks == maxlocks)
 		{
 			maxlocks *= 2;
-			rules =
-				(RewriteRule **) repalloc(rules, sizeof(RewriteRule *) * maxlocks);
+			rules = (RewriteRule **) repalloc(rules, sizeof(RewriteRule *) * maxlocks);
 		}
 	}
 
@@ -828,7 +807,6 @@ RelationBuildDesc(RelationBuildDescInfo buildinfo)
 	 */
 	if (!HeapTupleIsValid(pg_class_tuple))
 	{
-
 		MemoryContextSwitchTo(oldcxt);
 
 		return NULL;
@@ -838,7 +816,7 @@ RelationBuildDesc(RelationBuildDescInfo buildinfo)
 	 *	get information from the pg_class_tuple
 	 * ----------------
 	 */
-	relid = pg_class_tuple->t_oid;
+	relid = pg_class_tuple->t_data->t_oid;
 	relp = (Form_pg_class) GETSTRUCT(pg_class_tuple);
 	natts = relp->relnatts;
 
@@ -873,10 +851,7 @@ RelationBuildDesc(RelationBuildDescInfo buildinfo)
 	 * ----------------
 	 */
 	if (OidIsValid(relam))
-	{
-		relation->rd_am = (Form_pg_am)
-			AccessMethodObjectIdGetForm(relam);
-	}
+		relation->rd_am = (Form_pg_am) AccessMethodObjectIdGetForm(relam);
 
 	/* ----------------
 	 *	initialize the tuple descriptor (relation->rd_att).
@@ -934,7 +909,6 @@ RelationBuildDesc(RelationBuildDescInfo buildinfo)
 	 *	restore memory context and return the new reldesc.
 	 * ----------------
 	 */
-
 	RelationCacheInsert(relation);
 
 	/* -------------------
@@ -1045,8 +1019,7 @@ formrdesc(char *relationName,
 	if (IsSystemRelationName(relationName))
 	{
 		relation->rd_rel->relowner = 6; /* XXX use sym const */
-		relation->rd_rel->relisshared =
-			IsSharedSystemRelationName(relationName);
+		relation->rd_rel->relisshared = IsSharedSystemRelationName(relationName);
 	}
 	else
 	{
@@ -1066,8 +1039,7 @@ formrdesc(char *relationName,
 	 */
 	for (i = 0; i < natts; i++)
 	{
-		relation->rd_att->attrs[i] =
-			(Form_pg_attribute) palloc(ATTRIBUTE_TUPLE_SIZE);
+		relation->rd_att->attrs[i] = (Form_pg_attribute) palloc(ATTRIBUTE_TUPLE_SIZE);
 
 		MemSet((char *) relation->rd_att->attrs[i], 0,
 			   ATTRIBUTE_TUPLE_SIZE);
@@ -1129,7 +1101,6 @@ RelationIdCacheGetRelation(Oid relationId)
 		}
 
 		RelationIncrementReferenceCount(rd);
-		RelationSetLockForDescriptorOpen(rd);
 
 	}
 
@@ -1162,7 +1133,6 @@ RelationNameCacheGetRelation(char *relationName)
 		}
 
 		RelationIncrementReferenceCount(rd);
-		RelationSetLockForDescriptorOpen(rd);
 
 	}
 
@@ -1206,8 +1176,7 @@ RelationIdGetRelation(Oid relationId)
 	buildinfo.i.info_id = relationId;
 
 	rd = RelationBuildDesc(buildinfo);
-	return
-		rd;
+	return rd;
 }
 
 /* --------------------------------
@@ -1319,7 +1288,7 @@ RelationFlushRelation(Relation *relationPtr,
 
 		FreeTriggerDesc(relation);
 
-#if 0
+#ifdef NOT_USED
 		if (relation->rd_rules)
 		{
 			int			j;
@@ -1341,8 +1310,9 @@ RelationFlushRelation(Relation *relationPtr,
 
 /* --------------------------------
  *		RelationForgetRelation -
- *		   RelationFlushRelation + if the relation is local then get rid of
- *		   the relation descriptor from the newly created relation list.
+ *		   RelationFlushRelation + if the relation is myxactonly then
+ *		   get rid of the relation descriptor from the newly created
+ *		   relation list.
  * --------------------------------
  */
 void
@@ -1351,37 +1321,39 @@ RelationForgetRelation(Oid rid)
 	Relation	relation;
 
 	RelationIdCacheLookup(rid, relation);
-	Assert(PointerIsValid(relation));
 
-	if (relation->rd_islocal)
+	if (PointerIsValid(relation))
 	{
-		MemoryContext oldcxt;
-		List	   *curr;
-		List	   *prev = NIL;
-
-		oldcxt = MemoryContextSwitchTo((MemoryContext) CacheCxt);
-
-		foreach(curr, newlyCreatedRelns)
+		if (relation->rd_myxactonly)
 		{
-			Relation	reln = lfirst(curr);
+			MemoryContext oldcxt;
+			List	   *curr;
+			List	   *prev = NIL;
 
-			Assert(reln != NULL && reln->rd_islocal);
-			if (RelationGetRelid(reln) == rid)
-				break;
-			prev = curr;
+			oldcxt = MemoryContextSwitchTo((MemoryContext) CacheCxt);
+
+			foreach(curr, newlyCreatedRelns)
+			{
+				Relation	reln = lfirst(curr);
+
+				Assert(reln != NULL && reln->rd_myxactonly);
+				if (RelationGetRelid(reln) == rid)
+					break;
+				prev = curr;
+			}
+			if (curr == NIL)
+				elog(FATAL, "Local relation %s not found in list",
+					 (RelationGetRelationName(relation))->data);
+			if (prev == NIL)
+				newlyCreatedRelns = lnext(newlyCreatedRelns);
+			else
+				lnext(prev) = lnext(curr);
+			pfree(curr);
+			MemoryContextSwitchTo(oldcxt);
 		}
-		if (curr == NIL)
-			elog(FATAL, "Local relation %s not found in list",
-				 (RelationGetRelationName(relation))->data);
-		if (prev == NIL)
-			newlyCreatedRelns = lnext(newlyCreatedRelns);
-		else
-			lnext(prev) = lnext(curr);
-		pfree(curr);
-		MemoryContextSwitchTo(oldcxt);
-	}
 
-	RelationFlushRelation(&relation, false);
+		RelationFlushRelation(&relation, false);
+	}
 }
 
 /* --------------------------------
@@ -1402,7 +1374,7 @@ RelationIdInvalidateRelationCacheByRelationId(Oid relationId)
 	 * BufferSync also? But I'll leave it for now since I don't want to
 	 * break anything.) - ay 3/95
 	 */
-	if (PointerIsValid(relation) && !relation->rd_islocal)
+	if (PointerIsValid(relation) && !relation->rd_myxactonly)
 	{
 
 		/*
@@ -1446,7 +1418,7 @@ RelationFlushIndexes(Relation *r,
 void
 RelationIdInvalidateRelationCacheByAccessMethodId(Oid accessMethodId)
 {
-#if 0
+#ifdef NOT_USED
 
 	/*
 	 * 25 aug 1992:  mao commented out the ht walk below.  it should be
@@ -1511,13 +1483,13 @@ RelationRegisterRelation(Relation relation)
 
 	/*
 	 * we've just created the relation. It is invisible to anyone else
-	 * before the transaction is committed. Setting rd_islocal allows us
-	 * to use the local buffer manager for select/insert/etc before the
+	 * before the transaction is committed. Setting rd_myxactonly allows
+	 * us to use the local buffer manager for select/insert/etc before the
 	 * end of transaction. (We also need to keep track of relations
 	 * created during a transaction and does the necessary clean up at the
 	 * end of the transaction.)				- ay 3/95
 	 */
-	relation->rd_islocal = TRUE;
+	relation->rd_myxactonly = TRUE;
 	newlyCreatedRelns = lcons(relation, newlyCreatedRelns);
 
 	MemoryContextSwitchTo(oldcxt);
@@ -1525,7 +1497,7 @@ RelationRegisterRelation(Relation relation)
 
 /*
  * RelationPurgeLocalRelation -
- *	  find all the Relation descriptors marked rd_islocal and reset them.
+ *	  find all the Relation descriptors marked rd_myxactonly and reset them.
  *	  This should be called at the end of a transaction (commit/abort) when
  *	  the "local" relations will become visible to others and the multi-user
  *	  buffer pool should be used.
@@ -1545,7 +1517,7 @@ RelationPurgeLocalRelation(bool xactCommitted)
 		List	   *l = newlyCreatedRelns;
 		Relation	reln = lfirst(l);
 
-		Assert(reln != NULL && reln->rd_islocal);
+		Assert(reln != NULL && reln->rd_myxactonly);
 
 		if (!xactCommitted)
 		{
@@ -1554,18 +1526,18 @@ RelationPurgeLocalRelation(bool xactCommitted)
 			 * remove the file if we abort. This is so that files for
 			 * tables created inside a transaction block get removed.
 			 */
-			if (reln->rd_istemp)
+			if (reln->rd_isnoname)
 			{
-				if (!(reln->rd_tmpunlinked))
+				if (!(reln->rd_nonameunlinked))
 				{
 					smgrunlink(DEFAULT_SMGR, reln);
-					reln->rd_tmpunlinked = TRUE;
+					reln->rd_nonameunlinked = TRUE;
 				}
 			}
 			else
 				smgrunlink(DEFAULT_SMGR, reln);
 		}
-		else if (!IsBootstrapProcessingMode() && !(reln->rd_istemp))
+		else if (!IsBootstrapProcessingMode() && !(reln->rd_isnoname))
 
 			/*
 			 * RelationFlushRelation () below will flush relation
@@ -1577,7 +1549,7 @@ RelationPurgeLocalRelation(bool xactCommitted)
 			 */
 			smgrclose(DEFAULT_SMGR, reln);
 
-		reln->rd_islocal = FALSE;
+		reln->rd_myxactonly = FALSE;
 
 		if (!IsBootstrapProcessingMode())
 			RelationFlushRelation(&reln, FALSE);
@@ -1661,11 +1633,10 @@ AttrDefaultFetch(Relation relation)
 	Relation	adrel;
 	Relation	irel;
 	ScanKeyData skey;
-	HeapTuple	tuple;
+	HeapTupleData tuple;
 	Form_pg_attrdef adform;
 	IndexScanDesc sd;
 	RetrieveIndexResult indexRes;
-	ItemPointer iptr;
 	struct varlena *val;
 	bool		isnull;
 	int			found;
@@ -1680,7 +1651,7 @@ AttrDefaultFetch(Relation relation)
 	adrel = heap_openr(AttrDefaultRelationName);
 	irel = index_openr(AttrDefaultIndex);
 	sd = index_beginscan(irel, false, 1, &skey);
-	tuple = (HeapTuple) NULL;
+	tuple.t_data = NULL;
 
 	for (found = 0;;)
 	{
@@ -1690,13 +1661,13 @@ AttrDefaultFetch(Relation relation)
 		if (!indexRes)
 			break;
 
-		iptr = &indexRes->heap_iptr;
-		tuple = heap_fetch(adrel, SnapshotNow, iptr, &buffer);
+		tuple.t_self = indexRes->heap_iptr;
+		heap_fetch(adrel, SnapshotNow, &tuple, &buffer);
 		pfree(indexRes);
-		if (!HeapTupleIsValid(tuple))
+		if (tuple.t_data == NULL)
 			continue;
 		found++;
-		adform = (Form_pg_attrdef) GETSTRUCT(tuple);
+		adform = (Form_pg_attrdef) GETSTRUCT(&tuple);
 		for (i = 0; i < ndef; i++)
 		{
 			if (adform->adnum != attrdef[i].adnum)
@@ -1706,7 +1677,7 @@ AttrDefaultFetch(Relation relation)
 				relation->rd_att->attrs[adform->adnum - 1]->attname.data,
 					 relation->rd_rel->relname.data);
 
-			val = (struct varlena *) fastgetattr(tuple,
+			val = (struct varlena *) fastgetattr(&tuple,
 												 Anum_pg_attrdef_adbin,
 												 adrel->rd_att, &isnull);
 			if (isnull)
@@ -1714,7 +1685,7 @@ AttrDefaultFetch(Relation relation)
 				relation->rd_att->attrs[adform->adnum - 1]->attname.data,
 					 relation->rd_rel->relname.data);
 			attrdef[i].adbin = textout(val);
-			val = (struct varlena *) fastgetattr(tuple,
+			val = (struct varlena *) fastgetattr(&tuple,
 												 Anum_pg_attrdef_adsrc,
 												 adrel->rd_att, &isnull);
 			if (isnull)
@@ -1750,10 +1721,9 @@ RelCheckFetch(Relation relation)
 	Relation	rcrel;
 	Relation	irel;
 	ScanKeyData skey;
-	HeapTuple	tuple;
+	HeapTupleData tuple;
 	IndexScanDesc sd;
 	RetrieveIndexResult indexRes;
-	ItemPointer iptr;
 	Name		rcname;
 	struct varlena *val;
 	bool		isnull;
@@ -1768,7 +1738,7 @@ RelCheckFetch(Relation relation)
 	rcrel = heap_openr(RelCheckRelationName);
 	irel = index_openr(RelCheckIndex);
 	sd = index_beginscan(irel, false, 1, &skey);
-	tuple = (HeapTuple) NULL;
+	tuple.t_data = NULL;
 
 	for (found = 0;;)
 	{
@@ -1778,30 +1748,30 @@ RelCheckFetch(Relation relation)
 		if (!indexRes)
 			break;
 
-		iptr = &indexRes->heap_iptr;
-		tuple = heap_fetch(rcrel, SnapshotNow, iptr, &buffer);
+		tuple.t_self = indexRes->heap_iptr;
+		heap_fetch(rcrel, SnapshotNow, &tuple, &buffer);
 		pfree(indexRes);
-		if (!HeapTupleIsValid(tuple))
+		if (tuple.t_data == NULL)
 			continue;
 		if (found == ncheck)
 			elog(ERROR, "RelCheckFetch: unexpected record found for rel %s",
 				 relation->rd_rel->relname.data);
 
-		rcname = (Name) fastgetattr(tuple,
+		rcname = (Name) fastgetattr(&tuple,
 									Anum_pg_relcheck_rcname,
 									rcrel->rd_att, &isnull);
 		if (isnull)
 			elog(ERROR, "RelCheckFetch: rcname IS NULL for rel %s",
 				 relation->rd_rel->relname.data);
 		check[found].ccname = nameout(rcname);
-		val = (struct varlena *) fastgetattr(tuple,
+		val = (struct varlena *) fastgetattr(&tuple,
 											 Anum_pg_relcheck_rcbin,
 											 rcrel->rd_att, &isnull);
 		if (isnull)
 			elog(ERROR, "RelCheckFetch: rcbin IS NULL for rel %s",
 				 relation->rd_rel->relname.data);
 		check[found].ccbin = textout(val);
-		val = (struct varlena *) fastgetattr(tuple,
+		val = (struct varlena *) fastgetattr(&tuple,
 											 Anum_pg_relcheck_rcsrc,
 											 rcrel->rd_att, &isnull);
 		if (isnull)
@@ -1847,11 +1817,20 @@ RelCheckFetch(Relation relation)
  *			  from an initialization file in the data/base/... directory.
  *
  *		   +  If the initialization file isn't there, then we create the
- *			  relation descriptor using sequential scans and write it to
+ *			  relation descriptors using sequential scans and write 'em to
  *			  the initialization file for use by subsequent backends.
  *
- *		This is complicated and interferes with system changes, but
- *		performance is so bad that we're willing to pay the tax.
+ *		We could dispense with the initialization file and just build the
+ *		critical reldescs the hard way on every backend startup, but that
+ *		slows down backend startup noticeably if pg_class is large.
+ *
+ *		As of v6.5, vacuum.c deletes the initialization file at completion
+ *		of a VACUUM, so that it will be rebuilt at the next backend startup.
+ *		This ensures that vacuum-collected stats for the system indexes
+ *		will eventually get used by the optimizer --- otherwise the relcache
+ *		entries for these indexes will show zero sizes forever, since the
+ *		relcache entries are pinned in memory and will never be reloaded
+ *		from pg_class.
  */
 
 /* pg_attnumind, pg_classnameind, pg_classoidind */
@@ -1872,7 +1851,11 @@ init_irels(void)
 	int			i;
 	int			relno;
 
-	if ((fd = FileNameOpenFile(INIT_FILENAME, O_RDONLY, 0600)) < 0)
+#ifndef __CYGWIN32__
+	if ((fd = FileNameOpenFile(RELCACHE_INIT_FILENAME, O_RDONLY, 0600)) < 0)
+#else
+	if ((fd = FileNameOpenFile(RELCACHE_INIT_FILENAME, O_RDONLY | O_BINARY, 0600)) < 0)
+#endif
 	{
 		write_irels();
 		return;
@@ -1883,7 +1866,7 @@ init_irels(void)
 	for (relno = 0; relno < Num_indices_bootstrap; relno++)
 	{
 		/* first read the relation descriptor length */
-		if ((nread = FileRead(fd, (char *) &len, sizeof(int))) != sizeof(int))
+		if ((nread = FileRead(fd, (char *) &len, sizeof(len))) != sizeof(len))
 		{
 			write_irels();
 			return;
@@ -1906,7 +1889,7 @@ init_irels(void)
 		ird->lockInfo = (char *) NULL;
 
 		/* next, read the access method tuple form */
-		if ((nread = FileRead(fd, (char *) &len, sizeof(int))) != sizeof(int))
+		if ((nread = FileRead(fd, (char *) &len, sizeof(len))) != sizeof(len))
 		{
 			write_irels();
 			return;
@@ -1922,7 +1905,7 @@ init_irels(void)
 		ird->rd_am = am;
 
 		/* next read the relation tuple form */
-		if ((nread = FileRead(fd, (char *) &len, sizeof(int))) != sizeof(int))
+		if ((nread = FileRead(fd, (char *) &len, sizeof(len))) != sizeof(len))
 		{
 			write_irels();
 			return;
@@ -1944,7 +1927,7 @@ init_irels(void)
 		len = ATTRIBUTE_TUPLE_SIZE;
 		for (i = 0; i < relform->relnatts; i++)
 		{
-			if ((nread = FileRead(fd, (char *) &len, sizeof(int))) != sizeof(int))
+			if ((nread = FileRead(fd, (char *) &len, sizeof(len))) != sizeof(len))
 			{
 				write_irels();
 				return;
@@ -1960,7 +1943,7 @@ init_irels(void)
 		}
 
 		/* next, read the index strategy map */
-		if ((nread = FileRead(fd, (char *) &len, sizeof(int))) != sizeof(int))
+		if ((nread = FileRead(fd, (char *) &len, sizeof(len))) != sizeof(len))
 		{
 			write_irels();
 			return;
@@ -1992,7 +1975,7 @@ init_irels(void)
 		ird->rd_istrat = strat;
 
 		/* finally, read the vector of support procedures */
-		if ((nread = FileRead(fd, (char *) &len, sizeof(int))) != sizeof(int))
+		if ((nread = FileRead(fd, (char *) &len, sizeof(len))) != sizeof(len))
 		{
 			write_irels();
 			return;
@@ -2019,7 +2002,7 @@ init_irels(void)
 static void
 write_irels(void)
 {
-	int			len;
+	Size		len;
 	int			nwritten;
 	File		fd;
 	Relation	irel[Num_indices_bootstrap];
@@ -2033,9 +2016,13 @@ write_irels(void)
 	int			relno;
 	RelationBuildDescInfo bi;
 
-	fd = FileNameOpenFile(INIT_FILENAME, O_WRONLY | O_CREAT | O_TRUNC, 0600);
+#ifndef __CYGWIN32__
+	fd = FileNameOpenFile(RELCACHE_INIT_FILENAME, O_WRONLY | O_CREAT | O_TRUNC, 0600);
+#else
+	fd = FileNameOpenFile(RELCACHE_INIT_FILENAME, O_WRONLY | O_CREAT | O_TRUNC | O_BINARY, 0600);
+#endif
 	if (fd < 0)
-		elog(FATAL, "cannot create init file %s", INIT_FILENAME);
+		elog(FATAL, "cannot create init file %s", RELCACHE_INIT_FILENAME);
 
 	FileSeek(fd, 0L, SEEK_SET);
 
@@ -2085,8 +2072,8 @@ write_irels(void)
 		len = sizeof(RelationData);
 
 		/* first, write the relation descriptor length */
-		if ((nwritten = FileWrite(fd, (char *) &len, sizeof(int)))
-			!= sizeof(int))
+		if ((nwritten = FileWrite(fd, (char *) &len, sizeof(len)))
+			!= sizeof(len))
 			elog(FATAL, "cannot write init file -- descriptor length");
 
 		/* next, write out the Relation structure */
@@ -2095,8 +2082,8 @@ write_irels(void)
 
 		/* next, write the access method tuple form */
 		len = sizeof(FormData_pg_am);
-		if ((nwritten = FileWrite(fd, (char *) &len, sizeof(int)))
-			!= sizeof(int))
+		if ((nwritten = FileWrite(fd, (char *) &len, sizeof(len)))
+			!= sizeof(len))
 			elog(FATAL, "cannot write init file -- am tuple form length");
 
 		if ((nwritten = FileWrite(fd, (char *) am, len)) != len)
@@ -2104,8 +2091,8 @@ write_irels(void)
 
 		/* next write the relation tuple form */
 		len = sizeof(FormData_pg_class);
-		if ((nwritten = FileWrite(fd, (char *) &len, sizeof(int)))
-			!= sizeof(int))
+		if ((nwritten = FileWrite(fd, (char *) &len, sizeof(len)))
+			!= sizeof(len))
 			elog(FATAL, "cannot write init file -- relation tuple form length");
 
 		if ((nwritten = FileWrite(fd, (char *) relform, len)) != len)
@@ -2115,8 +2102,8 @@ write_irels(void)
 		len = ATTRIBUTE_TUPLE_SIZE;
 		for (i = 0; i < relform->relnatts; i++)
 		{
-			if ((nwritten = FileWrite(fd, (char *) &len, sizeof(int)))
-				!= sizeof(int))
+			if ((nwritten = FileWrite(fd, (char *) &len, sizeof(len)))
+				!= sizeof(len))
 				elog(FATAL, "cannot write init file -- length of attdesc %d", i);
 			if ((nwritten = FileWrite(fd, (char *) ird->rd_att->attrs[i], len))
 				!= len)
@@ -2126,8 +2113,8 @@ write_irels(void)
 		/* next, write the index strategy map */
 		len = AttributeNumberGetIndexStrategySize(relform->relnatts,
 												  am->amstrategies);
-		if ((nwritten = FileWrite(fd, (char *) &len, sizeof(int)))
-			!= sizeof(int))
+		if ((nwritten = FileWrite(fd, (char *) &len, sizeof(len)))
+			!= sizeof(len))
 			elog(FATAL, "cannot write init file -- strategy map length");
 
 		if ((nwritten = FileWrite(fd, (char *) strat, len)) != len)
@@ -2135,8 +2122,8 @@ write_irels(void)
 
 		/* finally, write the vector of support procedures */
 		len = relform->relnatts * (am->amsupport * sizeof(RegProcedure));
-		if ((nwritten = FileWrite(fd, (char *) &len, sizeof(int)))
-			!= sizeof(int))
+		if ((nwritten = FileWrite(fd, (char *) &len, sizeof(len)))
+			!= sizeof(len))
 			elog(FATAL, "cannot write init file -- support vector length");
 
 		if ((nwritten = FileWrite(fd, (char *) support, len)) != len)
