@@ -7,7 +7,7 @@
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/utils/cache/temprel.c,v 1.4 1999/05/25 22:42:16 momjian Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/utils/cache/temprel.c,v 1.6 1999/07/03 15:43:57 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -30,6 +30,7 @@
 #include "nodes/pg_list.h"
 #include "utils/mcxt.h"
 #include "utils/temprel.h"
+#include "access/xact.h"
 #include "access/htup.h"
 #include "access/heapam.h"
 #include "catalog/heap.h"
@@ -79,6 +80,9 @@ remove_all_temp_relations(void)
 	List	   *l,
 			   *next;
 
+	AbortOutOfAnyTransaction();
+	StartTransactionCommand();
+
 	l = temp_rels;
 	while (l != NIL)
 	{
@@ -102,6 +106,7 @@ remove_all_temp_relations(void)
 
 		l = next;
 	}
+	CommitTransactionCommand();
 }
 
 /* we don't have the relname for indexes, so we just pass the oid */
