@@ -1,13 +1,13 @@
 /*-------------------------------------------------------------------------
  *
- * elog.c--
+ * elog.c
  *	  error logger
  *
  * Copyright (c) 1994, Regents of the University of California
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/utils/error/elog.c,v 1.35 1998/09/01 04:33:07 momjian Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/utils/error/elog.c,v 1.45 1999/05/25 16:12:24 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -31,7 +31,9 @@
 #include "postgres.h"
 #include "miscadmin.h"
 #include "libpq/libpq.h"
+#include "libpq/pqformat.h"
 #include "storage/proc.h"
+#include "tcop/tcopprot.h"
 #include "utils/trace.h"
 
 #ifdef USE_SYSLOG
@@ -53,7 +55,7 @@ static int	Err_file = -1;
 static int	ElogDebugIndentLevel = 0;
 
 /*
- * elog --
+ * elog
  *		Old error logging function.
  */
 void
@@ -67,11 +69,6 @@ elog(int lev, const char *fmt,...)
 	extern int	errno,
 				sys_nerr;
 
-#ifndef PG_STANDALONE
-	extern FILE *Pfout;
-
-#endif
-
 #ifdef USE_SYSLOG
 	int			log_level;
 
@@ -133,7 +130,7 @@ elog(int lev, const char *fmt,...)
 		else
 			*bp++ = *cp;
 	*bp = '\0';
-	vsprintf(line, buf, ap);
+	vsnprintf(line, ELOG_MAXLEN - 1, buf, ap);
 	va_end(ap);
 
 #ifdef USE_SYSLOG
@@ -170,7 +167,7 @@ elog(int lev, const char *fmt,...)
 	 * front-end program, write to it first.  This is important because
 	 * there's a bug in the socket code on ultrix.  If the front end has
 	 * gone away (so the channel to it has been closed at the other end),
-	 * then writing here can cause this backend to exit without warning --
+	 * then writing here can cause this backend to exit without warning
 	 * that is, write() does an exit(). In this case, our only hope of
 	 * finding out what's going on is if Err_file was set to some disk
 	 * log.  This is a major pain.
@@ -190,18 +187,39 @@ elog(int lev, const char *fmt,...)
 
 #ifndef PG_STANDALONE
 	/* Send IPC message to the front-end program */
-	if (Pfout != NULL && lev > DEBUG)
+	if (IsUnderPostmaster && lev > DEBUG)
 	{
-		/* notices are not exactly errors, handle it differently */
+		/* notices are not errors, handle 'em differently */
+		char		msgtype;
+
 		if (lev == NOTICE)
-			pq_putnchar("N", 1);
+			msgtype = 'N';
 		else
-			pq_putnchar("E", 1);
-		/* pq_putint(-101, 4); *//* should be query id */
-		pq_putstr(line + TIMESTAMP_SIZE);		/* don't show timestamps */
+		{
+
+			/*
+			 * Abort any COPY OUT in progress when an error is detected.
+			 * This hack is necessary because of poor design of copy
+			 * protocol.
+			 */
+			pq_endcopyout(true);
+			msgtype = 'E';
+		}
+		/* exclude the timestamp from msg sent to frontend */
+		pq_puttextmessage(msgtype, line + TIMESTAMP_SIZE);
+
+		/*
+		 * This flush is normally not necessary, since postgres.c will
+		 * flush out waiting data when control returns to the main loop.
+		 * But it seems best to leave it here, so that the client has some
+		 * clue what happened if the backend dies before getting back to
+		 * the main loop ... error/notice messages should not be a
+		 * performance- critical path anyway, so an extra flush won't hurt
+		 * much ...
+		 */
 		pq_flush();
 	}
-	if (Pfout == NULL)
+	if (!IsUnderPostmaster)
 	{
 
 		/*
@@ -214,20 +232,12 @@ elog(int lev, const char *fmt,...)
 
 	if (lev == ERROR)
 	{
-		extern bool InError;
-
 		ProcReleaseSpins(NULL); /* get rid of spinlocks we hold */
 		if (!InError)
 		{
-			kill(MyProcPid, SIGQUIT);	/* abort to traffic cop */
-			pause();
+			/* exit to main loop */
+			siglongjmp(Warn_restart, 1);
 		}
-
-		/*
-		 * The pause(3) is just to avoid race conditions where the thread
-		 * of control on an MP system gets past here (i.e., the signal is
-		 * not received instantaneously).
-		 */
 	}
 
 	if (lev == FATAL)
@@ -298,8 +308,7 @@ DebugFileOpen(void)
 	fd = fileno(stderr);
 	if (fcntl(fd, F_GETFD, 0) < 0)
 	{
-		sprintf(OutputFileName, "%s/pg.errors.%d",
-				DataDir, (int) MyProcPid);
+		sprintf(OutputFileName, "%s/pg.errors.%d", DataDir, (int) MyProcPid);
 		fd = open(OutputFileName, O_CREAT | O_APPEND | O_WRONLY, 0666);
 	}
 	if (fd < 0)
