@@ -7,7 +7,7 @@
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/utils/error/format.c,v 1.11 1999/05/25 16:12:26 momjian Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/utils/error/format.c,v 1.13 1999/06/19 05:00:28 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -22,11 +22,11 @@ static char FormBuf[FormMaxSize];
 
 
 /* ----------------
- *		form
+ *		vararg_format
  * ----------------
  */
 char *
-form(const char *fmt,...)
+vararg_format(const char *fmt,...)
 {
 	va_list		args;
 
