@@ -1,13 +1,13 @@
 /*-------------------------------------------------------------------------
  *
- * enbl.c--
+ * enbl.c
  *	  POSTGRES module enable and disable support code.
  *
  * Copyright (c) 1994, Regents of the University of California
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/utils/init/enbl.c,v 1.4 1998/09/01 03:26:49 momjian Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/utils/init/enbl.c,v 1.7 1999/05/25 16:12:33 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -15,7 +15,7 @@
 #include "utils/module.h"		/* where the declarations go */
 
 /*
- * BypassEnable --
+ * BypassEnable
  *		False iff enable/disable processing is required given on and "*countP."
  *
  * Note:
@@ -38,7 +38,7 @@ BypassEnable(int *enableCountInOutP, bool on)
 		return (bool) (*enableCountInOutP >= 2);
 	}
 
-	AssertState(*enableCountInOutP >= 1);
+	Assert(*enableCountInOutP >= 1);
 
 	*enableCountInOutP -= 1;
 
