@@ -1,13 +1,13 @@
 /*-------------------------------------------------------------------------
  *
- * miscinit.c--
+ * miscinit.c
  *	  miscellanious initialization support stuff
  *
  * Copyright (c) 1994, Regents of the University of California
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/utils/init/miscinit.c,v 1.24 1998/10/08 18:30:16 momjian Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/utils/init/miscinit.c,v 1.29 1999/05/25 16:12:35 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -37,9 +37,10 @@
 #include "utils/syscache.h"
 
 #include "storage/fd.h"			/* for O_ */
+#include "storage/ipc.h"		/* for proc_exit */
 
 /*
- * EnableAbortEnvVarName --
+ * EnableAbortEnvVarName
  *		Enables system abort iff set to a non-empty string in environment.
  */
 #define EnableAbortEnvVarName	"POSTGRESABORT"
@@ -68,7 +69,7 @@ unsigned char RecodeBackTable[128];
  */
 
 /*
- * ExitPostgres --
+ * ExitPostgres
  *		Exit POSTGRES with a status code.
  *
  * Note:
@@ -88,7 +89,7 @@ ExitPostgres(ExitStatus status)
 }
 
 /*
- * AbortPostgres --
+ * AbortPostgres
  *		Abort POSTGRES dumping core.
  *
  * Note:
@@ -137,6 +138,7 @@ StatusPostmasterExit(int status)
 	/* someday, do some real cleanup and then call the LISP exit */
 	proc_exit(status);
 }
+
 #endif
 
 
@@ -148,7 +150,7 @@ static ProcessingMode Mode = NoProcessing;
 
 #ifdef NOT_USED
 /*
- * IsNoProcessingMode --
+ * IsNoProcessingMode
  *		True iff processing mode is NoProcessing.
  */
 bool
@@ -156,10 +158,11 @@ IsNoProcessingMode()
 {
 	return (bool) (Mode == NoProcessing);
 }
+
 #endif
 
 /*
- * IsBootstrapProcessingMode --
+ * IsBootstrapProcessingMode
  *		True iff processing mode is BootstrapProcessing.
  */
 bool
@@ -169,7 +172,7 @@ IsBootstrapProcessingMode()
 }
 
 /*
- * IsInitProcessingMode --
+ * IsInitProcessingMode
  *		True iff processing mode is InitProcessing.
  */
 bool
@@ -179,7 +182,7 @@ IsInitProcessingMode()
 }
 
 /*
- * IsNormalProcessingMode --
+ * IsNormalProcessingMode
  *		True iff processing mode is NormalProcessing.
  */
 bool
@@ -189,7 +192,7 @@ IsNormalProcessingMode()
 }
 
 /*
- * SetProcessingMode --
+ * SetProcessingMode
  *		Sets mode of processing as specified.
  *
  * Exceptions:
@@ -338,7 +341,11 @@ SetCharSet()
 		map_file = (char *) malloc((strlen(DataDir) +
 									strlen(p) + 2) * sizeof(char));
 		sprintf(map_file, "%s/%s", DataDir, p);
-		file = fopen(map_file, "r");
+#ifndef __CYGWIN32__
+		file = AllocateFile(map_file, "r");
+#else
+		file = AllocateFile(map_file, "rb");
+#endif
 		if (file == NULL)
 			return;
 		eof = false;
@@ -372,7 +379,7 @@ SetCharSet()
 				}
 			}
 		}
-		fclose(file);
+		FreeFile(file);
 		free(map_file);
 	}
 }
