@@ -1,13 +1,13 @@
 /*-------------------------------------------------------------------------
  *
- * postinit.c--
+ * postinit.c
  *	  postgres initialization utilities
  *
  * Copyright (c) 1994, Regents of the University of California
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/utils/init/postinit.c,v 1.35 1998/09/01 04:33:17 momjian Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/utils/init/postinit.c,v 1.40 1999/05/25 16:12:36 momjian Exp $
  *
  * NOTES
  *		InitPostgres() is the function called from PostgresMain
@@ -143,7 +143,7 @@ InitMyDatabaseInfo(char *name)
 
 
 /*
- * DoChdirAndInitDatabaseNameAndPath --
+ * DoChdirAndInitDatabaseNameAndPath
  *		Set current directory to the database directory for the database
  *		named <name>.
  *		Also set global variables DatabasePath and DatabaseName to those
@@ -176,7 +176,11 @@ VerifySystemDatabase()
 
 	errormsg[0] = '\0';
 
+#ifndef __CYGWIN32__
 	if ((fd = open(DataDir, O_RDONLY, 0)) == -1)
+#else
+	if ((fd = open(DataDir, O_RDONLY | O_DIROPEN, 0)) == -1)
+#endif
 		sprintf(errormsg, "Database system does not exist.  "
 				"PGDATA directory '%s' not found.\n\tNormally, you "
 				"create a database system by running initdb.",
@@ -212,7 +216,11 @@ VerifyMyDatabase()
 	name = DatabaseName;
 	myPath = DatabasePath;
 
+#ifndef __CYGWIN32__
 	if ((fd = open(myPath, O_RDONLY, 0)) == -1)
+#else
+	if ((fd = open(myPath, O_RDONLY | O_DIROPEN, 0)) == -1)
+#endif
 		sprintf(errormsg,
 				"Database '%s' does not exist."
 			"\n\tWe know this because the directory '%s' does not exist."
@@ -407,7 +415,7 @@ InitStdio()
 }
 
 /* --------------------------------
- * InitPostgres --
+ * InitPostgres
  *		Initialize POSTGRES.
  *
  * Note:
@@ -507,19 +515,14 @@ InitPostgres(char *name)		/* database name */
 	}
 
 	/*
-	 * ********************************
-	 *
-	 *
-	 *
-	 * code after this point assumes we are in the proper directory!
+	 * Code after this point assumes we are in the proper directory!
 	 *
 	 * So, how do we implement alternate locations for databases? There are
 	 * two possible locations for tables and we need to look in
 	 * DataDir/pg_database to find the true location of an individual
 	 * database. We can brute-force it as done in InitMyDatabaseInfo(), or
 	 * we can be patient and wait until we open pg_database gracefully.
-	 * Will try that, but may not work... - thomas 1997-11-01 ********************************
-	 *
+	 * Will try that, but may not work... - thomas 1997-11-01
 	 */
 
 	/* Does not touch files (?) - thomas 1997-11-01 */
@@ -563,7 +566,7 @@ InitPostgres(char *name)		/* database name */
 	 */
 	InitProcess(PostgresIpcKey);
 
-	if (MyBackendId > MaxBackendId || MyBackendId <= 0)
+	if (MyBackendId > MAXBACKENDS || MyBackendId <= 0)
 	{
 		elog(FATAL, "cinit2: bad backend id %d (%d)",
 			 MyBackendTag,
