@@ -1,13 +1,13 @@
 /*-------------------------------------------------------------------------
  *
- * database.c--
+ * database.c
  *	  miscellanious initialization support stuff
  *
  * Copyright (c) 1994, Regents of the University of California
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/utils/misc/database.c,v 1.20 1998/09/03 02:32:41 momjian Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/utils/misc/database.c,v 1.24 1999/05/25 16:12:47 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -188,7 +188,7 @@ GetRawDatabaseInfo(char *name, int4 *owner, Oid *db_id, char *path, int *encodin
 	int			nbytes;
 	int			max,
 				i;
-	HeapTuple	tup;
+	HeapTupleData tup;
 	Page		pg;
 	PageHeader	ph;
 	char	   *dbfname;
@@ -198,7 +198,11 @@ GetRawDatabaseInfo(char *name, int4 *owner, Oid *db_id, char *path, int *encodin
 	sprintf(dbfname, "%s%cpg_database", DataDir, SEP_CHAR);
 	fileflags = O_RDONLY;
 
+#ifndef __CYGWIN32__
 	if ((dbfd = open(dbfname, O_RDONLY, 0)) < 0)
+#else
+	if ((dbfd = open(dbfname, O_RDONLY | O_BINARY, 0)) < 0)
+#endif
 		elog(FATAL, "Cannot open %s", dbfname);
 
 	pfree(dbfname);
@@ -238,7 +242,7 @@ GetRawDatabaseInfo(char *name, int4 *owner, Oid *db_id, char *path, int *encodin
 
 			/* get a pointer to the tuple itself */
 			offset = (int) ph->pd_linp[i].lp_off;
-			tup = (HeapTuple) (((char *) pg) + offset);
+			tup.t_data = (HeapTupleHeader) (((char *) pg) + offset);
 
 			/*
 			 * if the tuple has been deleted (the database was destroyed),
@@ -253,7 +257,7 @@ GetRawDatabaseInfo(char *name, int4 *owner, Oid *db_id, char *path, int *encodin
 			 * if data is ever moved and no longer the first field this
 			 * will be broken!! -mer 11 Nov 1991.
 			 */
-			if (TransactionIdIsValid((TransactionId) tup->t_xmax))
+			if (TransactionIdIsValid((TransactionId) tup.t_data->t_xmax))
 				continue;
 
 			/*
@@ -267,7 +271,7 @@ GetRawDatabaseInfo(char *name, int4 *owner, Oid *db_id, char *path, int *encodin
 			 * you exactly how big the header actually is. use the PC
 			 * means of getting at sys cat attrs.
 			 */
-			tup_db = (Form_pg_database) GETSTRUCT(tup);
+			tup_db = (Form_pg_database) GETSTRUCT(&tup);
 #ifdef MULTIBYTE
 
 			/*
@@ -279,7 +283,7 @@ GetRawDatabaseInfo(char *name, int4 *owner, Oid *db_id, char *path, int *encodin
 #endif
 			if (strcmp(name, tup_db->datname.data) == 0)
 			{
-				*db_id = tup->t_oid;
+				*db_id = tup.t_data->t_oid;
 				strncpy(path, VARDATA(&(tup_db->datpath)),
 						(VARSIZE(&(tup_db->datpath)) - VARHDRSZ));
 				*(path + VARSIZE(&(tup_db->datpath)) - VARHDRSZ) = '\0';
