@@ -1,13 +1,13 @@
 /*-------------------------------------------------------------------------
  *
- * mcxt.c--
+ * mcxt.c
  *	  POSTGRES memory context code.
  *
  * Copyright (c) 1994, Regents of the University of California
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/utils/mmgr/mcxt.c,v 1.10 1998/09/01 04:33:36 momjian Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/utils/mmgr/mcxt.c,v 1.14 1999/05/25 16:12:53 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -54,7 +54,7 @@ static OrderedSetData ActiveGlobalMemorySetData;		/* uninitialized */
 #define PSIZESPACE(LEN) ((LEN) + sizeof (int32))
 
 /*
- * AllocSizeIsValid --
+ * AllocSizeIsValid
  *		True iff 0 < size and size <= MaxAllocSize.
  */
 #define AllocSizeIsValid(size)	(0 < (size) && (size) <= MaxAllocSize)
@@ -64,10 +64,10 @@ static OrderedSetData ActiveGlobalMemorySetData;		/* uninitialized */
  *****************************************************************************/
 
 /*
- * CurrentMemoryContext --
+ * CurrentMemoryContext
  *		Memory context for general global allocations.
  */
-MemoryContext CurrentMemoryContext = NULL;
+DLLIMPORT MemoryContext CurrentMemoryContext = NULL;
 
 /*****************************************************************************
  *	  PRIVATE DEFINITIONS													 *
@@ -103,16 +103,17 @@ static struct MemoryContextMethodsData GlobalContextMethodsData = {
  */
 /* extern bool EqualGlobalMemory(); */
 
-static struct GlobalMemory TopGlobalMemoryData = {
+static struct GlobalMemoryData TopGlobalMemoryData = {
 	T_GlobalMemory,				/* NodeTag				tag		  */
 	&GlobalContextMethodsData,	/* ContextMethods		method	  */
-	{{0}},						/* uninitialized OrderedSetData allocSetD */
+	{NULL, {NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL}},
+	/* free AllocSet   */
 	"TopGlobal",				/* char* name	   */
 	{0}							/* uninitialized OrderedElemData elemD */
 };
 
 /*
- * TopMemoryContext --
+ * TopMemoryContext
  *		Memory context for general global allocations.
  *
  * Note:
@@ -130,7 +131,7 @@ MemoryContext TopMemoryContext = (MemoryContext) &TopGlobalMemoryData;
  */
 
 /*
- * EnableMemoryContext --
+ * EnableMemoryContext
  *		Enables/disables memory management and global contexts.
  *
  * Note:
@@ -162,7 +163,7 @@ EnableMemoryContext(bool on)
 
 		/* make TopGlobalMemoryData member of ActiveGlobalMemorySet */
 		OrderedSetInit(ActiveGlobalMemorySet,
-					   offsetof(struct GlobalMemory, elemData));
+					   offsetof(struct GlobalMemoryData, elemData));
 		OrderedElemPushInto(&TopGlobalMemoryData.elemData,
 							ActiveGlobalMemorySet);
 
@@ -206,7 +207,7 @@ EnableMemoryContext(bool on)
 }
 
 /*
- * MemoryContextAlloc --
+ * MemoryContextAlloc
  *		Returns pointer to aligned allocated memory in the given context.
  *
  * Note:
@@ -230,7 +231,7 @@ MemoryContextAlloc(MemoryContext context, Size size)
 }
 
 /*
- * MemoryContextFree --
+ * MemoryContextFree
  *		Frees allocated memory referenced by pointer in the given context.
  *
  * Note:
@@ -251,7 +252,7 @@ MemoryContextFree(MemoryContext context, Pointer pointer)
 }
 
 /*
- * MemoryContextRelloc --
+ * MemoryContextRelloc
  *		Returns pointer to aligned allocated memory in the given context.
  *
  * Note:
@@ -277,7 +278,7 @@ MemoryContextRealloc(MemoryContext context,
 }
 
 /*
- * MemoryContextGetName --
+ * MemoryContextGetName
  *		Returns pointer to aligned allocated memory in the given context.
  *
  * Note:
@@ -300,7 +301,7 @@ MemoryContextGetName(MemoryContext context)
 #endif
 
 /*
- * PointerGetAllocSize --
+ * PointerGetAllocSize
  *		Returns size of aligned allocated memory given pointer to it.
  *
  * Note:
@@ -323,7 +324,7 @@ PointerGetAllocSize(Pointer pointer)
 #endif
 
 /*
- * MemoryContextSwitchTo --
+ * MemoryContextSwitchTo
  *		Returns the current context; installs the given context.
  *
  * Note:
@@ -350,7 +351,7 @@ MemoryContextSwitchTo(MemoryContext context)
  * External Functions
  */
 /*
- * CreateGlobalMemory --
+ * CreateGlobalMemory
  *		Returns new global memory context.
  *
  * Note:
@@ -371,7 +372,7 @@ CreateGlobalMemory(char *name)	/* XXX MemoryContextName */
 
 	savecxt = MemoryContextSwitchTo(TopMemoryContext);
 
-	context = (GlobalMemory) newNode(sizeof(struct GlobalMemory), T_GlobalMemory);
+	context = (GlobalMemory) newNode(sizeof(struct GlobalMemoryData), T_GlobalMemory);
 	context->method = &GlobalContextMethodsData;
 	context->name = name;		/* assumes name is static */
 	AllocSetInit(&context->setData, DynamicAllocMode, (Size) 0);
@@ -384,7 +385,7 @@ CreateGlobalMemory(char *name)	/* XXX MemoryContextName */
 }
 
 /*
- * GlobalMemoryDestroy --
+ * GlobalMemoryDestroy
  *		Destroys given global memory context.
  *
  * Exceptions:
@@ -412,7 +413,7 @@ GlobalMemoryDestroy(GlobalMemory context)
  *****************************************************************************/
 
 /*
- * GlobalMemoryAlloc --
+ * GlobalMemoryAlloc
  *		Returns pointer to aligned space in the global context.
  *
  * Exceptions:
@@ -425,7 +426,7 @@ GlobalMemoryAlloc(GlobalMemory this, Size size)
 }
 
 /*
- * GlobalMemoryFree --
+ * GlobalMemoryFree
  *		Frees allocated memory in the global context.
  *
  * Exceptions:
@@ -440,7 +441,7 @@ GlobalMemoryFree(GlobalMemory this,
 }
 
 /*
- * GlobalMemoryRealloc --
+ * GlobalMemoryRealloc
  *		Returns pointer to aligned space in the global context.
  *
  * Note:
@@ -460,7 +461,7 @@ GlobalMemoryRealloc(GlobalMemory this,
 }
 
 /*
- * GlobalMemoryGetName --
+ * GlobalMemoryGetName
  *		Returns name string for context.
  *
  * Exceptions:
@@ -473,7 +474,7 @@ GlobalMemoryGetName(GlobalMemory this)
 }
 
 /*
- * GlobalMemoryDump --
+ * GlobalMemoryDump
  *		Dumps global memory context for debugging.
  *
  * Exceptions:
@@ -498,7 +499,7 @@ GlobalMemoryDump(GlobalMemory this)
 }
 
 /*
- * DumpGlobalMemories --
+ * DumpGlobalMemories
  *		Dumps all global memory contexts for debugging.
  *
  * Exceptions:
