@@ -1,13 +1,13 @@
 /*-------------------------------------------------------------------------
  *
- * oset.c--
+ * oset.c
  *	  Fixed format ordered set definitions.
  *
  * Copyright (c) 1994, Regents of the University of California
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/backend/utils/mmgr/oset.c,v 1.9 1998/09/01 03:27:05 momjian Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/backend/utils/mmgr/oset.c,v 1.13 1999/05/26 12:56:05 momjian Exp $
  *
  * NOTE
  *	  XXX This is a preliminary implementation which lacks fail-fast
@@ -15,7 +15,7 @@
  *
  *-------------------------------------------------------------------------
  */
-#include "c.h"
+#include "postgres.h"
 
 #include "utils/memutils.h"		/* where declarations of this file goes */
 
@@ -24,7 +24,7 @@ static void OrderedElemPush(OrderedElem elem);
 static void OrderedElemPushHead(OrderedElem elem);
 
 /*
- * OrderedElemGetBase --
+ * OrderedElemGetBase
  *		Returns base of enclosing structure.
  */
 static Pointer
@@ -37,7 +37,7 @@ OrderedElemGetBase(OrderedElem elem)
 }
 
 /*
- * OrderedSetInit --
+ * OrderedSetInit
  */
 void
 OrderedSetInit(OrderedSet set, Offset offset)
@@ -49,17 +49,19 @@ OrderedSetInit(OrderedSet set, Offset offset)
 }
 
 /*
- * OrderedSetContains --
+ * OrderedSetContains
  *		True iff ordered set contains given element.
  */
+#ifdef NOT_USED
 bool
 OrderedSetContains(OrderedSet set, OrderedElem elem)
 {
 	return (bool) (elem->set == set && (elem->next || elem->prev));
 }
+#endif
 
 /*
- * OrderedSetGetHead --
+ * OrderedSetGetHead
  */
 Pointer
 OrderedSetGetHead(OrderedSet set)
@@ -73,7 +75,7 @@ OrderedSetGetHead(OrderedSet set)
 }
 
 /*
- * OrderedSetGetTail --
+ * OrderedSetGetTail
  */
 #ifdef NOT_USED
 Pointer
@@ -90,7 +92,7 @@ OrderedSetGetTail(OrderedSet set)
 #endif
 
 /*
- * OrderedElemGetPredecessor --
+ * OrderedElemGetPredecessor
  */
 Pointer
 OrderedElemGetPredecessor(OrderedElem elem)
@@ -102,7 +104,7 @@ OrderedElemGetPredecessor(OrderedElem elem)
 }
 
 /*
- * OrderedElemGetSuccessor --
+ * OrderedElemGetSuccessor
  */
 Pointer
 OrderedElemGetSuccessor(OrderedElem elem)
@@ -114,7 +116,7 @@ OrderedElemGetSuccessor(OrderedElem elem)
 }
 
 /*
- * OrderedElemPop --
+ * OrderedElemPop
  */
 void
 OrderedElemPop(OrderedElem elem)
@@ -127,7 +129,7 @@ OrderedElemPop(OrderedElem elem)
 }
 
 /*
- * OrderedElemPushInto --
+ * OrderedElemPushInto
  */
 void
 OrderedElemPushInto(OrderedElem elem, OrderedSet set)
@@ -140,7 +142,7 @@ OrderedElemPushInto(OrderedElem elem, OrderedSet set)
 }
 
 /*
- * OrderedElemPush --
+ * OrderedElemPush
  */
 static void
 OrderedElemPush(OrderedElem elem)
@@ -149,7 +151,7 @@ OrderedElemPush(OrderedElem elem)
 }
 
 /*
- * OrderedElemPushHead --
+ * OrderedElemPushHead
  */
 static void
 OrderedElemPushHead(OrderedElem elem)
