@@ -7,7 +7,7 @@
  *
  * Copyright (c) 1994, Regents of the University of California
  *
- * $Id: c.h,v 1.55 1999/05/25 16:13:22 momjian Exp $
+ * $Id: c.h,v 1.58 1999/07/01 19:47:25 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -71,8 +71,12 @@ typedef char bool;
 
 #endif	 /* ndef bool */
 #endif	 /* not C++ */
-#define false	((bool) 0)
+#ifndef true
 #define true	((bool) 1)
+#endif
+#ifndef false
+#define false	((bool) 0)
+#endif
 typedef bool *BoolPtr;
 
 #ifndef TRUE
@@ -591,7 +595,7 @@ extern int	assert_enabled;
 #define LogTrap(condition, exception, printArgs) \
 		{ if ((assert_enabled) && (condition)) \
 				ExceptionalCondition(CppAsString(condition), &(exception), \
-						form printArgs, __FILE__, __LINE__); }
+						vararg_format printArgs, __FILE__, __LINE__); }
 
 /*
  *	LogTrapMacro is the same as LogTrap but it's intended for use in macros:
@@ -602,7 +606,7 @@ extern int	assert_enabled;
 	((bool) ((! assert_enabled) || (! condition) || \
 			 (ExceptionalCondition(CppAsString(condition), \
 								   &(exception), \
-								   form printArgs, __FILE__, __LINE__))))
+								   vararg_format printArgs, __FILE__, __LINE__))))
 
 #ifndef USE_ASSERT_CHECKING
 #define LogAssert(condition, printArgs)
@@ -711,10 +715,10 @@ extern int ExceptionalCondition(char *conditionName,
 
 
 /* ----------------
- *		form is used by assert and the exception handling stuff
+ *		vararg_format is used by assert and the exception handling stuff
  * ----------------
  */
-extern char *form(const char *fmt,...);
+extern char *vararg_format(const char *fmt,...);
 
 
 
