@@ -1,13 +1,13 @@
 /*-------------------------------------------------------------------------
  *
- * pg_index.h--
+ * pg_index.h
  *	  definition of the system "index" relation (pg_index)
  *	  along with the relation's initial contents.
  *
  *
  * Copyright (c) 1994, Regents of the University of California
  *
- * $Id: pg_index.h,v 1.5 1997/09/08 02:35:12 momjian Exp $
+ * $Id: pg_index.h,v 1.12 1999/05/25 16:13:45 momjian Exp $
  *
  * NOTES
  *	  the genbki.sh script reads this file and generates .bki
@@ -48,11 +48,13 @@ CATALOG(pg_index)
 	int28		indkey;
 	oid8		indclass;
 	bool		indisclustered;
-	bool		indisarchived;
 	bool		indislossy;		/* do we fetch false tuples (lossy
 								 * compression)? */
 	bool		indhaskeytype;	/* does key type != attribute type? */
 	bool		indisunique;	/* is this a unique index? */
+	bool		indisprimary;	/* is this index for primary key */
+	Oid			indreference;	/* oid of index of referenced relation (ie
+								 * - this index for foreign key */
 	text		indpred;		/* query plan for partial index predicate */
 } FormData_pg_index;
 
@@ -64,23 +66,24 @@ CATALOG(pg_index)
  *		the format of pg_index relation.
  * ----------------
  */
-typedef FormData_pg_index *IndexTupleForm;
+typedef FormData_pg_index *Form_pg_index;
 
 /* ----------------
  *		compiler constants for pg_index
  * ----------------
  */
-#define Natts_pg_index					11
+#define Natts_pg_index					12
 #define Anum_pg_index_indexrelid		1
 #define Anum_pg_index_indrelid			2
 #define Anum_pg_index_indproc			3
 #define Anum_pg_index_indkey			4
 #define Anum_pg_index_indclass			5
 #define Anum_pg_index_indisclustered	6
-#define Anum_pg_index_indisarchived		7
-#define Anum_pg_index_indislossy		8
-#define Anum_pg_index_indhaskeytype		9
-#define Anum_pg_index_indisunique		10
-#define Anum_pg_index_indpred			11
+#define Anum_pg_index_indislossy		7
+#define Anum_pg_index_indhaskeytype		8
+#define Anum_pg_index_indisunique		9
+#define Anum_pg_index_indisprimary		10
+#define Anum_pg_index_indreference		11
+#define Anum_pg_index_indpred			12
 
-#endif							/* PG_INDEX_H */
+#endif	 /* PG_INDEX_H */
