@@ -1,13 +1,13 @@
 /*-------------------------------------------------------------------------
  *
- * execdesc.h--
+ * execdesc.h
  *	  plan and query descriptor accessor macros used by the executor
  *	  and related modules.
  *
  *
  * Copyright (c) 1994, Regents of the University of California
  *
- * $Id: execdesc.h,v 1.8 1998/02/26 04:41:17 momjian Exp $
+ * $Id: execdesc.h,v 1.10 1999/02/13 23:21:23 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -33,8 +33,7 @@ typedef struct QueryDesc
 } QueryDesc;
 
 /* in pquery.c */
-extern QueryDesc *
-CreateQueryDesc(Query *parsetree, Plan *plantree,
+extern QueryDesc *CreateQueryDesc(Query *parsetree, Plan *plantree,
 				CommandDest dest);
 
-#endif							/* EXECDESC_H  */
+#endif	 /* EXECDESC_H	*/
