@@ -1,12 +1,12 @@
 /*-------------------------------------------------------------------------
  *
- * executor.h--
+ * executor.h
  *	  support for the POSTGRES executor module
  *
  *
  * Copyright (c) 1994, Regents of the University of California
  *
- * $Id: executor.h,v 1.27 1998/10/14 05:10:05 momjian Exp $
+ * $Id: executor.h,v 1.34 1999/06/17 15:15:53 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -83,13 +83,11 @@ extern HeapTuple ExecRemoveJunk(JunkFilter *junkfilter, TupleTableSlot *slot);
  * prototypes from functions in execMain.c
  */
 extern TupleDesc ExecutorStart(QueryDesc *queryDesc, EState *estate);
-extern TupleTableSlot *ExecutorRun(QueryDesc *queryDesc, EState *estate, int feature, int count);
+extern TupleTableSlot *ExecutorRun(QueryDesc *queryDesc, EState *estate,
+			int feature, Node *limoffset, Node *limcount);
 extern void ExecutorEnd(QueryDesc *queryDesc, EState *estate);
-extern HeapTuple ExecConstraints(char *caller, Relation rel, HeapTuple tuple);
-#ifdef QUERY_LIMIT
-extern int ExecutorLimit(int limit);
-extern int ExecutorGetLimit(void);
-#endif
+extern void ExecConstraints(char *caller, Relation rel, HeapTuple tuple,
+				EState *estate);
 
 /*
  * prototypes from functions in execProcnode.c
@@ -149,7 +147,7 @@ extern TupleDesc ExecTypeFromTL(List *targetList);
 extern void SetChangedParamList(Plan *node, List *newchg);
 
 /*
- * prototypes from functions in execTuples.c
+ * prototypes from functions in execUtils.c
  */
 extern void ResetTupleCount(void);
 extern void ExecAssignNodeBaseInfo(EState *estate, CommonState *basenode,
@@ -163,6 +161,8 @@ extern void ExecAssignResultTypeFromTL(Plan *node, CommonState *commonstate);
 extern TupleDesc ExecGetResultType(CommonState *commonstate);
 extern void ExecAssignProjectionInfo(Plan *node, CommonState *commonstate);
 extern void ExecFreeProjectionInfo(CommonState *commonstate);
+extern void ExecFreeExprContext(CommonState *commonstate);
+extern void ExecFreeTypeInfo(CommonState *commonstate);
 extern TupleDesc ExecGetScanType(CommonScanState *csstate);
 extern void ExecAssignScanType(CommonScanState *csstate,
 				   TupleDesc tupDesc);
