@@ -1,24 +1,26 @@
 /*-------------------------------------------------------------------------
  *
- * nodeMergejoin.h--
+ * nodeMergejoin.h
  *
  *
  *
  * Copyright (c) 1994, Regents of the University of California
  *
- * $Id: nodeMergejoin.h,v 1.6 1997/09/08 21:52:09 momjian Exp $
+ * $Id: nodeMergejoin.h,v 1.10 1999/02/13 23:21:26 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
 #ifndef NODEMERGEJOIN_H
 #define NODEMERGEJOIN_H
 
+#include "executor/tuptable.h"
+#include "nodes/execnodes.h"
+#include "nodes/plannodes.h"
+
 extern TupleTableSlot *ExecMergeJoin(MergeJoin *node);
-
 extern bool ExecInitMergeJoin(MergeJoin *node, EState *estate, Plan *parent);
-
 extern int	ExecCountSlotsMergeJoin(MergeJoin *node);
-
 extern void ExecEndMergeJoin(MergeJoin *node);
+extern void ExecReScanMergeJoin(MergeJoin *node, ExprContext *exprCtxt, Plan *parent);
 
-#endif							/* NODEMERGEJOIN_H; */
+#endif	 /* NODEMERGEJOIN_H; */
