@@ -1,13 +1,13 @@
 /*-------------------------------------------------------------------------
  *
- * s_lock.h--
+ * s_lock.h
  *	   This file contains the implementation (if any) for spinlocks.
  *
  * Copyright (c) 1994, Regents of the University of California
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/include/storage/s_lock.h,v 1.56 1998/10/31 02:06:08 momjian Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/include/storage/s_lock.h,v 1.61 1999/06/13 00:07:43 ishii Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -68,6 +68,8 @@
 
 #include "storage/ipc.h"
 
+extern void s_lock_sleep(unsigned spin);
+
 #if defined(HAS_TEST_AND_SET)
 
 
@@ -122,6 +124,22 @@ __asm__("lock; xchgb %0,%1": "=q"(_res), "=m"(*lock):"0"(_res));
 
 
 
+#if defined(__arm32__)
+#define TAS(lock) tas(lock)
+
+static __inline__ int
+tas(volatile slock_t *lock)
+{
+        register slock_t _res = 1;
+
+__asm__("swpb %0, %0, [%3]": "=r"(_res), "=m"(*lock):"0"(_res), "r" (lock));
+        return (int) _res;
+}
+
+#endif   /* __arm32__ */
+
+
+
 #if defined(sparc)
 #define TAS(lock) tas(lock)
 
@@ -139,6 +157,24 @@ tas(volatile slock_t *lock)
 #endif	 /* sparc */
 
 
+#if defined(__mc68000__) && defined(__linux__)
+#define TAS(lock) tas(lock)
+
+static __inline__ int
+tas(volatile slock_t *lock)
+{
+	register int rv;
+	
+	__asm__ __volatile__ (
+		"tas %1; sne %0"
+		: "=d" (rv), "=m"(*lock)
+		: "1" (*lock)
+		: "cc" );
+	return rv;
+}
+
+#endif /* defined(__mc68000__) && defined(__linux__) */
+
 
 #if defined(NEED_VAX_TAS_ASM)
 /*
@@ -354,3 +390,4 @@ int			tas(volatile slock_t *lock);		/* port/.../tas.s, or
 
 #endif	 /* HAS_TEST_AND_SET */
 #endif	 /* S_LOCK_H */
+
