@@ -1,6 +1,6 @@
 /*-------------------------------------------------------------------------
  *
- * memutils.h--
+ * memutils.h
  *	  this file contains general memory alignment, allocation
  *	  and manipulation stuff that used to be spread out
  *	  between the following files:
@@ -15,7 +15,7 @@
  *
  * Copyright (c) 1994, Regents of the University of California
  *
- * $Id: memutils.h,v 1.18 1998/09/07 05:35:48 momjian Exp $
+ * $Id: memutils.h,v 1.28 1999/06/17 15:15:59 momjian Exp $
  *
  * NOTES
  *	  some of the information in this file will be moved to
@@ -27,70 +27,24 @@
 #define MEMUTILS_H
 
 
-#ifdef NOT_USED
-/*****************************************************************************
- *		align.h			- alignment macros									 *
- ****************************************************************************
- [TRH] Let the compiler decide what alignment it uses instead of
-tending
-   we know better.
-   GCC (at least v2.5.8 and up) has an __alignof__ keyword.
-   However, we cannot use it here since on some architectures it reports
-   just a _recommended_ alignment instead of the actual alignment used in
-   padding structures (or at least, this is how I understand gcc).
-   So define a macro that gives us the _actual_ alignment inside a struct.
-   {{note: assumes that alignment size is always a power of 2.}}
+/* ----------------
+ * Alignment macros: align a length or address appropriately for a given type.
+ *
+ * There used to be some incredibly crufty platform-dependent hackery here,
+ * but now we rely on the configure script to get the info for us. Much nicer.
+ *
+ * NOTE: TYPEALIGN will not work if ALIGNVAL is not a power of 2.
+ * That case seems extremely unlikely to occur in practice, however.
+ * ----------------
  */
-#define _ALIGNSIZE(TYPE)		offsetof(struct { char __c; TYPE __t;}, __t)
-#define _ALIGN(TYPE, LEN) \
-		(((long)(LEN) + (_ALIGNSIZE(TYPE) - 1)) & ~(_ALIGNSIZE(TYPE) - 1))
-#define SHORTALIGN(LEN)			_ALIGN(short, (LEN))
-#define INTALIGN(LEN)			_ALIGN(int, (LEN))
-#define LONGALIGN(LEN)			_ALIGN(long, (LEN))
-#define DOUBLEALIGN(LEN)		_ALIGN(double, (LEN))
-#define MAXALIGN(LEN)			_ALIGN(double, (LEN))
 
-#endif	 /* 0 */
+#define TYPEALIGN(ALIGNVAL,LEN)	(((long)(LEN) + (ALIGNVAL-1)) & ~(ALIGNVAL-1))
 
-/*
- *		SHORTALIGN(LEN) - length (or address) aligned for shorts
- */
-#define SHORTALIGN(LEN)\
-		(((long)(LEN) + (sizeof (short) - 1)) & ~(sizeof (short) - 1))
-
-#define INTALIGN(LEN)\
-		(((long)(LEN) + (sizeof (int) - 1)) & ~(sizeof (int) -1))
-
-/*
- *		LONGALIGN(LEN)	- length (or address) aligned for longs
- */
-#if defined(sun) && ! defined(sparc)
-#define LONGALIGN(LEN)	SHORTALIGN(LEN)
-#elif defined (__alpha)
-
- /*
-  * even though "long alignment" should really be on 8-byte boundaries for
-  * linuxalpha, we want the strictest alignment to be on 4-byte (int)
-  * boundaries, because otherwise things break when they try to use the
-  * FormData_pg_* structures.  --djm 12/12/96
-  */
-#define LONGALIGN(LEN)\
-		(((long)(LEN) + (sizeof (int) - 1)) & ~(sizeof (int) -1))
-#else
-#define LONGALIGN(LEN)\
-		(((long)(LEN) + (sizeof (long) - 1)) & ~(sizeof (long) -1))
-#endif
-
-#if ! defined(sco)
-#define DOUBLEALIGN(LEN)\
-		(((long)(LEN) + (sizeof (double) - 1)) & ~(sizeof (double) -1))
-
-#define MAXALIGN(LEN)\
-		(((long)(LEN) + (sizeof (double) - 1)) & ~(sizeof (double) -1))
-#else
-#define DOUBLEALIGN(LEN) INTALIGN(LEN)
-#define MAXALIGN(LEN)	 INTALIGN(LEN)
-#endif
+#define SHORTALIGN(LEN)			TYPEALIGN(ALIGNOF_SHORT, (LEN))
+#define INTALIGN(LEN)			TYPEALIGN(ALIGNOF_INT, (LEN))
+#define LONGALIGN(LEN)			TYPEALIGN(ALIGNOF_LONG, (LEN))
+#define DOUBLEALIGN(LEN)		TYPEALIGN(ALIGNOF_DOUBLE, (LEN))
+#define MAXALIGN(LEN)			TYPEALIGN(MAXIMUM_ALIGNOF, (LEN))
 
 /*****************************************************************************
  *	  oset.h --			Fixed format ordered set definitions.				 *
@@ -126,7 +80,6 @@ struct OrderedSetData
 };
 
 extern void OrderedSetInit(OrderedSet set, Offset offset);
-extern bool OrderedSetContains(OrderedSet set, OrderedElem elem);
 extern Pointer OrderedSetGetHead(OrderedSet set);
 extern Pointer OrderedElemGetPredecessor(OrderedElem elem);
 extern Pointer OrderedElemGetSuccessor(OrderedElem elem);
@@ -142,7 +95,7 @@ extern void OrderedElemPushInto(OrderedElem elem, OrderedSet Set);
  *		an allocation is requested for a set, memory is allocated and a
  *		pointer is returned.  Subsequently, this memory may be freed or
  *		reallocated.  In addition, an allocation set may be reset which
- *		will cause all allocated memory to be freed.
+ *		will cause all memory allocated within it to be freed.
  *
  *		Allocations may occur in four different modes.	The mode of
  *		allocation does not affect the behavior of allocations except in
@@ -155,7 +108,7 @@ extern void OrderedElemPushInto(OrderedElem elem, OrderedSet Set);
  *		and freed very frequently.	This is a good choice when allocation
  *		characteristics are unknown.  This is the default mode.
  *
- *		"Static" mode attemts to allocate space as efficiently as possible
+ *		"Static" mode attempts to allocate space as efficiently as possible
  *		without regard to freeing memory.  This mode should be chosen only
  *		when it is known that many allocations will occur but that very
  *		little of the allocated memory will be explicitly freed.
@@ -175,17 +128,17 @@ extern void OrderedElemPushInto(OrderedElem elem, OrderedSet Set);
  *		Allocation sets are not automatically reset on a system reset.
  *		Higher level code is responsible for cleaning up.
  *
- *		There may other modes in the future.
+ *		There may be other modes in the future.
  */
 
 /*
- * AllocPointer --
+ * AllocPointer
  *		Aligned pointer which may be a member of an allocation set.
  */
 typedef Pointer AllocPointer;
 
 /*
- * AllocMode --
+ * AllocMode
  *		Mode of allocation for an allocation set.
  *
  * Note:
@@ -202,25 +155,58 @@ typedef enum AllocMode
 #define DefaultAllocMode		DynamicAllocMode
 
 /*
- * AllocSet --
+ * AllocBlock
+ *		Small pieces of memory are taken from bigger blocks of
+ *		memory with a size aligned to a power of two. These
+ *		pieces are not free's separately, instead they are reused
+ *		for the next allocation of a fitting size.
+ */
+typedef struct AllocBlockData
+{
+	struct AllocSetData *aset;
+	struct AllocBlockData *next;
+	char	   *freeptr;
+	char	   *endptr;
+} AllocBlockData;
+
+typedef AllocBlockData *AllocBlock;
+
+/*
+ * AllocChunk
+ *		The prefix of each piece of memory in an AllocBlock
+ */
+typedef struct AllocChunkData
+{
+	/* aset is the owning aset if allocated, or the freelist link if free */
+	void	   *aset;
+	/* size is always the chunk size */
+	Size		size;
+} AllocChunkData;
+
+typedef AllocChunkData *AllocChunk;
+
+/*
+ * AllocSet
  *		Allocation set.
  */
 typedef struct AllocSetData
 {
-	OrderedSetData setData;
+	struct AllocBlockData *blocks;
+#define ALLOCSET_NUM_FREELISTS	8
+	struct AllocChunkData *freelist[ALLOCSET_NUM_FREELISTS];
 	/* Note: this will change in the future to support other modes */
 } AllocSetData;
 
 typedef AllocSetData *AllocSet;
 
 /*
- * AllocPointerIsValid --
+ * AllocPointerIsValid
  *		True iff pointer is valid allocation pointer.
  */
 #define AllocPointerIsValid(pointer) PointerIsValid(pointer)
 
 /*
- * AllocSetIsValid --
+ * AllocSetIsValid
  *		True iff set is valid allocation set.
  */
 #define AllocSetIsValid(set) PointerIsValid(set)
@@ -253,7 +239,7 @@ extern void AllocSetDump(AllocSet set);
 typedef int LibCCopyLength;
 
 /*
- * MemoryCopy --
+ * MemoryCopy
  *		Copies fixed length block of memory to another.
  */
 #define MemoryCopy(toBuffer, fromBuffer, length)\
