@@ -1,12 +1,12 @@
 /*-------------------------------------------------------------------------
  *
- * rel.h--
+ * rel.h
  *	  POSTGRES relation descriptor definitions.
  *
  *
  * Copyright (c) 1994, Regents of the University of California
  *
- * $Id: rel.h,v 1.20 1998/09/01 04:39:29 momjian Exp $
+ * $Id: rel.h,v 1.23 1999/05/25 16:14:57 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -49,10 +49,10 @@ typedef struct RelationData
 	File		rd_fd;			/* open file descriptor */
 	int			rd_nblocks;		/* number of blocks in rel */
 	uint16		rd_refcnt;		/* reference count */
-	bool		rd_islocal;		/* uses the local buffer mgr */
+	bool		rd_myxactonly;	/* uses the local buffer mgr */
 	bool		rd_isnailed;	/* rel is nailed in cache */
-	bool		rd_istemp;		/* rel is a temp rel */
-	bool		rd_tmpunlinked; /* temp rel already unlinked */
+	bool		rd_isnoname;	/* rel has no name */
+	bool		rd_nonameunlinked;		/* noname rel already unlinked */
 	Form_pg_am	rd_am;			/* AM tuple */
 	Form_pg_class rd_rel;		/* RELATION tuple */
 	Oid			rd_id;			/* relations's object id */
@@ -77,13 +77,13 @@ typedef Relation *RelationPtr;
 #define InvalidRelation ((Relation)NULL)
 
 /*
- * RelationIsValid --
+ * RelationIsValid
  *		True iff relation descriptor is valid.
  */
 #define RelationIsValid(relation) PointerIsValid(relation)
 
 /*
- * RelationGetSystemPort --
+ * RelationGetSystemPort
  *		Returns system port of a relation.
  *
  * Note:
@@ -92,14 +92,14 @@ typedef Relation *RelationPtr;
 #define RelationGetSystemPort(relation) ((relation)->rd_fd)
 
 /*
- * RelationGetLockInfo --
+ * RelationGetLockInfo
  *		Returns the lock information structure in the reldesc
  *
  */
 #define RelationGetLockInfo(relation) ((relation)->lockInfo)
 
 /*
- * RelationHasReferenceCountZero --
+ * RelationHasReferenceCountZero
  *		True iff relation reference count is zero.
  *
  * Note:
@@ -109,25 +109,25 @@ typedef Relation *RelationPtr;
 		((bool)((relation)->rd_refcnt == 0))
 
 /*
- * RelationSetReferenceCount --
+ * RelationSetReferenceCount
  *		Sets relation reference count.
  */
 #define RelationSetReferenceCount(relation,count) ((relation)->rd_refcnt = count)
 
 /*
- * RelationIncrementReferenceCount --
+ * RelationIncrementReferenceCount
  *		Increments relation reference count.
  */
 #define RelationIncrementReferenceCount(relation) ((relation)->rd_refcnt += 1);
 
 /*
- * RelationDecrementReferenceCount --
+ * RelationDecrementReferenceCount
  *		Decrements relation reference count.
  */
 #define RelationDecrementReferenceCount(relation) ((relation)->rd_refcnt -= 1)
 
 /*
- * RelationGetForm --
+ * RelationGetForm
  *		Returns relation attribute values for a relation.
  *
  * Note:
@@ -137,7 +137,7 @@ typedef Relation *RelationPtr;
 
 
 /*
- * RelationGetRelid --
+ * RelationGetRelid
  *
  *	returns the object id of the relation
  *
@@ -145,7 +145,7 @@ typedef Relation *RelationPtr;
 #define RelationGetRelid(relation) ((relation)->rd_id)
 
 /*
- * RelationGetFile --
+ * RelationGetFile
  *
  *	  Returns the open File decscriptor
  */
@@ -153,21 +153,21 @@ typedef Relation *RelationPtr;
 
 
 /*
- * RelationGetRelationName --
+ * RelationGetRelationName
  *
  *	  Returns a Relation Name
  */
 #define RelationGetRelationName(relation) (&(relation)->rd_rel->relname)
 
 /*
- * RelationGetRelationName --
+ * RelationGetRelationName
  *
  *	  Returns a the number of attributes.
  */
 #define RelationGetNumberOfAttributes(relation) ((relation)->rd_rel->relnatts)
 
 /*
- * RelationGetDescr --
+ * RelationGetDescr
  *		Returns tuple descriptor for a relation.
  *
  * Note:
