@@ -1,13 +1,13 @@
 /*-------------------------------------------------------------------------
  *
- * lotest.c--
+ * lotest.c
  *	  test using large objects with libpq
  *
  * Copyright (c) 1994, Regents of the University of California
  *
  *
  * IDENTIFICATION
- *	  $Header: /usr/local/cvsroot/pgsql/src/test/examples/testlo2.c,v 1.8 1998/06/15 19:30:31 momjian Exp $
+ *	  $Header: /usr/local/cvsroot/pgsql/src/test/examples/testlo2.c,v 1.12 1999/05/25 16:15:22 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -79,10 +79,7 @@ pickout(PGconn *conn, Oid lobjId, int start, int len)
 
 	lobj_fd = lo_open(conn, lobjId, INV_READ);
 	if (lobj_fd < 0)
-	{
-		fprintf(stderr, "can't open large object %d",
-				lobjId);
-	}
+		fprintf(stderr, "can't open large object %u", lobjId);
 
 	lo_lseek(conn, lobj_fd, start, SEEK_SET);
 	buf = malloc(len + 1);
@@ -110,10 +107,7 @@ overwrite(PGconn *conn, Oid lobjId, int start, int len)
 
 	lobj_fd = lo_open(conn, lobjId, INV_READ);
 	if (lobj_fd < 0)
-	{
-		fprintf(stderr, "can't open large object %d",
-				lobjId);
-	}
+		fprintf(stderr, "can't open large object %u", lobjId);
 
 	lo_lseek(conn, lobj_fd, start, SEEK_SET);
 	buf = malloc(len + 1);
@@ -152,10 +146,7 @@ exportFile(PGconn *conn, Oid lobjId, char *filename)
 	 */
 	lobj_fd = lo_open(conn, lobjId, INV_READ);
 	if (lobj_fd < 0)
-	{
-		fprintf(stderr, "can't open large object %d",
-				lobjId);
-	}
+		fprintf(stderr, "can't open large object %u", lobjId);
 
 	/*
 	 * open the file to be written to
@@ -233,15 +224,14 @@ main(int argc, char **argv)
 	printf("importing file \"%s\" ...\n", in_filename);
 /*	  lobjOid = importFile(conn, in_filename); */
 	lobjOid = lo_import(conn, in_filename);
-/*
-	printf("\tas large object %d.\n", lobjOid);
+
+	printf("\tas large object %u.\n", lobjOid);
 
 	printf("picking out bytes 1000-2000 of the large object\n");
 	pickout(conn, lobjOid, 1000, 1000);
 
 	printf("overwriting bytes 1000-2000 of the large object with X's\n");
 	overwrite(conn, lobjOid, 1000, 1000);
-*/
 
 	printf("exporting large object to file \"%s\" ...\n", out_filename);
 /*	  exportFile(conn, lobjOid, out_filename); */
